@@ -250,8 +250,8 @@ proc mkAuthor {} {
   set w .author.frame
 
   label $w.msg0   -text "Author and Maintainer of xlock"
-  label $w.msg1   -text "Maintained by: David A. Bagley (bagleyd@tux.org)"
-  label $w.msg2   -text "Original Author: Patrick J. Naughton (naughton@eng.sun.com)"
+  label $w.msg1   -text "Maintained by: David A. Bagley (bagleyd AT verizon.net)"
+  label $w.msg2   -text "Original Author: Patrick J. Naughton (naughton AT eng.sun.com)"
   label $w.msg3   -text "Mailstop 21-14 Sun Microsystems Laboratories,"
   label $w.msg4   -text "Inc.  Mountain View, CA  94043 15//336-1080"
   label $w.msg5   -text "with many additional contributors"
@@ -650,9 +650,12 @@ listbox $LISTSCROL.list -yscroll  "$LISTSCROL.scroll set"
 #insert all modes in list
 #---------------------------
 $LISTSCROL.list  insert 0 \
+anemone\
 ant\
+ant3d\
 apollonian\
 atlantis\
+atunnels\
 ball\
 bat\
 blot\
@@ -672,17 +675,19 @@ decay\
 deco\
 demon\
 dilemma\
-dragon\
 discrete\
+dragon\
 drift\
 euler2d\
 eyes\
 fadeplot\
+fiberlamp\
 fire\
 flag\
 flame\
 flow\
 forest\
+fzort\
 galaxy\
 gears\
 glplanet\
@@ -696,6 +701,7 @@ ifs\
 image\
 invert\
 juggle\
+juggler3d\
 julia\
 kaleid\
 kumppa\
@@ -718,15 +724,19 @@ molecule\
 morph3d\
 mountain\
 munch\
+noof\
 nose\
 pacman\
 penrose\
 petal\
+petri\
 pipes\
 polyominoes\
 puzzle\
 pyro\
+pyro2\
 qix\
+rain\
 roll\
 rotor\
 rubik\
@@ -737,7 +747,7 @@ sierpinski\
 sierpinski3d\
 skewb\
 slip\
-solitare\
+solitaire\
 space\
 sphere\
 spiral\
@@ -753,8 +763,10 @@ swirl\
 t3d\
 tetris\
 text3d\
+text3d2\
 thornbird\
 tik_tak\
+toneclock\
 triangle\
 tube\
 turtle\
