@@ -27,7 +27,7 @@ static const char sccsid[] = "@(#)bat.c	5.00 2000/11/01 xlockmore";
  * 18-Sep-1995: 5 bats now in color <patol@info.isbiel.ch>
  * 20-Sep-1994: 5 bats instead of bouncing balls, based on bounce.c
  *              <patol@info.isbiel.ch>
- * 02-Sep-1993: bounce version David Bagley <bagleyd@tux.org>
+ * 02-Sep-1993: bounce version David Bagley <bagleyd AT verizon.net>
  * 1986: Sun Microsystems
  */
 
@@ -59,6 +59,7 @@ static const char sccsid[] = "@(#)bat.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_bat
 #define PROGCLASS "Bat"
 #define HACK_INIT init_bat
 #define HACK_DRAW draw_bat
@@ -90,12 +91,8 @@ const ModStruct bat_description =
 
 #endif
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
-#if USE_XPMINC
-#include <xpm.h>
-#else
-#include <X11/xpm.h>		/* Normal spot */
-#endif
+#ifdef HAVE_XPM
+#include <X11/xpm.h>
 #include "pixmaps/bat-0.xpm"
 #include "pixmaps/bat-1.xpm"
 #include "pixmaps/bat-2.xpm"
@@ -116,7 +113,7 @@ const ModStruct bat_description =
 
 #include "bat.xbm"
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 #define BAT_NAME      image_name
 #include "bat.xpm"
 #define DEFAULT_XPM 1
@@ -180,7 +177,7 @@ static unsigned char *bits[] =
 	bat0_bits, bat1_bits, bat2_bits, bat3_bits, bat4_bits
 };
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 static char **pixs[] =
 {
 	bat0, bat1, bat2, bat3, bat4
@@ -346,7 +343,7 @@ collide(bouncestruct * bp, int a_bat)
 static void
 free_stuff(Display * display, bouncestruct * bp)
 {
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 	if (bp->graphics_format == IS_XPM) {
 		int         i;
 
@@ -379,14 +376,14 @@ free_bat(Display *display, bouncestruct *bp)
 {
 	free_stuff(display, bp);
 	if (bp->bats != NULL) {
-		(void) free((void *) bp->bats);
+		free(bp->bats);
 		bp->bats = (batstruct *) NULL;
 	}
 	if (bp->pixmap != None) {
 		XFreePixmap(display, bp->pixmap);
 		bp->pixmap = None;
 	}
-		
+
 }
 
 static Bool
@@ -400,7 +397,7 @@ init_stuff(ModeInfo * mi)
 	if (MI_BITMAP(mi) && strlen(MI_BITMAP(mi))) {
 		if (bp->logo == None) {
                 getImage(mi, &bp->logo, BAT_WIDTH, BAT_HEIGHT, BAT_BITS,
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
                          DEFAULT_XPM, BAT_NAME,
 #endif
                          &bp->graphics_format, &bp->cmap, &bp->black);
@@ -411,10 +408,10 @@ init_stuff(ModeInfo * mi)
 		}
 	} else {
 	if (!bp->cmap) {
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 		int         total = 0;
 
-		if (!MI_IS_FULLRANDOM(mi) || LRAND() & 1) {
+		if (!MI_IS_FULLRANDOM(mi) || (LRAND() & 1)) {
 			XpmAttributes attrib;
 
 #ifndef STANDALONE
@@ -529,7 +526,7 @@ init_bat(ModeInfo * mi)
 	if (bp->nbats < -MINBATS) {
 		/* if bp->nbats is random ... the size can change */
 		if (bp->bats != NULL) {
-			(void) free((void *) bp->bats);
+			free(bp->bats);
 			bp->bats = (batstruct *)NULL;
 		}
 		bp->nbats = NRAND(-bp->nbats - MINBATS + 1) + MINBATS;
@@ -684,7 +681,7 @@ release_bat(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_bat(MI_DISPLAY(mi), &bounces[screen]);
-		(void) free((void *) bounces);
+		free(bounces);
 		bounces = (bouncestruct *) NULL;
 	}
 }
@@ -700,7 +697,7 @@ refresh_bat(ModeInfo * mi)
 	if (bp->bats == NULL)
 		return;
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 	/* This is only needed when another program changes the colormap. */
 	if (MI_BITMAP(mi) && strlen(MI_BITMAP(mi))) {
 		init_bat(mi);
