@@ -23,13 +23,13 @@ static const char sccsid[] = "@(#)bouboule.c	5.00 2000/11/01 xlockmore";
  *
  * Revision History:
  * 01-Nov-2000: Allocation checks
- * 15-May-1997: jwz@jwz.org: turned into a standalone program.
+ * 15-May-1997: jwz AT jwz.org: turned into a standalone program.
  * 04-Sep-1996: Added 3d support Henrik Theiling <theiling@coli-uni-sb.de>
  * 20-Feb-1996: Added tests so that already malloced objects are not
  *              malloced twice, thanks to the report from
  *              <mccomb@interport.net>
- * 01-Feb-1996: Patched by Jouk Jansen <joukj@hrem.stm.tudelft.nl> for VMS
- *              Patched by <bagleyd@tux.org> for TrueColor displays
+ * 01-Feb-1996: Patched by Jouk Jansen <joukj AT hrem.nano.tudelft.nl> for VMS
+ *              Patched by <bagleyd AT verizon.net> for TrueColor displays
  * 30-Jan-1996: Wrote all that I wanted to.
  *
  * Sort of starfield with a 3D engine.  For a real starfield, I only scale
@@ -69,6 +69,7 @@ static const char sccsid[] = "@(#)bouboule.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_bouboule
 #define PROGCLASS "Bouboule"
 #define HACK_INIT init_bouboule
 #define HACK_DRAW draw_bouboule
@@ -312,7 +313,7 @@ sinfree(SinVariable * point)
 	while (next) {
 		temp = next;
 		next = temp->varrand;
-		(void) free((void *) temp);
+		free(temp);
 	}
 }
 
@@ -320,24 +321,24 @@ static void
 free_stars(StarField *sp)
 {
 	if (sp->star != NULL) {
-		(void) free((void *) sp->star);
+		free(sp->star);
 		sp->star = (Star *) NULL;
 	}
 	if (sp->xarc != NULL) {
-		(void) free((void *) sp->xarc);
+		free(sp->xarc);
 		sp->xarc = (XArc *) NULL;
 	}
 	if (sp->xarcleft != NULL) {
-		(void) free((void *) sp->xarcleft);
+		free(sp->xarcleft);
 		sp->xarcleft = (XArc *) NULL;
 	}
 #if ((USEOLDXARCS == 1) || (ADAPT_ERASE == 1))
 	if (sp->oldxarc != NULL) {
-		(void) free((void *) sp->oldxarc);
+		free(sp->oldxarc);
 		sp->oldxarc = (XArc *) NULL;
 	}
 	if (sp->oldxarcleft != NULL) {
-		(void) free((void *) sp->oldxarcleft);
+		free(sp->oldxarcleft);
 		sp->oldxarcleft = (XArc *) NULL;
 	}
 #endif
@@ -772,10 +773,10 @@ draw_bouboule(ModeInfo * mi)
 		if (sp->xarc_time >
 		    ADAPT_ARC_PREFERED * sp->rect_time) {
 			sp->hasbeenchecked = -2;	/* XFillRectangle mode */
-			(void) free((void *) sp->oldxarc);
+			free(sp->oldxarc);
 			sp->oldxarc = (XArc *) NULL;
 			if (MI_IS_USE3D(mi)) {
-				(void) free((void *) sp->oldxarcleft);
+				free(sp->oldxarcleft);
 				sp->oldxarcleft = (XArc *) NULL;
 			}
 		} else {
@@ -891,7 +892,7 @@ release_bouboule(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_bouboule(&starfield[screen]);
-		(void) free((void *) starfield);
+		free(starfield);
 		starfield = (StarField *) NULL;
 	}
 }
