@@ -40,7 +40,7 @@ static const char sccsid[] = "@(#)bounce.c	5.00 2000/11/01 xlockmore";
  *              <iarit@tara.mines.colorado.edu>
  * 22-Mar-1994: got rid of flashing problem by only erasing parts of the
  *              image that will not be covered up by the next image.
- * 02-Sep-1993: xlock version David Bagley <bagleyd@tux.org>
+ * 02-Sep-1993: xlock version David Bagley <bagleyd AT verizon.net>
  * 1986: Sun Microsystems
  */
 
@@ -81,6 +81,7 @@ static const char sccsid[] = "@(#)bounce.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_bounce
 #define PROGCLASS "Bounce"
 #define HACK_INIT init_bounce
 #define HACK_DRAW draw_bounce
@@ -92,7 +93,6 @@ static const char sccsid[] = "@(#)bounce.c	5.00 2000/11/01 xlockmore";
 #include "xlockmore.h"		/* in xscreensaver distribution */
 #else /* STANDALONE */
 #include "xlock.h"		/* in xlockmore distribution */
-#include "vis.h"
 #include "color.h"
 #endif /* STANDALONE */
 #include "iostuff.h"
@@ -129,7 +129,7 @@ const ModStruct bounce_description =
 
 #include "bounce.xbm"
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 #define BOUNCE_NAME      image_name
 #include "bounce.xpm"
 #define DEFAULT_XPM 1
@@ -257,11 +257,12 @@ drawball(ModeInfo * mi, ballstruct * ball)
 			XSetFillStyle(display, bp->stippledGC, FillStippled);
 			XSetTSOrigin(display, bp->stippledGC,
 				ball->xlast, ball->ylast);
+			XSetForeground(display, MI_GC(mi), MI_BLACK_PIXEL(mi));
 #ifdef FLASH
-			XFillRectangle(display, window, bp->stippledGC,
+			XFillRectangle(display, window, MI_GC(mi),
 				ball->xlast, ball->ylast, bp->xs, bp->ys);
 #else
-			ERASE_IMAGE(display, window, bp->stippledGC,
+			ERASE_IMAGE(display, window, MI_GC(mi),
 				ball->x, ball->y, ball->xlast, ball->ylast,
 				bp->xs, bp->ys);
 #endif
@@ -442,12 +443,12 @@ bounce_windows(ModeInfo * mi, bouncestruct * bp)
 	}
 	bp->nwindow = nchildren;
 	if (bp->windows != NULL)
-		(void) free((void *) bp->windows);
+		free(bp->windows);
 	if ((bp->windows = (ballwindow *) malloc(bp->nwindow *
 			sizeof (ballwindow))) == NULL) {
 		XFree((caddr_t) children);
 		bp->nwindow = 0;
-		return; 
+		return;
 	}
 	for (n = 0, i = 0; i < bp->nwindow; i++) {
 		XWindowAttributes att;
@@ -460,7 +461,7 @@ bounce_windows(ModeInfo * mi, bouncestruct * bp)
 		if (XGetWindowAttributes(MI_DISPLAY(mi), children[i], &att) == 0) {	/* failure */
 			XFree((caddr_t) children);
 			bp->nwindow = 0;
-			(void) free((void *) bp->windows);
+			free(bp->windows);
 			bp->windows = (ballwindow *) NULL;
 			return;
 		}
@@ -516,7 +517,7 @@ static void
 free_bounce(Display *display, bouncestruct *bp)
 {
 	if (bp->balls != NULL) {
-		(void) free((void *) bp->balls);
+		free(bp->balls);
 		bp->balls = (ballstruct *) NULL;
 	}
 	free_stuff(display, bp);
@@ -525,7 +526,7 @@ free_bounce(Display *display, bouncestruct *bp)
 		bp->stippledGC = None;
 	}
 	if (bp->windows != NULL) {
-		(void) free((void *) bp->windows);
+		free(bp->windows);
 		bp->windows = (ballwindow *) NULL;
 	}
 
@@ -541,7 +542,7 @@ init_stuff(ModeInfo * mi)
         if (MI_BITMAP(mi) && strlen(MI_BITMAP(mi))) {
                 if (bp->logo == None) {
                   getImage(mi, &bp->logo, BOUNCE_WIDTH, BOUNCE_HEIGHT, BOUNCE_BITS,
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
                          DEFAULT_XPM, BOUNCE_NAME,
 #endif
                          &bp->graphics_format, &bp->cmap, &bp->black);
@@ -607,7 +608,7 @@ init_bounce(ModeInfo * mi)
 	if (bp->nballs < -MINBALLS) {
 		/* if bp->nballs is random ... the size can change */
 		if (bp->balls != NULL) {
-			(void) free((void *) bp->balls);
+			free(bp->balls);
 			bp->balls = (ballstruct *) NULL;
 		}
 		bp->nballs = NRAND(-bp->nballs - MINBALLS + 1) + MINBALLS;
@@ -753,7 +754,7 @@ release_bounce(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_bounce(MI_DISPLAY(mi), &bounces[screen]);
-		(void) free((void *) bounces);
+		free(bounces);
 		bounces = (bouncestruct *) NULL;
 	}
 }
