@@ -26,12 +26,13 @@ static const char sccsid[] = "@(#)braid.c	5.00 2000/11/01 xlockmore";
  *
  * Revision History:
  * 01-Nov-2000: Allocation checks
- * 10-May-1997: Jamie Zawinski <jwz@jwz.org> compatible with xscreensaver
+ * 10-May-1997: Jamie Zawinski <jwz AT jwz.org> compatible with xscreensaver
  * 01-Sep-1995: color knotted components differently, J. Neil.
  * 29-Aug-1995: Written.  John Neil <neil@math.idbsu.edu>
  */
 
 #ifdef STANDALONE
+#define MODE_braid
 #define PROGCLASS "Braid"
 #define HACK_INIT init_braid
 #define HACK_DRAW draw_braid
@@ -436,7 +437,7 @@ void
 release_braid(ModeInfo * mi)
 {
 	if (braids != NULL) {
-		(void) free((void *) braids);
+		free(braids);
 		braids = (braidtype *) NULL;
 	}
 }
