@@ -27,13 +27,14 @@ static const char sccsid[] = "@(#)clock.c	5.00 2000/11/01 xlockmore";
  * Revision History:
  * 01-Nov-2000: Allocation checks
  * 10-May-1997: Compatible with xscreensaver
- * 24-Dec-1995: Ron Hitchens <ron@idiom.com> cycles range and defaults
+ * 24-Dec-1995: Ron Hitchens <ron AT idiom.com> cycles range and defaults
  *              changed.
  * 01-Dec-1995: Clock size changes each time it is displayed.
  * 01-Jun-1995: Written.
  */
 
 #ifdef STANDALONE
+#define MODE_clock
 #define PROGCLASS "Clock"
 #define HACK_INIT init_clock
 #define HACK_DRAW draw_clock
@@ -368,7 +369,7 @@ init_clock(ModeInfo * mi)
 	if (cp->nclocks < -MINCLOCKS) {
 		/* if cp->nclocks is random ... the size can change */
 		if (cp->oclocks != NULL) {
-			(void) free((void *) cp->oclocks);
+			free(cp->oclocks);
 			cp->oclocks = (oclock *) NULL;
 		}
 		cp->nclocks = NRAND(-cp->nclocks - MINCLOCKS + 1) + MINCLOCKS;
@@ -451,11 +452,11 @@ release_clock(ModeInfo * mi)
 			clockstruct *cp = &clocks[screen];
 
 			if (cp->oclocks != NULL) {
-				(void) free((void *) cp->oclocks);
+				free(cp->oclocks);
 				/* cp->oclocks = (oclock *) NULL; */
 			}
 		}
-		(void) free((void *) clocks);
+		free(clocks);
 		clocks = (clockstruct *) NULL;
 	}
 }
