@@ -22,7 +22,7 @@ static const char sccsid[] = "@(#)coral.c	5.00 2000/11/01 xlockmore";
  *
  * Revision History:
  * 01-Nov-2000: Allocation checks
- * 29-Oct-1997: xlock version (David Bagley <bagleyd@tux.org>)
+ * 29-Oct-1997: xlock version (David Bagley <bagleyd AT verizon.net>)
  * 15-Jul-1997: xscreensaver version Frederick G.M. Roeber
  *              <roeber@netscape.com>
  */
@@ -41,6 +41,7 @@ static const char sccsid[] = "@(#)coral.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_coral
 #define PROGCLASS "Coral"
 #define HACK_INIT init_coral
 #define HACK_DRAW draw_coral
@@ -90,7 +91,7 @@ typedef struct {
 
 static coralstruct *reefs = (coralstruct *) NULL;
 
-#define getdot(x,y) (cp->reef[(y*cp->widthb)+(x>>5)] &  (1<<(x & 31)))
+#define getdot(x,y) (cp->reef[(y*cp->widthb)+(x>>5)] & (1<<(x & 31)))
 /* Avoid array bounds reads and writes */
 #define setdot(x,y) (cp->reef[((y*cp->widthb)+(x>>5) >= 0 ? (y*cp->widthb)+(x>>5) : 0)] |= (1<<(x & 31)))
 
@@ -124,15 +125,15 @@ free_coral(coralstruct *cp)
 {
 
 	if (cp->reef != NULL) {
-		(void) free((void *) cp->reef);
+		free(cp->reef);
 		cp->reef = (unsigned int *) NULL;
 	}
 	if (cp->walkers != NULL) {
-		(void) free((void *) cp->walkers);
+		free(cp->walkers);
 		cp->walkers = (XPoint *) NULL;
 	}
 	if (cp->pointbuf != NULL) {
-		(void) free((void *) cp->pointbuf);
+		free(cp->pointbuf);
 		cp->pointbuf = (XPoint *) NULL;
 	}
 }
@@ -156,13 +157,12 @@ init_coral(ModeInfo * mi)
 	cp = &reefs[MI_SCREEN(mi)];
 
 
-	cp->width = MAX(MI_WIDTH(mi), 2);
-	cp->height = MAX(MI_HEIGHT(mi), 2);
+	cp->width = MAX(MI_WIDTH(mi), 4);
+	cp->height = MAX(MI_HEIGHT(mi), 4);
 
 	cp->widthb = ((cp->width + 31) >> 5);
-
 	if (cp->reef != NULL)
-		(void) free((void *) cp->reef);
+		free(cp->reef);
 	if ((cp->reef = (unsigned int *) calloc((cp->widthb + 1) * cp->height,
 			sizeof (unsigned int))) == NULL) {
 		free_coral(cp);
@@ -176,9 +176,9 @@ init_coral(ModeInfo * mi)
 	else
 		cp->density = MIN(MAXSIZE, size);
 
-	cp->nwalkers = (cp->width * cp->height * cp->density) / 100;
+	cp->nwalkers = MAX((cp->width * cp->height * cp->density) / 100, 1);
 	if (cp->walkers != NULL)
-		(void) free((void *) cp->walkers);
+		free(cp->walkers);
 	if ((cp->walkers = (XPoint *) calloc(cp->nwalkers,
 			sizeof (XPoint))) == NULL) {
 		free_coral(cp);
@@ -224,7 +224,7 @@ init_coral(ModeInfo * mi)
 		cp->walkers[i].y = NRAND(cp->height - 2) + 1;
 	}
 	if (cp->pointbuf) {
-		(void) free((void *) cp->pointbuf);
+		free(cp->pointbuf);
 	}
 	if ((cp->pointbuf = (XPoint *) calloc((MAXPOINTS + 2),
 			sizeof (XPoint))) == NULL) {
@@ -249,7 +249,7 @@ draw_coral(ModeInfo * mi)
 	cp = &reefs[MI_SCREEN(mi)];
 	if (cp->reef == NULL)
 		return;
-	
+
 	MI_IS_DRAWN(mi) = True;
 	for (i = 0; i < cp->nwalkers; i++) {
 		int         x = cp->walkers[i].x;
@@ -289,7 +289,7 @@ draw_coral(ModeInfo * mi)
 
 			if (0 == cp->nwalkers) {
 				if (cp->pointbuf)
-					(void) free((void *) cp->pointbuf);
+					free(cp->pointbuf);
 				cp->pointbuf = 0;
 				init_coral(mi);
 				return;
@@ -335,7 +335,7 @@ release_coral(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_coral(&reefs[screen]);
-		(void) free((void *) reefs);
+		free(reefs);
 		reefs = (coralstruct *) NULL;
 	}
 }
