@@ -2,12 +2,12 @@
 /* crystal --- polygons moving according to plane group rules */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)crystal.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)crystal.c	5.24 2007/01/18 xlockmore";
 
 #endif
 
 /*-
- * Copyright (c) 1997 by Jouk Jansen <joukj@hrem.stm.tudelft.nl>
+ * Copyright (c) 1997 by Jouk Jansen <joukj AT hrem.nano.tudelft.nl>
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted,
@@ -33,6 +33,14 @@ static const char sccsid[] = "@(#)crystal.c	5.00 2000/11/01 xlockmore";
  *   hexagonal groups 12 through 17 where the angle gamma is 120 degrees
  *
  * Revision History:
+ * 18-Jan-2007: Added vertical option.
+ *   Crystals (particularly the parallelagram ones) favor the lower
+ *     right corner in small windows.
+ *   Parallelagrams always look like this without -vertical:
+ *      __      __                   /|
+ *     /_/  or  \_\  but never like  |/  (coded this way because screen is
+ *     usually longer horizontally).
+ * 28-Apr-2003: Changed so it works in a 64x64 window
  * 01-Nov-2000: Allocation checks
  * 03-Dec-1998: Random inversion of y-axis included to simulate hexagonal
  *              groups with an angle of 60 degrees.
@@ -72,6 +80,7 @@ static const char sccsid[] = "@(#)crystal.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_crystal
 #define PROGCLASS "Crystal"
 #define HACK_INIT init_crystal
 #define HACK_DRAW draw_crystal
@@ -100,12 +109,11 @@ static const char sccsid[] = "@(#)crystal.c	5.00 2000/11/01 xlockmore";
 #define DEF_CENTRE "False"
 #define DEF_MAXSIZE "False"
 #define DEF_CYCLE "True"
-
-#define min(a,b) ((a) <= (b) ? (a) : (b))
+#define DEF_VERTICAL "False"
 
 static int  nx, ny;
 
-static Bool unit_cell, grid_cell, centre, maxsize, cycle_p;
+static Bool unit_cell, grid_cell, centre, maxsize, cycle_p, vertical;
 
 static XrmOptionDescRec opts[] =
 {
@@ -120,18 +128,21 @@ static XrmOptionDescRec opts[] =
 	{(char *) "-grid", (char *) ".crystal.grid", XrmoptionNoArg, (caddr_t) "on"},
 	{(char *) "+grid", (char *) ".crystal.grid", XrmoptionNoArg, (caddr_t) "off"},
 	{(char *) "-cycle", (char *) ".crystal.cycle", XrmoptionNoArg, (caddr_t) "on"},
-	{(char *) "+cycle", (char *) ".crystal.cycle", XrmoptionNoArg, (caddr_t) "off"}
+	{(char *) "+cycle", (char *) ".crystal.cycle", XrmoptionNoArg, (caddr_t) "off"},
+	{(char *) "-vertical", (char *) ".crystal.vertical", XrmoptionNoArg, (caddr_t) "on"},
+	{(char *) "+vertical", (char *) ".crystal.vertical", XrmoptionNoArg, (caddr_t) "off"}
 };
 
 static argtype vars[] =
 {
-	{(caddr_t *) & nx, (char *) "nx", (char *) "nx", (char *) DEF_NX, t_Int},
-	{(caddr_t *) & ny, (char *) "ny", (char *) "ny", (char *) DEF_NY, t_Int},
-	{(caddr_t *) & centre, (char *) "centre", (char *) "Centre", (char *) DEF_CENTRE, t_Bool},
-	{(caddr_t *) & maxsize, (char *) "maxsize", (char *) "Maxsize", (char *) DEF_MAXSIZE, t_Bool},
-	{(caddr_t *) & unit_cell, (char *) "cell", (char *) "Cell", (char *) DEF_CELL, t_Bool},
-	{(caddr_t *) & grid_cell, (char *) "grid", (char *) "Grid", (char *) DEF_GRID, t_Bool},
-	{(caddr_t *) & cycle_p, (char *) "cycle", (char *) "Cycle", (char *) DEF_CYCLE, t_Bool}
+	{(void *) & nx, (char *) "nx", (char *) "nx", (char *) DEF_NX, t_Int},
+	{(void *) & ny, (char *) "ny", (char *) "ny", (char *) DEF_NY, t_Int},
+	{(void *) & centre, (char *) "centre", (char *) "Centre", (char *) DEF_CENTRE, t_Bool},
+	{(void *) & maxsize, (char *) "maxsize", (char *) "Maxsize", (char *) DEF_MAXSIZE, t_Bool},
+	{(void *) & unit_cell, (char *) "cell", (char *) "Cell", (char *) DEF_CELL, t_Bool},
+	{(void *) & grid_cell, (char *) "grid", (char *) "Grid", (char *) DEF_GRID, t_Bool},
+	{(void *) & cycle_p, (char *) "cycle", (char *) "Cycle", (char *) DEF_CYCLE, t_Bool},
+	{(void *) & vertical, (char *) "vertical", (char *) "Vertical", (char *) DEF_VERTICAL, t_Bool}
 };
 static OptionStruct desc[] =
 {
@@ -141,7 +152,8 @@ static OptionStruct desc[] =
 	{(char *) "-/+maxsize", (char *) "turn on/off use of maximum part of screen"},
 	{(char *) "-/+cell", (char *) "turn on/off drawing of unit cell"},
 	{(char *) "-/+grid", (char *) "turn on/off drawing of grid of unit cells (if -cell is on)"},
-	{(char *) "-/+cycle", (char *) "turn on/off colour cycling"}
+	{(char *) "-/+cycle", (char *) "turn on/off colour cycling"},
+	{(char *) "-/+vertical", (char *) "change orientation"}
 };
 
 ModeSpecOpt crystal_opts =
@@ -257,7 +269,7 @@ typedef struct {
 } crystalatom;
 
 typedef struct {
-	Bool        painted;
+	Bool        painted, vertical;
 	int         win_width, win_height, num_atom;
 	int         planegroup, a, b, offset_w, offset_h, nx, ny;
 	float       gamma;
@@ -277,13 +289,13 @@ static crystalstruct *crystals = (crystalstruct *) NULL;
 
 static void
 trans_coor(XPoint * xyp, XPoint * new_xyp, int num_points,
-	   float gamma )
+	   float gamma)
 {
 	int         i;
 
 	for (i = 0; i <= num_points; i++) {
-		new_xyp[i].x = xyp[i].x +
-			(int) (xyp[i].y * sin((gamma - 90.0) * PI_RAD));
+		new_xyp[i].x = xyp[i].x + (int) (xyp[i].y *
+			sin((gamma - 90.0) * PI_RAD));
 		new_xyp[i].y = (int) (xyp[i].y / cos((gamma - 90.0) * PI_RAD));
 	}
 }
@@ -291,16 +303,26 @@ trans_coor(XPoint * xyp, XPoint * new_xyp, int num_points,
 static void
 trans_coor_back(XPoint * xyp, XPoint * new_xyp,
 		int num_points, float gamma, int offset_w, int offset_h ,
-		int winheight , int invert )
+		int winheight , int invert, Bool new_vertical)
 {
 	int         i;
 
 	for (i = 0; i <= num_points; i++) {
-		new_xyp[i].y = (int) (xyp[i].y * cos((gamma - 90) * PI_RAD)) +
-			offset_h;
-		new_xyp[i].x = xyp[i].x - (int) (xyp[i].y * sin((gamma - 90.0)
-						       * PI_RAD)) + offset_w;
-	   if ( invert ) new_xyp[i].y = winheight - new_xyp[i].y;
+		if (new_vertical) {
+			new_xyp[i].x = (int) (xyp[i].y *
+				cos((gamma - 90) * PI_RAD)) + offset_h;
+			new_xyp[i].y = xyp[i].x - (int) (xyp[i].y *
+				sin((gamma - 90.0) * PI_RAD)) + offset_w;
+			if (invert)
+				new_xyp[i].x = winheight - new_xyp[i].x;
+		} else {
+			new_xyp[i].y = (int) (xyp[i].y *
+				cos((gamma - 90) * PI_RAD)) + offset_h;
+			new_xyp[i].x = xyp[i].x - (int) (xyp[i].y *
+				sin((gamma - 90.0) * PI_RAD)) + offset_w;
+			if (invert)
+				new_xyp[i].y = winheight - new_xyp[i].y;
+		}
 	}
 }
 
@@ -469,7 +491,8 @@ crystal_drawatom(ModeInfo * mi, crystalatom * atom0)
 						cryst->gamma, cryst->offset_w,
 						cryst->offset_h ,
 						cryst->win_height,
-						cryst->invert);
+						cryst->invert,
+						cryst->vertical);
 				XFillPolygon(display, window, cryst->gc, new_xy,
 				  atom0->num_point, Convex, CoordModeOrigin);
 			}
@@ -489,9 +512,10 @@ crystal_drawatom(ModeInfo * mi, crystalatom * atom0)
 					trans_coor_back(xy_1, new_xy, atom0->num_point,
 							cryst->gamma,
 							cryst->offset_w,
-							cryst->offset_h ,
-							cryst->win_height ,
-							cryst->invert);
+							cryst->offset_h,
+							cryst->win_height,
+							cryst->invert,
+							cryst->vertical);
 					XFillPolygon(display, window, cryst->gc,
 						     new_xy,
 						     atom0->num_point, Convex,
@@ -524,7 +548,8 @@ crystal_drawatom(ModeInfo * mi, crystalatom * atom0)
 							cryst->offset_w,
 							cryst->offset_h ,
 							cryst->win_height,
-							cryst->invert);
+							cryst->invert,
+							cryst->vertical);
 					XFillPolygon(display, window, cryst->gc,
 						     new_xy,
 						     atom0->num_point, Convex,
@@ -550,7 +575,8 @@ crystal_drawatom(ModeInfo * mi, crystalatom * atom0)
 								cryst->offset_w,
 								cryst->offset_h ,
 								cryst->win_height,
-								cryst->invert);
+								cryst->invert,
+								cryst->vertical);
 						XFillPolygon(display, window,
 							     cryst->gc,
 						    new_xy, atom0->num_point,
@@ -577,7 +603,7 @@ free_crystal(Display *display, crystalstruct *cryst)
 		if (cryst->colors != NULL) {
 			if (cryst->ncolors && !cryst->no_colors)
 				free_colors(display, cryst->cmap, cryst->colors, cryst->ncolors);
-			(void) free((void *) cryst->colors);
+			free(cryst->colors);
 			cryst->colors = (XColor *) NULL;
 		}
 		if (cryst->cmap != None) {
@@ -590,11 +616,26 @@ free_crystal(Display *display, crystalstruct *cryst)
 		cryst->gc = None;
 	}
 	if (cryst->atom != NULL) {
-		(void) free((void *) cryst->atom);
+		free(cryst->atom);
 		cryst->atom = (crystalatom *) NULL;
 	}
 }
 
+
+static void
+DrawLine(Display *display, Window window, crystalstruct *cryst,
+		int x_1, int y_1, int x_2, int y_2)
+{
+
+	if (cryst->vertical) {
+		XDrawLine(display, window, cryst->gc,
+			y_1, x_1, y_2, x_2);
+	} else {
+		XDrawLine(display, window, cryst->gc,
+			x_1, y_1, x_2, y_2);
+	}
+}
+
 #ifndef STANDALONE
 extern char *background;
 extern char *foreground;
@@ -620,7 +661,7 @@ init_crystal(ModeInfo * mi)
 	cryst = &crystals[MI_SCREEN(mi)];
 	cryst->mi = mi;
 
-	if (!cryst->gc) {
+	if (cryst->gc == None) {
 		if (MI_IS_INSTALL(mi) && MI_NPIXELS(mi) > 2) {
 			XColor      color;
 
@@ -667,25 +708,28 @@ init_crystal(ModeInfo * mi)
 /*Set up crystal data */
 	cryst->direction = (LRAND() & 1) ? 1 : -1;
 	if (MI_IS_FULLRANDOM(mi)) {
-		if (LRAND() & 1)
-			cryst->unit_cell = True;
-		else
-			cryst->unit_cell = False;
-	} else
+		cryst->unit_cell = (Bool) (LRAND() & 1);
+		cryst->vertical = (Bool) (LRAND() & 1);
+	} else {
 		cryst->unit_cell = unit_cell;
-	if (cryst->unit_cell) {
-		if (MI_IS_FULLRANDOM(mi)) {
-			if (LRAND() & 1)
-				cryst->grid_cell = True;
-			else
-				cryst->grid_cell = False;
-		} else
-			cryst->grid_cell = grid_cell;
+		cryst->vertical = vertical;
 	}
-	cryst->win_width = MI_WIDTH(mi);
-	cryst->win_height = MI_HEIGHT(mi);
-	cell_min = min(cryst->win_width / 2 + 1, MIN_CELL);
-	cell_min = min(cell_min, cryst->win_height / 2 + 1);
+	if (cryst->unit_cell) {
+		if (MI_IS_FULLRANDOM(mi))
+			cryst->grid_cell = (Bool) (LRAND() & 1);
+		else
+			cryst->grid_cell = grid_cell;
+	
+	}
+	if (cryst->vertical) {
+		cryst->win_height = MAX(MI_WIDTH(mi) + 1, MIN_CELL);
+		cryst->win_width = MAX(MI_HEIGHT(mi) + 1, MIN_CELL);
+	} else {
+		cryst->win_width = MAX(MI_WIDTH(mi) + 1, MIN_CELL);
+		cryst->win_height = MAX(MI_HEIGHT(mi) + 1, MIN_CELL);
+	}
+	cell_min = MIN(cryst->win_width / 2 + 1, MIN_CELL);
+	cell_min = MIN(cell_min, cryst->win_height / 2 + 1);
 	cryst->planegroup = NRAND(17);
         cryst->invert = NRAND(2);
 	if (MI_IS_VERBOSE(mi))
@@ -739,7 +783,7 @@ init_crystal(ModeInfo * mi)
 			return;
 		}
 
-	if (maxsize) {
+	if (maxsize || MI_WIDTH(mi) < 92 || MI_HEIGHT(mi) < 92) {
 		if (cryst->planegroup < 13) {
 			cryst->gamma = 90.0;
 			cryst->offset_w = 0;
@@ -748,7 +792,7 @@ init_crystal(ModeInfo * mi)
 				cryst->b = cryst->win_height;
 				cryst->a = cryst->win_width;
 			} else {
-				cryst->b = min(cryst->win_height, cryst->win_width);
+				cryst->b = MIN(cryst->win_height, cryst->win_width);
 				cryst->a = cryst->b;
 			}
 		} else {
@@ -763,8 +807,9 @@ init_crystal(ModeInfo * mi)
 		cryst->offset_w = -1;
 		while (cryst->offset_w < 4 || (int) (cryst->offset_w - cryst->b *
 				    sin((cryst->gamma - 90) * PI_RAD)) < 4) {
-			cryst->b = NRAND((int) (cryst->win_height / (cos((cryst->gamma - 90) *
-					    PI_RAD))) - cell_min) + cell_min;
+			cryst->b = NRAND((int) (cryst->win_height /
+				(cos((cryst->gamma - 90) *
+				PI_RAD))) - cell_min) + cell_min;
 			if (cryst->planegroup > 8)
 				cryst->a = cryst->b;
 			else
@@ -795,7 +840,7 @@ init_crystal(ModeInfo * mi)
 		}
 	}
 
-	size_atom = min((int) ((float) (cryst->a) / 40.) + 1,
+	size_atom = MIN((int) ((float) (cryst->a) / 40.) + 1,
 			(int) ((float) (cryst->b) / 40.) + 1);
 	if (MI_SIZE(mi) < size_atom) {
 		if (MI_SIZE(mi) < -size_atom)
@@ -819,7 +864,7 @@ init_crystal(ModeInfo * mi)
 		     y_coor1 = y_coor2 = cryst->win_height - cryst->offset_h;
 		   else
 		     y_coor1 = y_coor2 = cryst->offset_h;
-			XDrawLine(display, window, cryst->gc, cryst->offset_w,
+			DrawLine(display, window, cryst, cryst->offset_w,
 				  y_coor1, cryst->offset_w + cryst->nx * cryst->a,
 				  y_coor2);
 		   if ( cryst->invert )
@@ -837,7 +882,7 @@ init_crystal(ModeInfo * mi)
 					 cos((cryst->gamma - 90) * PI_RAD)) +
 			  cryst->offset_h;
 		     }
-			XDrawLine(display, window, cryst->gc, cryst->offset_w,
+			DrawLine(display, window, cryst, cryst->offset_w,
 				  y_coor1, (int) (cryst->offset_w - cryst->ny * cryst->b *
 					  sin((cryst->gamma - 90) * PI_RAD)),
 				  y_coor2);
@@ -860,7 +905,7 @@ init_crystal(ModeInfo * mi)
 			y_coor2 = (int) (iny * cryst->b * cos((cryst->gamma - 90) * PI_RAD)) +
 					  cryst->offset_h;
 		     }
-				XDrawLine(display, window, cryst->gc,
+				DrawLine(display, window, cryst,
 					  (int) (cryst->offset_w +
 				     inx * cryst->a - (int) (iny * cryst->b *
 					 sin((cryst->gamma - 90) * PI_RAD))),
@@ -886,7 +931,7 @@ init_crystal(ModeInfo * mi)
 						    PI_RAD)) + cryst->offset_h;
 				y_coor2 =cryst->offset_h;
 			     }
-				XDrawLine(display, window, cryst->gc,
+				DrawLine(display, window, cryst,
 					  (int) (cryst->offset_w +
 				     inx * cryst->a - (int) (iny * cryst->b *
 					 sin((cryst->gamma - 90) * PI_RAD))),
@@ -923,22 +968,22 @@ init_crystal(ModeInfo * mi)
 						      PI_RAD)) +
 			  cryst->offset_h;
 		     }
-			XDrawLine(display, window, cryst->gc,
+			DrawLine(display, window, cryst,
 				  cryst->offset_w + inx * cryst->a - (int) (iny * cryst->b * sin((cryst->gamma - 90) * PI_RAD)),
 				  y_coor1,
 				  cryst->offset_w + (inx + 1) * cryst->a - (int) (iny * cryst->b * sin((cryst->gamma - 90) * PI_RAD)),
 				  y_coor1);
-			XDrawLine(display, window, cryst->gc,
+			DrawLine(display, window, cryst,
 				  cryst->offset_w + inx * cryst->a - (int) (iny * cryst->b * sin((cryst->gamma - 90) * PI_RAD)),
 				  y_coor1,
 				  cryst->offset_w + inx * cryst->a - (int) ((iny + 1) * cryst->b * sin((cryst->gamma - 90) * PI_RAD)),
 				  y_coor2);
-			XDrawLine(display, window, cryst->gc,
+			DrawLine(display, window, cryst,
 				  cryst->offset_w + (inx + 1) * cryst->a - (int) (iny * cryst->b * sin((cryst->gamma - 90) * PI_RAD)),
 				  y_coor1,
 				  cryst->offset_w + (inx + 1) * cryst->a - (int) ((iny + 1) * cryst->b * sin((cryst->gamma - 90) * PI_RAD)),
 				  y_coor2);
-			XDrawLine(display, window, cryst->gc,
+			DrawLine(display, window, cryst,
 				  cryst->offset_w + inx * cryst->a - (int) ((iny + 1) * cryst->b * sin((cryst->gamma - 90) * PI_RAD)),
 				  y_coor2,
 				  cryst->offset_w + (inx + 1) * cryst->a - (int) ((iny + 1) * cryst->b * sin((cryst->gamma - 90) * PI_RAD)),
@@ -951,7 +996,7 @@ init_crystal(ModeInfo * mi)
 		if (cryst->colors != NULL) {
 			if (cryst->ncolors && !cryst->no_colors)
 				free_colors(display, cryst->cmap, cryst->colors, cryst->ncolors);
-			(void) free((void *) cryst->colors);
+			free(cryst->colors);
 			cryst->colors = (XColor *) NULL;
 		}
 		cryst->ncolors = MI_NCOLORS(mi);
@@ -1160,7 +1205,7 @@ refresh_crystal(ModeInfo * mi)
 		     y_coor1 = y_coor2 = cryst->win_height - cryst->offset_h;
 		   else
 		     y_coor1 = y_coor2 = cryst->offset_h;
-			XDrawLine(display, window, cryst->gc, cryst->offset_w,
+			DrawLine(display, window, cryst, cryst->offset_w,
 				  y_coor1, cryst->offset_w + cryst->nx * cryst->a,
 				  y_coor2);
 		   if ( cryst->invert )
@@ -1178,7 +1223,7 @@ refresh_crystal(ModeInfo * mi)
 					 cos((cryst->gamma - 90) * PI_RAD)) +
 			  cryst->offset_h;
 		     }
-			XDrawLine(display, window, cryst->gc, cryst->offset_w,
+			DrawLine(display, window, cryst, cryst->offset_w,
 				  y_coor1, (int) (cryst->offset_w - cryst->ny * cryst->b *
 					  sin((cryst->gamma - 90) * PI_RAD)),
 				  y_coor2);
@@ -1201,7 +1246,7 @@ refresh_crystal(ModeInfo * mi)
 			y_coor2 = (int) (iny * cryst->b * cos((cryst->gamma - 90) * PI_RAD)) +
 					  cryst->offset_h;
 		     }
-				XDrawLine(display, window, cryst->gc,
+				DrawLine(display, window, cryst,
 					  (int) (cryst->offset_w +
 				     inx * cryst->a - (int) (iny * cryst->b *
 					 sin((cryst->gamma - 90) * PI_RAD))),
@@ -1227,7 +1272,7 @@ refresh_crystal(ModeInfo * mi)
 						    PI_RAD)) + cryst->offset_h;
 				y_coor2 =cryst->offset_h;
 			     }
-				XDrawLine(display, window, cryst->gc,
+				DrawLine(display, window, cryst,
 					  (int) (cryst->offset_w +
 				     inx * cryst->a - (int) (iny * cryst->b *
 					 sin((cryst->gamma - 90) * PI_RAD))),
@@ -1264,22 +1309,22 @@ refresh_crystal(ModeInfo * mi)
 						      PI_RAD)) +
 			  cryst->offset_h;
 		     }
-			XDrawLine(display, window, cryst->gc,
+			DrawLine(display, window, cryst,
 				  cryst->offset_w + inx * cryst->a - (int) (iny * cryst->b * sin((cryst->gamma - 90) * PI_RAD)),
 				  y_coor1,
 				  cryst->offset_w + (inx + 1) * cryst->a - (int) (iny * cryst->b * sin((cryst->gamma - 90) * PI_RAD)),
 				  y_coor1);
-			XDrawLine(display, window, cryst->gc,
+			DrawLine(display, window, cryst,
 				  cryst->offset_w + inx * cryst->a - (int) (iny * cryst->b * sin((cryst->gamma - 90) * PI_RAD)),
 				  y_coor1,
 				  cryst->offset_w + inx * cryst->a - (int) ((iny + 1) * cryst->b * sin((cryst->gamma - 90) * PI_RAD)),
 				  y_coor2);
-			XDrawLine(display, window, cryst->gc,
+			DrawLine(display, window, cryst,
 				  cryst->offset_w + (inx + 1) * cryst->a - (int) (iny * cryst->b * sin((cryst->gamma - 90) * PI_RAD)),
 				  y_coor1,
 				  cryst->offset_w + (inx + 1) * cryst->a - (int) ((iny + 1) * cryst->b * sin((cryst->gamma - 90) * PI_RAD)),
 				  y_coor2);
-			XDrawLine(display, window, cryst->gc,
+			DrawLine(display, window, cryst,
 				  cryst->offset_w + inx * cryst->a - (int) ((iny + 1) * cryst->b * sin((cryst->gamma - 90) * PI_RAD)),
 				  y_coor2,
 				  cryst->offset_w + (inx + 1) * cryst->a - (int) ((iny + 1) * cryst->b * sin((cryst->gamma - 90) * PI_RAD)),
@@ -1309,7 +1354,7 @@ release_crystal(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_crystal(MI_DISPLAY(mi), &crystals[screen]);
-		(void) free((void *) crystals);
+		free(crystals);
 		crystals = (crystalstruct *) NULL;
 	}
 }
