@@ -7,7 +7,7 @@ static const char sccsid[] = "@(#)decay.c	5.00 2000/11/01 xlockmore";
 #endif
 
 /* xscreensaver, Copyright (c) 1992, 1993, 1994, 1996, 1997
- * Jamie Zawinski <jwz@jwz.org>
+ * Jamie Zawinski <jwz AT jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -22,7 +22,7 @@ static const char sccsid[] = "@(#)decay.c	5.00 2000/11/01 xlockmore";
  * Revision History:
  * 01-Nov-2000: Allocation checks
  * 17-Mar-1999: Converted from xscreensaver's decayscreen
- * 
+ *
  * decayscreen
  *
  * Based on slidescreen program from the xscreensaver application and the
@@ -46,6 +46,7 @@ static const char sccsid[] = "@(#)decay.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_decay
 #define PROGCLASS "Decay"
 #define HACK_INIT init_decay
 #define HACK_DRAW draw_decay
@@ -83,7 +84,7 @@ ModStruct   decay_description =
 #define DECAY_BITS    image_bits
 #include "decay.xbm"
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 #define DECAY_NAME    image_name
 #include "decay.xpm"
 #define DEFAULT_XPM 1
@@ -114,8 +115,8 @@ static decaystruct *decay_info = (decaystruct *) NULL;
 #define DOWNLEFT 6
 #define UPRIGHT 7
 #define DOWNRIGHT 8
-#define IN 9
-#define OUT 10
+#define INSIDE 9
+#define OUTSIDE 10
 #define	DEGREE	1
 
 static void
@@ -146,7 +147,7 @@ alloc_decay(ModeInfo * mi)
 	if (dp->hide) {
 		if (dp->logo == None) {
 			getImage(mi, &dp->logo, DECAY_WIDTH, DECAY_HEIGHT, DECAY_BITS,
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 				DEFAULT_XPM, DECAY_NAME,
 #endif
 				&dp->graphics_format, &dp->cmap, &dp->black);
@@ -208,12 +209,12 @@ init_decay(ModeInfo * mi)
 	else if (s && !strcmp(s, "downleft")) dp->mode = DOWNLEFT;
 	else if (s && !strcmp(s, "upright")) dp->mode = UPRIGHT;
 	else if (s && !strcmp(s, "downright")) dp->mode = DOWNRIGHT;
-	else if (s && !strcmp(s, "in")) dp->mode = IN;
-	else if (s && !strcmp(s, "out")) dp->mode = OUT;
+	else if (s && !strcmp(s, "in")) dp->mode = INSIDE;
+	else if (s && !strcmp(s, "out")) dp->mode = OUTSIDE;
 	else {
 		if (s && *s && !!strcmp(s, "random"))
 			(void) fprintf(stderr, "%s: unknown mode %s\n", ProgramName, s);
-		dp->mode = (int) (LRAND() % (OUT+1));
+		dp->mode = (int) (LRAND() % (OUTSIDE+1));
 	}
 
 	if (MI_IS_FULLRANDOM(mi) && !hide)
@@ -285,6 +286,8 @@ draw_decay (ModeInfo * mi)
 	if (dp->backGC == None)
 		return;
 
+	MI_IS_DRAWN(mi) = True;
+
     switch (dp->mode) {
       case SHUFFLE:	bias = no_bias; break;
       case UP:		bias = up_bias; break;
@@ -295,8 +298,8 @@ draw_decay (ModeInfo * mi)
       case DOWNLEFT:	bias = downleft_bias; break;
       case UPRIGHT:	bias = upright_bias; break;
       case DOWNRIGHT:	bias = downright_bias; break;
-      case IN:		bias = no_bias; break;
-      case OUT:		bias = no_bias; break;
+      case INSIDE:	bias = no_bias; break;
+      case OUTSIDE:	bias = no_bias; break;
       default: bias = no_bias;
 	 if (MI_IS_VERBOSE(mi)) {
              (void) fprintf(stderr, "Weirdness in draw_decay()\n");
@@ -304,7 +307,7 @@ draw_decay (ModeInfo * mi)
          }
     }
 
-    left = NRAND(dp->windowsize.x - 1);
+    left = NRAND(dp->windowsize.x);
     top = NRAND(dp->windowsize.y);
     width = NRAND(dp->windowsize.x - left);
     height = NRAND(dp->windowsize.y - top);
@@ -312,12 +315,12 @@ draw_decay (ModeInfo * mi)
     toleft = left;
     totop = top;
 
-    if (dp->mode == IN || dp->mode == OUT) {
+    if (dp->mode == INSIDE || dp->mode == OUTSIDE) {
       int x = left+(width/2);
       int y = top+(height/2);
       int cx = dp->windowsize.x/2;
       int cy = dp->windowsize.y/2;
-      if (dp->mode == IN) {
+      if (dp->mode == INSIDE) {
 	if      (x > cx && y > cy)   bias = upleft_bias;
 	else if (x < cx && y > cy)   bias = upright_bias;
 	else if (x < cx && y < cy)   bias = downright_bias;
@@ -353,7 +356,7 @@ draw_decay (ModeInfo * mi)
 void
 refresh_decay(ModeInfo * mi)
 {
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 	decaystruct *dp;
 
 	if (decay_info == NULL)
@@ -377,7 +380,7 @@ release_decay(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_decay(MI_DISPLAY(mi), &decay_info[screen]);
-		(void) free((void *) decay_info);
+		free(decay_info);
 		decay_info = (decaystruct *) NULL;
 	}
 }
