@@ -6,7 +6,7 @@ static const char sccsid[] = "@(#)deco.c	5.00 2000/11/01 xlockmore";
 
 #endif
 /*
- * Copyright (c) 1997 by Jamie Zawinski <jwz@jwz.org>
+ * Copyright (c) 1997 by Jamie Zawinski <jwz AT jwz.org>
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted,
@@ -22,13 +22,13 @@ static const char sccsid[] = "@(#)deco.c	5.00 2000/11/01 xlockmore";
  *
  * Revision History:
  * 01-Nov-2000: Allocation checks
- * 29-Oct-1997: xlock version (David Bagley <bagleyd@tux.org>)
- * 1997: xscreensaver version Jamie Zawinski <jwz@jwz.org>
+ * 29-Oct-1997: xlock version (David Bagley <bagleyd AT verizon.net>)
+ * 1997: xscreensaver version Jamie Zawinski <jwz AT jwz.org>
  */
 
 /*-
  * original copyright
- * xscreensaver, Copyright (c) 1997 Jamie Zawinski <jwz@jwz.org>
+ * xscreensaver, Copyright (c) 1997 Jamie Zawinski <jwz AT jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -43,6 +43,7 @@ static const char sccsid[] = "@(#)deco.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_deco
 #define PROGCLASS "Deco"
 #define HACK_INIT init_deco
 #define HACK_DRAW draw_deco
@@ -183,7 +184,7 @@ void
 release_deco(ModeInfo * mi)
 {
 	if (decos != NULL) {
-		(void) free((void *) decos);
+		free(decos);
 		decos = (decostruct *) NULL;
 	}
 }
