@@ -7,7 +7,7 @@ static const char sccsid[] = "@(#)eyes.c	5.00 2000/11/01 xlockmore";
 #endif
 
 /*-
- * Copyright 1996 by Ron Hitchens <ron@idiom.com>
+ * Copyright 1996 by Ron Hitchens <ron AT idiom.com>
  *
  * Adapted from the ubiquitous xeyes demo supplied by MIT with the
  * X Window System.
@@ -43,6 +43,7 @@ static const char sccsid[] = "@(#)eyes.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_eyes
 #define PROGCLASS "Eyes"
 #define HACK_INIT init_eyes
 #define HACK_DRAW draw_eyes
@@ -75,7 +76,7 @@ static XrmOptionDescRec opts[] =
 
 static argtype vars[] =
 {
-	{(caddr_t *) & trackmouse, (char *) "trackmouse", (char *) "TrackMouse", (char *) DEF_TRACKMOUSE, t_Bool}
+	{(void *) & trackmouse, (char *) "trackmouse", (char *) "TrackMouse", (char *) DEF_TRACKMOUSE, t_Bool}
 };
 
 static OptionStruct desc[] =
@@ -475,15 +476,16 @@ init_fly(ModeInfo * mi, Fly * f)
 	(void) memset((char *) f, 0, sizeof (Fly));	/* clear everything to zero */
 
 	f->side = FLY_SIDE_LEFT;
-
-	if (MI_IS_ICONIC(mi)) {
-		/* image is just a dot when iconic */
-		f->width = f->height = FLY_ICON_SIZE;
-		f->vx = NRAND(4) + 1;	/* slower when iconic */
-	} else {
+	if (!MI_IS_ICONIC(mi))
+	{
 		f->width = ep->flywidth;
 		f->height = ep->flyheight;
 		f->vx = NRAND(15) + 1;	/* random horiz velocity */
+	} else
+	{
+		/* image is just a dot when iconic */
+		f->width = f->height = FLY_ICON_SIZE;
+		f->vx = NRAND(4) + 1;	/* slower when iconic */
 	}
 
 	f->y = NRAND(win_height);
@@ -763,7 +765,7 @@ create_eyes(ModeInfo * mi, Eyes * e, Eyes * eyes, int num_eyes)
 
 	/* clear the offscreen pixmap to background color */
 	XSetForeground(display, ep->eyeGC, black_pixel);
-	XFillRectangle(display, e->pixmap, ep->eyeGC, 0, 0, e->width, e->height);
+	XFillRectangle(display, (Drawable) e->pixmap, ep->eyeGC, 0, 0, e->width, e->height);
 
 	/* make the full eye images in the offscreen Pixmap */
 	make_eye(mi, e->pixmap, e, 0, True);
@@ -873,7 +875,7 @@ freePairsOfEyes(Display * display, EyeScrInfo * ep)
 		for (en = 0; en < ep->num_eyes; en++)
 			if (ep->eyes[en].pixmap != None)
 				XFreePixmap(display, ep->eyes[en].pixmap);
-		(void) free((void *) ep->eyes);
+		free(ep->eyes);
 		ep->eyes = (Eyes *) NULL;
 	}
 }
@@ -1076,7 +1078,7 @@ release_eyes(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_eyes(MI_DISPLAY(mi), &eye_info[screen]);
-		(void) free((void *) eye_info);
+		free(eye_info);
 	}
 	eye_info = (EyeScrInfo *) NULL;
 }
