@@ -30,6 +30,7 @@ static const char sccsid[] = "@(#)fadeplot.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_fadeplot
 #define PROGCLASS "Fadeplot"
 #define HACK_INIT init_fadeplot
 #define HACK_DRAW draw_fadeplot
@@ -79,11 +80,11 @@ static void
 free_fadeplot(fadeplotstruct *fp)
 {
 	if (fp->pts != NULL) {
-		(void) free((void *) fp->pts);
+		free(fp->pts);
 		fp->pts = (XPoint *) NULL;
 	}
 	if (fp->stab != NULL) {
-		(void) free((void *) fp->stab);
+		free(fp->stab);
 		fp->stab = (int *) NULL;
 	}
 }
@@ -114,7 +115,7 @@ init_fadeplot(ModeInfo * mi)
 
 	if (fadeplots == NULL) {
 		if ((fadeplots = (fadeplotstruct *) calloc(MI_NUM_SCREENS(mi),
-					   sizeof (fadeplotstruct))) == NULL)
+				sizeof (fadeplotstruct))) == NULL)
 			return;
 	}
 	fp = &fadeplots[MI_SCREEN(mi)];
@@ -142,8 +143,8 @@ init_fadeplot(ModeInfo * mi)
 		fp->maxpts = 1;
 
 	if (fp->pts == NULL) {
-		if ((fp->pts = (XPoint *) calloc(fp->maxpts, sizeof (XPoint))) ==
-				 NULL) {
+		if ((fp->pts = (XPoint *) calloc(
+				fp->maxpts, sizeof (XPoint))) == NULL) {
 			free_fadeplot(fp);
 			return;
 		}
@@ -152,7 +153,7 @@ init_fadeplot(ModeInfo * mi)
 		fp->pix = NRAND(MI_NPIXELS(mi));
 
 	if (fp->stab != NULL)
-		(void) free((void *) fp->stab);
+		free(fp->stab);
 	if (!initSintab(mi))
 		return;
 	MI_CLEARWINDOW(mi);
@@ -188,11 +189,13 @@ draw_fadeplot(ModeInfo * mi)
 	for (j = 0; j < fp->nbstep; j++) {
 		for (i = 0; i < fp->maxpts / fp->nbstep; i++) {
 			fp->pts[temp].x =
-				fp->stab[(fp->st.x + fp->speed.x * j + i * fp->step.x) % fp->angles] *
-				fp->factor.x + fp->width / 2 - fp->min;
+				fp->stab[(fp->st.x + fp->speed.x * j +
+				i * fp->step.x) % fp->angles] * fp->factor.x +
+				fp->width / 2 - fp->min;
 			fp->pts[temp].y =
-				fp->stab[(fp->st.y + fp->speed.y * j + i * fp->step.y) % fp->angles] *
-				fp->factor.y + fp->height / 2 - fp->min;
+				fp->stab[(fp->st.y + fp->speed.y * j +
+				i * fp->step.y) % fp->angles] * fp->factor.y +
+				fp->height / 2 - fp->min;
 			temp++;
 		}
 	}
@@ -204,13 +207,12 @@ draw_fadeplot(ModeInfo * mi)
 	if ((fp->temps % (fp->angles / 2)) == 0) {
 		fp->temps = fp->temps % fp->angles * 5;
 		if ((fp->temps % (fp->angles)) == 0)
-			fp->speed.y = (fp->speed.y++) % 30 + 1;
+			fp->speed.y = (fp->speed.y + 1) % 30 + 1;
 		if ((fp->temps % (fp->angles * 2)) == 0)
 			fp->speed.x = (fp->speed.x) % 20;
 		if ((fp->temps % (fp->angles * 3)) == 0)
-			fp->step.y = (fp->step.y++) % 2 + 1;
-
-		MI_CLEARWINDOW(mi);
+			fp->step.y = (fp->step.y + 1) % 2 + 1;
+		MI_CLEARWINDOWCOLORMAPFAST(mi, MI_GC(mi), MI_BLACK_PIXEL(mi));
 	}
 }
 void
@@ -227,7 +229,7 @@ release_fadeplot(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_fadeplot(&fadeplots[screen]);
-		(void) free((void *) fadeplots);
+		free(fadeplots);
 		fadeplots = (fadeplotstruct *) NULL;
 	}
 }
