@@ -2,7 +2,7 @@
 /* gears --- 3D gear wheels */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)gears.c	5.01 2001/02/22 xlockmore";
+static const char sccsid[] = "@(#)gears.c	5.03 2001/11/28 xlockmore";
 
 #endif
 
@@ -20,14 +20,15 @@ static const char sccsid[] = "@(#)gears.c	5.01 2001/02/22 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
- * 22-Feb-2001: backported from xscreensaver by lassauge@mail.dotcom.fr
- * 09-Feb-2001: "Planetary" gear system added by jwz@jwz.org.
+ * 28-Nov-2001: backported from xscreensaver-3.34 by lassauge AT users.sourceforge.net
+ * 22-Feb-2001: backported from xscreensaver-3.33 by lassauge AT users.sourceforge.net
+ * 09-Feb-2001: "Planetary" gear system added by jwz AT jwz.org.
  * 01-Nov-2000: Allocation checks
  * 10-May-1997: Compatible with xscreensaver
  * 22-Mar-1997: Added support for -mono mode, and monochrome X servers.
  *              Ed Mackey, emackey@netaxs.com
- * 13-Mar-1997: Memory leak fix by Tom Schmidt <tschmidt@micron.com>
- * 1996: "written" by Danny Sung <dannys@ucla.edu>
+ * 13-Mar-1997: Memory leak fix by Tom Schmidt <tschmidt AT micron.com>
+ * 1996: "written" by Danny Sung <danny@dannysung.com>
  *       Based on 3-D gear wheels by Brian Paul which is in the public domain.
  */
 
@@ -37,14 +38,17 @@ static const char sccsid[] = "@(#)gears.c	5.01 2001/02/22 xlockmore";
  * been fixed in MesaGL 2.2 and later releases.
  */
 
+#ifdef VMS
 /*-
  * due to a Bug/feature in VMS X11/Intrinsic.h has to be placed before xlock.
  * otherwise caddr_t is not defined correctly
  */
 
 #include <X11/Intrinsic.h>
+#endif
 
 #ifdef STANDALONE
+# define MODE_gears
 # define PROGCLASS					"Gears"
 # define HACK_INIT					init_gears
 # define HACK_DRAW					draw_gears
@@ -52,7 +56,7 @@ static const char sccsid[] = "@(#)gears.c	5.01 2001/02/22 xlockmore";
 # define gears_opts					xlockmore_opts
 # define DEFAULTS			"*count:		1       \n"	\
 					"*cycles:		2       \n"	\
-					"*delay:		20000   \n"	\
+					"*delay:		50000   \n"	\
 					"*size:			0 	\n"	\
 					"*planetary:		False   \n"	\
 					"*showFps:      	False   \n"	\
@@ -71,7 +75,7 @@ static const char sccsid[] = "@(#)gears.c	5.01 2001/02/22 xlockmore";
 #define countof(x) (sizeof((x))/sizeof((*x)))
 
 #define DEF_PLANETARY "False"
-#define DEF_PLANETSIZE "400"
+#define DEF_PLANETSIZE "0"
 
 static Bool planetary;
 static int planetsize;
@@ -83,8 +87,8 @@ static XrmOptionDescRec opts[] = {
 };
 
 static argtype vars[] = {
-  {(caddr_t *) &planetary, (char *) "planetary", (char *) "Planetary", DEF_PLANETARY, t_Bool},
-  {(caddr_t *) & planetsize, (char *) "planetsize", (char *) "PlanetSize", (char *) DEF_PLANETSIZE, t_Int}
+  {(void *) &planetary, (char *) "planetary", (char *) "Planetary", (char *) DEF_PLANETARY, t_Bool},
+  {(void *) & planetsize, (char *) "planetsize", (char *) "PlanetSize", (char *) DEF_PLANETSIZE, t_Int}
 };
 
 static OptionStruct desc[] = {
@@ -98,12 +102,25 @@ ModeSpecOpt gears_opts = {countof(opts), opts, countof(vars), vars, desc};
 ModStruct   gears_description =
 {"gears", "init_gears", "draw_gears", "release_gears",
  "draw_gears", "init_gears", (char *) NULL, &gears_opts,
- 1000, 1, 2, 0, 64, 1.0, "",
+ 50000, 1, 2, 0, 64, 1.0, "",
  "Shows GL's gears", 0, NULL};
 
 #endif
 
+#define SMOOTH_TUBE       /* whether to have smooth or faceted tubes */
+
+#ifdef SMOOTH_TUBE
+# define TUBE_FACES  20   /* how densely to render tubes */
+#else
+# define TUBE_FACES  6
+#endif
+
 typedef struct {
+#ifdef WIN32
+  HGLRC       glx_context;
+#else
+  GLXContext *glx_context;
+#endif
 
   GLfloat rotx, roty, rotz;	   /* current object rotation */
   GLfloat dx, dy, dz;		   /* current rotational velocity */
@@ -114,7 +131,6 @@ typedef struct {
   GLuint      gear_inner, gear_outer;
   GLuint      armature;
   GLfloat     angle;
-  GLXContext *glx_context;
   Window      window;
   Bool        planetary;
   int         planetsize;
@@ -317,8 +333,8 @@ gear(GLfloat inner_radius, GLfloat outer_radius, GLfloat width,
 	}
 
 	if (!wire) {
-		glVertex3f(r1 * cos(0), r1 * sin(0), width * 0.5);
-		glVertex3f(r1 * cos(0), r1 * sin(0), -width * 0.5);
+		glVertex3f(r1, 0.0, width * 0.5);
+		glVertex3f(r1, 0.0, -width * 0.5);
 		glEnd();
 	}
 	if (!wire)
@@ -352,78 +368,111 @@ gear(GLfloat inner_radius, GLfloat outer_radius, GLfloat width,
 
 }
 
+static void
+unit_tube (Bool wire)
+{
+  int i;
+  int faces = TUBE_FACES;
+  GLfloat step = M_PI * 2 / faces;
+  GLfloat th;
+  int z = 0;
+
+  /* side walls
+   */
+  glFrontFace(GL_CCW);
+
+# ifdef SMOOTH_TUBE
+  glBegin(wire ? GL_LINES : GL_QUAD_STRIP);
+# else
+  glBegin(wire ? GL_LINES : GL_QUADS);
+# endif
+
+  for (i = 0, th = 0; i <= faces; i++)
+    {
+      GLfloat x = cos (th);
+      GLfloat y = sin (th);
+      glNormal3f(x, 0, y);
+      glVertex3f(x, 0.0, y);
+      glVertex3f(x, 1.0, y);
+      th += step;
+
+# ifndef SMOOTH_TUBE
+      x = cos (th);
+      y = sin (th);
+      glVertex3f(x, 1.0, y);
+      glVertex3f(x, 0.0, y);
+# endif
+    }
+  glEnd();
+
+  /* End caps
+   */
+  for (z = 0; z <= 1; z++)
+    {
+      glFrontFace(z == 0 ? GL_CCW : GL_CW);
+      glNormal3f(0, (z == 0 ? -1 : 1), 0);
+      glBegin(wire ? GL_LINE_LOOP : GL_TRIANGLE_FAN);
+      if (! wire) glVertex3f(0, z, 0);
+      for (i = 0, th = 0; i <= faces; i++)
+        {
+          GLfloat x = cos (th);
+          GLfloat y = sin (th);
+          glVertex3f(x, z, y);
+          th += step;
+        }
+      glEnd();
+    }
+}
 
 
 static void
-tube(GLfloat radius, GLfloat width, GLint facets, Bool wire)
+tube (GLfloat x1, GLfloat y1, GLfloat z1,
+      GLfloat x2, GLfloat y2, GLfloat z2,
+      GLfloat diameter, GLfloat cap_size,
+      Bool wire)
 {
-  GLint i;
-  GLfloat da = 2.0 * M_PI / facets / 4.0;
+  GLfloat length, angle, a, b, c;
 
-  glFrontFace(GL_CCW);
+  if (diameter <= 0) abort();
 
-  /* draw bottom of tube */
+  a = (x2 - x1);
+  b = (y2 - y1);
+  c = (z2 - z1);
 
-  glShadeModel(GL_FLAT);
-  glNormal3f(0, 0, 1);
-  if (!wire)
+  length = sqrt (a*a + b*b + c*c);
+  angle = acos (a / length);
+
+  glPushMatrix();
+  glTranslatef(x1, y1, z1);
+  glScalef (length, length, length);
+
+  if (c == 0 && b == 0)
+    glRotatef (angle / (M_PI / 180), 0, 1, 0);
+  else
+    glRotatef (angle / (M_PI / 180), 0, -c, b);
+
+  glRotatef (-90, 0, 0, 1);
+  glScalef (diameter/length, 1, diameter/length);
+
+  /* extend the endpoints of the tube by the cap size in both directions */
+  if (cap_size != 0)
     {
-      glBegin(GL_TRIANGLE_FAN);
-      glVertex3f(0, 0, width * 0.5);
-      for (i = 0; i <= facets; i++) {
-        GLfloat angle = i * 2.0 * M_PI / facets;
-        glVertex3f(radius * cos(angle), radius * sin(angle), width * 0.5);
-      }
-      glEnd();
+      GLfloat c = cap_size/length;
+      glTranslatef (0, -c, 0);
+      glScalef (1, 1+c+c, 1);
     }
 
-  /* draw top of tube */
+  unit_tube (wire);
+  glPopMatrix();
+}
 
-  glShadeModel(GL_FLAT);
-  glNormal3f(0, 0, -1);
-  glFrontFace(GL_CW);
-  if (!wire)
-    {
-      glBegin(GL_TRIANGLE_FAN);
-      glVertex3f(0, 0, -width * 0.5);
-      for (i = 0; i <= facets; i++) {
-        GLfloat angle = i * 2.0 * M_PI / facets;
-        glVertex3f(radius * cos(angle), radius * sin(angle), -width * 0.5);
-      }
-      glEnd();
-    }
 
-  /* draw side of tube */
-  glFrontFace(GL_CW);
-  glShadeModel(GL_SMOOTH);
-
-  if (!wire)
-    glBegin(GL_QUAD_STRIP);
-
-  for (i = 0; i <= facets; i++) {
-    GLfloat angle = i * 2.0 * M_PI / facets;
-    
-    if (wire)
-      glBegin(GL_LINES);
-
-    glNormal3f(cos(angle), sin(angle), 0.0);
-
-    glVertex3f(radius * cos(angle), radius * sin(angle), -width * 0.5);
-    glVertex3f(radius * cos(angle), radius * sin(angle), width * 0.5);
-
-    if (wire) {
-      glVertex3f(radius * cos(angle), radius * sin(angle), -width * 0.5);
-      glVertex3f(radius * cos(angle + 4 * da), radius * sin(angle + 4 * da), -width * 0.5);
-      glVertex3f(radius * cos(angle), radius * sin(angle), width * 0.5);
-      glVertex3f(radius * cos(angle + 4 * da), radius * sin(angle + 4 * da), width * 0.5);
-      glEnd();
-    }
-  }
-
-  if (!wire)
-    glEnd();
-
-  glFrontFace(GL_CCW);
+static void
+ctube (GLfloat diameter, GLfloat width, Bool wire)
+{
+  tube (0, 0,  width/2,
+        0, 0, -width/2,
+        diameter, 0, wire);
 }
 
 
@@ -638,7 +687,11 @@ free_gears(Display *display, gearsstruct *gp)
 {
 	if (gp->glx_context) {
 		/* Display lists MUST be freed while their glXContext is current. */
+#ifdef WIN32
+		wglMakeCurrent(hdc, gp->glx_context);
+#else
 		glXMakeCurrent(display, gp->window, *(gp->glx_context));
+#endif
 		if (glIsList(gp->gear1)) {
 			glDeleteLists(gp->gear1, 1);
 			gp->gear1 = 0;
@@ -663,7 +716,11 @@ free_gears(Display *display, gearsstruct *gp)
 			glDeleteLists(gp->armature, 1);
 			gp->armature = 0;
 		}
+#ifdef WIN32
+		gp->glx_context = NULL;
+#else
 		gp->glx_context = (GLXContext *) NULL;
+#endif
 	}
 }
 
@@ -908,21 +965,21 @@ pinit(ModeInfo * mi)
       glPushMatrix();
       glTranslatef(7.0, 0, 0);
       glRotatef(90, 0, 1, 0);
-      tube(0.5, 0.5, 10, wire);   /* nub 1 */
+      ctube(0.5, 0.5, wire);   /* nub 1 */
       glPopMatrix();
 
       glPushMatrix();
       glRotatef(120, 0, 0, 1);
       glTranslatef(7.0, 0, 0);
       glRotatef(90, 0, 1, 0);
-      tube(0.5, 0.5, 10, wire);   /* nub 2 */
+      ctube(0.5, 0.5, wire);   /* nub 2 */
       glPopMatrix();
 
       glPushMatrix();
       glRotatef(240, 0, 0, 1);
       glTranslatef(7.0, 0, 0);
       glRotatef(90, 0, 1, 0);
-      tube(0.5, 0.5, 10, wire);   /* nub 3 */
+      ctube(0.5, 0.5, wire);   /* nub 3 */
       glPopMatrix();
 
 
@@ -953,33 +1010,33 @@ pinit(ModeInfo * mi)
 
       glTranslatef(0, 0, 1.5);
 
-      tube(0.5, 10, 15, wire);       /* center axle */
+      ctube(0.5, 10, wire);       /* center axle */
 
       glPushMatrix();
       glTranslatef(0.0, 4.2, -1);
-      tube(0.5, 3, 15, wire);       /* axle 1 */
+      ctube(0.5, 3, wire);       /* axle 1 */
       glTranslatef(0, 0, 1.8);
-      tube(0.7, 0.7, 15, wire);
+      ctube(0.7, 0.7, wire);
       glPopMatrix();
 
       glPushMatrix();
       glRotatef(120, 0.0, 0.0, 1.0);
       glTranslatef(0.0, 4.2, -1);
-      tube(0.5, 3, 15, wire);       /* axle 2 */
+      ctube(0.5, 3, wire);       /* axle 2 */
       glTranslatef(0, 0, 1.8);
-      tube(0.7, 0.7, 15, wire);
+      ctube(0.7, 0.7, wire);
       glPopMatrix();
 
       glPushMatrix();
       glRotatef(240, 0.0, 0.0, 1.0);
       glTranslatef(0.0, 4.2, -1);
-      tube(0.5, 3, 15, wire);       /* axle 3 */
+      ctube(0.5, 3, wire);       /* axle 3 */
       glTranslatef(0, 0, 1.8);
-      tube(0.7, 0.7, 15, wire);
+      ctube(0.7, 0.7, wire);
       glPopMatrix();
 
       glTranslatef(0, 0, 1.5);      /* center disk */
-      tube(1.5, 2, 20, wire);
+      ctube(1.5, 2, wire);
 
       glPushMatrix();
       glRotatef(270, 0, 0, 1);
@@ -1050,12 +1107,12 @@ rotate(GLfloat *pos, GLfloat *v, GLfloat *dv, GLfloat max_v, Bool verbose)
   /* If it stops, start it going in the other direction. */
   else if (*v < 0)
     {
-      if (random() % 4)
+      if (NRAND(4) != 0)
 	{
 	  *v = 0;
 
 	  /* keep going in the same direction */
-	  if (random() % 2)
+	  if (LRAND() & 1)
 	    *dv = 0;
 	  else if (*dv < 0)
 	    *dv = -*dv;
@@ -1070,15 +1127,15 @@ rotate(GLfloat *pos, GLfloat *v, GLfloat *dv, GLfloat max_v, Bool verbose)
     }
 
   /* Alter direction of rotational acceleration randomly. */
-  if (! (random() % 120))
+  if (!(NRAND(120)))
     *dv = -*dv;
 
   /* Change acceleration very occasionally. */
-  if (! (random() % 200))
+  if (!(NRAND(200)))
     {
       if (*dv == 0)
 	*dv = 0.00001;
-      else if (random() & 1)
+      else if (LRAND() & 1)
 	*dv *= 1.2;
       else
 	*dv *= 0.8;
@@ -1102,33 +1159,33 @@ init_gears(ModeInfo * mi)
 
 	gp->window = MI_WINDOW(mi);
 
-    if (MI_IS_FULLRANDOM(mi)) {
-      gp->planetary = (Bool) (LRAND() & 1);
-    } else {
-      gp->planetary = planetary;
-    }
+	if (MI_IS_FULLRANDOM(mi)) {
+		gp->planetary = (Bool) (LRAND() & 1);
+	} else {
+		gp->planetary = planetary;
+	}
 
-    gp->rotx = FLOATRAND(1.0) * RANDSIGN();
-    gp->roty = FLOATRAND(1.0) * RANDSIGN();
-    gp->rotz = FLOATRAND(1.0) * RANDSIGN();
+	gp->rotx = FLOATRAND(1.0) * RANDSIGN();
+	gp->roty = FLOATRAND(1.0) * RANDSIGN();
+	gp->rotz = FLOATRAND(1.0) * RANDSIGN();
 
-    /* bell curve from 0-1.5 degrees, avg 0.75 */
-    gp->dx = (FLOATRAND(1) + FLOATRAND(1) + FLOATRAND(1)) / (360*2);
-    gp->dy = (FLOATRAND(1) + FLOATRAND(1) + FLOATRAND(1)) / (360*2);
-    gp->dz = (FLOATRAND(1) + FLOATRAND(1) + FLOATRAND(1)) / (360*2);
+	/* bell curve from 0-1.5 degrees, avg 0.75 */
+	gp->dx = (FLOATRAND(1) + FLOATRAND(1) + FLOATRAND(1)) / (360*2);
+	gp->dy = (FLOATRAND(1) + FLOATRAND(1) + FLOATRAND(1)) / (360*2);
+	gp->dz = (FLOATRAND(1) + FLOATRAND(1) + FLOATRAND(1)) / (360*2);
 
-    gp->d_max = gp->dx * 2;
+	gp->d_max = gp->dx * 2;
 
-    gp->ddx = 0.00006 + FLOATRAND(0.00003);
-    gp->ddy = 0.00006 + FLOATRAND(0.00003);
-    gp->ddz = 0.00006 + FLOATRAND(0.00003);
-
-    gp->ddx = 0.00001;
-    gp->ddy = 0.00001;
-    gp->ddz = 0.00001;
+	gp->ddx = 0.00006 + FLOATRAND(0.00003);
+	gp->ddy = 0.00006 + FLOATRAND(0.00003);
+	gp->ddz = 0.00006 + FLOATRAND(0.00003);
 
+	gp->ddx = 0.00001;
+	gp->ddy = 0.00001;
+	gp->ddz = 0.00001;
 	if ((gp->glx_context = init_GL(mi)) != NULL) {
 		reshape_gears(mi, MI_WIDTH(mi), MI_HEIGHT(mi));
+		glDrawBuffer(GL_BACK);
 		if (!pinit(mi)) {
 			MI_CLEARWINDOW(mi);
 		}
@@ -1145,30 +1202,30 @@ draw_gears(ModeInfo * mi)
 	int         angle_incr = MI_CYCLES(mi) ? MI_CYCLES(mi) : 2;
 	gearsstruct *gp;
 
-    if (gears == NULL)
-       return;
-    gp = &gears[MI_SCREEN(mi)];
-
-    MI_IS_DRAWN(mi) = True;
-    if (gp->planetary)
-      angle_incr *= 3;
-
+	if (gears == NULL)
+		return;
+	gp = &gears[MI_SCREEN(mi)];
 	if (!gp->glx_context)
 		return;
-
+	MI_IS_DRAWN(mi) = True;
 	glDrawBuffer(GL_BACK);
-
+#ifdef WIN32
+	wglMakeCurrent(hdc, gp->glx_context);
+#else
 	glXMakeCurrent(display, window, *(gp->glx_context));
+#endif
+	if (gp->planetary)
+		angle_incr *= 3;
 	draw(mi);
 
 	/* let's do something so we don't get bored */
 	gp->angle = (int) (gp->angle + angle_incr) % 360;
 
-    rotate(&gp->rotx, &gp->dx, &gp->ddx, gp->d_max, MI_IS_VERBOSE(mi));
-    rotate(&gp->roty, &gp->dy, &gp->ddy, gp->d_max, MI_IS_VERBOSE(mi));
-    rotate(&gp->rotz, &gp->dz, &gp->ddz, gp->d_max, MI_IS_VERBOSE(mi));
+	rotate(&gp->rotx, &gp->dx, &gp->ddx, gp->d_max, MI_IS_VERBOSE(mi));
+	rotate(&gp->roty, &gp->dy, &gp->ddy, gp->d_max, MI_IS_VERBOSE(mi));
+	rotate(&gp->rotz, &gp->dz, &gp->ddz, gp->d_max, MI_IS_VERBOSE(mi));
 
-    if (MI_IS_FPS(mi)) do_fps (mi);
+	if (MI_IS_FPS(mi)) do_fps (mi);
 	glFinish();
 	glXSwapBuffers(display, window);
 }
@@ -1181,7 +1238,7 @@ release_gears(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_gears(MI_DISPLAY(mi), &gears[screen]);
-		(void) free((void *) gears);
+		free(gears);
 		gears = (gearsstruct *) NULL;
 	}
 	FreeAllGL(mi);
