@@ -22,7 +22,7 @@ static const char sccsid[] = "@(#)glplanet.c	5.01 01/04/13 xlockmore";
  * Revision History:
  *
  * 13-Apr-01: rolf@groppe.de Did this on the xlockmore version
- * (21-Mar-01: jwz@jwz.org   Broke sphere routine out into its own file.
+ * (21-Mar-01: jwz AT jwz.org   Broke sphere routine out into its own file.
  *                           Done on the xscreensaver version)
  *
  * 5-Apr-01:  rolf@groppe.de    Ported this mode from xscreensaver to xlock.
@@ -32,7 +32,7 @@ static const char sccsid[] = "@(#)glplanet.c	5.01 01/04/13 xlockmore";
  *
  * 9-Oct-98:  dek@cgl.ucsf.edu  Added stars.
  *
- * 8-Oct-98:  jwz@jwz.org   Made the 512x512x1 xearth image be built in.
+ * 8-Oct-98:  jwz AT jwz.org   Made the 512x512x1 xearth image be built in.
  *                          Made it possible to load XPM or XBM files.
  *                          Made the planet bounce and roll around.
  *
@@ -41,7 +41,7 @@ static const char sccsid[] = "@(#)glplanet.c	5.01 01/04/13 xlockmore";
  *
  * BUGS:
  * -bounce is broken
- * 
+ *
  *   For even more spectacular results, grab the images from the "SSysten"
  *   package (http://www.msu.edu/user/kamelkev/) and do this:
  *
@@ -64,10 +64,11 @@ static const char sccsid[] = "@(#)glplanet.c	5.01 01/04/13 xlockmore";
 
 #ifdef VMS
 #include "vms_x_fix.h"
-#endif
 #include <X11/Intrinsic.h>
+#endif
 
 #ifdef STANDALONE
+# define MODE_glplanet
 # define PROGCLASS						"Planet"
 # define HACK_INIT						init_planet
 # define HACK_DRAW						draw_glplanet
@@ -94,26 +95,22 @@ static const char sccsid[] = "@(#)glplanet.c	5.01 01/04/13 xlockmore";
 
 #ifdef MODE_glplanet /* whole file */
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
-#if USE_XPMINC
-#include <xpm.h>
-#else
-#include <X11/xpm.h>            /* Normal spot */
-#endif /* USE_XPMINC */
+#ifdef HAVE_XPM
+#include <X11/xpm.h>
 # ifndef PIXEL_ALREADY_TYPEDEFED
 #  define PIXEL_ALREADY_TYPEDEFED /* Sigh, Xmu/Drawing.h needs this... */
 # endif
 #endif
 
 #ifdef USE_XMU
-# ifndef VMS
-#  include <X11/Xmu/Drawing.h>
-#else  /* VMS */
+# ifdef VMS
 #  include <Xmu/Drawing.h>
-# endif /* VMS */
+#else  /* !VMS */
+#  include <X11/Xmu/Drawing.h>
+# endif /* !VMS */
 #endif
 
-#include "sphere.h" 
+#include "sphere.h"
 
 #include <GL/glu.h>
 
@@ -154,19 +151,19 @@ static XrmOptionDescRec opts[] = {
 };
 
 static argtype vars[] = {
-  {(caddr_t *) &do_rotate,   (char *) "rotate",  (char *) "Rotate",
+  {(void *) &do_rotate,   (char *) "rotate",  (char *) "Rotate",
     (char *) DEF_ROTATE,  t_Bool},
-  {(caddr_t *) &do_roll,     (char *) "roll",    (char *) "Roll",
+  {(void *) &do_roll,     (char *) "roll",    (char *) "Roll",
     (char *) DEF_ROLL,    t_Bool},
-  {(caddr_t *) &do_bounce,   (char *) "bounce",  (char *) "Bounce",
+  {(void *) &do_bounce,   (char *) "bounce",  (char *) "Bounce",
     (char *) DEF_BOUNCE,  t_Bool},
-  {(caddr_t *) &do_texture,  (char *) "texture", (char *) "Texture",
+  {(void *) &do_texture,  (char *) "texture", (char *) "Texture",
     (char *) DEF_TEXTURE, t_Bool},
-  {(caddr_t *) &do_stars,    (char *) "stars",   (char *) "Stars",
+  {(void *) &do_stars,    (char *) "stars",   (char *) "Stars",
     (char *) DEF_STARS,   t_Bool},
-  {(caddr_t *) &do_light,    (char *) "light",   (char *) "Light",
+  {(void *) &do_light,    (char *) "light",   (char *) "Light",
     (char *) DEF_LIGHT,   t_Bool},
-  {(caddr_t *) &which_image, (char *) "pimage",  (char *) "Pimage",
+  {(void *) &which_image, (char *) "pimage",  (char *) "Pimage",
     (char *) DEF_IMAGE,   t_String},
 };
 
@@ -186,7 +183,7 @@ ModeSpecOpt glplanet_opts = {countof(opts), opts, countof(vars), vars, desc};
 ModStruct   glplanet_description =
 {"glplanet", "init_glplanet", "draw_glplanet", "release_glplanet",
  "draw_glplanet", "init_glplanet", (char *) NULL, &glplanet_opts,
- 1000, 1, 2, 1, 64, 1.0, "",
+ 15000, 1, 2, 1, 64, 1.0, "",
  "Animates texture mapped sphere (planet)", 0, NULL};
 #endif
 
@@ -216,11 +213,15 @@ ModStruct   glplanet_description =
 
 /* structure for holding the planet data */
 typedef struct {
+#ifdef WIN32
+  HGLRC glx_context;
+#else
+  GLXContext *glx_context;
+#endif
+  Window window;
   GLuint platelist;
   GLuint starlist;
   int screen_width, screen_height;
-  GLXContext *glx_context;
-  Window window;
 
   XColor fg, bg;
 
@@ -230,7 +231,7 @@ typedef struct {
   GLfloat dx, dy, dz;
   GLfloat box_width, box_height, box_depth;
   int roll_div, rot_div;
-  
+
 } planetstruct;
 
 
@@ -253,7 +254,7 @@ normalize(GLfloat v[3])
 #endif
 
 /* Set up and enable texturing on our object */
-static void
+static Bool
 setup_xbm_texture (unsigned char *bits, int width, int height,
 				   XColor *fgc, XColor *bgc)
 {
@@ -274,7 +275,7 @@ setup_xbm_texture (unsigned char *bits, int width, int height,
 	  {
 		unsigned char byte = bits [(y * (width / 8) + (x / 8))];
 		unsigned char bit = (byte & (1 << (x % 8))) >> (x % 8);
-		unsigned int word = (bit ? bg : fg); 
+		unsigned int word = (bit ? bg : fg);
 		*out++ = (word & 0xFF0000) >> 16;
 		*out++ = (word & 0x00FF00) >> 8;
 		*out++ = (word & 0x0000FF);
@@ -283,7 +284,12 @@ setup_xbm_texture (unsigned char *bits, int width, int height,
   clear_gl_error();
   glTexImage2D(GL_TEXTURE_2D, 0, 3, width, height, 0,
 			   GL_RGB, GL_UNSIGNED_BYTE, data);
-  check_gl_error("texture");
+  if (check_gl_error("texture")) {
+	if (data != NULL) {
+		free(data);
+	}
+	return False;
+  }
 
   /* setup parameters for texturing */
   glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
@@ -294,29 +300,32 @@ setup_xbm_texture (unsigned char *bits, int width, int height,
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 
   if (data != NULL) {
-	(void) free((void *) data);
+	free(data);
   }
+	return True;
 }
 
 
-static void
+static Bool
 setup_file_texture (ModeInfo *mi, char *filename)
 {
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
   {
 	char **xpm_data = 0;
-	int result = XpmReadFileToData (filename, &xpm_data); 
+	int result = XpmReadFileToData (filename, &xpm_data);
 	switch (result) {
 	case XpmSuccess:
 	  {
 		XImage *image = xpm_to_ximage (MI_DISPLAY(mi), MI_VISUAL(mi),
 			 MI_COLORMAP(mi), xpm_data);
 
-        clear_gl_error();
+		clear_gl_error();
 		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
 					 image->width, image->height, 0,
 					 GL_RGBA, GL_UNSIGNED_BYTE, image->data);
-        check_gl_error("texture");
+		if (check_gl_error("texture A")) {
+			return False;
+		}
 
 		/* setup parameters for texturing */
 		glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
@@ -327,31 +336,27 @@ setup_file_texture (ModeInfo *mi, char *filename)
 		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
 		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-		return;
+		return True;
 	  }
-	  break;
 
 	case XpmOpenFailed:
-	  fprintf (stderr, "Glplanet: file %s doesn't exist.\n", filename);
-	  exit (-1);
-	  break;
+	  (void) fprintf (stderr, "Glplanet: file %s doesn't exist.\n", filename);
+	  return False;
 
 	case XpmFileInvalid:
 	  /* Fall through and try it as an XBM. */
 	  break;
 
 	case XpmNoMemory:
-	  fprintf (stderr, "Glplanet: XPM: out of memory\n");
-	  exit (-1);
-	  break;
+	  (void) fprintf (stderr, "Glplanet: XPM: out of memory\n");
+	  return False;
 
 	default:
-	  fprintf (stderr, "Glplanet: XPM: unknown error code %d\n", result);
-	  exit (-1);
-	  break;
+	  (void) fprintf (stderr, "Glplanet: XPM: unknown error code %d\n", result);
+	  return False;
 	}
   }
-#endif /* USE_XPM */
+#endif /* HAVE_XPM */
 
 #ifdef USE_XMU
   {
@@ -364,39 +369,46 @@ setup_file_texture (ModeInfo *mi, char *filename)
 											&xhot, &yhot);
 	if (status != Success)
 	  {
-#if defined( USE_XPM ) || defined( USE_XPMINC )
-		fprintf (stderr, "Glplanet: not an XPM file: %s\n", filename);
+#ifdef HAVE_XPM
+		(void) fprintf (stderr, "Glplanet: not an XPM file: %s\n", filename);
 # endif
-		fprintf (stderr, "Glplanet: not an XBM file: %s\n", filename);
-		exit (1);
+		(void) fprintf (stderr, "Glplanet: not an XBM file: %s\n", filename);
+		return False;
 	  }
 
-	setup_xbm_texture ((unsigned char *) data, width, height, &gp->fg, &gp->bg);
+	if (!setup_xbm_texture ((unsigned char *) data, width, height,
+			&gp->fg, &gp->bg))
+		return False;
   }
+  return True;
 #else  /* !XMU */
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
-  fprintf (stderr, "Glplanet: not an XPM file: %s\n", filename);
+#ifdef HAVE_XPM
+  (void) fprintf (stderr, "Glplanet: not an XPM file: %s\n", filename);
 # endif
-  fprintf (stderr, "Glplanet: your vendor doesn't ship the standard Xmu library.\n");
-  fprintf (stderr, "Glplanet: we can't load XBM files without it.\n");
-  exit (1);
+  (void) fprintf (stderr, "Glplanet: your vendor doesn't ship the standard Xmu library.\n");
+  (void) fprintf (stderr, "Glplanet: we can't load XBM files without it.\n");
+  return False;
 #endif /* !XMU */
 }
 
 
-static void
+static Bool
 setup_texture(ModeInfo * mi)
 {
   planetstruct *gp = &planets[MI_SCREEN(mi)];
   if (!which_image ||
 	  !*which_image ||
-	  !strcmp(which_image, "BUILTIN"))
-	setup_xbm_texture (earth_bits, earth_width, earth_height,
-					   &gp->fg, &gp->bg);
+	  !strcmp(which_image, "BUILTIN")) {
+	if (!setup_xbm_texture (earth_bits, earth_width, earth_height,
+					   &gp->fg, &gp->bg))
+		return False;
 
-  else
-	setup_file_texture (mi, which_image);
+  } else {
+	if (!setup_file_texture (mi, which_image))
+		return False;
+  }
+  return True;
 }
 
 
@@ -416,21 +428,22 @@ static void
 setup_face(void)
 {
   glEnable(GL_CULL_FACE);
-  glCullFace(GL_BACK); 
+  glCullFace(GL_BACK);
 }
 
 /* lame way to generate some random stars */
-void generate_stars(ModeInfo * mi, int width, int height)
+static void
+generate_stars(ModeInfo * mi, int width, int height)
 {
   int i;
 /*  GLfloat size_range[2], size;*/
   GLfloat x, y;
 
   planetstruct *gp = &planets[MI_SCREEN(mi)];
-  
+
 /*    glGetFloatv(GL_POINT_SIZE_RANGE, size_range); */
-  
-/*    printf("size range: %f\t%f\n", size_range[0], size_range[1]); */
+
+/*    (void) printf("size range: %f\t%f\n", size_range[0], size_range[1]); */
   gp->starlist = glGenLists(1);
   glNewList(gp->starlist, GL_COMPILE);
 
@@ -448,7 +461,7 @@ void generate_stars(ModeInfo * mi, int width, int height)
   glEnable(GL_POINT_SMOOTH);
   glEnable(GL_BLEND);
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-  
+
   glBegin(GL_POINTS);
   for(i = 0 ; i < NUM_STARS ; i++)
 	{
@@ -483,8 +496,10 @@ pinit(ModeInfo * mi)
   }
 
   /* turn on various options we like */
-  if (do_texture)
-	setup_texture(mi);
+  if (do_texture) {
+	if (!setup_texture(mi))
+		do_texture = False;
+  }
   if (do_light)
 	setup_light();
 
@@ -516,7 +531,7 @@ draw_sphere_glp(ModeInfo * mi)
 	glEnable(GL_TEXTURE_2D);
 
   if (do_light)
-	{ 
+	{
 	  glEnable(GL_LIGHTING);
 	  glEnable(GL_LIGHT0);
 	  glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
@@ -540,7 +555,7 @@ pick_velocity (ModeInfo * mi)
   if (gp->roll_div <= MINROLL) gp->roll_div = MINROLL * 10;
   gp->rot_div = MI_CYCLES(mi);
   if (gp->rot_div <= MINROT) gp->rot_div = MINROT * 50;
-  
+
   roll1 = gp->roll_div / 100.0;
   roll2 = roll1 * 0.75;
   rot1  = gp->rot_div  / 100.0;
@@ -561,7 +576,7 @@ pick_velocity (ModeInfo * mi)
 /* following values are unused
   gp->dx = (FLOATRAND(0.2) + FLOATRAND(0.2)) * RANDSIGN();
   gp->dy = (FLOATRAND(0.2) + FLOATRAND(0.2)) * RANDSIGN();
-  gp->dz = (FLOATRAND(0.2) + FLOATRAND(0.2)) * RANDSIGN(); */ 
+  gp->dz = (FLOATRAND(0.2) + FLOATRAND(0.2)) * RANDSIGN(); */
 }
 
 
@@ -601,7 +616,7 @@ rotate_and_move (ModeInfo * mi)
 */
 /* Original function replaced for faster execution */
 #     define SINOID(SCALE,SIZE) \
-        (sin(frame * (SCALE)) / 2.0) * (SIZE) 
+        (sin(frame * (SCALE)) / 2.0) * (SIZE)
       gp->xpos = SINOID(0.006086836, gp->box_width);
       gp->ypos = SINOID(0.004516039, gp->box_height);
       gp->zpos = SINOID(0.003337942, gp->box_depth);
@@ -670,18 +685,18 @@ init_glplanet(ModeInfo * mi)
 
     if (!XParseColor(MI_DISPLAY(mi), MI_COLORMAP(mi), f, &gp->fg))
       {
-		fprintf(stderr, "Glplanet: unparsable color: \"%s\"\n", f);
+		(void) fprintf(stderr, "Glplanet: unparsable color: \"%s\"\n", f);
 		exit(1);
       }
     if (!XParseColor(MI_DISPLAY(mi), MI_COLORMAP(mi), b, &gp->bg))
       {
-		fprintf(stderr, "Glplanet: unparsable color: \"%s\"\n", f);
+		(void) fprintf(stderr, "Glplanet: unparsable color: \"%s\"\n", f);
 		exit(1);
       }
 
-	(void) free((void *) f);
-	(void) free((void *) b);
-  } 
+	free(f);
+	free(b);
+  }
 
 
   gp->window = MI_WINDOW(mi);
@@ -703,11 +718,15 @@ draw_glplanet(ModeInfo * mi)
   if (!gp->glx_context)
 	return;
 
+  MI_IS_DRAWN(mi) = True;
+
   glDrawBuffer(GL_BACK);
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-
+#ifdef WIN32
+  wglMakeCurrent(hdc, gp->glx_context);
+#else
   glXMakeCurrent(display, window, *(gp->glx_context));
-
+#endif
 
   if (do_stars) {
 	/* protect our modelview matrix and attributes */
@@ -761,7 +780,11 @@ release_glplanet(ModeInfo * mi)
 
 	  if (gp->glx_context) {
 		/* Display lists MUST be freed while their glXContext is current. */
+#ifdef WIN32
+		wglMakeCurrent(hdc, gp->glx_context);
+#else
 		glXMakeCurrent(MI_DISPLAY(mi), gp->window, *(gp->glx_context));
+#endif
 
 		if (glIsList(gp->platelist))
 		  glDeleteLists(gp->platelist, 1);
@@ -769,12 +792,10 @@ release_glplanet(ModeInfo * mi)
 		  glDeleteLists(gp->starlist, 1);
 	  }
 	}
-	(void) free((void *) planets);
+	free(planets);
 	planets = (planetstruct *) NULL;
   }
   FreeAllGL(mi);
 }
 
-
 #endif
-
