@@ -68,7 +68,7 @@ void printSpline(FILE *fp, TwoJetVec v00, TwoJetVec v01,
 			   TwoJetVec v10, TwoJetVec v11,
 			   double us, double vs,
 			   double s0, double s1, double t0, double t1, int binary) {
-    if (bezier) {
+  if (bezier) {
     print_point(fp, v00, 1, 0, 0, 0, binary);
     print_point(fp, v00, 1, us, 0, 0, binary);
     print_point(fp, v10, 1,-us, 0, 0, binary);
@@ -90,11 +90,14 @@ void printSpline(FILE *fp, TwoJetVec v00, TwoJetVec v01,
     print_point(fp, v11, 1, 0, 0, 0, binary);
 
     if (binary) {
-      float sts[8] = {s0,t0, s1,t0, s0,t1, s1,t1};
+      float sts[8] = {(float) s0, (float) t0,
+		(float) s1, (float) t0,
+		(float) s0, (float) t1,
+		(float) s1, (float) t1};
       (void) fwrite(&sts, sizeof(float), 8, fp);
     } else {
       (void) fprintf(fp, "%g %g  %g %g  %g %g  %g %g\n\n",
-  	s0,t0,  s1,t0,  s0,t1, s1,t1);
+	s0,t0,  s1,t0,  s0,t1, s1,t1);
     }
   }
   else {
@@ -148,12 +151,14 @@ char *parse_parts(char *parts)
 	    if(cp == ncp) {
 		(void) fprintf(stderr,
 "evert -parts: expected string with alternating signs and strip numbers\n");
-		return NULL;
+		free(partlist);
+		return (char *) NULL;
 	    }
 	    if(j < 0 || j >= n_strips) {
 		(void) fprintf(stderr,
 "evert -parts: bad strip number %d; must be in range 0..%d\n", j, n_strips-1);
-		return NULL;
+		free(partlist);
+		return (char *) NULL;
 	    }
 	    partlist[j] |= bits;
 	    cp = ncp;
@@ -271,6 +276,7 @@ void printScene(
 	    printMesh(values[j][k]);
 	}
 	glEnd();
+    }
 #else
     for(j = 0; j <= jmax; j++) {
         for(k = 0; k <= kmax; k++)
@@ -278,9 +284,9 @@ void printScene(
 
 	if(!binary)
 	    fputc('\n', fp);
+    }
 #endif
 
-    }
   }
 
 #ifndef XLOCK
@@ -291,21 +297,21 @@ void printScene(
 
   for (j = 0; j <= jmax; j++) {
     if (values[j] != NULL) {
-      (void) free((void *) values[j]);
-      values[j] = NULL;
+      free(values[j]);
+      values[j] = (TwoJetVec *) NULL;
     }
     if (speedu[j] != NULL) {
-      (void) free((void *) speedu[j]);
-      speedu[j] = NULL;
+      free(speedu[j]);
+      speedu[j] = (double *) NULL;
     }
   }
-  (void) free((void *) values);
-  (void) free((void *) speedu);
-  (void) free((void *) speedv);
+  free(values);
+  free(speedu);
+  free(speedv);
 }
 
+#if 0
 void impossible(char *msg) {
   (void) fprintf(stderr, "%s\n", msg);
-  exit(1);
 }
-
+#endif
