@@ -6,7 +6,7 @@ static const char sccsid[] = "@(#)lament.c	5.03 2001/11/28 xlockmore";
 
 #endif
 
-/* xscreensaver, Copyright (c) 1998 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1998 Jamie Zawinski <jwz AT jwz.org>
 
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -43,7 +43,7 @@ static const char sccsid[] = "@(#)lament.c	5.03 2001/11/28 xlockmore";
         the images so that the edges may be symmetrical.
 
      *  I want the gold leaf to seem to be raised up from the surface, but I
-        think this isn't possible with OpenGL.  Supposedly, OpenGL only 
+        think this isn't possible with OpenGL.  Supposedly, OpenGL only
         supports Gouraud shading (interpolating edge normals from face normals,
         and shading smoothly) but bump-maps only work with Phong shading
         (computing a normal for each rendered pixel.)
@@ -57,11 +57,11 @@ static const char sccsid[] = "@(#)lament.c	5.03 2001/11/28 xlockmore";
 
 	But apparently there are tricky ways around this:
 	http://reality.sgi.com/opengl/tips/rts/
-	I think these techniques require GLUT, however, which isn't 
+	I think these techniques require GLUT, however, which isn't
 	(currently) required by any other xscreensaver hacks.
 
      *  There should be strange lighting effects playing across the surface:
-        electric sparks, or little glittery blobs of light.  
+        electric sparks, or little glittery blobs of light.
         http://reality.sgi.com/opengl/tips/lensflare/ might provide guidance.
 
      *  Need to add some more modes, to effect the transition from the cube
@@ -73,7 +73,7 @@ static const char sccsid[] = "@(#)lament.c	5.03 2001/11/28 xlockmore";
         simply be rotated along the axis to which one cannot point.)
 
         The other hard thing about this is that the "leviathan" shapes contain
-        a much larger number of facets, and I modelled this whole thing by 
+        a much larger number of facets, and I modelled this whole thing by
         hand, since I don't have any 3d-object-editing tools that I know how
         to use (or that look like they would take any less than several months
         to become even marginally proficient with...)
@@ -131,7 +131,7 @@ static XrmOptionDescRec opts[] =
 
 static argtype vars[] =
 {
-	{(caddr_t *) & do_texture, (char *) "texture", (char *) "Texture", (char *) DEF_TEXTURE, t_Bool},
+	{(void *) & do_texture, (char *) "texture", (char *) "Texture", (char *) DEF_TEXTURE, t_Bool},
 };
 
 static OptionStruct desc[] =
@@ -202,8 +202,11 @@ static GLfloat interior_color[] = { 0.20, 0.20, 0.15, 1.00,  /* ambient    */
                                   };
 
 typedef struct {
-	GLXContext *glx_context;
-
+#ifdef WIN32
+        HGLRC       glx_context;
+#else
+        GLXContext *glx_context;
+#endif
 	GLuint      box;	/* display list IDs */
 	GLuint      star1, star2;
 	GLuint      tetra_une, tetra_usw, tetra_dwn, tetra_dse, tetra_mid;
@@ -2212,7 +2215,11 @@ static void
 free_lament(Display *display, lament_configuration *lc)
 {
 	if (lc->glx_context) {
+#ifdef WIN32
+		wglMakeCurrent(hdc, lc->glx_context);
+#else
 		glXMakeCurrent(display, lc->window, *(lc->glx_context));
+#endif
 		if (glIsList(lc->box)) {
 			glDeleteLists(lc->box, 16);
 			lc->box = 0;
@@ -2430,8 +2437,11 @@ draw_lament(ModeInfo * mi)
 	MI_IS_DRAWN(mi) = True;
 
 	glDrawBuffer(GL_BACK);
-
+#ifdef WIN32
+	wglMakeCurrent(hdc, lc->glx_context);
+#else
 	glXMakeCurrent(display, window, *(lc->glx_context));
+#endif
 	draw(mi);
         if (MI_IS_FPS(mi)) do_fps (mi);
 
