@@ -52,6 +52,165 @@ static const char sccsid[] = "@(#)morph3d.c	5.01 2001/03/01 xlockmore";
  *
  */
 
+/* Mathematics.txt doc
+ Basics of morph3d Engine
+
+This document is VERY incomplete, but tries to describe the mathematics used
+in the program. At this moment it just describes how the polyhedra are 
+generated. On further versions, this document will probably be improved.
+
+Marcelo Fernandes Vianna 
+- Feb-13-1997
+
+POLYHEDRA GENERATION
+
+For the purpose of this program it's not sufficient to know the polyhedra
+vertexes coordinates. Since the morphing algorithm applies a nonlinear 
+transformation over the surfaces (faces) of the polyhedron, each face has
+to be divided into smaller ones. The morphing algorithm needs to transform 
+each vertex of these smaller faces individually. It's a very time consuming
+task.
+
+In order to reduce calculation overload, and since all the macro faces of
+the polyhedron are transformed by the same way, the generation is made by 
+creating only one face of the polyhedron, morphing it and then rotating it
+around the polyhedron center. 
+
+What we need to know is the face radius of the polyhedron (the radius of 
+the inscribed sphere) and the angle between the center of two adjacent 
+faces using the center of the sphere as the angle's vertex.
+
+The face radius of the regular polyhedra are known values which I decided
+to not waste my time calculating. Following is a table of face radius for
+the regular polyhedra with edge length = 1:
+
+    TETRAHEDRON  : 1/(2*sqrt(2))/sqrt(3)
+    CUBE         : 1/2
+    OCTAHEDRON   : 1/sqrt(6)
+    DODECAHEDRON : T^2 * sqrt((T+2)/5) / 2     -> where T=(sqrt(5)+1)/2
+    ICOSAHEDRON  : (3*sqrt(3)+sqrt(15))/12
+
+I've not found any reference about the mentioned angles, so I needed to
+calculate them, not a trivial task until I figured out how :)
+Curiously these angles are the same for the tetrahedron and octahedron.
+A way to obtain this value is inscribing the tetrahedron inside the cube
+by matching their vertexes. So you'll notice that the remaining unmatched
+vertexes are in the same straight line starting in the cube/tetrahedron
+center and crossing the center of each tetrahedron's face. At this point
+it's easy to obtain the bigger angle of the isosceles triangle formed by
+the center of the cube and two opposite vertexes on the same cube face.
+The edges of this triangle have the following lengths: sqrt(2) for the base
+and sqrt(3)/2 for the other two other edges. So the angle we want is:
+     +-----------------------------------------------------------+
+     | 2*ARCSIN(sqrt(2)/sqrt(3)) = 109.47122063449069174 degrees |
+     +-----------------------------------------------------------+
+For the cube this angle is obvious, but just for formality it can be
+easily obtained because we also know it's isosceles edge lengths:
+sqrt(2)/2 for the base and 1/2 for the other two edges. So the angle we 
+want is:
+     +-----------------------------------------------------------+
+     | 2*ARCSIN((sqrt(2)/2)/1)   = 90.000000000000000000 degrees |
+     +-----------------------------------------------------------+
+For the octahedron we use the same idea used for the tetrahedron, but now
+we inscribe the cube inside the octahedron so that all cubes's vertexes
+matches exactly the center of each octahedron's face. It's now clear that
+this angle is the same of the tetrahedron one:
+     +-----------------------------------------------------------+
+     | 2*ARCSIN(sqrt(2)/sqrt(3)) = 109.47122063449069174 degrees |
+     +-----------------------------------------------------------+
+For the dodecahedron it's a little bit harder because it's only relationship
+with the cube is useless to us. So we need to solve the problem by another
+way. The concept of Face radius also exists on 2D polygons with the name
+Edge radius:
+  Edge Radius For Pentagon (ERp)
+  ERp = (1/2)/TAN(36 degrees) * VRp = 0.6881909602355867905
+  (VRp is the pentagon's vertex radio).
+  Face Radius For Dodecahedron
+  FRd = T^2 * sqrt((T+2)/5) / 2 = 1.1135163644116068404
+Why we need ERp? Well, ERp and FRd segments forms a 90 degrees angle, 
+completing this triangle, the lesser angle is a half of the angle we are 
+looking for, so this angle is:
+     +-----------------------------------------------------------+
+     | 2*ARCTAN(ERp/FRd)         = 63.434948822922009981 degrees |
+     +-----------------------------------------------------------+
+For the icosahedron we can use the same method used for dodecahedron (well
+the method used for dodecahedron may be used for all regular polyhedra)
+  Edge Radius For Triangle (this one is well known: 1/3 of the triangle height)
+  ERt = sin(60)/3 = sqrt(3)/6 = 0.2886751345948128655
+  Face Radius For Icosahedron
+  FRi= (3*sqrt(3)+sqrt(15))/12 = 0.7557613140761707538
+So the angle is:
+     +-----------------------------------------------------------+
+     | 2*ARCTAN(ERt/FRi)         = 41.810314895778596167 degrees |
+     +-----------------------------------------------------------+
+
+Update May-23-2005  [Maintainer wants to add a rhombic dodecahedron and the
+    rhombic triacontahedron]
+
+   In the below figure, you will find most of the entities I mention in
+   the "Mathematics.txt" doc I provided years ago along with morph3d.
+   I'm writing this new document because I foresee a pitfall you will
+   have to deal to add non-regular polyhedra to the mode.  Please, read
+   the Mathematics.txt document together with this e-mail for the sake of
+   completeness.
+   One important thing you need to elaborate before adding any solid to
+   morph3d is how to divide each of its faces in smaller ones (triangles
+   or squares).  This is the only way you will have inner vertexes  (as P
+   in the example of the below figure) for each face that will be subject
+   for the transformations in the mode.
+
+    Figure/Diagram example (not here) shows a dodecahedron
+       SC = Solid Center (Center of polyhedron)
+       FC = Face Center (Center of a polygon) 
+       P =  Arbitrary Point inside the face plan
+       Alpha = Faces Angle (internal angle from center of polyhedron)
+
+   Face Radius (see Mathematics.txt) = Distance between FC and SC
+   (mentioned in Mathematics.txt
+   Alpha = the angle between two faces using SC as the vertex.
+   Both entities (Face Radius and alpha) are useful for regular
+   polyhedra, but for non-regular ones you will have to make other
+   calculation and probably need more than one macro (actually one for
+   each different face shape and radius).  They are actually used to
+   minimize calculation by drawing one face only and than replicate it.
+   This won't be possible on non-regular polyhedron, at least not for all
+   the faces, but only for the equivalent ones.
+   The distortion effect is obtained by multiplying the P vector ( P-SC
+   where SC is (0,0,0) ) by a factor that is proportional to the P-FC
+   vector, so the face center simply is unchanged and the face edges are
+   the most affected.
+   The big pitfall here is that the distortion very near the edges are
+   always equivalent for neighbor faces, while it may not be true for
+   non-regular ones, so you will probably need to use an adjustment
+   factor, otherwise, if the distances are different, you will se non
+   continuous faces, actually a rip will occur.
+   Thinking about this now, makes me think you could probably use a
+   factor other than P-FC for the distortion intensity.  Considering that
+   for some non-regular solids you still have all the face radius with
+   the same value (it seems to be the case of some you mentioned to me)
+   you can think of imaginarily inscribing a sphere inside the solid and
+   using the distance between P and the sphere as the multiplyer.  This
+   surely will avoid the rips, but won't work for a number of other
+   solids.
+
+Update May-24-2005
+
+   Actually the edge lengths are not of much importance, but the face
+   radius is.  You'll understand it better when you read Mathematics.txt
+   together with the previous e-mail.
+
+   About my comment on using the distance between an inscribed sphere and
+   the point P instead of the face center and point P, this actually shall
+   work for all solids AFAICS.
+
+   Dividing the faces into triangles is not a big problem, actually the
+   pentagon really was the only difficult one (I made it dividing it into 5
+   triangles at first, and then dividing each triangle into smaller ones).
+   For cube, as far as I remember, I divided the faces into smaller
+   squares, not triangles, although triangles would also be possible.
+
+*/
+
 #ifdef VMS
 /*-
  * due to a Bug/feature in VMS X11/Intrinsic.h has to be placed before xlock.
@@ -72,7 +231,7 @@ static const char sccsid[] = "@(#)morph3d.c	5.01 2001/03/01 xlockmore";
 #include "xlockmore.h"		/* from the xscreensaver distribution */
 #else /* !STANDALONE */
 #include "xlock.h"		/* from the xlockmore distribution */
-#include "vis.h"
+#include "visgl.h"
 #endif /* !STANDALONE */
 
 #ifdef MODE_moebius
@@ -125,6 +284,11 @@ ModStruct   morph3d_description =
 /*************************************************************************/
 
 typedef struct {
+#ifdef WIN32
+	HGLRC       glx_context;
+#else
+	GLXContext *glx_context;
+#endif
 	GLint       WindH, WindW;
 	GLfloat     step;
 	GLfloat     seno;
@@ -134,7 +298,7 @@ typedef struct {
 	void        (*draw_object) (ModeInfo * mi);
 	float       Magnitude;
 	float      *MaterialColor[20];
-	GLXContext *glx_context;
+
 } morph3dstruct;
 
 static float front_shininess[] =
@@ -614,7 +778,7 @@ draw_icosa(ModeInfo * mi)
 }
 
 static void
-reshape(ModeInfo * mi, int width, int height)
+reshape_morph3d(ModeInfo * mi, int width, int height)
 {
 	morph3dstruct *mp = &morph3d[MI_SCREEN(mi)];
 
@@ -750,10 +914,9 @@ init_morph3d(ModeInfo * mi)
 	mp = &morph3d[MI_SCREEN(mi)];
 	mp->step = NRAND(90);
 	mp->VisibleSpikes = 1;
-
 	if ((mp->glx_context = init_GL(mi)) != NULL) {
 
-		reshape(mi, MI_WIDTH(mi), MI_HEIGHT(mi));
+		reshape_morph3d(mi, MI_WIDTH(mi), MI_HEIGHT(mi));
 		glDrawBuffer(GL_BACK);
 		mp->object = MI_COUNT(mi);
 		if (mp->object <= 0 || mp->object > 5)
@@ -774,14 +937,14 @@ draw_morph3d(ModeInfo * mi)
 	if (morph3d == NULL)
 		return;
 	mp = &morph3d[MI_SCREEN(mi)];
-
-	MI_IS_DRAWN(mi) = True;
-
 	if (!mp->glx_context)
 		return;
-
+	MI_IS_DRAWN(mi) = True;
+#ifdef WIN32
+	wglMakeCurrent(hdc, mp->glx_context);
+#else
 	glXMakeCurrent(display, window, *(mp->glx_context));
-
+#endif
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
 	glPushMatrix();
