@@ -55,11 +55,12 @@ static const char sccsid[] = "@(#)sproingiewrap.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_sproingies
 #define PROGCLASS "Sproingies"
 #define HACK_INIT init_sproingies
 #define HACK_DRAW draw_sproingies
 #define sproingies_opts xlockmore_opts
-#define DEFAULTS "*delay: 100 \n" \
+#define DEFAULTS "*delay: 80000 \n" \
  "*count: 5 \n" \
  "*cycles: 0 \n" \
  "*size: 0 \n" \
@@ -67,7 +68,7 @@ static const char sccsid[] = "@(#)sproingiewrap.c	5.00 2000/11/01 xlockmore";
 #include "xlockmore.h"		/* from the xscreensaver distribution */
 #else /* !STANDALONE */
 #include "xlock.h"		/* from the xlockmore distribution */
-#include "vis.h"
+#include "visgl.h"
 #endif /* !STANDALONE */
 
 #ifdef MODE_sproingies
@@ -79,7 +80,7 @@ ModeSpecOpt sproingies_opts =
 ModStruct   sproingies_description =
 {"sproingies", "init_sproingies", "draw_sproingies", "release_sproingies",
  "refresh_sproingies", "init_sproingies", (char *) NULL, &sproingies_opts,
- 1000, 5, 0, 400, 64, 1.0, "",
+ 80000, 5, 0, 0, 64, 1.0, "",
  "Shows Sproingies!  Nontoxic.  Safe for pets and small children", 0, NULL};
 
 #endif
@@ -101,14 +102,18 @@ void        CleanupSproingies(int screen);
 Bool        InitSproingies(int wfmode, int grnd, int mspr, int screen, int numscreens, int mono);
 
 typedef struct {
+#ifdef WIN32
+	HGLRC       glx_context;
+#else
+	GLXContext *glx_context;
+#endif
+	Window      window;
 	GLfloat     view_rotx, view_roty, view_rotz;
 	GLint       gear1, gear2, gear3;
 	GLfloat     angle;
 	GLuint      limit;
 	GLuint      count;
-	GLXContext *glx_context;
 	int         mono;
-	Window      window;
 	int         init;
 } sproingiesstruct;
 
@@ -208,9 +213,12 @@ draw_sproingies(ModeInfo * mi)
 	MI_IS_DRAWN(mi) = True;
 	if (!sp->glx_context)
 		return;
-
 	glDrawBuffer(GL_BACK);
+#ifdef WIN32
+	wglMakeCurrent(hdc, sp->glx_context);
+#else
 	glXMakeCurrent(display, window, *(sp->glx_context));
+#endif
 
 	swap_display = display;
 	swap_window = window;
@@ -238,13 +246,16 @@ release_sproingies(ModeInfo * mi)
 			sproingiesstruct *sp = &sproingies[screen];
 
 			if (sp->glx_context) {
-
+#ifdef WIN32
+				wglMakeCurrent(hdc, sp->glx_context);
+#else
 				glXMakeCurrent(MI_DISPLAY(mi), sp->window, *(sp->glx_context));
+#endif
 				CleanupSproingies(screen);
 			}
 		}
 
-		(void) free((void *) sproingies);
+		free(sproingies);
 		sproingies = (sproingiesstruct *) NULL;
 	}
 	FreeAllGL(mi);
