@@ -34,12 +34,12 @@ static const char sccsid[] = "@(#)stairs.c	5.01 2001/03/01 xlockmore";
  * mistakes.
  *
  * My e-mail address is
- * mfvianna@centroin.com.br
+ * mfvianna AT centroin.com.br
  *
  * Marcelo F. Vianna (Jun-01-1997)
  *
  * Revision History:
- * 01-Mar-2001: Added FPS stuff - Eric Lassauge <lassauge@mail.dotcom.fr>
+ * 01-Mar-2001: Added FPS stuff - Eric Lassauge <lassauge AT users.sourceforge.net>
  * 01-Nov-2000: Allocation checks
  * 07-Jan-1998: This would be a scene for the escher mode, but now escher mode
  *              was splitted in different modes for each scene. This is the
@@ -58,10 +58,12 @@ static const char sccsid[] = "@(#)stairs.c	5.01 2001/03/01 xlockmore";
  *
  * In real OpenGL, PseudoColor DO NOT support texture map (as far as I know).
  */
-
+#ifdef VMS
 #include <X11/Intrinsic.h>
+#endif
 
 #ifdef STANDALONE
+#define MODE_stairs
 #define PROGCLASS "stairs"
 #define HACK_INIT init_stairs
 #define HACK_DRAW draw_stairs
@@ -107,6 +109,11 @@ ModStruct   stairs_description =
 /*************************************************************************/
 
 typedef struct {
+#ifdef WIN32
+	HGLRC       glx_context;
+#else
+	GLXContext *glx_context;
+#endif
 	GLint       WindH, WindW;
 	GLfloat     step;
 	Bool        direction;
@@ -114,7 +121,6 @@ typedef struct {
 	Bool        AreObjectsDefined[MaxObj];
 #endif
 	int         sphere_position;
-	GLXContext *glx_context;
 } stairsstruct;
 
 static float front_shininess[] =
@@ -335,7 +341,7 @@ draw_stairs_internal(ModeInfo * mi)
 }
 
 static void
-reshape(ModeInfo * mi, int width, int height)
+reshape_stairs(ModeInfo * mi, int width, int height)
 {
 	stairsstruct *sp = &stairs[MI_SCREEN(mi)];
 	int i;
@@ -408,7 +414,7 @@ release_stairs(ModeInfo * mi)
 			}
 		}
 #endif
-		(void) free((void *) stairs);
+		free(stairs);
 		stairs = (stairsstruct *) NULL;
 	}
 	FreeAllGL(mi);
@@ -429,17 +435,15 @@ init_stairs(ModeInfo * mi)
 	sp->step = 0.0;
 	sp->direction = LRAND() & 1;
 	sp->sphere_position = NRAND(NPOSITIONS / 3) * 3;
-
 	if ((sp->glx_context = init_GL(mi)) != NULL) {
-
-		reshape(mi, MI_WIDTH(mi), MI_HEIGHT(mi));
-		glDrawBuffer(GL_BACK);
 #if 0
 		if (!glIsList(objects))
 			if ((objects = glGenLists(MaxObj)) == 0) {
 				MI_CLEARWINDOW(mi);
 			}
 #endif
+		reshape_stairs(mi, MI_WIDTH(mi), MI_HEIGHT(mi));
+		glDrawBuffer(GL_BACK);
 		pinit();
 	} else {
 		MI_CLEARWINDOW(mi);
@@ -456,13 +460,15 @@ draw_stairs(ModeInfo * mi)
 	if (stairs == NULL)
 			return;
 	sp = &stairs[MI_SCREEN(mi)];
-
-	MI_IS_DRAWN(mi) = True;
-	if (!sp->glx_context)
+	if (!sp->glx_context) {
 		return;
-
+	}
+#ifdef WIN32
+	wglMakeCurrent(hdc, sp->glx_context);
+#else
 	glXMakeCurrent(display, window, *(sp->glx_context));
-
+#endif
+	MI_IS_DRAWN(mi) = True;
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
 	glPushMatrix();
@@ -508,8 +514,11 @@ change_stairs(ModeInfo * mi)
 
 	if (!sp->glx_context)
 		return;
-
+#ifdef WIN32
+	wglMakeCurrent(hdc, sp->glx_context);
+#else
 	glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(sp->glx_context));
+#endif
 	pinit();
 }
 
