@@ -26,7 +26,12 @@ extern      "C" {
 #include <GL/gl.h>
 #include <GL/glx.h>
 } typedef struct {
-
+#ifdef WIN32
+	HGLRC       glx_context;
+#else
+	GLXContext *glx_context;
+#endif
+	GLint       WinH, WinW;
 	/* global Parameters */
 	int         wire, counter, animation;
 	float       extrusion;
@@ -34,9 +39,7 @@ extern      "C" {
 	float       rfreq;
 	int 	    direction;
 	char       *words, *words_start;
-	/* per Screen variables */
-	GLint       WinH, WinW;
-	GLXContext *glx_context;
+	/* per Screen varialbles */
 	FTFace     *face;
 	float       center_x, center_y, center_z;
 	float       phi, theta, radius;
