@@ -23,13 +23,13 @@ static const char sccsid[] = "@(#)goop.c	5.00 2000/11/01 xlockmore";
  *
  * Revision History:
  * 01-Nov-2000: Allocation checks
- * 24-Mar-1998: xlock version David Bagley <bagleyd@tux.org>
- * 1997: xscreensaver version Jamie Zawinski <jwz@jwz.org>
+ * 24-Mar-1998: xlock version David Bagley <bagleyd AT verizon.net>
+ * 1997: xscreensaver version Jamie Zawinski <jwz AT jwz.org>
  */
 
 /*-
  * original copyright
- * xscreensaver, Copyright (c) 1997 Jamie Zawinski <jwz@jwz.org>
+ * xscreensaver, Copyright (c) 1997 Jamie Zawinski <jwz AT jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -71,6 +71,7 @@ static const char sccsid[] = "@(#)goop.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_goop
 #define PROGCLASS "Goop"
 #define HACK_INIT init_goop
 #define HACK_DRAW draw_goop
@@ -320,8 +321,8 @@ make_layer(ModeInfo * mi, layer * l, int nblobs)
 	blob_max = (width < height ? width : height) / 2;
 	blob_min = (blob_max * 2) / 3;
 	for (i = 0; i < l->nblobs; i++)
-		if (!make_blob(&(l->blobs[i]), width, height,
-			  (int) (LRAND() % (blob_max - blob_min)) + blob_min))
+		if (!make_blob(&(l->blobs[i]), width, height, (int) (LRAND() %
+				(blob_max - blob_min + 1)) + blob_min))
 			return False;
 
 	if ((l->pixmap = XCreatePixmap(MI_DISPLAY(mi), MI_WINDOW(mi),
@@ -376,17 +377,17 @@ free_goop(Display * display, goopstruct * gp)
 
 				for (b = 0; b < gp->layers[l].nblobs; b++) {
 					if (gp->layers[l].blobs[b].r != NULL)
-						(void) free((void *) gp->layers[l].blobs[b].r);
+						free(gp->layers[l].blobs[b].r);
 					free_spline(gp->layers[l].blobs[b].splines);
 				}
-				(void) free((void *) gp->layers[l].blobs);
+				free(gp->layers[l].blobs);
 			}
 			if (gp->layers[l].gc != None)
 				XFreeGC(display, gp->layers[l].gc);
 			if (gp->layers[l].pixmap != None)
 				XFreePixmap(display, gp->layers[l].pixmap);
 		}
-		(void) free((void *) gp->layers);
+		free(gp->layers);
 		gp->layers = (layer *) NULL;
 	}
 	if (gp->pixmap_gc != None) {
@@ -407,7 +408,7 @@ init_goop(ModeInfo * mi)
 	int         i;
 	XGCValues   gcv;
 	int         nblobs;
-	unsigned long *plane_masks = 0;
+	unsigned long *plane_masks = NULL;
 	unsigned long base_pixel = 0;
 	goopstruct *gp;
 
@@ -478,7 +479,7 @@ init_goop(ModeInfo * mi)
 			if (!make_layer(mi, &(gp->layers[i]),
 				   (nblobs > 0 ? nblobs : lblobs[i])))
 				free_goop(display, gp);
-		(void) free((void *) lblobs);
+		free(lblobs);
 	}
 
 	if (gp->mode == transparent && plane_masks) {
@@ -486,8 +487,8 @@ init_goop(ModeInfo * mi)
 			gp->layers[i].pixel = base_pixel | plane_masks[i];
 		gp->background = base_pixel;
 	}
-	if (plane_masks)
-		(void) free((void *) plane_masks);
+	if (plane_masks != NULL)
+		free(plane_masks);
 
 	if (gp->mode != transparent) {
 		gp->background = 0;	/* init */
@@ -609,7 +610,7 @@ release_goop(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_goop(MI_DISPLAY(mi), &goops[screen]);
-		(void) free((void *) goops);
+		free(goops);
 		goops = (goopstruct *) NULL;
 	}
 }
