@@ -29,6 +29,7 @@ static const char sccsid[] = "@(#)grav.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_grav
 #define PROGCLASS "Grav"
 #define HACK_INIT init_grav
 #define HACK_DRAW draw_grav
@@ -60,8 +61,8 @@ static XrmOptionDescRec opts[] =
 };
 static argtype vars[] =
 {
-	{(caddr_t *) & decay, (char *) "decay", (char *) "Decay", (char *) DEF_DECAY, t_Bool},
-	{(caddr_t *) & trail, (char *) "trail", (char *) "Trail", (char *) DEF_TRAIL, t_Bool}
+	{(void *) & decay, (char *) "decay", (char *) "Decay", (char *) DEF_DECAY, t_Bool},
+	{(void *) & trail, (char *) "trail", (char *) "Trail", (char *) DEF_TRAIL, t_Bool}
 };
 static OptionStruct desc[] =
 {
@@ -255,7 +256,7 @@ init_grav(ModeInfo * mi)
 	gp->nplanets = MI_COUNT(mi);
 	if (gp->nplanets < 0) {
 		if (gp->planets) {
-			(void) free((void *) gp->planets);
+			free(gp->planets);
 			gp->planets = (planetstruct *) NULL;
 		}
 		gp->nplanets = NRAND(-gp->nplanets) + 1;	/* Add 1 so its not too boring */
@@ -334,9 +335,9 @@ release_grav(ModeInfo * mi)
 			gravstruct *gp = &gravs[screen];
 
 			if (gp->planets)
-				(void) free((void *) gp->planets);
+				free(gp->planets);
 		}
-		(void) free((void *) gravs);
+		free(gravs);
 		gravs = (gravstruct *) NULL;
 	}
 }
