@@ -24,7 +24,7 @@ static const char sccsid[] = "@(#)ico.c	5.00 2000/11/01 xlockmore";
  * 01-Nov-2000: Allocation checks
  * 27-Mar-2000: Added double buffering for drawings
  * 10-May-1997: Compatible with xscreensaver
- * 25-Mar-1997: David Bagley <bagleyd@tux.org>
+ * 25-Mar-1997: David Bagley <bagleyd AT verizon.net>
  *              Took ico from the X11R6 distribution.  Stripped out
  *              anything complicated... to be added back in later.
  *              added dodecahedron, tetrahedron, and star octahedron.
@@ -90,6 +90,7 @@ SOFTWARE.
  */
 
 #ifdef STANDALONE
+#define MODE_ico
 #define PROGCLASS "Ico"
 #define HACK_INIT init_ico
 #define HACK_DRAW draw_ico
@@ -127,9 +128,9 @@ static XrmOptionDescRec opts[] =
 };
 static argtype vars[] =
 {
-	{(caddr_t *) & faces, (char *) "faces", (char *) "Faces", (char *) DEF_FACES, t_Bool},
-	{(caddr_t *) & edges, (char *) "edges", (char *) "Edges", (char *) DEF_EDGES, t_Bool},
-	{(caddr_t *) & opaque, (char *) "opaque", (char *) "Opaque", (char *) DEF_OPAQUE, t_Bool}
+	{(void *) & faces, (char *) "faces", (char *) "Faces", (char *) DEF_FACES, t_Bool},
+	{(void *) & edges, (char *) "edges", (char *) "Edges", (char *) DEF_EDGES, t_Bool},
+	{(void *) & opaque, (char *) "opaque", (char *) "Opaque", (char *) DEF_OPAQUE, t_Bool}
 };
 static OptionStruct desc[] =
 {
@@ -187,12 +188,16 @@ typedef struct {
 
 /*-
  *                              faces/edges/vert Vertex Config   Wythoff Symbol
+ * Platonic Solids:
  * tetrahedron                  4/6/4            {3,3,3}         3|2 3
  * cube                         6/12/8           {4,4,4}         3|2 4
  * octahedron                   8/12/6           {3,3,3,3}       4|2 3
- * dodecahedron                 12/30/12         {5,5,5}         3|2 5
+ * dodecahedron                 12/30/20         {5,5,5}         3|2 5
  * icosahedron                  20/30/12         {3,3,3,3,3}     5|2 3
- *Nice additions would be the Kepler-Poinsot Solids:
+ * Edge Transitive Catalan Solids:
+ * rhombic dodecahedron         12/24/14
+ * rhombic triacontahedron      30/60/32
+ * Kepler-Poinsot Solids (not added yet):
  * small stellated dodecahedron 12/30/12      {5/2,5/2,5/2,5/2,5/2}   5|2 5/2
  * great stellated dodecahedron 12/30/20         {5/2,5/2,5/2}   3|2 5/2
  * great dodecahedron           12/30/12         {5,5,5,5,5}/2   5/2|2 5
@@ -219,7 +224,7 @@ static Polyinfo polygons[] =
 #undef T
 		},
 		{		/* faces (numfaces + indexes into vertices) */
-		/*  faces must be specified clockwise from the outside */
+		/* faces must be specified clockwise from the outside */
 			3, 2, 1, 0,
 			3, 1, 3, 0,
 			3, 3, 2, 0,
@@ -249,7 +254,7 @@ static Polyinfo polygons[] =
 #undef T
 		},
 		{		/* faces (numfaces + indexes into vertices) */
-		/*  faces must be specified clockwise from the outside */
+		/* faces must be specified clockwise from the outside */
 			4, 0, 1, 2, 3,
 			4, 7, 6, 5, 4,
 			4, 1, 0, 4, 5,
@@ -279,7 +284,7 @@ static Polyinfo polygons[] =
 #undef T
 		},
 		{		/* faces (numfaces + indexes into vertices) */
-		/*  faces must be specified clockwise from the outside */
+		/* faces must be specified clockwise from the outside */
 			3, 0, 4, 2,
 			3, 0, 2, 5,
 			3, 0, 5, 3,
@@ -291,7 +296,7 @@ static Polyinfo polygons[] =
 		}
 	},
 
-/* objdodec - structure values for dodecahedron */
+/* objdodeca - structure values for dodecahedron */
 
 	{
 #ifdef DEFUNCT
@@ -301,29 +306,35 @@ static Polyinfo polygons[] =
 		20, 30, 12,	/* number of vertices, edges, and faces */
 		{		/* vertices (x,y,z) */
 			/* all points must be within radius 1 of the origin */
-			{0.000000, 0.309017, 0.809015},
-			{0.000000, -0.309017, 0.809015},
-			{0.000000, -0.309017, -0.809015},
-			{0.000000, 0.309017, -0.809015},
-			{0.809015, 0.000000, 0.309017},
-			{-0.809015, 0.000000, 0.309017},
-			{-0.809015, 0.000000, -0.309017},
-			{0.809015, 0.000000, -0.309017},
-			{0.309017, 0.809015, 0.000000},
-			{-0.309017, 0.809015, 0.000000},
-			{-0.309017, -0.809015, 0.000000},
-			{0.309017, -0.809015, 0.000000},
-			{0.500000, 0.500000, 0.500000},
-			{-0.500000, 0.500000, 0.500000},
-			{-0.500000, -0.500000, 0.500000},
-			{0.500000, -0.500000, 0.500000},
-			{0.500000, -0.500000, -0.500000},
-			{0.500000, 0.500000, -0.500000},
-			{-0.500000, 0.500000, -0.500000},
-			{-0.500000, -0.500000, -0.500000},
+#define G 0.3090169943749474241 /* cos 72 */
+#define D 0.8090169943749474241 /* cos 36 */
+#define H 0.5
+			{0, G, D},
+			{0, -G, D},
+			{0, -G, -D},
+			{0, G, -D},
+			{D, 0, G},
+			{-D, 0, G},
+			{-D, 0, -G},
+			{D, 0, -G},
+			{G, D, 0},
+			{-G, D, 0},
+			{-G, -D, 0},
+			{G, -D, 0},
+			{H, H, H},
+			{-H, H, H},
+			{-H, -H, H},
+			{H, -H, H},
+			{H, -H, -H},
+			{H, H, -H},
+			{-H, H, -H},
+			{-H, -H, -H},
+#undef G
+#undef D
+#undef H
 		},
 		{		/* faces (numfaces + indexes into vertices) */
-		/*  faces must be specified clockwise from the outside */
+		/* faces must be specified clockwise from the outside */
 			5, 12, 8, 17, 7, 4,
 			5, 5, 6, 18, 9, 13,
 			5, 14, 10, 19, 6, 5,
@@ -349,21 +360,35 @@ static Polyinfo polygons[] =
 		12, 30, 20,	/* number of vertices, edges, and faces */
 		{		/* vertices (x,y,z) */
 			/* all points must be within radius 1 of the origin */
-			{0.00000000, 0.00000000, -0.95105650},
-			{0.00000000, 0.85065080, -0.42532537},
-			{0.80901698, 0.26286556, -0.42532537},
-			{0.50000000, -0.68819095, -0.42532537},
-			{-0.50000000, -0.68819095, -0.42532537},
-			{-0.80901698, 0.26286556, -0.42532537},
-			{0.50000000, 0.68819095, 0.42532537},
-			{0.80901698, -0.26286556, 0.42532537},
-			{0.00000000, -0.85065080, 0.42532537},
-			{-0.80901698, -0.26286556, 0.42532537},
-			{-0.50000000, 0.68819095, 0.42532537},
-			{0.00000000, 0.00000000, 0.95105650}
+#define A 0.26286556
+#define B 0.42532537
+#define C 0.68819095
+#define D 0.8090169943749474241 /* cos 36 */
+#define E 0.8506508083520399322 /* cossec 36 / 2 */
+#define F 0.9510565162951535721 /* sin 72 */
+#define H 0.5
+			{0, 0, -F},
+			{0, E, -B},
+			{D, A, -B},
+			{H, -C, -B},
+			{-H, -C, -B},
+			{-D, A, -B},
+			{H, C, B},
+			{D, -A, B},
+			{0, -E, B},
+			{-D, -A, B},
+			{-H, C, B},
+			{0, 0, F}
+#undef A
+#undef B
+#undef C
+#undef D
+#undef E
+#undef F
+#undef H
 		},
 		{		/* faces (numfaces + indexes into vertices) */
-		/*  faces must be specified clockwise from the outside */
+		/* faces must be specified clockwise from the outside */
 			3, 0, 2, 1,
 			3, 0, 3, 2,
 			3, 0, 4, 3,
@@ -387,6 +412,218 @@ static Polyinfo polygons[] =
 		}
 	},
 
+/* objrhombicdoeca - structure values for rhombic dodecahedron */
+
+	{
+#ifdef DEFUNCT
+		"rhombic_dodecahedron", "rhombic",	/* long and short names */
+		"cubooctahedron",	/* long name of dual */
+#endif
+		14, 24, 12,	/* number of vertices, edges, and faces */
+		{		/* vertices (x,y,z) */
+			/* all points must be within radius 1 of the origin */
+#define T 1.0
+#define H 0.5
+			{T, 0, 0},
+			{0, T, 0},
+			{0, 0, T},
+			{H, H, H},
+			{H, H, -H},
+			{H, -H, H},
+			{H, -H, -H},
+			{-H, H, H},
+			{-H, H, -H},
+			{-H, -H, H},
+			{-H, -H, -H},
+			{-T, 0, 0},
+			{0, -T, 0},
+			{0, 0, -T},
+#undef H
+#undef T
+		},
+		{		/* faces (numfaces + indexes into vertices) */
+		/* faces must be specified clockwise from the outside */
+			4, 0, 3, 2, 5,
+			4, 2, 9, 12, 5,
+			4, 2, 7, 11, 9,
+			4, 1, 7, 2, 3,
+			4, 0, 4, 1, 3,
+			4, 0, 5, 12, 6,
+			4, 9, 11, 10, 12,
+			4, 1, 8, 11, 7,
+			4, 0, 6, 13, 4,
+			4, 6, 12, 10, 13,
+			4, 8, 13, 10, 11,
+			4, 1, 4, 13, 8,
+		}
+	},
+
+/* objtriacont - structure values for rhombic triacontahedron */
+
+	{
+#ifdef DEFUNCT
+		"rhombic_triacontahedron", "triaconta",	/* long and short names */
+		"icosidodecahedron",	/* long name of dual */
+#endif
+		32, 60, 30,	/* number of vertices, edges, and faces */
+		{		/* vertices (x,y,z) */
+			/* all points must be within radius 1 of the origin */
+#define G 0.3090169943749474241 /* cos 72 */
+#define D 0.8090169943749474241 /* cos 36 */
+#define H 0.5
+			{0, G, D},
+			{0, -G, D},
+			{0, -G, -D},
+			{0, G, -D},
+			{D, 0, G},
+			{-D, 0, G},
+			{-D, 0, -G},
+			{D, 0, -G},
+			{G, D, 0},
+			{-G, D, 0},
+			{-G, -D, 0},
+			{G, -D, 0},
+			{H, H, H},
+			{-H, H, H},
+			{-H, -H, H},
+			{H, -H, H},
+			{H, -H, -H},
+			{H, H, -H},
+			{-H, H, -H},
+			{-H, -H, -H},
+			{0, D, H},
+			{0, -D, H},
+			{0, D, -H},
+			{0, -D, -H},
+			{H, 0, D},
+			{-H, 0, D},
+			{H, 0, -D},
+			{-H, 0, -D},
+			{D, H, 0},
+			{-D, H, 0},
+			{D, -H, 0},
+			{-D, -H, 0},
+#undef D
+#undef H
+#undef G
+		},
+		{		/* faces (numfaces + indexes into vertices) */
+		/* faces must be specified clockwise from the outside */
+			4, 8, 20, 12, 28,
+			4, 0, 24, 12, 20,
+			4, 4, 28, 12, 24,
+			4, 8, 22, 9, 20,
+			4, 8, 28, 17, 22,
+			4, 9, 29, 13, 20,
+			4, 0, 20, 13, 25,
+			4, 0, 25, 1, 24,
+			4, 5, 25, 13, 29,
+			4, 9, 22, 18, 29,
+			4, 3, 27, 18, 22,
+			4, 3, 22, 17, 26,
+			4, 6, 29, 18, 27,
+			4, 7, 26, 17, 28,
+			4, 2, 27, 3, 26,
+			4, 4, 30, 7, 28,
+			4, 4, 24, 15, 30,
+			4, 7, 30, 16, 26,
+			4, 1, 21, 15, 24,
+			4, 1, 25, 14, 21,
+			4, 11, 30, 15, 21,
+			4, 11, 23, 16, 30,
+			4, 2, 26, 16, 23,
+			4, 10, 23, 11, 21,
+			4, 2, 23, 19, 27,
+			4, 10, 31, 19, 23,
+			4, 10, 21, 14, 31,
+			4, 5, 31, 14, 25,
+			4, 5, 29, 6, 31,
+			4, 6, 27, 19, 31,
+		}
+	},
+
+/* ico does not draw non-convex polyhedra well. */
+/* objstar - structure values for octahedron star (stellated octahedron?) */
+	{
+#ifdef DEFUNCT
+		"star", "star",	/* long and short names */
+		"star",		/* long name of dual */
+#endif
+		32, 96, 24,	/* number of vertices, edges, and faces */
+		/*8, 12, 8,*/	/* number of real vertices, edges, and faces */
+		{		/* vertices (x,y,z) */
+			/* all points must be within radius 1 of the origin */
+#define T 0.577
+			{T, T, T},
+			{T, 0, 0},
+			{0, T, 0},
+			{0, 0, T},
+			{T, T, -T},
+			{T, 0, 0},
+			{0, T, 0},
+			{0, 0, -T},
+			{T, -T, T},
+			{T, 0, 0},
+			{0, -T, 0},
+			{0, 0, T},
+			{T, -T, -T},
+			{T, 0, 0},
+			{0, -T, 0},
+			{0, 0, -T},
+			{-T, T, T},
+			{-T, 0, 0},
+			{0, T, 0},
+			{0, 0, T},
+			{-T, T, -T},
+			{-T, 0, 0},
+			{0, T, 0},
+			{0, 0, -T},
+			{-T, -T, T},
+			{-T, 0, 0},
+			{0, -T, 0},
+			{0, 0, T},
+			{-T, -T, -T},
+			{-T, 0, 0},
+			{0, -T, 0},
+			{0, 0, -T},
+#undef T
+		},
+		{		/* faces (numfaces + indexes into vertices) */
+		/* faces must be specified clockwise from the outside */
+			3, 2, 1, 0,
+			3, 1, 3, 0,
+			3, 3, 2, 0,
+			3, 4, 5, 6,
+			3, 4, 7, 5,
+			3, 4, 6, 7,
+			3, 8, 9, 10,
+			3, 8, 11, 9,
+			3, 8, 10, 11,
+			3, 14, 13, 12,
+			3, 13, 15, 12,
+			3, 15, 14, 12,
+			3, 16, 17, 18,
+			3, 16, 19, 17,
+			3, 16, 18, 19,
+			3, 22, 21, 20,
+			3, 21, 23, 20,
+			3, 23, 22, 20,
+			3, 26, 25, 24,
+			3, 25, 27, 24,
+			3, 27, 26, 24,
+			3, 28, 29, 30,
+			3, 28, 31, 29,
+			3, 28, 30, 31,
+			3, 2, 3, 1,
+			3, 5, 7, 6,
+			3, 9, 11, 10,
+			3, 14, 15, 13,
+			3, 17, 19, 18,
+			3, 22, 23, 21,
+			3, 26, 27, 25,
+			3, 29, 31, 30,
+		}
+	},
 /* objplane - structure values for plane */
 
 	{
@@ -405,7 +642,7 @@ static Polyinfo polygons[] =
 #undef T
 		},
 		{		/* faces (numfaces + indexes into vertices) */
-		/*  faces must be specified clockwise from the outside */
+		/* faces must be specified clockwise from the outside */
 			4, 0, 2, 1, 3,
 		}
 	},
@@ -426,11 +663,11 @@ static Polyinfo polygons[] =
 			{0, T, 0},
 			{0, -T, 0},
 			{0, 0, T},
-		/* {  0,  0, -T }, */
+		/* {0, 0, -T }, */
 #undef T
 		},
-		{		/* faces (numfaces + indexes into vertices) */
-			/*  faces must be specified clockwise from the outside */
+		{	/* faces (numfaces + indexes into vertices) */
+		/* faces must be specified clockwise from the outside */
 			3, 0, 4, 2,
 			/* 3, 0, 2, 5, */
 			/* 3, 0, 5, 3, */
@@ -465,7 +702,7 @@ static Polyinfo polygons[] =
 #undef T
 		},
 		{		/* faces (numfaces + indexes into vertices) */
-		/*  faces must be specified clockwise from the outside */
+		/* faces must be specified clockwise from the outside */
 			3, 2, 1, 0,
 			3, 1, 3, 0,
 			3, 3, 2, 0,
@@ -481,7 +718,8 @@ static Polyinfo polygons[] =
 };
 
 static int  polysize = sizeof (polygons) / sizeof (polygons[0]);
-#define POLYSIZE 5  /* Only the 5 Platonic solids work, why is that? */
+#define POLYSIZE 7	/* Only the 5 Platonic solids and the 2 edge */
+			/* transitive Catalan solids work, why is that? */
 
 #define POLYBITS(n,w,h)\
   if ((ip->pixmaps[ip->init_bits]=\
@@ -773,10 +1011,10 @@ drawPoly(ModeInfo * mi, Polyinfo * poly, GC gc,
 
 	if (ip->dbuf) {
 		XSetForeground(display, ip->dbuf_gc, MI_BLACK_PIXEL(mi));
-		XFillRectangle(display, ip->dbuf, ip->dbuf_gc, 0, 0,
+		XFillRectangle(display, (Drawable) ip->dbuf, ip->dbuf_gc, 0, 0,
 			polyW + ip->deltaWidth + ip->linewidth,
 			polyH + ip->deltaHeight + ip->linewidth);
-		lwindow = ip->dbuf;
+		lwindow = (Drawable) ip->dbuf;
 		lgc = ip->dbuf_gc;
 	} else {
 		lwindow = window;
@@ -896,13 +1134,13 @@ drawPoly(ModeInfo * mi, Polyinfo * poly, GC gc,
 #endif
 		{
 			for (j = 0; j < pp-pts; j++)
-			XFillArc(display, lwindow, lgc,
-                         pts[j].x - ip->linewidth / 2, pts[j].y - ip->linewidth / 2, ip->linewidth, ip->linewidth, 0, 23040);
+			  XFillArc(display, lwindow, lgc,
+                             pts[j].x - ip->linewidth / 2, pts[j].y - ip->linewidth / 2, ip->linewidth, ip->linewidth, 0, 23040);
 
 		}
 	}
 	if (ip->dbuf) {
-		XCopyArea(display, ip->dbuf, window, gc, 0, 0,
+		XCopyArea(display, (Drawable) ip->dbuf, window, gc, 0, 0,
 			polyW + 1 + ip->deltaWidth + ip->linewidth,
 			polyH + 1 + ip->deltaHeight + ip->linewidth,
 			currX - ip->deltaWidth / 2 - ip->linewidth / 2,
@@ -961,7 +1199,7 @@ init_ico(ModeInfo * mi)
 
 			gcv.fill_style = FillOpaqueStippled;
 			if ((ip->stippledGC = XCreateGC(display, window, GCFillStyle,
-					 &gcv)) == NULL) {
+					 &gcv)) == None) {
 				free_ico(display, ip);
 				return;
 			}
@@ -1044,17 +1282,17 @@ init_ico(ModeInfo * mi)
 
 		if (ip->dbuf_gc != None)
 			XFreeGC(display, ip->dbuf_gc);
-		if ((ip->dbuf_gc = XCreateGC(display, ip->dbuf,
+		if ((ip->dbuf_gc = XCreateGC(display, (Drawable) ip->dbuf,
 				GCForeground | GCBackground | GCGraphicsExposures | GCFunction,
 				&gcv)) == None) {
 			XFreePixmap(display, ip->dbuf);
 			ip->dbuf = None;
 		} else {
-			XFillRectangle(display, ip->dbuf, ip->dbuf_gc,
+			XFillRectangle(display, (Drawable) ip->dbuf, ip->dbuf_gc,
 				0, 0, ip->polyW + ip->deltaWidth + ip->linewidth,
 				ip->polyH + ip->deltaHeight + ip->linewidth);
-			XSetBackground(display, MI_GC(mi), MI_BLACK_PIXEL(mi));
-			XSetFunction(display, MI_GC(mi), GXcopy);
+			/*XSetBackground(display, MI_GC(mi), MI_BLACK_PIXEL(mi));
+			XSetFunction(display, MI_GC(mi), GXcopy);*/
 		}
 	}
 #endif
@@ -1118,7 +1356,7 @@ release_ico(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_ico(MI_DISPLAY(mi), &icos[screen]);
-		(void) free((void *) icos);
+		free(icos);
 		icos = (icostruct *) NULL;
 	}
 }
