@@ -26,8 +26,8 @@ static const char sccsid[] = "@(#)julia.c	5.00 2000/11/01 xlockmore";
  *
  * Revision History:
  * 01-Nov-2000: Allocation checks
- * 28-May-1997: jwz@jwz.org: added interactive frobbing with the mouse.
- * 10-May-1997: jwz@jwz.org: turned into a standalone program.
+ * 28-May-1997: jwz AT jwz.org: added interactive frobbing with the mouse.
+ * 10-May-1997: jwz AT jwz.org: turned into a standalone program.
  * 02-Dec-1995: snagged boilerplate from hop.c
  *              used ifs {w0 = sqrt(x-c), w1 = -sqrt(x-c)} with random
  *              iteration to plot the julia set, and sinusoidially varied
@@ -75,7 +75,7 @@ static XrmOptionDescRec opts[] =
 
 static argtype vars[] =
 {
-        {(caddr_t *) & trackmouse, (char *) "trackmouse", (char *) "TrackMouse", (char *) DEF_TRACKMOUSE, t_Bool}
+        {(void *) & trackmouse, (char *) "trackmouse", (char *) "TrackMouse", (char *) DEF_TRACKMOUSE, t_Bool}
 };
 
 static OptionStruct desc[] =
@@ -249,9 +249,10 @@ init_julia(ModeInfo * mi)
 			free_julia(display, jp);
 			return;
 		}
-			
-		if ((jp->cursor = XCreatePixmapCursor(display, bit, bit,
-				 &black, &black, 0, 0)) == None) {
+
+		if ((jp->cursor = XCreatePixmapCursor(display,
+				bit, bit,
+				&black, &black, 0, 0)) == None) {
 			XFreePixmap(display, bit);
 			free_julia(display, jp);
 			return;
@@ -274,24 +275,25 @@ init_julia(ModeInfo * mi)
 			return;
 		}
 		gcv.foreground = 1;
-		if ((fg_gc = XCreateGC(display, jp->pixmap, GCForeground,
-				&gcv)) == None) {
+		if ((fg_gc = XCreateGC(display, (Drawable) jp->pixmap,
+				GCForeground, &gcv)) == None) {
 			free_julia(display, jp);
 			return;
 		}
 		gcv.foreground = 0;
-		if ((bg_gc = XCreateGC(display, jp->pixmap, GCForeground,
-				&gcv)) == None) {
+		if ((bg_gc = XCreateGC(display, (Drawable) jp->pixmap,
+				GCForeground, &gcv)) == None) {
 			XFreeGC(display, fg_gc);
 			free_julia(display, jp);
 			return;
 		}
-		XFillRectangle(display, jp->pixmap, bg_gc,
+		XFillRectangle(display, (Drawable) jp->pixmap, bg_gc,
 			       0, 0, jp->circsize, jp->circsize);
 		if (jp->circsize < 2)
-			XDrawPoint(display, jp->pixmap, fg_gc, 0, 0);
+			XDrawPoint(display, (Drawable) jp->pixmap, fg_gc,
+				0, 0);
 		else
-			XFillArc(display, jp->pixmap, fg_gc,
+			XFillArc(display, (Drawable) jp->pixmap, fg_gc,
 				 0, 0, jp->circsize, jp->circsize, 0, 23040);
 		XFreeGC(display, fg_gc);
 		XFreeGC(display, bg_gc);
