@@ -35,7 +35,7 @@ static const char sccsid[] = "@(#)kumppa.c	5.00 2000/11/01 xlockmore";
 
    * Revision History:
    * 01-Nov-2000: Allocation checks
-   * 16-Jul-1998: xlockmore version by Jouk Jansen <joukj@hrem.stm.tudelft.nl>
+   * 16-Jul-1998: xlockmore version by Jouk Jansen <joukj AT hrem.nano.tudelft.nl>
    * Feb 1998 : original xscreensaver version by Teemu Suutari <temisu@utu.fi>
  */
 
@@ -99,13 +99,13 @@ static XrmOptionDescRec opts[] =
 
 static argtype vars[] =
 {
-	{(caddr_t *) & speed, (char *) "speed", (char *) "speed", (char *) DEF_SPEED, t_Float},
+	{(void *) & speed, (char *) "speed", (char *) "speed", (char *) DEF_SPEED, t_Float},
 #if 0
 #ifdef HAVE_XDBE_EXTENSION
-	{(caddr_t *) & usedouble, (char *) "dbuf", (char *) "dbuf", (char *) DEF_USEDOUBLE, t_Bool},
+	{(void *) & usedouble, (char *) "dbuf", (char *) "dbuf", (char *) DEF_USEDOUBLE, t_Bool},
 #endif				/* HAVE_XDBE_EXTENSION */
 #endif
-	{(caddr_t *) & cosilines, (char *) "rrandom", (char *) "rrandom", (char *) DEF_COSILINES, t_Bool}
+	{(void *) & cosilines, (char *) "rrandom", (char *) "rrandom", (char *) DEF_COSILINES, t_Bool}
 };
 static OptionStruct desc[] =
 {
@@ -384,7 +384,7 @@ make_rots(ModeInfo * mi, double xspeed, double yspeed)
 	    ((chks = (Bool *) malloc(((s->midx > s->midy) ? s->midx : s->midy) *
 			sizeof (Bool))) == NULL)) {
 		free_kumppa(MI_DISPLAY(mi), s);
-		return; 
+		return;
 	}
 
 
@@ -450,7 +450,7 @@ make_rots(ModeInfo * mi, double xspeed, double yspeed)
 			sizeof (int))) == NULL) {
 		free_kumppa(MI_DISPLAY(mi), s);
 		free(chks);
-		return; 
+		return;
 	}
 
 	maxi = 0;
@@ -534,8 +534,8 @@ init_kumppa(ModeInfo * mi)
 	Window      window = MI_WINDOW(mi);
 	XGCValues   xgcv;
 	int         n, i;
-	double      rspeed;
 	kumppastruct *s;
+	double      rspeed;
 
 	if (kumppas == NULL) {
 		if ((kumppas = (kumppastruct *) calloc(MI_NUM_SCREENS(mi),
@@ -671,12 +671,13 @@ init_kumppa(ModeInfo * mi)
 #endif /* HAVE_XDBE_EXTENSION */
 #endif
 
-	rspeed = (double) speed;
-	if (rspeed < 0.0001 || rspeed > 0.2) {
+	if (speed < (float) 0.0001 || speed > (float) 0.2) {
 		(void) fprintf(stderr,
-		   "Speed not in valid range! (0.0001 - 0.2), using 0.1 \n");
-		rspeed = 0.1;
+			"Speed %g not in valid range! (0.0001 - 0.2), using 0.1\n",
+			speed);
+		speed = 0.1;
 	}
+	rspeed = (double) speed;
 	s->sizx = MI_WIDTH(mi);
 	s->sizy = MI_HEIGHT(mi);
 	s->midx = s->sizx >> 1;
@@ -686,7 +687,6 @@ init_kumppa(ModeInfo * mi)
 	s->c = 0;
 	s->dir = (LRAND() & 1) ? -1 : 1;
 	MI_CLEARWINDOW(mi);
-
 	make_rots(mi, rspeed, rspeed);
 }
 
