@@ -7,7 +7,7 @@ static const char sccsid[] = "@(#)loop.c	5.24 2007/01/18 xlockmore";
 #endif
 
 /*-
- * Copyright (c) 1996 - 2004 by David Bagley.
+ * Copyright (c) 1996 - 2011 by David Bagley.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted,
@@ -22,6 +22,7 @@ static const char sccsid[] = "@(#)loop.c	5.24 2007/01/18 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
+ * 02-Mar-2011: Byl and Chou, Reggia Loops
  * 18-Jan-2007: Added vertical option.
  * 10-Apr-2004: EvoLoopS - Evolving SDSR Loop Simulator idea stolen
  *              from Hiroki Sayama <sayama AT is.s.u-tokyo.ac.jp>.
@@ -116,6 +117,9 @@ ping.
  */
 #define DEF_NEIGHBORS  "0"      /* choose random value */
 #define DEF_LANGTON  "False"    /* neighbors 4 or 6 */
+#define DEF_BYL  "False"        /* neighbors 4 only */
+#define DEF_CHOU1  "False"      /* neighbors 4 only */
+#define DEF_CHOU2  "False"      /* neighbors 4 only */
 #define DEF_EVOLVE  "False"     /* neighbors 4 only */
 #define DEF_DISSOLVE  "False"   /* neighbors 4 only and no evolve */
 #define DEF_SHEATH  "False"     /* neighbors 4 only and no evolve */
@@ -124,6 +128,9 @@ ping.
 
 static int  neighbors;
 static Bool langton;
+static Bool byl;
+static Bool chou1;
+static Bool chou2;
 static Bool evolve;
 static Bool dissolve;
 static Bool sheath;
@@ -135,6 +142,12 @@ static XrmOptionDescRec opts[] =
 	{(char *) "-neighbors", (char *) ".loop.neighbors", XrmoptionSepArg, (caddr_t) NULL},
 	{(char *) "-langton", (char *) ".loop.langton", XrmoptionNoArg, (caddr_t) "on"},
 	{(char *) "+langton", (char *) ".loop.langton", XrmoptionNoArg, (caddr_t) "off"},
+	{(char *) "-byl", (char *) ".loop.byl", XrmoptionNoArg, (caddr_t) "on"},
+	{(char *) "+byl", (char *) ".loop.byl", XrmoptionNoArg, (caddr_t) "off"},
+	{(char *) "-chou1", (char *) ".loop.chou1", XrmoptionNoArg, (caddr_t) "on"},
+	{(char *) "+chou1", (char *) ".loop.chou1", XrmoptionNoArg, (caddr_t) "off"},
+	{(char *) "-chou2", (char *) ".loop.chou2", XrmoptionNoArg, (caddr_t) "on"},
+	{(char *) "+chou2", (char *) ".loop.chou2", XrmoptionNoArg, (caddr_t) "off"},
 	{(char *) "-evolve", (char *) ".loop.evolve", XrmoptionNoArg, (caddr_t) "on"},
 	{(char *) "+evolve", (char *) ".loop.evolve", XrmoptionNoArg, (caddr_t) "off"},
 	{(char *) "-dissolve", (char *) ".loop.dissolve", XrmoptionNoArg, (caddr_t) "on"},
@@ -151,6 +164,9 @@ static argtype vars[] =
 {
 	{(void *) & neighbors, (char *) "neighbors", (char *) "Neighbors", (char *) DEF_NEIGHBORS, t_Int},
 	{(void *) & langton, (char *) "langton", (char *) "Langton", (char *) DEF_LANGTON, t_Bool},
+	{(void *) & byl, (char *) "byl", (char *) "Byl", (char *) DEF_BYL, t_Bool},
+	{(void *) & chou1, (char *) "chou1", (char *) "Chou1", (char *) DEF_CHOU1, t_Bool},
+	{(void *) & chou2, (char *) "chou2", (char *) "Chou2", (char *) DEF_CHOU2, t_Bool},
 	{(void *) & evolve, (char *) "evolve", (char *) "Evolve", (char *) DEF_EVOLVE, t_Bool},
 	{(void *) & dissolve, (char *) "dissolve", (char *) "Dissolve", (char *) DEF_DISSOLVE, t_Bool},
 	{(void *) & sheath, (char *) "sheath", (char *) "Extension", (char *) DEF_SHEATH, t_Bool},
@@ -162,6 +178,9 @@ static OptionStruct desc[] =
 {
 	{(char *) "-neighbors num", (char *) "squares 4 or hexagons 6"},
 	{(char *) "-/+langton", (char *) "turn on/off Langton's Loops"},
+	{(char *) "-/+byl", (char *) "turn on/off Byl's Loops"},
+	{(char *) "-/+chou1", (char *) "turn on/off Chou-Reggia 1 Loops"},
+	{(char *) "-/+chou2", (char *) "turn on/off Chou-Reggia 2 Loops"},
 	{(char *) "-/+evolve", (char *) "turn on/off Evolving Loops"},
 	{(char *) "-/+dissolve", (char *) "turn on/off structurally dissolvable loops"},
 	{(char *) "-/+sheath", (char *) "turn on/off sheath extension"},
@@ -206,10 +225,19 @@ static int  local_neighbors = 0;
 #define MINLOOPS 1
 #define REDRAWSTEP 2000		/* How many cells to draw per cycle */
 #define ADAM_SIZE 8 /* MIN 5 */
+#define BYL_ADAM_SIZE 4
+#define CHOU1_ADAM_SIZE 4
+#define CHOU2_ADAM_SIZE 3
 #define EVO_ADAM_SIZE 15
 #if 1
 #define ADAM_LOOPX  (ADAM_SIZE+2)
 #define ADAM_LOOPY  (ADAM_SIZE+2)
+#define BYL_ADAM_LOOPX  (BYL_ADAM_SIZE+2)
+#define BYL_ADAM_LOOPY  (BYL_ADAM_SIZE+2)
+#define CHOU1_ADAM_LOOPX  (CHOU1_ADAM_SIZE+2)
+#define CHOU1_ADAM_LOOPY  (CHOU1_ADAM_SIZE+2)
+#define CHOU2_ADAM_LOOPX  (CHOU2_ADAM_SIZE+2)
+#define CHOU2_ADAM_LOOPY  (CHOU2_ADAM_SIZE+2)
 #define EVO_ADAM_LOOPX  (EVO_ADAM_SIZE+2)
 #define EVO_ADAM_LOOPY  (EVO_ADAM_SIZE+2)
 #define SQ_ADAM_LOOPX(evolve) ((evolve)?EVO_ADAM_LOOPX:ADAM_LOOPX)
@@ -275,9 +303,6 @@ typedef struct {
 
 static loopstruct *loops = (loopstruct *) NULL;
 
-static const int M8[]={1, 8, 64, 512, 4096, 32768, 262144, 2097152, 16777216};
-static const int M9[]={1, 9, 81, 729, 6561, 59049, 531441, 4782969, 43046721,
-	387420489};
 /*#define INEF2 1*/
 #ifndef INEF2
 #define TRANSITION(TT,V) V=TT&7;TT>>=3
@@ -285,15 +310,20 @@ static const int M9[]={1, 9, 81, 729, 6561, 59049, 531441, 4782969, 43046721,
 #define TABLE(R,T,L,B) (table[((B)<<9)|((L)<<6)|((T)<<3)|(R)])
 #define TABLE_OUT(C,R,T,L,B) ((TABLE(R,T,L,B)>>((C)*3))&7)
 #else
+static const int M8[]={1, 8, 64, 512, 4096, 32768, 262144, 2097152, 16777216};
 #define TRANSITION(TT,V) V=TT%8;TT/=8
 #define FINALTRANSITION(TT,V) V=TT%8
 #define TABLE(R,T,L,B) (table[((B)*512)+((L)*64)+((T)*8)+(R)])
 #define TABLE_OUT(C,R,T,L,B) ((TABLE(R,T,L,B)/(M8[C]))%8)
 #endif
+#if 0
+static const int M9[]={1, 9, 81, 729, 6561, 59049, 531441, 4782969, 43046721,
+	387420489};
 #define EXT_TRANSITION(TT,V) V=TT%9;TT/=9
 #define EXT_FINALTRANSITION(TT,V) V=TT%9
 #define EXT_TABLE(R,T,L,B) (table[((B)*729)+((L)*81)+((T)*9)+(R)])
 #define EXT_TABLE_OUT(C,R,T,L,B) ((EXT_TABLE(R,T,L,B)/(M9[C]))%9)
+#endif
 #define INEF_TABLE_OUT(C,R,T,L,B) table[(B)*6561+(L)*729+(T)*81+(R)*9+(C)]
 #define HEX_TABLE(R,T,t,l,b,B) (table[((B)<<15)|((b)<<12)|((l)<<9)|((t)<<6)|((T)<<3)|(R)])
 #define HEX_TABLE_OUT(C,R,T,t,l,b,B) ((HEX_TABLE(R,T,t,l,b,B)>>((C)*3))&7)
@@ -309,8 +339,10 @@ static const int M9[]={1, 9, 81, 729, 6561, 59049, 531441, 4782969, 43046721,
 #ifdef RAND_RULES
 #define TABLE_IN(C,R,T,L,B,I) (TABLE(R,T,L,B)&=~(7<<((C)*3)));\
 (TABLE(R,T,L,B)|=((I)<<((C)*3)))
+#if 0
 #define EXT_TABLE_IN(C,R,T,L,B,I) (EXT_TABLE(R,T,L,B)-=(9*M9[C]-1));\
 (EXT_TABLE(R,T,L,B)+=((I)*(M9[C])))
+#endif
 #define HEX_TABLE_IN(C,R,T,t,l,b,B,I) (HEX_TABLE(R,T,t,l,b,B)&=~(7<<((C)*3)));\
 (HEX_TABLE(R,T,t,l,b,B)|=((I)<<((C)*3)))
 #else
@@ -319,7 +351,9 @@ static const int M9[]={1, 9, 81, 729, 6561, 59049, 531441, 4782969, 43046721,
 #else
 #define TABLE_IN(C,R,T,L,B,I) if(TABLE_OUT(C,R,T,L,B)==0) (TABLE(R,T,L,B)+=((I)*(M8[C])))
 #endif
+#if 0
 #define EXT_TABLE_IN(C,R,T,L,B,I) if (EXT_TABLE_OUT(C,R,T,L,B)==0) (EXT_TABLE(R,T,L,B)+=((I)*(M9[C])))
+#endif
 #define HEX_TABLE_IN(C,R,T,t,l,b,B,I) (HEX_TABLE(R,T,t,l,b,B)|=((I)<<((C)*3)))
 #endif
 #define INEF_TABLE_IN(C,R,T,L,B,I) INEF_TABLE_OUT(C,R,T,L,B)=(I)
@@ -335,10 +369,14 @@ static char plots[NEIGHBORKINDS] =
   4, 6 /* Neighborhoods */
 };
 
-/* From Chris Langton */
-static unsigned int transition_table[] =
-{	/* Octal  CBLTR->I */
-      /* CBLTRI   CBLTRI   CBLTRI   CBLTRI   CBLTRI */
+/* C.G.Langton. "Self-reproduction in cellular automata." */
+/* Physica D, Vol. 10, pages 135-144, 1984. */
+/* transition rules from: http://necsi.org/postdocs/sayama/sdsr/java/loops.java */
+/* credits: "Self-Replicating Loops & Ant, Programmed by Eli Bachmutsky, Copyleft Feb.1999" */
+/* states: 8, rules: 219, format: CNESWC' */
+static unsigned int transitionTable[] =
+{	/* Octal  CNESW->I */
+      /* CNESWI   CNESWI   CNESWI   CNESWI   CNESWI */
 	0000000, 0000012, 0000020, 0000030, 0000050,
 	0000063, 0000071, 0000112, 0000122, 0000132,
 	0000212, 0000220, 0000230, 0000262, 0000272,
@@ -385,8 +423,116 @@ static unsigned int transition_table[] =
 	0702251, 0702321, 0702525, 0702720
 };
 
+/* J. Byl. "Self-Reproduction in small cellular automata." */
+/* Physica D, Vol. 34, pages 295-299, 1989. */
+/* transition rules from: http://necsi.org/postdocs/sayama/sdsr/java/loops.java */
+/* credits: "Self-Replicating Loops & Ant, Programmed by Eli Bachmutsky, Copyleft Feb.1999" */
+/* states: 6, rules: 146, format: CNESWC' */
+static unsigned int bylTransitionTable[] =
+{
+	0000000, 0000010, 0000031, 0000420, 0000110,
+	0000120, 0000311, 0000330, 0000320, 0000200,
+	0000242, 0000233, 0004330, 0005120, 0001400,
+	0001120, 0001305, 0001310, 0001320, 0003020,
+	0003400, 0003420, 0003120, 0002040, 0002050,
+	0002010, 0002020, 0002420, 0002500, 0002520,
+	0002200, 0002250, 0002220, 0040000, 0050000,
+	0050500, 0010052, 0010022, 0010400, 0010100,
+	0020055, 0400233, 0405235, 0401233, 0442024,
+	0452020, 0415233, 0411233, 0412024, 0412533,
+	0432024, 0421433, 0422313, 0501302, 0502230,
+	0540022, 0542002, 0512024, 0530025, 0520025,
+	0520442, 0523242, 0522020, 0100000, 0100010,
+	0100033, 0100330, 0101233, 0103401, 0103244,
+	0111244, 0113244, 0112404, 0133244, 0121351,
+	0123444, 0123543, 0122414, 0122434, 0300100,
+	0300300, 0300211, 0300233, 0304233, 0301100,
+	0301211, 0303211, 0303233, 0302233, 0344233,
+	0343233, 0351202, 0353215, 0314233, 0311233,
+	0313251, 0313211, 0312211, 0335223, 0333211,
+	0320533, 0324433, 0325415, 0321433, 0321511,
+	0321321, 0323411, 0200000, 0200042, 0200032,
+	0200022, 0200442, 0200515, 0200112, 0200122,
+	0200342, 0200332, 0200242, 0200212, 0201502,
+	0203202, 0202302, 0244022, 0245022, 0242042,
+	0242022, 0254202, 0255042, 0252025, 0210042,
+	0214022, 0215022, /*0212052,*/ 0212055, 0212022,
+	0230052, 0234022, 0235002, 0235022, 0232042,
+	0232022, 0220042, 0220020, 0220533, 0221552,
+	0322343, 0204122,
+ /* last 2 are not listed but implied by initializing rules to be stable */
+};
+
+/* J. A. Reggia, S.L.Armentrout, H.-H. Chou, and Y. Peng. */
+/* Simple systems that exhibit self-directed replication. */
+/* Science, Vol. 259, pages 1282-1287, February 1993. */
+/* transition rules from: http://necsi.org/postdocs/sayama/sdsr/java/loops.java */
+/* credits: "Self-Replicating Loops & Ant, Programmed by Eli Bachmutsky, Copyleft Feb.1999" */
+/* rules: 174, states: 8, format: CNESWC' */
+static unsigned int chou1TransitionTable[] =
+{
+	0000000, 0000200, 0000110, 0004420, 0006301,
+	0001447, 0003030, 0046000, 0060400, 0062102,
+	0051060, 0010420, 0010360, 0012152, 0032000,
+	0401033, 0410011, 0431206, 0640406, 0660630,
+	0616600, 0636000, 0000440, 0000260, 0000330,
+	0004512, 0005103, 0001540, 0003100, 0045002,
+	0060600, 0061110, 0051710, 0010506, 0010320,
+	0030400, 0076010, 0460313, 0410633, 0600000,
+	0640106, 0610006, 0616100, 0636100, 0000420,
+	0000231, 0000705, 0006030, 0002020, 0001210,
+	0040000, 0045600, 0060100, 0050006, 0020320,
+	0010140, 0016000, 0036000, 0400035, 0466633,
+	0410100, 0606100, 0660060, 0610106, 0613001,
+	0676600, 0000600, 0000100, 0004040, 0006440,
+	0001010, 0001110, 0040160, 0041040, 0060166,
+	0050306, 0024200, 0010154, 0016100, 0036606,
+	0400313, 0423106, 0411033, 0606313, 0660460,
+	0614004, 0630160, 0500000, 0000540, 0000152,
+	0004010, 0006110, 0001030, 0003000, 0044400,
+	0060066, 0062030, 0054040, 0023010, 0010340,
+	0012000, 0036300, 0405033, 0410061, 0431033,
+	0640006, 0660615, 0616000, 0630360, 0500044,
+	0500011, 0230000, 0100111, 0101044, 0146004,
+	0143321, 0166611, 0121121, 0117044, 0300111,
+	0340066, 0366611, 0374017, 0200100, 0100044,
+	0104414, 0101601, 0141403, 0160414, 0161101,
+	0112244, 0130414, 0304011, 0340611, 0314001,
+	0703010, 0203010, 0100011, 0104674, 0101301,
+	0141424, 0160111, 0161301, 0111044, 0133001,
+	0301471, 0340261, 0314676, 0703330, 0240400,
+	0100441, 0106101, 0103011, 0141141, 0164104,
+	0120414, 0113011, 0133011, 0301303, 0340211,
+	0313023, 0733630, 0210102, 0100414, 0105011,
+	0103103, 0141121, 0166644, 0124104, 0113601,
+	0300101, 0307141, 0344011, 0331001,
+};
+
+/* J. A. Reggia, S.L.Armentrout, H.-H. Chou, and Y. Peng. */
+/* Simple systems that exhibit self-directed replication. */
+/* Science, Vol. 259, pages 1282-1287, February 1993. */
+/* transition rules from: http://necsi.org/postdocs/sayama/sdsr/java/loops.java */
+/* credits: "Self-Replicating Loops & Ant, Programmed by Eli Bachmutsky, Copyleft Feb.1999" */
+/* rules: 65, states: 8, format: CNESWC' */
+static unsigned int chou2TransitionTable[] =
+{
+	0000000, 0000440, 0000547, 0000100, 0000110,
+	0000330, 0004040, 0004445, 0004100, 0001040,
+	0001010, 0001740, 0003000, 0003010, 0003030,
+	0007040, 0007030, 0007104, 0007110, 0040000,
+	0040070, 0047100, 0050007, 0017000, 0070000,
+	0070077, 0071010, 0400101, 0400313, 0401033,
+	0407103, 0411033, 0431033, 0500033, 0503330,
+	0100045, 0100011, 0100414, 0101044, 0101301,
+	0103011, 0107131, 0140414, 0141044, 0111044,
+	0133011, 0177711, 0304011, 0305011, 0305141,
+	0307141, 0344011, 0345011, 0354010, 0314001,
+	0377711, 0700000, 0700337, 0707100, 0707141,
+	0707110, 0717000, 0730007, 0770070, 0770711,
+};
+
 /* From Hiroki Sayama */
-static unsigned int evo_transition_table[] =
+static unsigned int evoTransitionTable[] =
 {
 	0000012, 0000043, 0000122, 0000152, 0000212,
 	0000242, 0000422, 0000452, 0000752, 0001022,
@@ -443,9 +589,9 @@ static unsigned int evo_transition_table[] =
 };
 
 /* From David Bagley */
-static unsigned int hex_transition_table[] =
-{				/* Octal CBbltTR->I */
-  /* CBbltTRI   CBbltTRI   CBbltTRI   CBbltTRI   CBbltTRI */
+static unsigned int hexTransitionTable[] =
+{				/* Octal CNESswn->I */
+  /* CNESswnI   CNESswnI   CNESswnI   CNESswnI   CNESswnI */
 
 #ifdef TRIA
     000000000, 000000020, 000000220, 000002220, 000022220,
@@ -798,16 +944,16 @@ static unsigned int hex_transition_table[] =
 
 /*-
 Neighborhoods are read as follows (rotations are not listed):
-    T
-  L C R  ==>  I
-    B
+    N
+  W C E  ==>  I
+    S
 
-   t T
-  l C R  ==>  I
-   b B
+   n N
+  w C E  ==>  I
+   s S
  */
 
-static unsigned char self_reproducing_loop[ADAM_LOOPY][ADAM_LOOPX] =
+static unsigned char selfReproducingLoop[ADAM_LOOPY][ADAM_LOOPX] =
 {
 /* 10x10 */
 	{0, 2, 2, 2, 2, 2, 2, 2, 2, 0},
@@ -821,7 +967,31 @@ static unsigned char self_reproducing_loop[ADAM_LOOPY][ADAM_LOOPX] =
 	{2, 1, 0, 6, 1, 0, 7, 1, 0, 2},
 	{0, 2, 2, 2, 2, 2, 2, 2, 2, 0}
 };
-static unsigned char evo_self_reproducing_loop[EVO_ADAM_LOOPY][EVO_ADAM_LOOPX] =
+
+static unsigned char bylReproducingLoop[BYL_ADAM_LOOPY][BYL_ADAM_LOOPX] =
+{
+	{0, 2, 2, 0},
+	{2, 3, 1, 2},
+	{2, 3, 4, 2},
+	{0, 2, 5, 0}
+};
+
+static unsigned char chou1ReproducingLoop[CHOU1_ADAM_LOOPY][CHOU1_ADAM_LOOPX] =
+{
+	{0, 0, 0, 0},
+	{0, 0, 0, 0},
+	{1, 1, 0, 0},
+	{3, 4, 1, 1}
+};
+
+static unsigned char chou2ReproducingLoop[CHOU2_ADAM_LOOPY][CHOU2_ADAM_LOOPX] =
+{
+	{0, 0, 0},
+	{1, 1, 0},
+	{3, 4, 1}
+};
+
+static unsigned char evoReproducingLoop[EVO_ADAM_LOOPY][EVO_ADAM_LOOPX] =
 {
 /* 17x17 */
 	{0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
@@ -842,7 +1012,7 @@ static unsigned char evo_self_reproducing_loop[EVO_ADAM_LOOPY][EVO_ADAM_LOOPX] =
 	{2, 0, 7, 1, 0, 7, 1, 0, 7, 1, 0, 4, 1, 0, 4, 1, 2},
 	{0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 5, 0}
 };
-static unsigned char hex_self_reproducing_loop[HEX_ADAM_LOOPY][HEX_ADAM_LOOPX] =
+static unsigned char hexReproducingLoop[HEX_ADAM_LOOPY][HEX_ADAM_LOOPX] =
 {
 #if 0
 /* Experimental TRIA5:7x7 */
@@ -921,7 +1091,7 @@ static unsigned char hex_self_reproducing_loop[HEX_ADAM_LOOPY][HEX_ADAM_LOOPX] =
 };
 
 static void
-position_of_neighbor(loopstruct * lp, int dir, int *pcol, int *prow)
+positionOfNeighbor(loopstruct * lp, int dir, int *pcol, int *prow)
 {
 	int         col = *pcol, row = *prow;
 
@@ -1408,29 +1578,47 @@ init_table(ModeInfo *mi)
 		int mult = 1;
 		unsigned int tt, c, n[MAXNEIGHBORS], i;
 		int         j, k;
-		int  size_transition_table = sizeof (transition_table) /
+		int  sizeTransitionTable = sizeof (transitionTable) /
 			sizeof (unsigned int);
-		int  size_evo_transition_table = sizeof (evo_transition_table) /
+		int  sizeBylTransitionTable = sizeof (bylTransitionTable) /
 			sizeof (unsigned int);
-		int  size_hex_transition_table = sizeof (hex_transition_table) /
+		int  sizeChou1TransitionTable = sizeof (chou1TransitionTable) /
+			sizeof (unsigned int);
+		int  sizeChou2TransitionTable = sizeof (chou2TransitionTable) /
+			sizeof (unsigned int);
+		int  sizeEvoTransitionTable = sizeof (evoTransitionTable) /
+			sizeof (unsigned int);
+		int  sizeHexTransitionTable = sizeof (hexTransitionTable) /
 			sizeof (unsigned int);
 
 		if (local_neighbors == 4) {
 			if (MI_IS_FULLRANDOM(mi) ||
-			    !(langton || dissolve || evolve || sheath)) {
-				int i = NRAND(4);
+			    !(langton || byl || chou1 || chou2 || dissolve || evolve || sheath)) {
+				int i = NRAND(5);
 
-				evolve = (!i);
-				dissolve = (i == 1);
-				sheath = (!evolve && (LRAND() & 1));
+				if (i == 1) /* Make 1-3 less likely */
+					i += NRAND(3);
+				else if (i > 1)
+					i += 2;
+				byl = (i == 1);
+				chou1 = (i == 2);
+				chou2 = (i == 3);
+				evolve = (i == 4);
+				dissolve = (i == 5);
+				if (i == 5)
+					sheath = (LRAND() & 1);
+				sheath = (i == 6);
 			} else {
-				evolve = (evolve && !langton);
-				dissolve = (dissolve && !evolve && !langton);
-				sheath = (sheath && !evolve && !langton);
+				evolve = (evolve && !langton && !byl && !chou1 && !chou2);
+				dissolve = (dissolve && !evolve && !langton && !byl && !chou1 && !chou2);
+				sheath = (sheath && !evolve && !langton && !byl && !chou1 && !chou2);
 			}
-			langton = (!(dissolve || evolve || sheath));
+			langton = (!(dissolve || evolve || sheath || byl || chou1 || chou2));
 		} else {
 			langton = True;
+			byl = False;
+			chou1 = False;
+			chou2 = False;
 			evolve = False;
 			dissolve = False;
 			sheath = False;
@@ -1438,6 +1626,12 @@ init_table(ModeInfo *mi)
 		if (MI_IS_VERBOSE(mi)) {
 			if (langton)
 				(void) fprintf(stdout, "Langton's Loops\n");
+			else if (byl)
+				(void) fprintf(stdout, "Byl's Loops\n");
+			else if (chou1)
+				(void) fprintf(stdout, "Chou-Reggia 1 Loops\n");
+			else if (chou2)
+				(void) fprintf(stdout, "Chou-Reggia 2 Loops\n");
 			else if (evolve)
 				(void) fprintf(stdout, "Evolving Loops\n");
 			else if (dissolve && !sheath)
@@ -1450,21 +1644,21 @@ init_table(ModeInfo *mi)
 				(void) fprintf(stderr, "Unknown Loop\n");
 		}
 		for (j = 0; j < local_neighbors; j++)
-			mult *= (8 + !langton);
+			mult *= (8 + !(langton || byl || chou1 || chou2));
 #if 1
 /* using space-wise inefficient method,
    uhhh I'll try to think harder on how to do it later %-) */
 /* also (!dissolve && sheath) should only be multiplied by 8... here and
    above, but lets keep it simple for now */
 /* Because of this will be using INEF_TABLE instead of EXT_TABLE */
-		if (!langton)
+		if (!(langton || byl || chou1 || chou2))
 			mult *= 9;
 #endif
 
 		if ((table = (unsigned int *) calloc(mult, sizeof (unsigned int))) == NULL) {
 			return False;
 		}
-		if (dissolve || evolve || sheath) {
+		if ((!(langton || byl || chou1 || chou2)) && (dissolve || evolve || sheath)) {
 	int a, b, c, d, e;
 
 	for (a = 0; a < EXT_STATES; a++)
@@ -1491,8 +1685,8 @@ init_table(ModeInfo *mi)
 		}
 #endif
 		if (local_neighbors == 6) {
-			for (j = 0; j < size_hex_transition_table; j++) {
-				tt = hex_transition_table[j];
+			for (j = 0; j < sizeHexTransitionTable; j++) {
+				tt = hexTransitionTable[j];
 				TRANSITION(tt, i);
 				for (k = 0; k < local_neighbors; k++) {
 					TRANSITION(tt, n[k]);
@@ -1505,11 +1699,63 @@ init_table(ModeInfo *mi)
 				HEX_TABLE_IN(c, n[4], n[5], n[0], n[1], n[2], n[3], i);
 				HEX_TABLE_IN(c, n[5], n[0], n[1], n[2], n[3], n[4], i);
 			}
+		} else if (byl) {
+			for (j = 0; j < sizeBylTransitionTable; j++) {
+				tt = bylTransitionTable[j];
+				TRANSITION(tt, i);
+				for (k = 0; k < local_neighbors; k++) {
+					TRANSITION(tt, n[k]);
+				}
+				FINALTRANSITION(tt, c);
+				TABLE_IN(c, n[0], n[1], n[2], n[3], i);
+				TABLE_IN(c, n[1], n[2], n[3], n[0], i);
+				TABLE_IN(c, n[2], n[3], n[0], n[1], i);
+				TABLE_IN(c, n[3], n[0], n[1], n[2], i);
+			}
+		} else if (chou1) {
+			for (j = 0; j < sizeChou1TransitionTable; j++) {
+				tt = chou1TransitionTable[j];
+				TRANSITION(tt, i);
+				for (k = 0; k < local_neighbors; k++) {
+					TRANSITION(tt, n[k]);
+				}
+				FINALTRANSITION(tt, c);
+				TABLE_IN(c, n[0], n[1], n[2], n[3], i);
+				TABLE_IN(c, n[1], n[2], n[3], n[0], i);
+				TABLE_IN(c, n[2], n[3], n[0], n[1], i);
+				TABLE_IN(c, n[3], n[0], n[1], n[2], i);
+			}
+		} else if (chou2) {
+			for (j = 0; j < sizeChou2TransitionTable; j++) {
+				tt = chou2TransitionTable[j];
+				TRANSITION(tt, i);
+				for (k = 0; k < local_neighbors; k++) {
+					TRANSITION(tt, n[k]);
+				}
+				FINALTRANSITION(tt, c);
+				TABLE_IN(c, n[0], n[1], n[2], n[3], i);
+				TABLE_IN(c, n[1], n[2], n[3], n[0], i);
+				TABLE_IN(c, n[2], n[3], n[0], n[1], i);
+				TABLE_IN(c, n[3], n[0], n[1], n[2], i);
+			}
+		} else if (langton) {
+			for (j = 0; j < sizeTransitionTable; j++) {
+				tt = transitionTable[j];
+				TRANSITION(tt, i);
+				for (k = 0; k < local_neighbors; k++) {
+					TRANSITION(tt, n[k]);
+				}
+				FINALTRANSITION(tt, c);
+				TABLE_IN(c, n[0], n[1], n[2], n[3], i);
+				TABLE_IN(c, n[1], n[2], n[3], n[0], i);
+				TABLE_IN(c, n[2], n[3], n[0], n[1], i);
+				TABLE_IN(c, n[3], n[0], n[1], n[2], i);
+			}
 		} else if (dissolve || evolve || sheath) {
-			for (j = 0; j < ((evolve) ? size_evo_transition_table :
-					size_transition_table);  j++) {
-				tt = (evolve) ? evo_transition_table[j] :
-					transition_table[j];
+			for (j = 0; j < ((evolve) ? sizeEvoTransitionTable :
+					sizeTransitionTable);  j++) {
+				tt = (evolve) ? evoTransitionTable[j] :
+					transitionTable[j];
 				TRANSITION(tt, i);
 				for (k = 0; k < local_neighbors; k++) {
 					TRANSITION(tt, n[k]);
@@ -1524,31 +1770,20 @@ init_table(ModeInfo *mi)
 				setUndefinedEvoRule();
 			else
 				setUndefinedRule();
-		} else {
-			for (j = 0; j < size_transition_table; j++) {
-				tt = transition_table[j];
-				TRANSITION(tt, i);
-				for (k = 0; k < local_neighbors; k++) {
-					TRANSITION(tt, n[k]);
-				}
-				FINALTRANSITION(tt, c);
-				TABLE_IN(c, n[0], n[1], n[2], n[3], i);
-				TABLE_IN(c, n[1], n[2], n[3], n[0], i);
-				TABLE_IN(c, n[2], n[3], n[0], n[1], i);
-				TABLE_IN(c, n[3], n[0], n[1], n[2], i);
-			}
 		}
 	}
 	return True;
 }
 
+#define	BLUE 1
+#define RED 2
 static void
 init_flaw(ModeInfo * mi)
 {
 	loopstruct *lp = &loops[MI_SCREEN(mi)];
 	int a, b;
+	int color = RED;
 
-#define	BLUE 2
 	if (lp->bncols <= 6 || lp->bnrows <= 6)
 		return;
 	a = MIN(lp->bncols - 6, 2 * ((local_neighbors == 6) ?
@@ -1578,38 +1813,99 @@ init_flaw(ModeInfo * mi)
 			lp->maxrow = lp->nrows - 1 - !lp->wrap;
 	}
 	if (local_neighbors == 6) {
-		lp->newcells[b * lp->bncols + a + (!lp->wrap + b) % 2 ] = BLUE;
+		lp->newcells[b * lp->bncols + a + (!lp->wrap + b) % 2 ] = color;
 		lp->newcells[b * lp->bncols + a + 1 +
-			(!lp->wrap + b) % 2] = BLUE;
-		lp->newcells[(b + 1) * lp->bncols + a] = BLUE;
-		lp->newcells[(b + 1) * lp->bncols + a + 2] = BLUE;
+			(!lp->wrap + b) % 2] = color;
+		lp->newcells[(b + 1) * lp->bncols + a] = color;
+		lp->newcells[(b + 1) * lp->bncols + a + 2] = color;
 		lp->newcells[(b + 2) * lp->bncols + a +
-			(!lp->wrap + b) % 2] = BLUE;
+			(!lp->wrap + b) % 2] = color;
 		lp->newcells[(b + 2) * lp->bncols + a + 1 +
-			(!lp->wrap + b) % 2] = BLUE;
+			(!lp->wrap + b) % 2] = color;
+	} else if (byl || chou1 || chou2) {
+		int orient = NRAND(2);
+		if (chou1 || chou2)
+			color = BLUE;
+		lp->newcells[lp->bncols * (b + 1) + a + 1] = color;
+		if (orient == 0)
+			lp->newcells[lp->bncols * b + a + 1] = color;
+		else
+			lp->newcells[lp->bncols * (b + 1) + a] = color;
 	} else {
 		int orient = NRAND(4);
-		lp->newcells[lp->bncols * (b + 1) + a + 1] = BLUE;
+		lp->newcells[lp->bncols * (b + 1) + a + 1] = color;
 		if (orient == 0 || orient == 1) {
-			lp->newcells[lp->bncols * b + a + 1] = BLUE;
+			lp->newcells[lp->bncols * b + a + 1] = color;
 		}
 		if (orient == 1 || orient == 2) {
-			lp->newcells[lp->bncols * (b + 1) + a + 2] = BLUE;
+			lp->newcells[lp->bncols * (b + 1) + a + 2] = color;
 		}
 		if (orient == 2 || orient == 3) {
-			lp->newcells[lp->bncols * (b + 2) + a + 1] = BLUE;
+			lp->newcells[lp->bncols * (b + 2) + a + 1] = color;
 		}
 		if (orient == 3 || orient == 0) {
-			lp->newcells[lp->bncols * (b + 1) + a] = BLUE;
+			lp->newcells[lp->bncols * (b + 1) + a] = color;
 		}
 	}
 }
 
+static int
+sqAdamLoopX()
+{
+	if (byl) {
+		return BYL_ADAM_LOOPX;
+	} else if (chou1) {
+		return CHOU1_ADAM_LOOPX;
+	} else if (chou2) {
+		return CHOU2_ADAM_LOOPX;
+	} else {
+		return SQ_ADAM_LOOPX(evolve);
+	}
+}
+
+static int
+sqAdamLoopY()
+{
+	if (byl) {
+		return BYL_ADAM_LOOPY;
+	} else if (chou1) {
+		return CHOU1_ADAM_LOOPY;
+	} else if (chou2) {
+		return CHOU2_ADAM_LOOPY;
+	} else {
+		return SQ_ADAM_LOOPY(evolve);
+	}
+}
+
+static int loopCell(int i, int j, Bool clockwise)
+{
+	int loopX = sqAdamLoopX();
+	if (byl) {
+		return ((clockwise) ?
+			bylReproducingLoop[j][loopX - i - 1] :
+			bylReproducingLoop[j][i]);
+	} else if (chou1) {
+		return ((clockwise) ?
+			chou1ReproducingLoop[j][loopX - i - 1] :
+			chou1ReproducingLoop[j][i]);
+	} else if (chou2) {
+		return ((clockwise) ?
+			chou2ReproducingLoop[j][loopX - i - 1] :
+			chou2ReproducingLoop[j][i]);
+	}
+	return ((evolve) ? ((clockwise) ?
+		evoReproducingLoop[j][loopX - i - 1] :
+		evoReproducingLoop[j][i]) :
+		((clockwise) ?
+		selfReproducingLoop[j][loopX - i - 1] :
+		selfReproducingLoop[j][i]));
+}
+
 static void
 init_adam(ModeInfo * mi)
 {
 	loopstruct *lp = &loops[MI_SCREEN(mi)];
-	XPoint      start, dirx, diry;
+	XPoint      start;
 	int         i, j, dir;
 
 #ifdef DELAYDEBUGLOOP
@@ -1622,7 +1918,8 @@ init_adam(ModeInfo * mi)
 	if (!MI_COUNT(mi)) /* Probably doing testing so do not confuse */
 #endif
 	dir = NRAND(local_neighbors);
-
+	start.x = 0;
+	start.y = 0;
 	if (local_neighbors == 6) {
 		int k;
 
@@ -1643,8 +1940,8 @@ init_adam(ModeInfo * mi)
 						k = (((lp->bnrows / 2 + HEX_ADAM_LOOPY / 2) % 2) ? -j / 2 : -(j + 1) / 2);
 						lp->newcells[(start.y + j) * lp->bncols + start.x + i + k] =
 		 					(lp->clockwise) ?
-					hex_self_reproducing_loop[i][j] :
-					hex_self_reproducing_loop[j][i];
+					hexReproducingLoop[i][j] :
+					hexReproducingLoop[j][i];
 					}
 				}
 				break;
@@ -1665,8 +1962,8 @@ init_adam(ModeInfo * mi)
               ? -(i + j + 1) / 2 : -(i + j) / 2);
 						lp->newcells[(start.y + j - i - lp->wrap) * lp->bncols + lp->wrap * (lp->bncols - (start.y + j - i + 1) % 2) + start.x + i + j + k] =
 		 					(lp->clockwise) ?
-		 					hex_self_reproducing_loop[i][j] :
-		 					hex_self_reproducing_loop[j][i];
+		 					hexReproducingLoop[i][j] :
+		 					hexReproducingLoop[j][i];
 					}
 				}
 				break;
@@ -1686,8 +1983,8 @@ init_adam(ModeInfo * mi)
 						k = (((lp->bnrows / 2 + HEX_ADAM_LOOPX / 2) % 2) ? -(HEX_ADAM_LOOPX - j - 1) / 2 : -(HEX_ADAM_LOOPX - j) / 2);
 						lp->newcells[(start.y + j) * lp->bncols + start.x + i + k] =
 		 					(lp->clockwise) ?
-		 					hex_self_reproducing_loop[j][HEX_ADAM_LOOPX - i - 1] :
-		 					hex_self_reproducing_loop[i][HEX_ADAM_LOOPY - j - 1];
+		 					hexReproducingLoop[j][HEX_ADAM_LOOPX - i - 1] :
+		 					hexReproducingLoop[i][HEX_ADAM_LOOPY - j - 1];
 					}
 				}
 				break;
@@ -1707,8 +2004,8 @@ init_adam(ModeInfo * mi)
 						k = (((lp->bnrows / 2 + HEX_ADAM_LOOPY / 2) % 2) ? -j / 2 : -(j + 1) / 2);
 						lp->newcells[(start.y + j) * lp->bncols + start.x + i + k] =
 		 					(lp->clockwise) ?
-		 					hex_self_reproducing_loop[HEX_ADAM_LOOPX - i - 1][HEX_ADAM_LOOPY - j - 1] :
-		 					hex_self_reproducing_loop[HEX_ADAM_LOOPY - j - 1][HEX_ADAM_LOOPX - i - 1];
+		 					hexReproducingLoop[HEX_ADAM_LOOPX - i - 1][HEX_ADAM_LOOPY - j - 1] :
+		 					hexReproducingLoop[HEX_ADAM_LOOPY - j - 1][HEX_ADAM_LOOPX - i - 1];
 					}
 				}
 				break;
@@ -1729,8 +2026,8 @@ init_adam(ModeInfo * mi)
               ? -(i + j + 1) / 2 : -(i + j) / 2);
 						lp->newcells[(start.y + j - i) * lp->bncols - lp->wrap * ((start.y + j - i + 1) % 2) + start.x + i + j + k] =
 		 					(lp->clockwise) ?
-		 					hex_self_reproducing_loop[HEX_ADAM_LOOPX - i - 1][HEX_ADAM_LOOPY - j - 1] :
-		 					hex_self_reproducing_loop[HEX_ADAM_LOOPY - j - 1][HEX_ADAM_LOOPX - i - 1];
+		 					hexReproducingLoop[HEX_ADAM_LOOPX - i - 1][HEX_ADAM_LOOPY - j - 1] :
+		 					hexReproducingLoop[HEX_ADAM_LOOPY - j - 1][HEX_ADAM_LOOPX - i - 1];
 					}
 				}
 				break;
@@ -1750,8 +2047,8 @@ init_adam(ModeInfo * mi)
 						k = (((lp->bnrows / 2 + HEX_ADAM_LOOPX / 2) % 2) ? -(HEX_ADAM_LOOPX - j - 1) / 2 : -(HEX_ADAM_LOOPX - j) / 2);
 						lp->newcells[(start.y + j) * lp->bncols + start.x + i + k] =
 		 					(lp->clockwise) ?
-		 					hex_self_reproducing_loop[HEX_ADAM_LOOPY - j - 1][i] :
-		 					hex_self_reproducing_loop[HEX_ADAM_LOOPX - i - 1][j];
+		 					hexReproducingLoop[HEX_ADAM_LOOPY - j - 1][i] :
+		 					hexReproducingLoop[HEX_ADAM_LOOPX - i - 1][j];
 					}
 				}
 				break;
@@ -1760,87 +2057,85 @@ init_adam(ModeInfo * mi)
 				/* (void) printf ("s %d  s %d \n", start.x, start.y); */
 		(void) printf ("%d %d %d %d %d\n",
      start.x + i + ((lp->bnrows / 2 % 2) ? -j / 2 : -(j + 1) / 2) - lp->bx,
-		 start.y + j - lp->by, i, j, hex_self_reproducing_loop[j][i]);
+		 start.y + j - lp->by, i, j, hexReproducingLoop[j][i]);
 		/* Draw right away */
 		drawcell(mi, start.x + i + ((lp->bnrows / 2 % 2) ? -j / 2 : -(j + 1) / 2) - lp->bx,
 		 start.y + j - lp->by,
-		 hex_self_reproducing_loop[j][i]);
+		 hexReproducingLoop[j][i]);
 #endif
 	} else {
+		XPoint dirx, diry;
+		dirx.x = 0, dirx.y = 0;
+		diry.x = 0, diry.y = 0;
 		switch (dir) {
 			case 0:
-				start.x = (lp->bncols - SQ_ADAM_LOOPX(evolve)) / 2;
-				start.y = (lp->bnrows - SQ_ADAM_LOOPY(evolve)) / 2;
+				start.x = (lp->bncols - sqAdamLoopX()) / 2;
+				start.y = (lp->bnrows - sqAdamLoopY()) / 2;
 				dirx.x = 1, dirx.y = 0;
 				diry.x = 0, diry.y = 1;
 				if (lp->mincol > start.x)
 					lp->mincol = start.x;
 				if (lp->minrow > start.y)
 					lp->minrow = start.y;
-				if (lp->maxcol < start.x + SQ_ADAM_LOOPX(evolve))
-					lp->maxcol = start.x + SQ_ADAM_LOOPX(evolve);
-				if (lp->maxrow < start.y + SQ_ADAM_LOOPY(evolve))
-					lp->maxrow = start.y + SQ_ADAM_LOOPY(evolve);
+				if (lp->maxcol < start.x + sqAdamLoopX())
+					lp->maxcol = start.x + sqAdamLoopX();
+				if (lp->maxrow < start.y + sqAdamLoopY())
+					lp->maxrow = start.y + sqAdamLoopY();
 				break;
 			case 1:
-				start.x = (lp->bncols + SQ_ADAM_LOOPY(evolve)) / 2;
-				start.y = (lp->bnrows - SQ_ADAM_LOOPX(evolve)) / 2;
+				start.x = (lp->bncols + sqAdamLoopY()) / 2;
+				start.y = (lp->bnrows - sqAdamLoopX()) / 2;
 				dirx.x = 0, dirx.y = 1;
 				diry.x = -1, diry.y = 0;
-				if (lp->mincol > start.x - SQ_ADAM_LOOPY(evolve))
-					lp->mincol = start.x - SQ_ADAM_LOOPY(evolve);
+				if (lp->mincol > start.x - sqAdamLoopY())
+					lp->mincol = start.x - sqAdamLoopY();
 				if (lp->minrow > start.y)
 					lp->minrow = start.y;
 				if (lp->maxcol < start.x)
 					lp->maxcol = start.x;
-				if (lp->maxrow < start.y + SQ_ADAM_LOOPX(evolve))
-					lp->maxrow = start.y + SQ_ADAM_LOOPX(evolve);
+				if (lp->maxrow < start.y + sqAdamLoopX())
+					lp->maxrow = start.y + sqAdamLoopX();
 				break;
 			case 2:
-				start.x = (lp->bncols + SQ_ADAM_LOOPX(evolve)) / 2;
-				start.y = (lp->bnrows + SQ_ADAM_LOOPY(evolve)) / 2;
+				start.x = (lp->bncols + sqAdamLoopX()) / 2;
+				start.y = (lp->bnrows + sqAdamLoopY()) / 2;
 				dirx.x = -1, dirx.y = 0;
 				diry.x = 0, diry.y = -1;
-				if (lp->mincol > start.x - SQ_ADAM_LOOPX(evolve))
-					lp->mincol = start.x - SQ_ADAM_LOOPX(evolve);
-				if (lp->minrow > start.y - SQ_ADAM_LOOPY(evolve))
-					lp->minrow = start.y - SQ_ADAM_LOOPY(evolve);
+				if (lp->mincol > start.x - sqAdamLoopX())
+					lp->mincol = start.x - sqAdamLoopX();
+				if (lp->minrow > start.y - sqAdamLoopY())
+					lp->minrow = start.y - sqAdamLoopY();
 				if (lp->maxcol < start.x)
 					lp->maxcol = start.x;
 				if (lp->maxrow < start.y)
 					lp->maxrow = start.y;
 				break;
 			case 3:
-				start.x = (lp->bncols - SQ_ADAM_LOOPY(evolve)) / 2;
-				start.y = (lp->bnrows + SQ_ADAM_LOOPX(evolve)) / 2;
+				start.x = (lp->bncols - sqAdamLoopY()) / 2;
+				start.y = (lp->bnrows + sqAdamLoopX()) / 2;
 				dirx.x = 0, dirx.y = -1;
 				diry.x = 1, diry.y = 0;
 				if (lp->mincol > start.x)
 					lp->mincol = start.x;
-				if (lp->minrow > start.y - SQ_ADAM_LOOPX(evolve))
-					lp->minrow = start.y - SQ_ADAM_LOOPX(evolve);
-				if (lp->maxcol < start.x + SQ_ADAM_LOOPX(evolve))
-					lp->maxcol = start.x + SQ_ADAM_LOOPX(evolve);
+				if (lp->minrow > start.y - sqAdamLoopX())
+					lp->minrow = start.y - sqAdamLoopX();
+				if (lp->maxcol < start.x + sqAdamLoopX())
+					lp->maxcol = start.x + sqAdamLoopX();
 				if (lp->maxrow < start.y)
 					lp->maxrow = start.y;
 				break;
 		}
-		for (j = 0; j < SQ_ADAM_LOOPY(evolve); j++)
-			for (i = 0; i < SQ_ADAM_LOOPX(evolve); i++) {
+		for (j = 0; j < sqAdamLoopY(); j++)
+			for (i = 0; i < sqAdamLoopX(); i++) {
+			  int value = loopCell(i, j, lp->clockwise);
+
 			  lp->newcells[lp->bncols * (start.y + dirx.y * i + diry.y * j) +
-			    start.x + dirx.x * i + diry.x * j] =
-			    (evolve) ? ((lp->clockwise) ?
-			      evo_self_reproducing_loop[j][EVO_ADAM_LOOPX - i - 1] :
-			      evo_self_reproducing_loop[j][i]) :
-			    ((lp->clockwise) ?
-			      self_reproducing_loop[j][ADAM_LOOPX - i - 1] :
-			      self_reproducing_loop[j][i]);
+			    start.x + dirx.x * i + diry.x * j] = value;
 #ifdef DEBUG
 			  /* Draw right away */
 			  drawcell(mi, start.x + dirx.x * i + diry.x * j - lp->bx,
 			   start.y + dirx.y * i + diry.y * j - lp->by,
-			   (evolve) ? ((lp->clockwise) ? evo_self_reproducing_loop[j][EVO_ADAM_LOOPX - i - 1] : evo_self_reproducing_loop[j][i]) :
-			   ((lp->clockwise) ? self_reproducing_loop[j][ADAM_LOOPX - i - 1] : self_reproducing_loop[j][i]));
+			   value);			   
 #endif
 			}
 	}
@@ -1869,7 +2164,7 @@ do_gen(loopstruct * lp)
 			for (k = 0; k < local_neighbors; k++) {
 				int         newi = i, newj = j;
 
-				position_of_neighbor(lp,
+				positionOfNeighbor(lp,
 					k * ANGLES / local_neighbors,
 					&newi, &newj);
 				n[k] = 0;
@@ -1887,14 +2182,14 @@ do_gen(loopstruct * lp)
 				*z = (lp->clockwise) ?
 					HEX_TABLE_OUT(c, n[5], n[4], n[3], n[2], n[1], n[0]) :
 					HEX_TABLE_OUT(c, n[0], n[1], n[2], n[3], n[4], n[5]);
+			} else if (langton || byl || chou1 || chou2) {
+				*z = (lp->clockwise) ?
+					TABLE_OUT(c, n[3], n[2], n[1], n[0]) :
+					TABLE_OUT(c, n[0], n[1], n[2], n[3]);
 			} else if (dissolve || evolve || sheath) {
 				*z = (lp->clockwise) ?
 					INEF_TABLE_OUT(c, n[3], n[2], n[1], n[0]) :
 					INEF_TABLE_OUT(c, n[0], n[1], n[2], n[3]);
-			} else {
-				*z = (lp->clockwise) ?
-					TABLE_OUT(c, n[3], n[2], n[1], n[0]) :
-					TABLE_OUT(c, n[0], n[1], n[2], n[3]);
 			}
 		}
 	}
@@ -1929,7 +2224,7 @@ init_loop(ModeInfo * mi)
 
 	if (loops == NULL) {
 		if ((loops = (loopstruct *) calloc(MI_NUM_SCREENS(mi),
-					       sizeof (loopstruct))) == NULL)
+				sizeof (loopstruct))) == NULL)
 			return;
 	}
 	lp = &loops[MI_SCREEN(mi)];
@@ -1960,13 +2255,13 @@ init_loop(ModeInfo * mi)
 	if (MI_NPIXELS(mi) >= COLORS) {
 		/* Maybe these colors should be randomized */
 		lp->colors[0] = MI_BLACK_PIXEL(mi);
-		lp->colors[1] = MI_PIXEL(mi, 0);	/* RED */
-		lp->colors[5] = MI_PIXEL(mi, MI_NPIXELS(mi) / REALCOLORS);	/* YELLOW */
-		lp->colors[4] = MI_PIXEL(mi, 2 * MI_NPIXELS(mi) / REALCOLORS);	/* GREEN */
-		lp->colors[6] = MI_PIXEL(mi, 3 * MI_NPIXELS(mi) / REALCOLORS);	/* CYAN */
-		lp->colors[2] = MI_PIXEL(mi, 4 * MI_NPIXELS(mi) / REALCOLORS);	/* BLUE */
-		lp->colors[3] = MI_PIXEL(mi, 5 * MI_NPIXELS(mi) / REALCOLORS);	/* MAGENTA */
-		lp->colors[7] = MI_WHITE_PIXEL(mi);
+		lp->colors[2] = MI_PIXEL(mi, 0);	/* RED */
+		lp->colors[4] = MI_PIXEL(mi, MI_NPIXELS(mi) / REALCOLORS);	/* YELLOW */
+		lp->colors[3] = MI_PIXEL(mi, 2 * MI_NPIXELS(mi) / REALCOLORS);	/* GREEN */
+		lp->colors[7] = MI_PIXEL(mi, 3 * MI_NPIXELS(mi) / REALCOLORS);	/* CYAN */
+		lp->colors[1] = MI_PIXEL(mi, 4 * MI_NPIXELS(mi) / REALCOLORS);	/* BLUE */
+		lp->colors[5] = MI_PIXEL(mi, 5 * MI_NPIXELS(mi) / REALCOLORS);	/* MAGENTA */
+		lp->colors[6] = MI_WHITE_PIXEL(mi);
 	}
 	lp->generation = 0;
 	lp->width = MI_WIDTH(mi);
@@ -1979,7 +2274,7 @@ init_loop(ModeInfo * mi)
 				break;
 			}
 			if (i == NEIGHBORKINDS - 1) {
-				if (NRAND(4))
+				if (NRAND(7))
 					local_neighbors = 4;
 				else
 					local_neighbors = 6;
@@ -2054,8 +2349,8 @@ init_loop(ModeInfo * mi)
 			lp->ys = MIN(size, MAX(MINSIZE, MIN(lp->width, lp->height) /
 					       MINGRIDSIZE));
 		lp->xs = lp->ys;
-		lp->ncols = MAX(lp->width / lp->xs, SQ_ADAM_LOOPX(evolve) + 1);
-		lp->nrows = MAX(lp->height / lp->ys, SQ_ADAM_LOOPX(evolve) + 1);
+		lp->ncols = MAX(lp->width / lp->xs, sqAdamLoopX() + 1);
+		lp->nrows = MAX(lp->height / lp->ys, sqAdamLoopY() + 1);
 		lp->xb = (lp->width - lp->xs * lp->ncols) / 2;
 		lp->yb = (lp->height - lp->ys * lp->nrows) / 2;
 	}
@@ -2102,7 +2397,7 @@ init_loop(ModeInfo * mi)
 void
 draw_loop(ModeInfo * mi)
 {
-	int         offset, i, j;
+	int         offset, i, j, factor = 1;
 	unsigned char *z, *znew;
 	loopstruct *lp;
 
@@ -2155,7 +2450,9 @@ draw_loop(ModeInfo * mi)
 			free_loop(MI_DISPLAY(mi), lp);
 			return;
 		}
-	if (++lp->generation > MI_CYCLES(mi) || lp->dead) {
+	if (byl | chou1 | chou2)
+		factor = 2; /* kill earlier */
+	if (++lp->generation * factor > MI_CYCLES(mi) || lp->dead) {
 		init_loop(mi);
 		return;
 	} else
