@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 4 -*- */
+/* -*- Mode: C; tab-width: 4; c-basic-offset: 4 -*- */
 /* mandelbrot --- animated mandelbrot sets */
 
 #if !defined( lint ) && !defined( SABER )
@@ -39,7 +39,7 @@ static const char sccsid[] = "@(#)mandelbrot.c	5.09 2003/06/30 xlockmore";
  *              xscreensaver's starfish
  * 01-Nov-2000: Allocation checks
  * 24-Mar-1999: DEM and Binary decomp options added by Tim Auckland
- *              <Tim.Auckland@Procket.com>.  Ideas from Peitgen & Saupe's
+ *              <tda10.geo AT yahoo.com>.  Ideas from Peitgen & Saupe's
  *              "The Science of Fractal Images"
  * 17-Nov-1998: Many Changes by Stromberg, including selection of
  *              interesting subregions, more extreme color ranges,
@@ -77,6 +77,9 @@ static const char sccsid[] = "@(#)mandelbrot.c	5.09 2003/06/30 xlockmore";
 #define DEF_INCREMENT  "1.00"
 #define DEF_BINARY     "False"
 #define DEF_DEM        "False"
+#define DEF_LYAP       "False"
+#define DEF_ALPHA      "False"
+#define DEF_INDEX      "False"
 #define DEF_POW        "False"
 #define DEF_SIN        "False"
 #define DEF_CYCLE "True"
@@ -87,13 +90,24 @@ static const char sccsid[] = "@(#)mandelbrot.c	5.09 2003/06/30 xlockmore";
 #define ESCAPE 13.0
 #define FLOATRAND(min,max) ((min)+((double) LRAND()/((double) MAXRAND))*((max)-(min)))
 
+typedef enum {
+	NONE,
+	LYAPUNOV,
+	ALPHA,
+	INDEX,
+	interior_size,
+} interior_t;
+
 	/* incr also would be nice as a parameter.  It controls how fast
 	   the order is advanced.  Non-integral values are not true orders,
 	   but it's a somewhat interesting function anyway
 	 */
 static float increment;
-static Bool  binary;
-static Bool  dem;
+static Bool  binary_p;
+static Bool  dem_p;
+static Bool  lyap_p;
+static Bool  alpha_p;
+static Bool  index_p;
 static Bool  pow_p;
 static Bool  sin_p;
 static Bool  cycle_p;
@@ -105,6 +119,12 @@ static XrmOptionDescRec opts[] =
   {(char *) "+binary", (char *) ".mandelbrot.binary", XrmoptionNoArg, (caddr_t) "off"},
   {(char *) "-dem", (char *) ".mandelbrot.dem", XrmoptionNoArg, (caddr_t) "on"},
   {(char *) "+dem", (char *) ".mandelbrot.dem", XrmoptionNoArg, (caddr_t) "off"},
+  {(char *) "-lyap", (char *) ".mandelbrot.lyap", XrmoptionNoArg, (caddr_t) "on"},
+  {(char *) "+lyap", (char *) ".mandelbrot.lyap", XrmoptionNoArg, (caddr_t) "off"},
+  {(char *) "-alpha", (char *) ".mandelbrot.alpha", XrmoptionNoArg, (caddr_t) "on"},
+  {(char *) "+alpha", (char *) ".mandelbrot.alpha", XrmoptionNoArg, (caddr_t) "off"},
+  {(char *) "-index", (char *) ".mandelbrot.index", XrmoptionNoArg, (caddr_t) "on"},
+  {(char *) "+index", (char *) ".mandelbrot.index", XrmoptionNoArg, (caddr_t) "off"},
   {(char *) "-pow", (char *) ".mandelbrot.pow", XrmoptionNoArg, (caddr_t) "on"},
   {(char *) "+pow", (char *) ".mandelbrot.pow", XrmoptionNoArg, (caddr_t) "off"},
   {(char *) "-sin", (char *) ".mandelbrot.sin", XrmoptionNoArg, (caddr_t) "on"},
@@ -115,19 +135,24 @@ static XrmOptionDescRec opts[] =
 
 static argtype vars[] =
 {
-  {(caddr_t *) & increment, (char *) "increment", (char *) "Increment", (char *) DEF_INCREMENT, t_Float},
-  {(caddr_t *) & binary, (char *) "binary", (char *) "Binary", (char *) DEF_BINARY, t_Bool},
-  {(caddr_t *) & dem, (char *) "dem", (char *) "Dem", (char *) DEF_DEM, t_Bool},
-  {(caddr_t *) & pow_p, (char *) "pow", (char *) "Pow", (char *) DEF_POW, t_Bool},
-  {(caddr_t *) & sin_p, (char *) "sin", (char *) "Sin", (char *) DEF_SIN, t_Bool},
-  {(caddr_t *) & cycle_p, (char *) "cycle", (char *) "Cycle", (char *) DEF_CYCLE
-, t_Bool}
+  {(void *) & increment, (char *) "increment", (char *) "Increment", (char *) DEF_INCREMENT, t_Float},
+  {(void *) & binary_p, (char *) "binary", (char *) "Binary", (char *) DEF_BINARY, t_Bool},
+  {(void *) & dem_p, (char *) "dem", (char *) "Dem", (char *) DEF_DEM, t_Bool},
+  {(void *) & lyap_p, (char *) "lyap", (char *) "Lyap", (char *) DEF_LYAP, t_Bool},
+  {(void *) & alpha_p, (char *) "alpha", (char *) "Alpha", (char *) DEF_ALPHA, t_Bool},
+  {(void *) & index_p, (char *) "index", (char *) "Index", (char *) DEF_INDEX, t_Bool},
+  {(void *) & pow_p, (char *) "pow", (char *) "Pow", (char *) DEF_POW, t_Bool},
+  {(void *) & sin_p, (char *) "sin", (char *) "Sin", (char *) DEF_SIN, t_Bool},
+  {(void *) & cycle_p, (char *) "cycle", (char *) "Cycle", (char *) DEF_CYCLE, t_Bool}
 };
 static OptionStruct desc[] =
 {
   {(char *) "-increment value", (char *) "increasing orders"},
   {(char *) "-/+binary", (char *) "turn on/off Binary Decomposition colour modulation"},
   {(char *) "-/+dem", (char *) "turn on/off Distance Estimator Method (instead of escape time)"},
+  {(char *) "-/+lyap", (char *) "render interior with Lyapunov measure"},
+  {(char *) "-/+alpha", (char *) "render interior with Alpha level sets"},
+  {(char *) "-/+index", (char *) "render interior with Alpha indexes"},
   {(char *) "-/+pow", (char *) "turn on/off adding z^z"},
   {(char *) "-/+sin", (char *) "turn on/off adding sin(z)"},
   {(char *) "-/+cycle", (char *) "turn on/off colour cycling"}
@@ -184,7 +209,7 @@ cln(complex * a)
 }
 
 static void
-cexp(complex * a)
+complex_exp(complex * a)
 {
 	/* e^z = e^x * (cos(y) + i * sin(y)) */
 	double      tr;
@@ -195,22 +220,22 @@ cexp(complex * a)
 }
 
 static complex
-cpow(complex a, complex b)
+complex_pow(complex a, complex b)
 {
 	/* a^z = e^(z * ln(a)) */
 
 	complex c, d;
-	
+
 	c = a;
 	d = b;
 	cln(&c);
 	mult(&d, c);
-	cexp(&d);
+	complex_exp(&d);
 	return d;
 }
 
 static complex
-csin(complex a)
+complex_sin(complex a)
 {
 	/* sin(z) = -i * (e^(i * z) - e^(-i * z)) / 2 */
 	/* cos(z) = (e^(i * z) - e^(-i * z)) / 2 */
@@ -220,12 +245,12 @@ csin(complex a)
 	d = a;
 	i.real = 0; i.imag = 1;
 
-	/* inefficient but I just want to see the picture */	
+	/* inefficient but I just want to see the picture */
 	mult(&c, i);
 	i.imag = -i.imag;
 	mult(&d, i);
-	cexp(&c);
-	cexp(&d);
+	complex_exp(&c);
+	complex_exp(&d);
 	d.real = -d.real;
 	d.imag = -d.imag;
 	add(&c, d);
@@ -284,6 +309,7 @@ typedef struct {
 	int         reptop;
 	Bool        dem, pow, sin, cycle_p, mono_p, no_colors;
 	Bool        binary;
+    interior_t  interior;
 	ModeInfo   *mi;
 } mandelstruct;
 
@@ -292,10 +318,20 @@ static mandelstruct *mandels = (mandelstruct *) NULL;
 /* do the iterations
  * if binary is true, check halfplane of last iteration.
  * if demrange is non zero, estimate lower bound of dist(c, M)
- * Loosely based on  Peitgen & Saupe's "The Science of Fractal Images"
+ *
+ * DEM - Distance Estimator Method
+ *      Based on Peitgen & Saupe's "The Science of Fractal Images"
+ *
+ * ALPHA - level sets of closest return.
+ * INDEX - index of ALPHA.
+ *      Based on Peitgen & Richter's "The Beauty of Fractals" (Fig 33,34)
+ *
+ * LYAPUNOV - lyapunov exponent estimate
+ *      Based on an idea by Jan Thor
+ *
  */
 static int
-reps(complex c, double p, int r, Bool binary, double demrange, Bool zpow, Bool zsin)
+reps(complex c, double p, int r, Bool binary, interior_t interior, double demrange, Bool zpow, Bool zsin)
 {
 	int         rep;
 	int         escaped = 0;
@@ -304,6 +340,14 @@ reps(complex c, double p, int r, Bool binary, double demrange, Bool zpow, Bool z
 		ESCAPE*ESCAPE*ESCAPE*ESCAPE); /* 2 more iterations */
 	complex     t1;
 	complex     dt;
+	double      L = 0.0;
+	double      l2;
+	double      dl2 = 1.0;
+	double      alpha2 = ESCAPE;
+	int         index = 0;
+#if defined(USE_LOG)
+	double      log_top = log((double) r);
+#endif
 
 	t = c;
 	dt.real = 1; dt.imag = 0;
@@ -312,12 +356,25 @@ reps(complex c, double p, int r, Bool binary, double demrange, Bool zpow, Bool z
 		ipow(&t, (int) p);
 		add(&t, c);
 		if (zpow)
-			add(&t, cpow(t1, t1));
+			add(&t, complex_pow(t1, t1));
 		if (zsin)
-			add(&t, csin(t1));
-		if (t.real * t.real + t.imag * t.imag >= escape) {
+			add(&t, complex_sin(t1));
+		l2 = t.real * t.real + t.imag * t.imag;
+		if (l2 <= alpha2) { 
+			alpha2 = l2;
+			index = rep;
+		}
+		if (l2 >= escape) {
 			escaped = 1;
 			break;
+			} else if (interior == LYAPUNOV) {
+		    /* Crude estimate of Lyapunov exponent.  The stronger the
+			   attractor, the more negative the exponent will be. */
+			/*                     n=N
+			L = lim        1/N *  Sum    log(abs(dx(n+1)/dx(n)))/ln(2)
+                   N->inf          n=1     
+			*/
+		    L += log(sqrt(l2));
 		}
 		if (demrange){
 			/* compute dt/dc
@@ -325,11 +382,14 @@ reps(complex c, double p, int r, Bool binary, double demrange, Bool zpow, Bool z
 			 * dt    =  p * t  * dt + 1
 			 *   k+1         k     k
 			 */
+			/* Note this is incorrect for zpow or zsin, but a correct
+			   implementation is too slow to be useful. */
 			dt.real *= p; dt.imag *= p;
 			if(p > 2) ipow(&t1, (int) (p - 1));
 			mult(&dt, t1);
 			dt.real += 1;
-			if (dt.real * dt.real + dt.imag * dt.imag >= 1e300) {
+			dl2 = dt.real * dt.real + dt.imag * dt.imag;			
+			if (dl2 >= 1e300) {
 				escaped = 2;
 				break;
 			}
@@ -339,15 +399,24 @@ reps(complex c, double p, int r, Bool binary, double demrange, Bool zpow, Bool z
 		if(demrange) {
 			double mt = sqrt(t1.real * t1.real + t1.imag * t1.imag);
 			 /* distance estimate */
-			double dist = 0.5 * mt * log(mt) /
-				sqrt(dt.real * dt.real + dt.imag * dt.imag);
-			rep = (int) (1 + 10*r*dist/demrange); /* scale for viewing */
+			double dist = 0.5 * mt * log(mt) / sqrt(dl2);
+			/* scale for viewing.  Allow black when showing interior. */
+			rep = (int) (((interior > NONE)?0:1) + 10*r*dist/demrange);
 			if(rep > r-1) rep = r-1; /* chop into color range */
-		} else {
-			if(binary && t.imag > 0)
-				rep = (r + rep / 2) % r; /* binary decomp */
 		}
+		if(binary && t.imag > 0)
+		    rep = (r + rep / 2) % r; /* binary decomp */
+#ifdef USE_LOG
+		if ( rep > 0 )
+		    rep = (int) (r * log((double) rep)/log_top); /* Log Scale */
+#endif
 		return rep;
+	} else if (interior == LYAPUNOV) {
+		return -(int)(L/M_LN2) % r;
+	} else if (interior == INDEX) {
+		return 1 + index;
+	} else if (interior == ALPHA) {
+		return (int) (r * sqrt(alpha2));
 	} else
 		return r;
 }
@@ -401,7 +470,7 @@ Select(/* input variables first */
 					temp.real = selected_ul->real +
 						(selected_ul->real - selected_lr->real) *
 						(((double)column)/sample_step);
-					r = reps(temp,(double) power,top,0,0.0, zpow, zsin);
+					r = reps(temp,(double) power,top,0,(interior_t) 0,0.0, zpow, zsin);
 					/* Here, we just want to see if the point is in the set,
 					** not if we can make something pretty
 					*/
@@ -445,8 +514,8 @@ free_mandelbrot(Display *display, mandelstruct *mp)
 			if (mp->ncolors && !mp->no_colors)
 				free_colors(display, mp->cmap, mp->colors,
 					mp->ncolors);
-				free(mp->colors);
-				mp->colors = (XColor *) NULL;
+			free(mp->colors);
+			mp->colors = (XColor *) NULL;
 		}
 		if (mp->cmap != None) {
 			XFreeColormap(display, mp->cmap);
@@ -492,19 +561,26 @@ init_mandelbrot(ModeInfo * mi)
 	MI_CLEARWINDOW(mi);
 
 	if (MI_IS_FULLRANDOM(mi)) {
-	  switch(NRAND(3)){
-	  case 0: mp->binary = 1; mp->dem = 0; break;
-	  case 1: mp->binary = 0; mp->dem = 1; break;
-	  default:mp->binary = 0; mp->dem = 0; break;
-	  }
+		mp->binary = (Bool) (LRAND() & 1);
+		mp->dem = (Bool) (LRAND() & 1);
+		mp->interior = (interior_t) NRAND(interior_size);
 #if 0
 	/* too slow */
 	  mp->pow = (NRAND(5) == 0);
 	  mp->sin = (NRAND(5) == 0);
 #endif
 	} else {
-	  mp->binary = binary;
-	  mp->dem = dem;
+	  mp->binary = binary_p;
+	  mp->dem = dem_p;
+	  if (index_p) {
+		  mp->interior = INDEX;
+	  } else if(alpha_p) {
+		  mp->interior = ALPHA;
+	  } else if(lyap_p) {
+		  mp->interior = LYAPUNOV;
+	  } else {
+		  mp->interior = NONE;
+	  }
 	  mp->pow = pow_p;
 	  mp->sin = sin_p;
 	}
@@ -560,7 +636,7 @@ init_mandelbrot(ModeInfo * mi)
 		}
 	}
 	MI_CLEARWINDOW(mi);
- 
+
   /* Set up colour map */
   mp->direction = (LRAND() & 1) ? 1 : -1;
   if (MI_IS_INSTALL(mi) && MI_NPIXELS(mi) > 2) {
@@ -657,9 +733,6 @@ draw_mandelbrot(ModeInfo * mi)
 	Window      window = MI_WINDOW(mi);
 	int         h;
 	complex     c;
-#if defined(USE_LOG)
-	double      log_top;
-#endif
 	double      demrange;
 	mandelstruct *mp;
 
@@ -668,9 +741,6 @@ draw_mandelbrot(ModeInfo * mi)
 	mp = &mandels[MI_SCREEN(mi)];
 
 	MI_IS_DRAWN(mi) = True;
-#if defined(USE_LOG)
-	log_top = log((float) mp->reptop);
-#endif
         if (MI_IS_INSTALL(mi) && MI_NPIXELS(mi) > 2) {
                 if (mp->mono_p) {
                         XSetForeground(display, mp->gc, mp->cur_color);
@@ -734,15 +804,11 @@ draw_mandelbrot(ModeInfo * mi)
 			(mp->ul.real-mp->lr.real)*(((double)(mp->column))/mp->screen_width);
 		c.imag = mp->ul.imag +
 			(mp->ul.imag - mp->lr.imag)*(((double) h) / mp->screen_height);
-		result = reps(c, mp->power, mp->reptop, mp->binary, demrange, mp->pow, mp->sin);
-		if (result == 0 || result == mp->reptop)
+		result = reps(c, mp->power, mp->reptop, mp->binary, mp->interior, demrange, mp->pow, mp->sin);
+		if (result < 0 || result >= mp->reptop)
 			XSetForeground(display, mp->gc, MI_BLACK_PIXEL(mi));
 		else {
-#if defined(USE_LOG)
-			color=(unsigned int) (MI_NPIXELS(mi) * log((float) result) / log_top);
-#else
 			color=(unsigned int) ((MI_NPIXELS(mi) * (float)result) / mp->reptop);
-#endif
 			XSetForeground(display, mp->gc, MI_PIXEL(mi, color));
 		}
 		/* we no longer have vertical symmetry - so we compute all points
