@@ -29,6 +29,7 @@ static const char sccsid[] = "@(#)marquee.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_marquee
 #define PROGCLASS "Marquee"
 #define HACK_INIT init_marquee
 #define HACK_DRAW draw_marquee
@@ -61,7 +62,17 @@ ModStruct   marquee_description =
 
 #endif
 
+#if USE_MB
+static int font_height(XFontStruct *f)
+{
+	XRectangle mbRect;
+
+	XmbTextExtents(fontset, "Aj", 1, NULL, &mbRect);
+	return mbRect.height;
+}
+#else
 #define font_height(f) ((f==None)?8:f->ascent + f->descent)
+#endif
 
 extern XFontStruct *getFont(Display * display);
 extern char *getWords(int screen, int screens);
@@ -194,6 +205,53 @@ fixup_back(char *s)
 	return s;
 }
 
+static int
+charBytes(unsigned char cb)
+{
+#ifdef USE_MB
+	int mb = 1;
+
+	/*if ((cb & 0x80) && !(cb & 0x40))
+		an extended byte*/
+	if ((cb & 0x80) && (cb & 0x40))
+		mb = 2;
+	else
+		return mb;
+	if (cb & 0x20)
+		mb = 3;
+	else
+		return mb;
+	if (cb & 0x10)
+		mb = 4;
+	else
+		return mb;
+	if (cb & 0x08)
+		mb = 5;
+	else
+		return mb;
+	if (cb & 0x04)
+		return 6;
+	else
+		return mb;
+#else
+	return 1;
+#endif
+}
+
+static int
+charWidth(char * string)
+{
+#ifdef USE_MB
+	XRectangle  rect;
+	int mb = charBytes(*string);
+
+	XmbTextExtents(fontset, string, mb, NULL, &rect);
+	return rect.width;
+#else
+	return char_width[(int) (unsigned char) *string];
+#endif
+}
+
 static int
 text_font_width(char *string)
 {
@@ -217,11 +275,11 @@ text_font_width(char *string)
 				 * a monospaced font. */
 				if (t) {
 					t--;
-					x -= char_width[' '];
+					x -= char_width[(int) (' ')];
 				}
 				break;
 			case '\t':
-				x += char_width[' '] * (8 - (t % 8));
+				x += char_width[(int) (' ')] * (8 - (t % 8));
 				t = ((t + 8) / 8) * 8;
 				break;
 			case '\r':
@@ -233,7 +291,7 @@ text_font_width(char *string)
 					x += char_back_char_width(string);
 					string += 2;
 				} else {
-					x += char_width[(int) (unsigned char) *string];
+					x += charWidth(string);
 				}
 		}
 		string++;
@@ -257,6 +315,8 @@ text_height(char *string)
 static int
 add_blanks(marqueestruct * mp)
 {
+	int width;
+
 	if (mp->t < 251) {
 		mp->modwords[mp->t] = ' ';
 		mp->t++;
@@ -265,9 +325,10 @@ add_blanks(marqueestruct * mp)
 		mp->modwords[mp->t] = '\0';
 		(void) strcat(mp->modwords, "  ");
 	}
-	mp->x -= 2 * char_width[' '];
-	if (mp->x <= -char_width[(int) (unsigned char) mp->modwords[0]]) {
-		mp->x += char_width[(int) (unsigned char) mp->modwords[0]];
+	mp->x -= 2 * char_width[(int) (' ')];
+	width = charWidth(mp->modwords);
+	if (mp->x <= -width) {
+		mp->x += width;
 		(void) memcpy(mp->modwords, &(mp->modwords[1]), mp->nonblanks);
 		mp->nonblanks--;
 	}
@@ -275,20 +336,29 @@ add_blanks(marqueestruct * mp)
 }
 
 static void
-add_letter(marqueestruct * mp, char letter)
+add_letter(marqueestruct * mp, char * letter)
 {
-	if (mp->t < 252) {
-		mp->modwords[mp->t] = letter;
-		mp->t++;
+	int width;
+	int b = charBytes(letter[0]);
+	int i;
+
+	if (mp->t < 253 - b) {
+		for (i = 0; i < b; i++) {
+			mp->modwords[mp->t] = letter[i];
+			mp->t++;
+		}
 		mp->modwords[mp->t] = '\0';
 		(void) strcat(mp->modwords, "  ");
 	}
-	mp->x -= char_width[(int) letter];
-	if (mp->x <= -char_width[(int) (unsigned char) mp->modwords[0]]) {
-		mp->x += char_width[(int) (unsigned char) mp->modwords[0]];
+	mp->x -= charWidth(letter);
+	width = charWidth(mp->modwords);
+	if (mp->x <= -width) {
+		mp->x += width;
 		(void) memcpy(mp->modwords, &(mp->modwords[1]), mp->t);
-		mp->modwords[mp->t] = ' ';
-		mp->t--;
+		for (i = 0; i < b; i++) {
+			mp->modwords[mp->t] = ' ';
+			mp->t--;
+		}
 	} else
 		mp->nonblanks = mp->t;
 }
@@ -323,7 +393,6 @@ init_marquee(ModeInfo * mi)
 		mode_font = getFont(display);
 	if (mp->gc == NULL && mode_font != None) {
 		gcv.font = mode_font->fid;
-		XSetFont(display, MI_GC(mi), mode_font->fid);
 		gcv.graphics_exposures = False;
 		gcv.foreground = MI_WHITE_PIXEL(mi);
 		gcv.background = MI_BLACK_PIXEL(mi);
@@ -376,7 +445,7 @@ draw_marquee(ModeInfo * mi)
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
 	char       *space = (char *) "        ";
-	char       *ch;
+	unsigned char       *ch;
 	marqueestruct *mp = &marquees[MI_SCREEN(mi)];
 
 	if (marquees == NULL)
@@ -386,9 +455,9 @@ draw_marquee(ModeInfo * mi)
 		return;
 
 	MI_IS_DRAWN(mi) = True;
-	ch = mp->words;
+	ch = (unsigned char*) mp->words;
 	if (isRibbon()) {
-		ch = mp->words;
+		ch = (unsigned char*) mp->words;
 		switch (*ch) {
 			case '\0':
 				if (add_blanks(mp)) {
@@ -402,11 +471,11 @@ draw_marquee(ModeInfo * mi)
 			case '\t':
 			case '\v':
 			case '\f':
-				add_letter(mp, ' ');
+				add_letter(mp, (char *) " ");
 				mp->words++;
 				break;
 			default:
-				add_letter(mp, *ch);
+				add_letter(mp, (char *) ch);
 				mp->words++;
 		}
 		if (MI_NPIXELS(mi) > 2) {
@@ -427,7 +496,7 @@ draw_marquee(ModeInfo * mi)
 				if (mp->t) {
 					/* see note in text_font_width */
 					mp->t--;
-					mp->x -= char_width[' '];
+					mp->x -= char_width[(int) (' ')];
 				}
 				break;
 			case '\v':
@@ -448,7 +517,7 @@ draw_marquee(ModeInfo * mi)
 			case '\t':
 				(void) XDrawString(display, window, mp->gc, mp->x, mp->y + mp->ascent,
 						   space, 8 - (mp->t % 8));
-				mp->x += char_width[' '] * (8 - (mp->t % 8));
+				mp->x += char_width[(int) (' ')] * (8 - (mp->t % 8));
 				mp->t = ((mp->t + 8) / 8) * 8;
 				break;
 			case '\r':
@@ -460,44 +529,27 @@ draw_marquee(ModeInfo * mi)
 						mp->color = 0;
 				} else
 					XSetForeground(display, mp->gc, MI_WHITE_PIXEL(mi));
-				if (is_char_back_char(ch)) {
-					int         xmid = mp->x + (char_back_char_width(ch) + 1) / 2;
+				if (is_char_back_char((char *) ch)) {
+					int         xmid = mp->x + (char_back_char_width((char *) ch) + 1) / 2;
 
 					(void) XDrawString(display, window, mp->gc,
-							   xmid - char_width[(int) (unsigned char) *ch] / 2,
-						  mp->y + mp->ascent, ch, 1);
+							   xmid - char_width[(int) (const char) *ch] / 2,
+						mp->y + mp->ascent, (char *) ch, 1);
 					(void) XDrawString(display, window, mp->gc,
-							   xmid - char_width[(int) (unsigned char) *(ch + 2)] / 2,
-					      mp->y + mp->ascent, ch + 2, 1);
-					mp->x += char_back_char_width(ch);
+							   xmid - char_width[(int) (const char) *(ch + 2)] / 2,
+						mp->y + mp->ascent, (char *) ch + 2, 1);
+					mp->x += char_back_char_width((char *) ch);
 					mp->words += 2;
 				} else {
-#ifdef USE_MB
-					int         mb = (*ch & 0x80) ? 2 : 1;
+					int mb = charBytes(*ch);
 
 					(void) XDrawString(display, window, mp->gc,
-					  mp->x, mp->y + mp->ascent, ch, mb);
-					if (mb == 1)
-						mp->x += char_width[(int) (unsigned char) *ch];
-					else {
-						XRectangle  rect;
-
-						XmbTextExtents(fontset, ch, 2, NULL, &rect);
-						mp->x += rect.width;
-					}
-#else
-					(void) XDrawString(display, window, mp->gc,
-					   mp->x, mp->y + mp->ascent, ch, 1);
-					mp->x += char_width[(int) (unsigned char) *ch];
-#endif
+						mp->x, mp->y + mp->ascent, (char *) ch, mb);
+					mp->x += charWidth((char *) ch);
 				}
 				mp->t++;
 		}
-#ifdef USE_MB
-		mp->words += (*mp->words & 0x80) ? 2 : 1;
-#else
-		mp->words++;
-#endif
+		mp->words += charBytes(*ch);
 	}
 }
 
@@ -513,7 +565,7 @@ release_marquee(ModeInfo * mi)
 			if (mp->gc != None)
 				XFreeGC(MI_DISPLAY(mi), mp->gc);
 		}
-		(void) free((void *) marquees);
+		free(marquees);
 		marquees = (marqueestruct *) NULL;
 	}
 	if (mode_font != None) {
