@@ -70,7 +70,6 @@ static const char sccsid[] = "@(#)matrix.c	5.00 2000/11/01 xlockmore";
 
 #ifdef VMS
 #include "vms_x_fix.h"
-#endif
 
 /*-
  * due to a Bug/feature in VMS X11/Intrinsic.h has to be placed before xlock.
@@ -78,8 +77,10 @@ static const char sccsid[] = "@(#)matrix.c	5.00 2000/11/01 xlockmore";
  */
 
 #include <X11/Intrinsic.h>
+#endif
 
 #ifdef STANDALONE
+#define MODE_matrix
 #define PROGCLASS "Matrix"
 #define HACK_INIT init_matrix
 #define HACK_DRAW draw_matrix
@@ -101,6 +102,12 @@ ModStruct   matrix_description =
  "Shows the Matrix", 0, NULL};
 #endif
 
+#ifdef MODE_matrix
+#ifndef VMS
+#ifndef WIN32
+#include <X11/Intrinsic.h>
+#endif /* WIN32 */
+#endif
 
 /*
  * state associated with a single column of the waterfall display
@@ -390,8 +397,8 @@ static void new_column(const ModeInfo *mi, column_t *c)
    * clear the pixmap to get rid of previous data or initialize
    */
   XSetForeground(MI_DISPLAY(mi), MI_GC(mi), mp->bg);
-  XFillRectangle(MI_DISPLAY(mi), c->pixmap, MI_GC(mi),
-				 0, 0, PIXMAP_WIDTH, mp->pixmapHeight);
+  XFillRectangle(MI_DISPLAY(mi), (Drawable) c->pixmap, MI_GC(mi),
+	 0, 0, PIXMAP_WIDTH, mp->pixmapHeight);
 
   /*
    * Write characters into the column pixmap, starting at the
@@ -434,10 +441,11 @@ static void new_column(const ModeInfo *mi, column_t *c)
 			  kOffset = MATRIX_RANDOM(0, katakana_num_cells);
 			}
 
-		  XCopyArea(MI_DISPLAY(mi), src, c->pixmap, MI_GC(mi),
-					kOffset * katakana_cell_width, 0,
-					katakana_cell_width, katakana_cell_height,
-					0, currChar * katakana_cell_height);
+		  XCopyArea(MI_DISPLAY(mi), (Drawable) src,
+			(Drawable) c->pixmap, MI_GC(mi),
+			kOffset * katakana_cell_width, 0,
+			katakana_cell_width, katakana_cell_height,
+			0, currChar * katakana_cell_height);
 
 		  c->strLen--;
 		}
@@ -461,7 +469,7 @@ free_matrix(Display *display, matrix_t *mp)
 			XFreePixmap(display, mp->columns[c].pixmap);
 	  }
 
-	  (void) free((void *) mp->columns);
+	  free(mp->columns);
 	  mp->columns = (column_t *) NULL;
 	}
 
@@ -525,7 +533,7 @@ static void setup_matrix(const ModeInfo *mi)
   if ((mp->columns = (column_t *) calloc(mp->num_cols,
 		sizeof(column_t))) == NULL) {
 	free_matrix(display, mp);
-	return; 
+	return;
   }
 
   /*
@@ -655,11 +663,11 @@ void draw_matrix(ModeInfo *mi)
 	  if (c->yPtr <= 0) /* we're about to finish the current pixmap */
 		{
 		  /* exhaust the pixels in the current pixmap */
-		  XCopyArea(MI_DISPLAY(mi), c->pixmap,
-					MI_WINDOW(mi), MI_GC(mi),
-					0, 0,
-					PIXMAP_WIDTH, yDelta + c->yPtr,
-					xOffset, -(c->yPtr));
+		  XCopyArea(MI_DISPLAY(mi), (Drawable) c->pixmap,
+			MI_WINDOW(mi), MI_GC(mi),
+			0, 0,
+			PIXMAP_WIDTH, yDelta + c->yPtr,
+			xOffset, -(c->yPtr));
 
 #ifdef RANDOMIZE_COLUMNS
 		  /* regenerate the column pixmap with new text */
@@ -672,22 +680,22 @@ void draw_matrix(ModeInfo *mi)
 #endif
 
 		  /* fill in the remainder of the column from the new pixmap */
-		  XCopyArea(MI_DISPLAY(mi), c->pixmap,
-					MI_WINDOW(mi), MI_GC(mi),
-					0, mp->pixmapHeight + c->yPtr,
-					PIXMAP_WIDTH, -(c->yPtr),
-					xOffset, 0);
+		  XCopyArea(MI_DISPLAY(mi), (Drawable) c->pixmap,
+			MI_WINDOW(mi), MI_GC(mi),
+			0, mp->pixmapHeight + c->yPtr,
+			PIXMAP_WIDTH, -(c->yPtr),
+			xOffset, 0);
 
 		  c->yPtr += mp->pixmapHeight;
 		}
 	  else
 		{
 		  /* fill in the remainder of the column from the pixmap */
-		  XCopyArea(MI_DISPLAY(mi), c->pixmap,
-					MI_WINDOW(mi), MI_GC(mi),
-					0, c->yPtr,
-					PIXMAP_WIDTH, yDelta,
-					xOffset, 0);
+		  XCopyArea(MI_DISPLAY(mi), (Drawable) c->pixmap,
+			MI_WINDOW(mi), MI_GC(mi),
+			0, c->yPtr,
+			PIXMAP_WIDTH, yDelta,
+			xOffset, 0);
 		}
 	}
 }
@@ -709,8 +717,8 @@ void release_matrix(ModeInfo *mi)
 
 	for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 		free_matrix(MI_DISPLAY(mi), &matrix[screen]);
-	  (void) free((void *) matrix);
-	  matrix = (matrix_t *) NULL;
+	free(matrix);
+	matrix = (matrix_t *) NULL;
   }
 }
 
@@ -771,3 +779,5 @@ void change_matrix(ModeInfo *mi)
 	  new_column(mi, column);
   }
 }
+
+#endif /* MODE_matrix */
