@@ -28,17 +28,17 @@ static const char sccsid[] = "@(#)maze.c	5.00 2000/11/01 xlockmore";
  * 10-May-1997: Compatible with xscreensaver
  * 27-Feb-1996: Add ModeInfo args to init and callback hooks, use new
  *              ModeInfo handle to specify long pauses (eliminate onepause).
- *		        Ron Hitchens <ron@idiom.com>
+ *		        Ron Hitchens <ron AT idiom.com>
  * 20-Jul-1995: minimum size fix Peter Schmitzberger <schmitz@coma.sbg.ac.at>
  * 17-Jun-1995: removed sleep statements
  * 22-Mar-1995: multidisplay fix Caleb Epstein <epstein_caleb@jpmorgan.com>
  * 09-Mar-1995: changed how batchcount is used
  * 27-Feb-1995: patch for VMS
- * 04-Feb-1995: patch to slow down maze Heath Kehoe <hakehoe@icaen.uiowa.edu>
+ * 04-Feb-1995: patch to slow down maze Heath Kehoe <hakehoe AT icaen.uiowa.edu>
  * 17-Jun-1994: HP ANSI C compiler needs a type cast for gray_bits
  *              Richard Lloyd <R.K.Lloyd@csc.liv.ac.uk>
- * 02-Sep-1993: xlock version David Bagley <bagleyd@tux.org>
- * 07-Mar-1993: Good ideas from xscreensaver Jamie Zawinski <jwz@jwz.org>
+ * 02-Sep-1993: xlock version David Bagley <bagleyd AT verizon.net>
+ * 07-Mar-1993: Good ideas from xscreensaver Jamie Zawinski <jwz AT jwz.org>
  * 06-Jun-1985: Martin Weiss Sun Microsystems
  */
 
@@ -77,7 +77,7 @@ static const char sccsid[] = "@(#)maze.c	5.00 2000/11/01 xlockmore";
 #define maze_opts xlockmore_opts
 #define DEFAULTS "*delay: 1000 \n" \
  "*cycles: 3000 \n" \
- "*size: -40 \n" \
+ "*size: -10 \n" \
  "*ncolors: 64 \n" \
  "*bitmap: \n" \
  "*trackmouse: False \n"
@@ -91,31 +91,44 @@ static const char sccsid[] = "@(#)maze.c	5.00 2000/11/01 xlockmore";
 #ifdef MODE_maze
 
 #define DEF_TRACKMOUSE  "False"
+#define DEF_THICK  "False"
+#define DEF_THREED  "False"
 static Bool trackmouse;
+static Bool thick;
+static Bool threed;
 
-#ifdef DISABLE_INTERACTIVE
-ModeSpecOpt maze_opts =
-{0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
-#else
 static XrmOptionDescRec opts[] =
 {
+#ifdef DISABLE_INTERACTIVE
   {(char *) "-trackmouse", (char *) ".maze.trackmouse", XrmoptionNoArg, (caddr_t) "on"},
-  {(char *) "+trackmouse", (char *) ".maze.trackmouse", XrmoptionNoArg, (caddr_t) "off"}
+  {(char *) "+trackmouse", (char *) ".maze.trackmouse", XrmoptionNoArg, (caddr_t) "off"},
+#endif
+  {(char *) "-thick", (char *) ".maze.thick", XrmoptionNoArg, (caddr_t) "on"},
+  {(char *) "+thick", (char *) ".maze.thick", XrmoptionNoArg, (caddr_t) "off"},
+  {(char *) "-threed", (char *) ".maze.threed", XrmoptionNoArg, (caddr_t) "on"},
+  {(char *) "+threed", (char *) ".maze.threed", XrmoptionNoArg, (caddr_t) "off"},
 };
 
 static argtype vars[] =
 {
-  {(caddr_t *) & trackmouse, (char *) "trackmouse", (char *) "TrackMouse", (char *) DEF_TRACKMOUSE, t_Bool}
+#ifdef DISABLE_INTERACTIVE
+  {(void *) & trackmouse, (char *) "trackmouse", (char *) "TrackMouse", (char *) DEF_TRACKMOUSE, t_Bool},
+#endif
+  {(void *) & thick, (char *) "thick", (char *) "Thick", (char *) DEF_THICK, t_Bool},
+  {(void *) & threed, (char *) "threed", (char *) "ThreeD", (char *) DEF_THREED, t_Bool},
 };
 
 static OptionStruct desc[] =
 {
+#ifdef DISABLE_INTERACTIVE
   {(char *) "-/+trackmouse", (char *) "turn on/off the tracking of the mouse"},
+#endif
+  {(char *) "-/+thick", (char *) "turn on/off the thick tracing"},
+  {(char *) "-/+threed", (char *) "turn on/off the Three D look"},
 };
 
 ModeSpecOpt maze_opts =
 {sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
-#endif
 
 #ifdef USE_MODULES
 ModStruct   maze_description =
@@ -141,8 +154,7 @@ ModStruct   maze_description =
 #define DEFAULT_XPM 1
 #endif
 
-#define MINGRIDSIZE	3
-#define MINSIZE        8
+#define MINGRIDSIZE	2
 
 #define WALL_TOP	0x8000
 #define WALL_RIGHT	0x4000
@@ -193,7 +205,7 @@ typedef struct {
 	unsigned long black, color;
 	int         graphics_format;
 	GC          backGC;
-	int         time;
+	int         time, space, threed;
 } mazestruct;
 
 static mazestruct *mazes = (mazestruct *) NULL;
@@ -249,27 +261,31 @@ draw_solid_square(ModeInfo * mi, GC gc,
 	switch (dir) {
 		case 0:
 			XFillRectangle(display, window, gc,
-				       mp->xb + 3 + mp->xs * i,
-				       mp->yb - 3 + mp->ys * j,
-				       mp->xs - 6, mp->ys);
+				       mp->xb + 3 * mp->space + mp->xs * i,
+				       mp->yb - 3 * mp->space + mp->ys * j,
+				       mp->xs + 1 - 6 * mp->space - mp->threed,
+				       mp->ys + 1 - mp->threed);
 			break;
 		case 1:
 			XFillRectangle(display, window, gc,
-				       mp->xb + 3 + mp->xs * i,
-				       mp->yb + 3 + mp->ys * j,
-				       mp->xs, mp->ys - 6);
+				       mp->xb + 3 * mp->space + mp->xs * i,
+				       mp->yb + 3 * mp->space + mp->ys * j,
+				       mp->xs + 1 - mp->threed,
+				       mp->ys + 1 - 6 * mp->space - mp->threed);
 			break;
 		case 2:
 			XFillRectangle(display, window, gc,
-				       mp->xb + 3 + mp->xs * i,
-				       mp->yb + 3 + mp->ys * j,
-				       mp->xs - 6, mp->ys);
+				       mp->xb + 3 * mp->space + mp->xs * i,
+				       mp->yb + 3 * mp->space + mp->ys * j,
+				       mp->xs + 1 - 6 * mp->space - mp->threed,
+				       mp->ys + 1 - mp->threed);
 			break;
 		case 3:
 			XFillRectangle(display, window, gc,
-				       mp->xb - 3 + mp->xs * i,
-				       mp->yb + 3 + mp->ys * j,
-				       mp->xs, mp->ys - 6);
+				       mp->xb - 3 * mp->space + mp->xs * i,
+				       mp->yb + 3 * mp->space + mp->ys * j,
+				       mp->xs + 1 - mp->threed,
+				       mp->ys + 1 - 6 * mp->space - mp->threed);
 			break;
 	}
 
@@ -365,18 +381,29 @@ set_maze_sizes(ModeInfo * mi)
 	mazestruct *mp = &mazes[MI_SCREEN(mi)];
 	Display *display = MI_DISPLAY(mi);
 	int         size = MI_SIZE(mi);
+	int         minsize;
 
-	if (size < -MINSIZE) {
+	if (MI_IS_FULLRANDOM(mi)) {
+		int randNum = NRAND(4);
+
+		mp->threed = (randNum == 1) ? 1 : 0;
+		mp->space = (randNum == 0) ? 0 : 1;
+	} else {
+		mp->threed = (threed) ? 1 : 0;
+		mp->space = (!threed && thick) ? 0 : 1;
+	}
+	minsize = mp->space * 4 + 3 + mp->threed;
+	if (size < -minsize) {
 		free_path(mp);
-		mp->ys = NRAND(MIN(-size, MAX(MINSIZE, (MIN(mp->width, mp->height) - 1) /
-				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
-	} else if (size < MINSIZE) {
+		mp->ys = NRAND(MIN(-size, MAX(minsize, (MIN(mp->width, mp->height) - 1) /
+				      MINGRIDSIZE)) - minsize + 1) + minsize;
+	} else if (size < minsize) {
 		if (!size)
-			mp->ys = MAX(MINSIZE, (MIN(mp->width, mp->height) - 1) / MINGRIDSIZE);
+			mp->ys = MAX(minsize, (MIN(mp->width, mp->height) - 1) / MINGRIDSIZE);
 		else
-			mp->ys = MINSIZE;
+			mp->ys = minsize;
 	} else
-		mp->ys = MIN(size, MAX(MINSIZE, (MIN(mp->width, mp->height) - 1) /
+		mp->ys = MIN(size, MAX(minsize, (MIN(mp->width, mp->height) - 1) /
 				       MINGRIDSIZE));
 	mp->xs = mp->ys;
 	mp->logo_size_x = mp->logo->width / mp->xs + 1;
@@ -515,10 +542,12 @@ initialize_maze(ModeInfo * mi)
 	mp->end_dir = wall;
 
 	/* set logo */
-	if ((mp->ncols > mp->logo_size_x + 6) &&
-	    (mp->nrows > mp->logo_size_y + 6)) {
-		mp->logo_x = NRAND(mp->ncols - mp->logo_size_x - 6) + 3;
-		mp->logo_y = NRAND(mp->nrows - mp->logo_size_y - 6) + 3;
+	if ((mp->ncols > mp->logo_size_x + 2 + 4 * mp->space) &&
+	    (mp->nrows > mp->logo_size_y + 2 + 4 * mp->space)) {
+		mp->logo_x = NRAND(mp->ncols - mp->logo_size_x - 2 -
+			4 * mp->space) + 1 + 2 * mp->space;
+		mp->logo_y = NRAND(mp->nrows - mp->logo_size_y - 2 -
+			4 * mp->space) + 1 + 2 * mp->space;
 
 		for (i = 0; i < mp->logo_size_x; i++)
 			for (j = 0; j < mp->logo_size_y; j++)
@@ -681,7 +710,8 @@ create_maze_walls(ModeInfo * mi)
 			(DOOR_IN_TOP >> ((newdoor + 2) % 4));
 
 		/* if end square set path length and save path */
-		if (mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] & END_SQUARE) {
+		if (mp->maze[mp->cur_sq_x * mp->nrows + mp->cur_sq_y] &
+				END_SQUARE) {
 			mp->path_length = mp->sqnum;
 			for (i = 0; i < mp->path_length; i++) {
 				mp->save_path[i].x = mp->move_list[i].x;
@@ -996,7 +1026,7 @@ draw_maze(ModeInfo * mi)
 	if (mp->solving) {
 		if (trackmouse)
 			mouse_maze(mi);
-		else 
+		else
 			solve_maze(mi);
 		return;
 	}
