@@ -28,7 +28,7 @@ static const char sccsid[] = "@(#)munch.c	5.00 2000/11/01 xlockmore";
  *
  * Some code stolen from / This is meant to work with
  * xscreensaver, Copyright (c) 1992, 1995, 1996
- * Jamie Zawinski <jwz@jwz.org>
+ * Jamie Zawinski <jwz AT jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -52,6 +52,7 @@ static const char sccsid[] = "@(#)munch.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_munch
 #define PROGCLASS "Munch"
 #define HACK_INIT init_munch
 #define HACK_DRAW draw_munch
@@ -77,7 +78,7 @@ ModStruct   munch_description =
 
 #endif
 
-#if DEBUG
+#ifdef DEBUG
 #include <assert.h>
 #endif
 
@@ -128,7 +129,7 @@ munchBit(ModeInfo * mi, int width,	/* pixels */
 		   while that might be a pretty feature, I didn't want it to do
 		   that yet.  if these trigger, please let me know.
 		 */
-#if DEBUG
+#ifdef DEBUG
 		assert(drawX >= 0 && drawX < MI_WIDTH(mi));
 		assert(drawY >= 0 && drawY < MI_HEIGHT(mi));
 #endif
@@ -256,10 +257,10 @@ release_munch(ModeInfo * mi)
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
 			munchstruct *mp = &munches[screen];
 
-			if (mp->gc != NULL)
+			if (mp->gc != None)
 				XFreeGC(MI_DISPLAY(mi), mp->gc);
 		}
-		(void) free((void *) munches);
+		free(munches);
 		munches = (munchstruct *) NULL;
 	}
 }
