@@ -28,17 +28,17 @@ static const char sccsid[] = "@(#)nose.c	5.00 2000/11/01 xlockmore";
  *              xscreensaver but colors are random).
  * 27-Feb-1996: Added new ModeInfo arg to init and callback hooks.  Removed
  *              references to onepause, now uses MI_PAUSE(mi) interface.
- *              Ron Hitchens <ron@idiom.com>
+ *              Ron Hitchens <ron AT idiom.com>
  * 10-Oct-1995: A better way of handling fortunes from a file, thanks to
- *              Jouk Jansen <joukj@hrem.stm.tudelft.nl>.
+ *              Jouk Jansen <joukj AT hrem.nano.tudelft.nl>.
  * 21-Sep-1995: font option added, debugged for multiscreens
  * 12-Aug-1995: xlock version
- * 1992: xscreensaver version, noseguy (Jamie Zawinski <jwz@jwz.org>)
+ * 1992: xscreensaver version, noseguy (Jamie Zawinski <jwz AT jwz.org>)
  * 1990: X11 version, xnlock (Dan Heller <argv@sun.com>)
  */
 
 /*-
- * xscreensaver, Copyright (c) 1992 Jamie Zawinski <jwz@mcom.com>
+ * xscreensaver, Copyright (c) 1992 Jamie Zawinski <jwz AT jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -50,6 +50,7 @@ static const char sccsid[] = "@(#)nose.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_nose
 #define PROGCLASS "Nose"
 #define HACK_INIT init_nose
 #define HACK_DRAW draw_nose
@@ -108,10 +109,44 @@ ModStruct   nose_description =
 #define TALK 1
 #define FREEZE 2
 
-#ifndef USE_MB
-extern XFontStruct *getFont(Display * display);
-#else
+#ifdef USE_MB
+static XFontSet mode_font = None;
+static int getFontHeight(XFontSet f)
+{
+	XRectangle ink, log;
+
+	if (f == None) {
+		return 8;
+	} else {
+		XmbTextExtents(mode_font, "My", strlen("My"), &ink, &log);
+		/*XmbTextExtents(mode_font, "M", strlen("M"), &ink, &log);*/
+		return log.height;
+	}
+}
+static int getTextWidth(char *string)
+{
+	XRectangle ink, logical;
+
+	XmbTextExtents(mode_font, string, strlen(string), &ink, &logical);
+	return logical.width;
+}
 extern XFontSet getFontSet(Display * display);
+#define DrawString(d, p, gc, start, ascent, string, len) (void) XmbDrawString(\
+	d, p, mode_font, gc, start, ascent, string, len)
+#else
+static XFontStruct *mode_font = None;
+#define getFontHeight(f) ((f == None) ? 8 : (f->ascent + f->descent == 0) ? 16 :  f->ascent + f->descent)
+static int getTextWidth(char *string)
+{
+	int width =  XTextWidth(mode_font, string, strlen(string));
+
+	if (width == 0 && strlen(string) > 0)
+		return 7 * strlen(string);
+	return width;
+}      
+extern XFontStruct *getFont(Display * display);
+#define DrawString(d, p, gc, start, ascent, string, len) (void) XDrawString(\
+	d, p, gc, start, ascent, string, len)
 #endif
 extern char *getWords(int screen, int screens);
 extern int  isRibbon(void);
@@ -139,26 +174,6 @@ typedef struct {
 
 static nosestruct *noses = (nosestruct *) NULL;
 
-#ifdef USE_MB
-static XFontSet mode_font = None;
-#else
-static XFontStruct *mode_font = None;
-#endif
-
-#ifdef USE_MB
-static int font_height(XFontSet f) {
-  XRectangle ink, log;
-  if (f == None) {
-	return 8;
-  } else {
-	XmbTextExtents(mode_font, "M", strlen("M"), &ink, &log);
-	return log.height;
-  }
-}
-#else
-#define font_height(f) ((f == None) ? 8 : f->ascent + f->descent)
-#endif
-
 #define LEFT 001
 #define RIGHT 002
 #define DOWN 004
@@ -179,11 +194,11 @@ free_nose(Display *display, nosestruct *np)
 
 	if (np->text_fg_gc != None) {
 		XFreeGC(display, np->text_fg_gc);
-	    np->text_fg_gc = None;
+		np->text_fg_gc = None;
 	}
 	if (np->text_bg_gc != None) {
 		XFreeGC(display, np->text_bg_gc);
-	    np->text_bg_gc = None;
+		np->text_bg_gc = None;
 	}
 	for (pix = 0; pix < PIXMAPS; pix++) {
 		if (np->position[pix] != None) {
@@ -233,7 +248,8 @@ pickClothes(ModeInfo * mi)
 	/* DOWN NOSE GUY */
 	if ((shoe_pix = XCreateBitmapFromData(display, window,
 			(char *) nose_shoe_front_bits,
-			nose_shoe_front_width, nose_shoe_front_height)) == None){
+			nose_shoe_front_width,
+			nose_shoe_front_height)) == None) {
 		return False;
 	}
 	COPY(display, gc, shoe_color, shoe_pix, np->position[D],
@@ -242,7 +258,8 @@ pickClothes(ModeInfo * mi)
 	     nose_shoe_front_width, nose_shoe_front_height);
 	if ((face_pix = XCreateBitmapFromData(display, window,
 			(char *) nose_face_down_bits,
-			nose_face_down_width, nose_face_down_height)) == None) {
+			nose_face_down_width,
+			nose_face_down_height)) == None) {
 		XFreePixmap(display, shoe_pix);
 		return False;
 	}
@@ -290,7 +307,8 @@ pickClothes(ModeInfo * mi)
 	}
 	if ((face_pix = XCreateBitmapFromData(display, window,
 			(char *) nose_face_front_bits,
-			nose_face_front_width, nose_face_front_height)) == None) {
+			nose_face_front_width,
+			nose_face_front_height)) == None) {
 		XFreePixmap(display, hat_pix);
 		return False;
 	}
@@ -300,7 +318,8 @@ pickClothes(ModeInfo * mi)
 	/* FRONT LEFT NOSE GUY */
 	if ((shoel_pix = XCreateBitmapFromData(display, window,
 			(char *) nose_shoe_left_bits,
-			nose_shoe_left_width, nose_shoe_left_height)) == None) {
+			nose_shoe_left_width,
+			nose_shoe_left_height)) == None) {
 		XFreePixmap(display, hat_pix);
 		XFreePixmap(display, face_pix);
 		return False;
@@ -325,7 +344,8 @@ pickClothes(ModeInfo * mi)
 	/* FRONT RIGHT NOSE GUY */
 	if ((shoer_pix = XCreateBitmapFromData(display, window,
 			(char *) nose_shoe_right_bits,
-			nose_shoe_right_width, nose_shoe_right_height)) == None) {
+			nose_shoe_right_width,
+			nose_shoe_right_height)) == None) {
 		XFreePixmap(display, hat_pix);
 		XFreePixmap(display, shoel_pix);
 		XFreePixmap(display, face_pix);
@@ -367,7 +387,8 @@ pickClothes(ModeInfo * mi)
 	}
 	if ((face_pix = XCreateBitmapFromData(display, window,
 			(char *) nose_face_left_bits,
-			nose_face_left_width, nose_face_left_height)) == None) {
+			nose_face_left_width,
+			nose_face_left_height)) == None) {
 		XFreePixmap(display, hat_pix);
 		XFreePixmap(display, shoer_pix);
 		return False;
@@ -378,7 +399,8 @@ pickClothes(ModeInfo * mi)
 	/* LEFT NOSE GUY STEPPING */
 	if ((shoel_pix = XCreateBitmapFromData(display, window,
 			(char *) nose_step_left_bits,
-			nose_step_left_width, nose_step_left_height)) == None) {
+			nose_step_left_width,
+			nose_step_left_height)) == None) {
 		XFreePixmap(display, hat_pix);
 		XFreePixmap(display, face_pix);
 		XFreePixmap(display, shoer_pix);
@@ -406,7 +428,8 @@ pickClothes(ModeInfo * mi)
 	/* RIGHT NOSE GUY */
 	if ((face_pix = XCreateBitmapFromData(display, window,
 			(char *) nose_face_right_bits,
-			nose_face_right_width, nose_face_right_height)) == None) {
+			nose_face_right_width,
+			nose_face_right_height)) == None) {
 		XFreePixmap(display, hat_pix);
 		XFreePixmap(display, shoer_pix);
 		return False;
@@ -432,7 +455,8 @@ pickClothes(ModeInfo * mi)
 	/* RIGHT NOSE GUY STEPPING */
 	if ((shoer_pix = XCreateBitmapFromData(display, window,
 			(char *) nose_step_right_bits,
-			nose_step_right_width, nose_step_right_height)) == None) {
+			nose_step_right_width,
+			nose_step_right_height)) == None) {
 		XFreePixmap(display, face_pix);
 		XFreePixmap(display, hat_pix);
 		return False;
@@ -479,9 +503,9 @@ walk(ModeInfo * mi, register int dir)
 			np->frames = (np->up < 0) ? R : RSTEP;
 		}
 		/* note that maybe neither UP nor DOWN is set! */
-		if (dir & UP && np->y > Y_INCR)
+		if ((dir & UP) && np->y > Y_INCR)
 			np->y -= Y_INCR;
-		else if (dir & DOWN && np->y < np->height - np->ys)
+		else if ((dir & DOWN) && np->y < np->height - np->ys)
 			np->y += Y_INCR;
 	} else if (dir == UP) {	/* Explicit up/down movement only (no left/right) */
 		np->y -= Y_INCR;
@@ -556,19 +580,17 @@ talk(ModeInfo * mi, Bool force_erase)
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
 	nosestruct *np = &noses[MI_SCREEN(mi)];
-	int         width = 0, height, Y, Z, total = 0;
+	int         width = 0, height = 0, Y, Z, total = 0;
 	register char *p, *p2;
 	char        buf[BUFSIZ], args[MAXLINES][MAXWIDTH];
-#ifdef USE_MB
-	XRectangle ink, log;
-#endif
 
 	/* clear what we've written */
 	if (np->talking || force_erase) {
 		if (!np->talking)
 			return;
-		XFillRectangle(display, window, np->text_bg_gc, np->s.x - 5, np->s.y - 5,
-			       np->s.width + 10, np->s.height + 10);
+		XFillRectangle(display, window, np->text_bg_gc,
+			np->s.x - 5, np->s.y - 5,
+			np->s.width + 10, np->s.height + 10);
 		np->talking = False;
 		if (!force_erase)
 			np->state = MOVE;
@@ -577,51 +599,46 @@ talk(ModeInfo * mi, Bool force_erase)
 	np->talking = True;
 	walk(mi, FRONT);
 	p = strncpy(buf, np->words, BUFSIZ);
-	if (!(p2 = (char *) strchr(p, '\n')) || !p2[1]) {
-		(void) strncpy(args[0], np->words, MAXWIDTH - 1);
-		total = strlen(args[0]);
-		args[0][MAXWIDTH - 1] = '\0';
-		if (mode_font == None)
-			width = 8;
-		else {
-#ifdef USE_MB
-			XmbTextExtents(mode_font, np->words, total, &ink, &log);
-			width = ink.width;
-#else
-			width = XTextWidth(mode_font, np->words, total);
-#endif
-		}
-		height = 0;
-	} else
-		/* p2 now points to the first '\n' */
-		for (height = 0; p && height < MAXLINES; height++) {
-			int         w;
-
+	while (strlen(p) > 0) {
+		p2 = p + strlen(p) - 1;
+		if (*p2 == '\n')
 			*p2 = 0;
-#ifdef USE_MB
-			XmbTextExtents(mode_font, p, p2 - p, &ink, &log);
-			if (mode_font != None && (w = log.width) > width)
-			  width = w;
-#else
-			if (mode_font != None &&
-					(w = XTextWidth(mode_font, p, p2 - p)) > width)
-				width = w;
-#endif
-			total += p2 - p;	/* total chars; count to determine reading time */
-			(void) strncpy(args[height], p, MAXWIDTH - 1);
-			args[height][MAXWIDTH - 1] = '\0';
-			p = p2 + 1;
-			if (!(p2 = (char *) strchr(p, '\n')))
-				break;
-		}
+		else
+			break;
+	}
+	for (height = 0; p && height < MAXLINES; height++) {
+		int w;
+		char *p3;
 
-	height++;
+		if (!(p2 = (char *) strchr(p, '\n')) || !p2[1]) {
+			p2 = p + strlen(p);
+		
+		}
+		/* p2 now points to the first '\n' */
+		*p2 = 0;
+		for (w = 0; w < p2 - p; w++) {
+			p3 = p + w;
+			if (*p3 == '\t') /* this should be improved */
+				*p3 = ' ';
+		}
+		(void) strncpy(args[height], p, MAXWIDTH - 1);
+		args[height][MAXWIDTH - 1] = '\0';
+		if (mode_font == None && height == 0)
+			width = 8;
+		if (mode_font != None && (w =
+				getTextWidth(args[height])) > width)
+			width = w;
+		total += p2 - p;  /* total chars; count to determine reading time */
+		if (p == p2)
+			break;
+		p = p2 + 1;
+	}
 	/*
 	 * Figure out the height and width in imagepixels (height, width) extend the
 	 * new box by 15 pixels on the sides (30 total) top and bottom.
 	 */
 	np->s.width = width + 30;
-	np->s.height = height * font_height(mode_font) + 30;
+	np->s.height = height * getFontHeight(mode_font) + 30;
 	if (np->x - np->s.width - 10 < 5)
 		np->s.x = 5;
 	else if ((np->s.x = np->x + 32 - (np->s.width + 15) / 2)
@@ -643,18 +660,14 @@ talk(ModeInfo * mi, Bool force_erase)
 	XDrawRectangle(display, window, np->text_fg_gc,
 	      np->s.x + 7, np->s.y + 7, np->s.width - 15, np->s.height - 15);
 
-	Y = 15 + font_height(mode_font);
+	Y = 15 + getFontHeight(mode_font);
 
 	/* now print each string in reverse order (start at bottom of box) */
 	for (Z = 0; Z < height; Z++) {
-#ifdef USE_MB
-		(void) XmbDrawString(display, window, mode_font, np->text_fg_gc,
-			 np->s.x + 15, np->s.y + Y, args[Z], strlen(args[Z]));
-#else
-		(void) XDrawString(display, window, np->text_fg_gc,
-			np->s.x + 15, np->s.y + Y, args[Z], strlen(args[Z]));
-#endif
-		Y += font_height(mode_font);
+		DrawString(display, window, np->text_fg_gc,
+			np->s.x + 15, np->s.y + Y,
+			args[Z], strlen(args[Z]));
+		Y += getFontHeight(mode_font);
 	}
 	np->busyLoop = (total / 15) * 10;
 	if (np->busyLoop < 30)
@@ -807,20 +820,18 @@ init_nose(ModeInfo * mi)
 			return;
 		}
 	np->words = getWords(MI_SCREEN(mi), MI_NUM_SCREENS(mi));
-	if (np->text_fg_gc == NULL && mode_font != None) {
+	if (np->text_fg_gc == None && mode_font != None) {
 #ifndef USE_MB
 		gcv.font = mode_font->fid;
-		XSetFont(display, gc, mode_font->fid);
 #endif
 		gcv.graphics_exposures = False;
 		gcv.foreground = MI_WHITE_PIXEL(mi);
 		gcv.background = MI_BLACK_PIXEL(mi);
 		if ((np->text_fg_gc = XCreateGC(display, window,
-#ifdef USE_MB
-				 GCForeground | GCBackground | GCGraphicsExposures,
-#else
-				 GCForeground | GCBackground | GCGraphicsExposures | GCFont,
+#ifndef USE_MB
+				 GCFont |
 #endif
+				 GCForeground | GCBackground | GCGraphicsExposures,
 				 &gcv)) == None) {
 			free_nose(display, np);
 			return;
@@ -828,11 +839,10 @@ init_nose(ModeInfo * mi)
 		gcv.foreground = MI_BLACK_PIXEL(mi);
 		gcv.background = MI_WHITE_PIXEL(mi);
 		if ((np->text_bg_gc = XCreateGC(display, window,
-#ifdef USE_MB
-				 GCForeground | GCBackground | GCGraphicsExposures,
-#else
-				 GCForeground | GCBackground | GCGraphicsExposures | GCFont,
+#ifndef USE_MB
+				 GCFont |
 #endif
+				 GCForeground | GCBackground | GCGraphicsExposures,
 				 &gcv)) == None) {
 			free_nose(display, np);
 			return;
@@ -891,7 +901,7 @@ release_nose(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_nose(MI_DISPLAY(mi), &noses[screen]);
-		(void) free((void *) noses);
+		free(noses);
 		noses = (nosestruct *) NULL;
 	}
 	if (mode_font != None) {
