@@ -2,7 +2,7 @@
 /* penrose --- quasiperiodic tilings */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)penrose.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)penrose.c	5.09 2003/07/08 xlockmore";
 
 #endif
 
@@ -22,8 +22,9 @@ static const char sccsid[] = "@(#)penrose.c	5.00 2000/11/01 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
+ * 08-Jul-2003: Mono made a little more interesting.
  * 01-Nov-2000: Allocation checks
- * 10-May-1997: Jamie Zawinski <jwz@jwz.org> compatible with xscreensaver
+ * 10-May-1997: Jamie Zawinski <jwz AT jwz.org> compatible with xscreensaver
  * 09-Sep-1996: Written.
  */
 
@@ -66,6 +67,7 @@ If one of these are hit penrose will reinitialize.
  */
 
 #ifdef STANDALONE
+#define MODE_penrose
 #define PROGCLASS "Penrose"
 #define HACK_INIT init_penrose
 #define HACK_DRAW draw_penrose
@@ -93,7 +95,7 @@ static XrmOptionDescRec opts[] =
 };
 static argtype vars[] =
 {
-	{(caddr_t *) & ammann, (char *) "ammann", (char *) "Ammann", (char *) DEF_AMMANN, t_Bool}
+	{(void *) & ammann, (char *) "ammann", (char *) "Ammann", (char *) DEF_AMMANN, t_Bool}
 };
 static OptionStruct desc[] =
 {
@@ -398,13 +400,13 @@ free_penrose(tiling_c * tp)
 	do {
 		fp2 = fp1;
 		fp1 = fp1->next;
-		(void) free((void *) fp2);
+		free(fp2);
 	} while (fp1 != tp->fringe.nodes);
 	tp->fringe.nodes = (fringe_node_c *) NULL;
 	for (lp1 = tp->forced.first; lp1 != 0;) {
 		lp2 = lp1;
 		lp1 = lp1->next;
-		(void) free((void *) lp2);
+		free(lp2);
 	}
 	tp->forced.first = 0;
 }
@@ -439,6 +441,14 @@ init_penrose(ModeInfo * mi)
 		/* Insure good contrast */
 		tp->thin_color = (NRAND(2 * MI_NPIXELS(mi) / 3) + tp->thick_color +
 				  MI_NPIXELS(mi) / 6) % MI_NPIXELS(mi);
+	} else {
+		if (LRAND() & 1) {
+			tp->thick_color = MI_WHITE_PIXEL(mi);
+			tp->thin_color = MI_BLACK_PIXEL(mi);
+		} else {
+			tp->thick_color = MI_BLACK_PIXEL(mi);
+			tp->thin_color = MI_WHITE_PIXEL(mi);
+		}
 	}
 	size = MI_SIZE(mi);
 	if (size < -MINSIZE)
@@ -539,8 +549,7 @@ match_rules(fringe_node_c * vertex, rule_match_c * matches, int first_only)
 	   congruence arithmetic. */
 	register int i, j;
 	int         hits = 0, good_rules[N_VERTEX_RULES], n_good = 0;
-	unsigned long
-	            vertex_hash = 0, lower_bits_mask = ~(VT_TOTAL_MASK << VT_BITS * (vertex->n_tiles - 1));
+	unsigned long vertex_hash = 0, lower_bits_mask = ~(VT_TOTAL_MASK << (VT_BITS * (vertex->n_tiles - 1)));
 	unsigned    new_rule_mask = 0;
 
 	for (i = 0; i < N_VERTEX_RULES; i++)
@@ -661,10 +670,20 @@ draw_tile(fringe_node_c * v1, fringe_node_c * v2,
 			XSetForeground(display, gc, MI_PIXEL(mi, tp->thick_color));
 		else
 			XSetForeground(display, gc, MI_PIXEL(mi, tp->thin_color));
-	} else
-		XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
+	} else {
+		if ((vtype & VT_TYPE_MASK) == VT_THICK)
+			XSetForeground(display, gc, tp->thick_color);
+		else
+			XSetForeground(display, gc, tp->thin_color);
+	}
 	XFillPolygon(display, window, gc, pts, 4, Convex, CoordModeOrigin);
-	XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
+	if (MI_NPIXELS(mi) <= 2) {
+		if ((vtype & VT_TYPE_MASK) == VT_THICK)
+			XSetForeground(display, gc, tp->thin_color);
+		else
+			XSetForeground(display, gc, tp->thick_color);
+	} else
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	XDrawLines(display, window, gc, pts, 5, CoordModeOrigin);
 
 	if (tp->ammann) {
@@ -682,7 +701,7 @@ draw_tile(fringe_node_c * v1, fringe_node_c * v2,
 			if (MI_NPIXELS(mi) > 2)
 				XSetForeground(display, gc, MI_PIXEL(mi, tp->thin_color));
 			else {
-				XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
+				XSetForeground(display, gc, tp->thin_color);
 				XSetLineAttributes(display, gc, 1, LineOnOffDash, CapNotLast, JoinMiter);
 			}
 			XDrawLine(display, window, gc,
@@ -696,7 +715,7 @@ draw_tile(fringe_node_c * v1, fringe_node_c * v2,
 			if (MI_NPIXELS(mi) > 2)
 				XSetForeground(display, gc, MI_PIXEL(mi, tp->thick_color));
 			else {
-				XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
+				XSetForeground(display, gc, tp->thick_color);
 				XSetLineAttributes(display, gc, 1, LineOnOffDash, CapNotLast, JoinMiter);
 			}
 			XDrawLine(display, window, gc,
@@ -750,7 +769,7 @@ check_vertex(ModeInfo * mi, fringe_node_c * vertex, tiling_c * tp)
 			*vertex->list_ptr = node->next;
 			if (node->next != 0)
 				node->next->vertex->list_ptr = vertex->list_ptr;
-			(void) free((void *) node);
+			free(node);
 			tp->forced.n_nodes--;
 			if (!vertex->off_screen)
 				tp->forced.n_visible--;
@@ -801,14 +820,14 @@ delete_vertex(ModeInfo * mi, fringe_node_c * vertex, tiling_c * tp)
 		*vertex->list_ptr = node->next;
 		if (node->next != 0)
 			node->next->vertex->list_ptr = vertex->list_ptr;
-		(void) free((void *) node);
+		free(node);
 		tp->forced.n_nodes--;
 		if (!vertex->off_screen)
 			tp->forced.n_visible--;
 	}
 	if (!vertex->off_screen)
 		tp->fringe.n_nodes--;
-	(void) free((void *) vertex);
+	free(vertex);
 }
 
 
@@ -850,14 +869,14 @@ fills_vertex(ModeInfo * mi, vertex_type_c vtype, fringe_node_c * vertex)
 static unsigned
 fringe_changes(ModeInfo * mi, fringe_node_c * vertex,
 	       unsigned side, vertex_type_c vtype,
-	       fringe_node_c ** right, fringe_node_c ** far,
+	       fringe_node_c ** right, fringe_node_c ** faraway,
 	       fringe_node_c ** left)
 {
 	fringe_node_c *v, *f = (fringe_node_c *) NULL;
 	unsigned    result = FC_NEW_FAR;	/* We clear this later if necessary. */
 
-	if (far)
-		*far = 0;
+	if (faraway)
+		*faraway = 0;
 	if (fills_vertex(mi, vtype, vertex)) {
 		result |= FC_CUT_THIS;
 	} else if (side == S_LEFT) {
@@ -877,8 +896,8 @@ fringe_changes(ModeInfo * mi, fringe_node_c * vertex,
 		if (fills_vertex(mi, VT_LEFT(vtype), v)) {
 			result = (result & ~FC_NEW_FAR) | FC_CUT_LEFT;
 			f = v->next;
-			if (far)
-				*far = f;
+			if (faraway)
+				*faraway = f;
 		}
 	}
 	if (!(result & FC_NEW_RIGHT)) {
@@ -888,8 +907,8 @@ fringe_changes(ModeInfo * mi, fringe_node_c * vertex,
 		if (fills_vertex(mi, VT_RIGHT(vtype), v)) {
 			result = (result & ~FC_NEW_FAR) | FC_CUT_RIGHT;
 			f = v->prev;
-			if (far)
-				*far = f;
+			if (faraway)
+				*faraway = f;
 		}
 	}
 	if (!(result & FC_NEW_FAR)
@@ -975,9 +994,9 @@ add_tile(ModeInfo * mi,
 	fringe_node_c
 		*left = (fringe_node_c *) NULL,
 		*right = (fringe_node_c *) NULL,
-		*far = (fringe_node_c *) NULL,
+		*faraway = (fringe_node_c *) NULL,
 		*node;
-	unsigned    fc = fringe_changes(mi, vertex, side, vtype, &right, &far, &left);
+	unsigned    fc = fringe_changes(mi, vertex, side, vtype, &right, &faraway, &left);
 
 	vertex_type_c
 		ltype = VT_LEFT(vtype),
@@ -1000,8 +1019,8 @@ add_tile(ModeInfo * mi,
 			if ((right = alloc_vertex(mi, vertex_dir(mi, vertex, S_LEFT) -
 					vtype_angle(vtype), vertex, tp)) == NULL)
 				return False;
-		if (far == NULL)
-			if ((far = alloc_vertex(mi, vertex_dir(mi, left, S_RIGHT) +
+		if (faraway == NULL)
+			if ((faraway = alloc_vertex(mi, vertex_dir(mi, left, S_RIGHT) +
 					vtype_angle(ltype), left, tp)) == NULL)
 				return False;
 	} else {
@@ -1009,8 +1028,8 @@ add_tile(ModeInfo * mi,
 			if ((left = alloc_vertex(mi, vertex_dir(mi, vertex, S_RIGHT) +
 					vtype_angle(vtype), vertex, tp)) == NULL)
 				return False;
-		if (far == NULL)
-			if ((far = alloc_vertex(mi, vertex_dir(mi, right, S_LEFT) -
+		if (faraway == NULL)
+			if ((faraway = alloc_vertex(mi, vertex_dir(mi, right, S_LEFT) -
 					vtype_angle(rtype), right, tp)) == NULL)
 				return False;
 	}
@@ -1022,14 +1041,14 @@ add_tile(ModeInfo * mi,
 	do {
 		if (((fc & FC_NEW_LEFT) && fived_equal(node->fived, left->fived))
 		    || ((fc & FC_NEW_RIGHT) && fived_equal(node->fived, right->fived))
-		    || ((fc & FC_NEW_FAR) && fived_equal(node->fived, far->fived))) {
+		    || ((fc & FC_NEW_FAR) && fived_equal(node->fived, faraway->fived))) {
 			/* Better luck next time. */
 			if (fc & FC_NEW_LEFT)
 				delete_vertex(mi, left, tp);
 			if (fc & FC_NEW_RIGHT)
 				delete_vertex(mi, right, tp);
 			if (fc & FC_NEW_FAR)
-				delete_vertex(mi, far, tp);
+				delete_vertex(mi, faraway, tp);
 			return False;
 		}
 		node = node->next;
@@ -1047,19 +1066,19 @@ add_tile(ModeInfo * mi,
 	}
 	if (!(fc & FC_CUT_FAR)) {
 		if (!(fc & FC_CUT_LEFT)) {
-			far->next = left;
-			left->prev = far;
+			faraway->next = left;
+			left->prev = faraway;
 		}
 		if (!(fc & FC_CUT_RIGHT)) {
-			far->prev = right;
-			right->next = far;
+			faraway->prev = right;
+			right->next = faraway;
 		}
 	}
-	draw_tile(vertex, right, far, left, vtype, mi);
+	draw_tile(vertex, right, faraway, left, vtype, mi);
 
 	/* Delete vertices that are no longer on the fringe.  Check the others. */
 	if (fc & FC_CUT_THIS) {
-		tp->fringe.nodes = far;
+		tp->fringe.nodes = faraway;
 		delete_vertex(mi, vertex, tp);
 	} else {
 		add_vtype(vertex, side, vtype);
@@ -1067,10 +1086,10 @@ add_tile(ModeInfo * mi,
 		tp->fringe.nodes = vertex;
 	}
 	if (fc & FC_CUT_FAR)
-		delete_vertex(mi, far, tp);
+		delete_vertex(mi, faraway, tp);
 	else {
-		add_vtype(far, fc & FC_CUT_RIGHT ? S_LEFT : S_RIGHT, ftype);
-		check_vertex(mi, far, tp);
+		add_vtype(faraway, fc & FC_CUT_RIGHT ? S_LEFT : S_RIGHT, ftype);
+		check_vertex(mi, faraway, tp);
 	}
 	if (fc & FC_CUT_LEFT)
 		delete_vertex(mi, left, tp);
@@ -1155,7 +1174,7 @@ legal_move(fringe_node_c * vertex, unsigned side, vertex_type_c vtype)
 static void
 add_random_tile(fringe_node_c * vertex, ModeInfo * mi)
 {
-	fringe_node_c *right, *left, *far;
+	fringe_node_c *right, *left, *faraway;
 	int         i, j, n, n_hits, n_good;
 	unsigned    side, fc, no_good, s;
 	vertex_type_c vtypes[MAX_COMPL];
@@ -1167,8 +1186,12 @@ add_random_tile(fringe_node_c * vertex, ModeInfo * mi)
 		/* Insure good contrast */
 		tp->thin_color = (NRAND(2 * MI_NPIXELS(mi) / 3) + tp->thick_color +
 				  MI_NPIXELS(mi) / 6) % MI_NPIXELS(mi);
-	} else
-		tp->thick_color = tp->thin_color = MI_WHITE_PIXEL(mi);
+	} else {
+		unsigned long temp = tp->thick_color;
+
+		tp->thick_color = tp->thin_color;
+		tp->thin_color = temp;
+	}
 	n_hits = match_rules(vertex, hits, False);
 	side = NRAND(2) ? S_LEFT : S_RIGHT;
 	n = find_completions(vertex, hits, n_hits, side, vtypes /*, False */ );
@@ -1183,7 +1206,7 @@ add_random_tile(fringe_node_c * vertex, ModeInfo * mi)
 	no_good = 0;
 	n_good = n;
 	for (i = 0; i < n; i++) {
-		fc = fringe_changes(mi, vertex, side, vtypes[i], &right, &far, &left);
+		fc = fringe_changes(mi, vertex, side, vtypes[i], &right, &faraway, &left);
 		if (fc & FC_BAG) {
 			tp->done = True;
 			if (MI_IS_VERBOSE(mi)) {
@@ -1207,9 +1230,9 @@ add_random_tile(fringe_node_c * vertex, ModeInfo * mi)
 				continue;
 			}
 		}
-		if (far) {
+		if (faraway) {
 			s = ((fc & FC_CUT_LEFT) ? S_RIGHT : S_LEFT);
-			if (!legal_move(far, s, VT_FAR(vtypes[i]))) {
+			if (!legal_move(faraway, s, VT_FAR(vtypes[i]))) {
 				no_good |= (1 << i);
 				n_good--;
 			}
@@ -1319,7 +1342,7 @@ release_penrose(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_penrose(&tilings[screen]);
-		(void) free((void *) tilings);
+		free(tilings);
 		tilings = (tiling_c *) NULL;
 	}
 }
