@@ -23,7 +23,7 @@ static const char sccsid[] = "@(#)polyominoes.c 5.01 2000/12/18 xlockmore";
  *
  * Revision History:
  * 07-Jan-2001: Made improvement to the search algorithm for the puzzles
- *              involving identical polyominoes (via the variable 
+ *              involving identical polyominoes (via the variable
  *              reason_to_not_attach).  By Stephen Montgomery-Smith.
  * 20-Dec-2000: Added more puzzles at David Bagley's request.
  * 27-Nov-2000: Adapted from euler2d.c Copyright (c) 2000 by Stephen
@@ -34,6 +34,7 @@ static const char sccsid[] = "@(#)polyominoes.c 5.01 2000/12/18 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_polyominoes
 #define PROGCLASS "Polyominoes"
 #define HACK_INIT init_polyominoes
 #define HACK_DRAW draw_polyominoes
@@ -64,8 +65,8 @@ static XrmOptionDescRec opts[] =
 };
 static argtype vars[] =
 {
-  {(caddr_t *) &identical, (char *) "identical", (char *) "Identical", (char *) DEF_IDENTICAL, t_Bool},
-  {(caddr_t *) & plain, (char *) "plain", (char *) "Plain", (char *) DEF_PLAIN, t_Bool}
+  {(void *) &identical, (char *) "identical", (char *) "Identical", (char *) DEF_IDENTICAL, t_Bool},
+  {(void *) & plain, (char *) "plain", (char *) "Plain", (char *) DEF_PLAIN, t_Bool}
 };
 static OptionStruct desc[] =
 {
@@ -220,7 +221,7 @@ typedef struct _polyominoesstruct{
 #define G ((sp->box/45)+1)         /* 1/2 of gap between polyominoes. */
 #define T ((sp->box<=12)?1:(G*2))  /* Thickness of walls of polyominoes. */
 #define R ((sp->box<=12)?1:(G*6))  /* Amount of rounding. */
-#define RT ((sp->box<=12)?1:(G*3)) /* Thickness of wall of rounded parts. 
+#define RT ((sp->box<=12)?1:(G*3)) /* Thickness of wall of rounded parts.
                                       Here 3 is an approximation to 2 sqrt(2). */
 #define RR 0   /* Roof ridge thickness */
 
@@ -293,29 +294,29 @@ static void print_board(polyominoesstruct *sp) {
   for (y=0;y<sp->height;y++) {
     for (x=0;x<sp->width;x++)
       if (ARRAY(x,y) == -1)
-        fprintf(stderr," ");
+        (void) fprintf(stderr," ");
       else
-        fprintf(stderr,"%c",'a'+ARRAY(x,y));
-    fprintf(stderr,"\n");
+        (void) fprintf(stderr,"%c",'a'+ARRAY(x,y));
+    (void) fprintf(stderr,"\n");
   }
-  fprintf(stderr,"\n");
+  (void) fprintf(stderr,"\n");
 }
 
 static void print_points(point_type *point, int len) {
   int i;
 
   for (i=0;i<len;i++)
-    fprintf(stderr,"(%d %d) ",point[i].x,point[i].y);
+    (void) fprintf(stderr,"(%d %d) ",point[i].x,point[i].y);
 }
 
 static void print_polyomino(polyomino_type poly) {
   int i;
 
   print_points(poly.point,poly.len);
-  fprintf(stderr,"\n");
+  (void) fprintf(stderr,"\n");
   for (i=0;i<poly.transform_len;i++)
-    fprintf(stderr,"%d ",poly.transform_list[i]);
-  fprintf(stderr,"\n");
+    (void) fprintf(stderr,"%d ",poly.transform_list[i]);
+  (void) fprintf(stderr,"\n");
 }
 */
 
@@ -344,7 +345,7 @@ These are the routines for manipulating the polyominoes and
 attaching and detaching them from the rectangle.
 ************************************************************/
 
-static void transform(point_type in, point_type offset, int transform_no, 
+static void transform(point_type in, point_type offset, int transform_no,
                       point_type attach_point, point_type *out) {
   switch (transform_no) {
     case 0: out->x=in.x-offset.x+attach_point.x;
@@ -394,7 +395,7 @@ static void next_poly_no(polyominoesstruct *sp, int *poly_no) {
   }
 }
 
-/* check_all_regions_multiple_of looks for connected regions of 
+/* check_all_regions_multiple_of looks for connected regions of
    blank spaces, and returns 0 if it finds a connected region containing
    a number of blanks that is not a multiple of n.
 */
@@ -420,9 +421,10 @@ static int check_all_regions_multiple_of(polyominoesstruct *sp, int n) {
     good = count%n == 0;
   }
 
-  for (x=0;x<sp->width;x++) for (y=0;y<sp->height;y++)
-    if (ARRAY(x,y) == -2)
-      ARRAY(x,y) = -1;
+  for (x=0;x<sp->width;x++)
+    for (y=0;y<sp->height;y++)
+      if (ARRAY(x,y) == -2)
+        ARRAY(x,y) = -1;
 
   return good;
 }
@@ -438,9 +440,10 @@ static int check_all_regions_positive_combination_of(polyominoesstruct *sp, int
       good = count%n == 0;
   }
 
-  for (x=0;x<sp->width;x++) for (y=0;y<sp->height;y++)
-    if (ARRAY(x,y) == -2)
-      ARRAY(x,y) = -1;
+  for (x=0;x<sp->width;x++)
+    for (y=0;y<sp->height;y++)
+      if (ARRAY(x,y) == -2)
+        ARRAY(x,y) = -1;
 
   return good;
 }
@@ -498,24 +501,24 @@ score_point(polyominoesstruct *sp, int x, int y, int min_score_so_far) {
   int score = 0;
 
   if (x>=1 && x<sp->width-1 && y>=1 && y<sp->height-1 &&
-      ARRAY(x-1,y-1)<0 && ARRAY(x-1,y)<0 && ARRAY(x-1,y+1)<0 && 
-      ARRAY(x+1,y-1)<0 && ARRAY(x+1,y)<0 && ARRAY(x+1,y+1)<0 && 
+      ARRAY(x-1,y-1)<0 && ARRAY(x-1,y)<0 && ARRAY(x-1,y+1)<0 &&
+      ARRAY(x+1,y-1)<0 && ARRAY(x+1,y)<0 && ARRAY(x+1,y+1)<0 &&
       ARRAY(x,y-1)<0 && ARRAY(x,y+1)<0)
     return 10000;
 
   attach_point.x = x;
   attach_point.y = y;
   for (poly_no=first_poly_no(sp);poly_no<sp->nr_polyominoes;next_poly_no(sp,&poly_no))
-  if (!sp->polyomino[poly_no].attached) {
+   if (!sp->polyomino[poly_no].attached) {
     for (point_no=0;point_no<sp->polyomino[poly_no].len;point_no++)
-    for (transform_index=0;transform_index<sp->polyomino[poly_no].transform_len;transform_index++) {
+     for (transform_index=0;transform_index<sp->polyomino[poly_no].transform_len;transform_index++) {
       attachable = 1;
       for (i=0;i<sp->polyomino[poly_no].len;i++) {
         transform(sp->polyomino[poly_no].point[i],
                   sp->polyomino[poly_no].point[point_no],
-                  sp->polyomino[poly_no].transform_list[transform_index], 
+                  sp->polyomino[poly_no].transform_list[transform_index],
                   attach_point, &target_point);
-        if ( ! ((target_point.x>=0) && (target_point.x<sp->width) 
+        if ( ! ((target_point.x>=0) && (target_point.x<sp->width)
                   && (target_point.y>=0) && (target_point.y<sp->height)
                   && (ARRAY_P(target_point)<0))) {
           attachable = 0;
@@ -527,8 +530,8 @@ score_point(polyominoesstruct *sp, int x, int y, int min_score_so_far) {
         if (score>=min_score_so_far)
           return score;
       }
-    }
-  }
+     }
+   }
 
   return score;
 }
@@ -556,8 +559,9 @@ static void find_blank(polyominoesstruct *sp, point_type *point) {
     }
   }
 
-  for (x=0;x<sp->width;x++) for (y=0;y<sp->height;y++) 
-    if (ARRAY(x,y)<0) ARRAY(x,y) = -1;
+  for (x=0;x<sp->width;x++)
+    for (y=0;y<sp->height;y++)
+      if (ARRAY(x,y)<0) ARRAY(x,y) = -1;
 }
 
 /* Detaches the most recently attached polyomino. */
@@ -576,7 +580,7 @@ void detach(polyominoesstruct *sp, int *poly_no, int *point_no, int *transform_i
   for (i=0;i<sp->polyomino[*poly_no].len;i++) {
     transform(sp->polyomino[*poly_no].point[i],
               sp->polyomino[*poly_no].point[*point_no],
-              sp->polyomino[*poly_no].transform_list[*transform_index]^(rot180<<1), 
+              sp->polyomino[*poly_no].transform_list[*transform_index]^(rot180<<1),
               *attach_point, &target_point);
     ARRAY_P(target_point) = -1;
     CHANGED_ARRAY_P(target_point) = 1;
@@ -585,7 +589,7 @@ void detach(polyominoesstruct *sp, int *poly_no, int *point_no, int *transform_i
   sp->polyomino[*poly_no].attached = 0;
 }
 
-/* Attempts to attach a polyomino at point (x,y) at the 
+/* Attempts to attach a polyomino at point (x,y) at the
    point_no-th point of that polyomino, using the transform
    transform_no.  Returns 1 if successful.
 */
@@ -608,15 +612,15 @@ int attach(polyominoesstruct *sp, int poly_no, int point_no, int transform_index
   for (i=0;i<sp->polyomino[poly_no].len;i++) {
     transform(sp->polyomino[poly_no].point[i],
               sp->polyomino[poly_no].point[point_no],
-              sp->polyomino[poly_no].transform_list[transform_index]^(rot180<<1), 
+              sp->polyomino[poly_no].transform_list[transform_index]^(rot180<<1),
               attach_point, &target_point);
-    if ( ! ((target_point.x>=0) && (target_point.x<sp->width) 
+    if ( ! ((target_point.x>=0) && (target_point.x<sp->width)
               && (target_point.y>=0) && (target_point.y<sp->height)
               && (ARRAY_P(target_point) == -1))) {
       if (sp->identical) {
         attachable = 0;
-        if ((target_point.x>=0) && (target_point.x<sp->width) 
-           && (target_point.y>=0) && (target_point.y<sp->height) 
+        if ((target_point.x>=0) && (target_point.x<sp->width)
+           && (target_point.y>=0) && (target_point.y<sp->height)
            && (ARRAY_P(target_point) >= 0)
            && (ARRAY_P(target_point)<worst_reason_not_to_attach))
           worst_reason_not_to_attach = ARRAY_P(target_point);
@@ -692,14 +696,15 @@ draw_without_bitmaps(ModeInfo * mi, polyominoesstruct *sp) {
 
   for (poly_no=-1;poly_no<sp->nr_polyominoes;poly_no++) {
     nr_rectangles = 0;
-    for (x=0;x<sp->width;x++) for (y=0;y<sp->height;y++)
-    if (CHANGED_ARRAY(x,y) && ARRAY(x,y) == poly_no) {
-      sp->rectangles[nr_rectangles].x = sp->x_margin + sp->box*x;
-      sp->rectangles[nr_rectangles].y = sp->y_margin + sp->box*y;
-      sp->rectangles[nr_rectangles].width = sp->box;
-      sp->rectangles[nr_rectangles].height = sp->box;
-      nr_rectangles++;
-    }
+    for (x=0;x<sp->width;x++)
+      for (y=0;y<sp->height;y++)
+        if (CHANGED_ARRAY(x,y) && ARRAY(x,y) == poly_no) {
+          sp->rectangles[nr_rectangles].x = sp->x_margin + sp->box*x;
+          sp->rectangles[nr_rectangles].y = sp->y_margin + sp->box*y;
+          sp->rectangles[nr_rectangles].width = sp->box;
+          sp->rectangles[nr_rectangles].height = sp->box;
+          nr_rectangles++;
+        }
     if (poly_no == -1)
       XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
     else
@@ -734,10 +739,8 @@ draw_without_bitmaps(ModeInfo * mi, polyominoesstruct *sp) {
     }
   }
   XDrawSegments(display, window, gc, sp->lines, nr_lines);
-
   XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
   XDrawRectangle(display, window, gc, sp->x_margin, sp->y_margin, sp->box*sp->width, sp->box*sp->height);
-
   XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 
   nr_lines = 0;
@@ -817,30 +820,38 @@ static void create_bitmaps(ModeInfo * mi, polyominoesstruct *sp) {
           THIRDBIT(n,x,y)
       }
 
-      if (IS_LEFT(n)) 
-        for (y=0;y<sp->box;y++) for (x=G;x<G+T;x++)
-          SETBIT(n,x,y)
-      if (IS_RIGHT(n)) 
-        for (y=0;y<sp->box;y++) for (x=G;x<G+T;x++)
-          SETBIT(n,sp->box-1-x,y)
+      if (IS_LEFT(n))
+        for (y=0;y<sp->box;y++)
+          for (x=G;x<G+T;x++)
+            SETBIT(n,x,y)
+      if (IS_RIGHT(n))
+        for (y=0;y<sp->box;y++)
+          for (x=G;x<G+T;x++)
+            SETBIT(n,sp->box-1-x,y)
       if (IS_UP(n))
-        for (x=0;x<sp->box;x++) for (y=G;y<G+T;y++)
-          SETBIT(n,x,y)
+        for (x=0;x<sp->box;x++)
+          for (y=G;y<G+T;y++)
+            SETBIT(n,x,y)
       if (IS_DOWN(n))
-        for (x=0;x<sp->box;x++) for (y=G;y<G+T;y++)
-          SETBIT(n,x,sp->box-1-y)
-      if (IS_LEFT(n)) 
-        for (y=0;y<sp->box;y++) for (x=0;x<G;x++)
-          RESBIT(n,x,y)
-      if (IS_RIGHT(n)) 
-        for (y=0;y<sp->box;y++) for (x=0;x<G;x++)
-          RESBIT(n,sp->box-1-x,y)
+        for (x=0;x<sp->box;x++)
+          for (y=G;y<G+T;y++)
+            SETBIT(n,x,sp->box-1-y)
+      if (IS_LEFT(n))
+        for (y=0;y<sp->box;y++)
+          for (x=0;x<G;x++)
+            RESBIT(n,x,y)
+      if (IS_RIGHT(n))
+        for (y=0;y<sp->box;y++)
+          for (x=0;x<G;x++)
+            RESBIT(n,sp->box-1-x,y)
       if (IS_UP(n))
-        for (x=0;x<sp->box;x++) for (y=0;y<G;y++)
-          RESBIT(n,x,y)
+        for (x=0;x<sp->box;x++)
+          for (y=0;y<G;y++)
+            RESBIT(n,x,y)
       if (IS_DOWN(n))
-        for (x=0;x<sp->box;x++) for (y=0;y<G;y++)
-          RESBIT(n,x,sp->box-1-y)
+        for (x=0;x<sp->box;x++)
+          for (y=0;y<G;y++)
+            RESBIT(n,x,sp->box-1-y)
 
       if (IS_LEFT(n) && IS_UP(n))
         for (x=G;x<=G+R;x++)
@@ -876,69 +887,89 @@ static void create_bitmaps(ModeInfo * mi, polyominoesstruct *sp) {
           }
 
       if (!IS_LEFT(n) && !IS_UP(n) && IS_LEFT_UP(n)) {
-        for (x=0;x<G;x++) for(y=0;y<G;y++)
-          RESBIT(n,x,y)
-        for (x=G;x<G+T;x++) for(y=0;y<G;y++)
-          SETBIT(n,x,y)
-        for (x=0;x<G+T;x++) for(y=G;y<G+T;y++)
-          SETBIT(n,x,y)
+        for (x=0;x<G;x++)
+          for(y=0;y<G;y++)
+            RESBIT(n,x,y)
+        for (x=G;x<G+T;x++)
+          for(y=0;y<G;y++)
+            SETBIT(n,x,y)
+        for (x=0;x<G+T;x++)
+          for(y=G;y<G+T;y++)
+            SETBIT(n,x,y)
       }
       if (!IS_LEFT(n) && !IS_DOWN(n) && IS_LEFT_DOWN(n)) {
-        for (x=0;x<G;x++) for(y=0;y<G;y++)
-          RESBIT(n,x,sp->box-1-y)
-        for (x=G;x<G+T;x++) for(y=0;y<G;y++)
-          SETBIT(n,x,sp->box-1-y)
-        for (x=0;x<G+T;x++) for(y=G;y<G+T;y++)
-          SETBIT(n,x,sp->box-1-y)
+        for (x=0;x<G;x++)
+          for(y=0;y<G;y++)
+            RESBIT(n,x,sp->box-1-y)
+        for (x=G;x<G+T;x++)
+          for(y=0;y<G;y++)
+            SETBIT(n,x,sp->box-1-y)
+        for (x=0;x<G+T;x++)
+          for(y=G;y<G+T;y++)
+            SETBIT(n,x,sp->box-1-y)
       }
       if (!IS_RIGHT(n) && !IS_UP(n) && IS_RIGHT_UP(n)) {
-        for (x=0;x<G;x++) for(y=0;y<G;y++)
-          RESBIT(n,sp->box-1-x,y)
-        for (x=G;x<G+T;x++) for(y=0;y<G;y++)
-          SETBIT(n,sp->box-1-x,y)
-        for (x=0;x<G+T;x++) for(y=G;y<G+T;y++)
-          SETBIT(n,sp->box-1-x,y)
+        for (x=0;x<G;x++)
+          for(y=0;y<G;y++)
+            RESBIT(n,sp->box-1-x,y)
+        for (x=G;x<G+T;x++)
+          for(y=0;y<G;y++)
+            SETBIT(n,sp->box-1-x,y)
+        for (x=0;x<G+T;x++)
+          for(y=G;y<G+T;y++)
+            SETBIT(n,sp->box-1-x,y)
       }
       if (!IS_RIGHT(n) && !IS_DOWN(n) && IS_RIGHT_DOWN(n)) {
-        for (x=0;x<G;x++) for(y=0;y<G;y++)
-          RESBIT(n,sp->box-1-x,sp->box-1-y)
-        for (x=G;x<G+T;x++) for(y=0;y<G;y++)
-          SETBIT(n,sp->box-1-x,sp->box-1-y)
-        for (x=0;x<G+T;x++) for(y=G;y<G+T;y++)
-          SETBIT(n,sp->box-1-x,sp->box-1-y)
+        for (x=0;x<G;x++)
+          for(y=0;y<G;y++)
+            RESBIT(n,sp->box-1-x,sp->box-1-y)
+        for (x=G;x<G+T;x++) 
+          for(y=0;y<G;y++)
+            SETBIT(n,sp->box-1-x,sp->box-1-y)
+        for (x=0;x<G+T;x++)
+          for(y=G;y<G+T;y++)
+            SETBIT(n,sp->box-1-x,sp->box-1-y)
       }
 
 #ifdef LARGE_BELLYBUTTON
       if (!sp->use3D) {
         if (!IS_LEFT(n) && !IS_UP(n) && !IS_LEFT_UP(n))
-          for (x=0;x<G+T;x++) for(y=0;y<G+T;y++)
-            SETBIT(n,x,y)
+          for (x=0;x<G+T;x++)
+            for(y=0;y<G+T;y++)
+              SETBIT(n,x,y)
         if (!IS_LEFT(n) && !IS_DOWN(n) && !IS_LEFT_DOWN(n))
-          for (x=0;x<G+T;x++) for(y=sp->box-G-T;y<sp->box;y++)
-            SETBIT(n,x,y)
+          for (x=0;x<G+T;x++)
+            for(y=sp->box-G-T;y<sp->box;y++)
+              SETBIT(n,x,y)
         if (!IS_RIGHT(n) && !IS_UP(n) && !IS_RIGHT_UP(n))
-          for (x=sp->box-G-T;x<sp->box;x++) for(y=0;y<G+T;y++)
-            SETBIT(n,x,y)
+          for (x=sp->box-G-T;x<sp->box;x++)
+            for(y=0;y<G+T;y++)
+              SETBIT(n,x,y)
         if (!IS_RIGHT(n) && !IS_DOWN(n) && !IS_RIGHT_DOWN(n))
-          for (x=sp->box-G-T;x<sp->box;x++) for(y=sp->box-G-T;y<sp->box;y++)
-            SETBIT(n,x,y)
+          for (x=sp->box-G-T;x<sp->box;x++)
+            for(y=sp->box-G-T;y<sp->box;y++)
+              SETBIT(n,x,y)
       } else
 #else
       if (sp->use3D)
 #endif
       {
         if (!IS_LEFT(n) && !IS_UP(n) && !IS_LEFT_UP(n))
-          for (x=0;x<sp->box/2-RR;x++) for(y=0;y<sp->box/2-RR;y++)
-            THREEQUARTERSBIT(n,x,y)
+          for (x=0;x<sp->box/2-RR;x++)
+            for(y=0;y<sp->box/2-RR;y++)
+              THREEQUARTERSBIT(n,x,y)
         if (!IS_LEFT(n) && !IS_DOWN(n) && !IS_LEFT_DOWN(n))
-          for (x=0;x<sp->box/2-RR;x++) for(y=sp->box/2+RR;y<sp->box;y++)
-            THREEQUARTERSBIT(n,x,y)
+          for (x=0;x<sp->box/2-RR;x++)
+            for(y=sp->box/2+RR;y<sp->box;y++)
+              THREEQUARTERSBIT(n,x,y)
         if (!IS_RIGHT(n) && !IS_UP(n) && !IS_RIGHT_UP(n))
-          for (x=sp->box/2+RR;x<sp->box;x++) for(y=0;y<sp->box/2-RR;y++)
-            THREEQUARTERSBIT(n,x,y)
+          for (x=sp->box/2+RR;x<sp->box;x++)
+            for(y=0;y<sp->box/2-RR;y++)
+              THREEQUARTERSBIT(n,x,y)
         if (!IS_RIGHT(n) && !IS_DOWN(n) && !IS_RIGHT_DOWN(n))
-          for (x=sp->box/2+RR;x<sp->box;x++) for(y=sp->box/2+RR;y<sp->box;y++)
-            THREEQUARTERSBIT(n,x,y)
+          for (x=sp->box/2+RR;x<sp->box;x++)
+            for(y=sp->box/2+RR;y<sp->box;y++)
+              THREEQUARTERSBIT(n,x,y)
       }
 
       sp->bitmaps[n] = XCreateImage(MI_DISPLAY(mi), MI_VISUAL(mi), 1, XYBitmap,
@@ -1008,7 +1039,7 @@ Routines to initialise and close down polyominoes.
 
 static void free_bitmaps(polyominoesstruct *sp) {
   int n;
-  
+
   for (n=0;n<256;n++)
 /* Don't bother to free duplicates */
     if (IS_LEFT_UP(n) && (IS_LEFT(n) || IS_UP(n)))
@@ -1047,12 +1078,12 @@ static void free_polyominoes(polyominoesstruct *sp) {
 }
 
 #define set_allocate(p,type,size) p = (type *) malloc(size);		\
-                             if ((p)==NULL) {free_polyominoes(sp);return 0;}
+  if ((p)==NULL) {free_polyominoes(sp);return 0;}
 
 #define copy_polyomino(dst,src,new_rand)				\
   (dst).len=(src).len;							\
   (dst).max_white = (src).max_white;					\
-  set_allocate((dst).point,point_type,sizeof(point_type)*(src).len);		\
+  set_allocate((dst).point,point_type,sizeof(point_type)*(src).len);	\
   (dst).len = (src).len;						\
   if (new_rand)								\
     random_permutation((src).len,perm_point);				\
@@ -1114,31 +1145,31 @@ static struct {int len; point_type point[4];
                int transform_len, transform_list[8], max_white;} tetromino[5] =
 {
 /*
-xxxx 
+xxxx
 */
   {4, {{0,0}, {1,0}, {2,0}, {3,0}},
    2, {0, 1, -1, -1, -1, -1, -1, -1}, 2},
 /*
-xxx  
-  x  
+xxx
+  x
 */
   {4, {{0,0}, {1,0}, {2,0}, {2,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 2},
 /*
-xxx  
- x   
+xxx
+ x
 */
   {4, {{0,0}, {1,0}, {1,1}, {2,0}},
    4, {0, 1, 2, 3, -1, -1, -1, -1}, 3},
 /*
-xx   
- xx  
+xx
+ xx
 */
   {4, {{0,0}, {1,0}, {1,1}, {2,1}},
    4, {0, 1, 4, 5, -1, -1, -1, -1}, 2},
 /*
-xx   
-xx   
+xx
+xx
 */
   {4, {{0,0}, {0,1}, {1,0}, {1,1}},
    1, {0, -1, -1, -1, -1, -1, -1, -1}, 2}};
@@ -1149,79 +1180,79 @@ static struct pentomino_struct {int len; point_type point[5];
   pentomino[12] =
 {
 /*
-xxxxx 
+xxxxx
 */
   {5, {{0,0}, {1,0}, {2,0}, {3,0}, {4,0}},
    2, {0, 1, -1, -1, -1, -1, -1, -1}, 3},
 /*
-xxxx  
-   x  
+xxxx
+   x
 */
   {5, {{0,0}, {1,0}, {2,0}, {3,0}, {3,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
-xxxx  
-  x   
+xxxx
+  x
 */
   {5, {{0,0}, {1,0}, {2,0}, {2,1}, {3,0}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
-  x   
-xxx   
-  x   
+  x
+xxx
+  x
 */
   {5, {{0,0}, {1,0}, {2,-1}, {2,0}, {2,1}},
    4, {0, 1, 2, 3, -1, -1, -1, -1}, 3},
 /*
-xxx   
-  xx  
+xxx
+  xx
 */
   {5, {{0,0}, {1,0}, {2,0}, {2,1}, {3,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
-xxx   
- xx   
+xxx
+ xx
 */
   {5, {{0,0}, {1,0}, {1,1}, {2,0}, {2,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
-xxx   
-  x   
-  x   
+xxx
+  x
+  x
 */
   {5, {{0,0}, {1,0}, {2,0}, {2,1}, {2,2}},
    4, {0, 1, 2, 3, -1, -1, -1, -1}, 3},
 /*
- x    
-xxx   
-  x   
+ x
+xxx
+  x
 */
   {5, {{0,0}, {1,-1}, {1,0}, {2,0}, {2,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
-xxx   
-x x   
+xxx
+x x
 */
   {5, {{0,0}, {0,1}, {1,0}, {2,0}, {2,1}},
    4, {0, 1, 2, 3, -1, -1, -1, -1}, 3},
 /*
-  x   
-xxx   
-x     
+  x
+xxx
+x
 */
   {5, {{0,0}, {0,1}, {1,0}, {2,-1}, {2,0}},
    4, {0, 1, 4, 5, -1, -1, -1, -1}, 3},
 /*
- x    
-xxx   
- x    
+ x
+xxx
+ x
 */
   {5, {{0,0}, {1,-1}, {1,0}, {1,1}, {2,0}},
    1, {0, -1, -1, -1, -1, -1, -1, -1}, 4},
 /*
-xx    
- xx   
-  x   
+xx
+ xx
+  x
 */
   {5, {{0,0}, {1,0}, {1,1}, {2,1}, {2,2}},
    4, {0, 1, 2, 3, -1, -1, -1, -1}, 3}};
@@ -1232,244 +1263,245 @@ static struct hexomino_struct {int len; point_type point[6];
   hexomino[35] =
 {
 /*
-xxxxxx 
+xxxxxx
 */
   {6, {{0,0}, {1,0}, {2,0}, {3,0}, {4,0}, {5,0}},
    2, {0, 1, -1, -1, -1, -1, -1, -1}, 3},
 /*
-xxxxx  
-    x  
+xxxxx
+    x
 */
   {6, {{0,0}, {1,0}, {2,0}, {3,0}, {4,0}, {4,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
-xxxxx  
-   x   
+xxxxx
+   x
 */
   {6, {{0,0}, {1,0}, {2,0}, {3,0}, {3,1}, {4,0}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 4},
 /*
-xxxxx  
-  x    
+xxxxx
+  x
 */
   {6, {{0,0}, {1,0}, {2,0}, {2,1}, {3,0}, {4,0}},
    4, {0, 1, 2, 3, -1, -1, -1, -1}, 3},
 /*
-   x   
-xxxx   
-   x   
+   x
+xxxx
+   x
 */
   {6, {{0,0}, {1,0}, {2,0}, {3,-1}, {3,0}, {3,1}},
    4, {0, 1, 2, 3, -1, -1, -1, -1}, 4},
 /*
-xxxx   
-   xx  
+xxxx
+   xx
 */
   {6, {{0,0}, {1,0}, {2,0}, {3,0}, {3,1}, {4,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
-xxxx   
-  xx   
+xxxx
+  xx
 */
   {6, {{0,0}, {1,0}, {2,0}, {2,1}, {3,0}, {3,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
-xxxx   
-   x   
-   x   
+xxxx
+   x
+   x
 */
   {6, {{0,0}, {1,0}, {2,0}, {3,0}, {3,1}, {3,2}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
-  x    
-xxxx   
-   x   
+  x
+xxxx
+   x
 */
   {6, {{0,0}, {1,0}, {2,-1}, {2,0}, {3,0}, {3,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
-xxxx   
- x x   
+xxxx
+ x x
 */
   {6, {{0,0}, {1,0}, {1,1}, {2,0}, {3,0}, {3,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 4},
 /*
- x     
-xxxx   
-   x   
+ x
+xxxx
+   x
 */
   {6, {{0,0}, {1,-1}, {1,0}, {2,0}, {3,0}, {3,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 4},
 /*
-xxxx   
-x  x   
+xxxx
+x  x
 */
   {6, {{0,0}, {0,1}, {1,0}, {2,0}, {3,0}, {3,1}},
    4, {0, 1, 2, 3, -1, -1, -1, -1}, 3},
 /*
-   x   
-xxxx   
-x      
+   x
+xxxx
+x
 */
   {6, {{0,0}, {0,1}, {1,0}, {2,0}, {3,-1}, {3,0}},
    4, {0, 1, 4, 5, -1, -1, -1, -1}, 3},
 /*
-  x    
-xxxx   
-  x    
+  x
+xxxx
+  x
 */
   {6, {{0,0}, {1,0}, {2,-1}, {2,0}, {2,1}, {3,0}},
    4, {0, 1, 2, 3, -1, -1, -1, -1}, 4},
 /*
-xxxx   
- xx    
+xxxx
+ xx
 */
   {6, {{0,0}, {1,0}, {1,1}, {2,0}, {2,1}, {3,0}},
    4, {0, 1, 2, 3, -1, -1, -1, -1}, 3},
 /*
-xxxx   
-  x    
-  x    
+xxxx
+  x
+  x
 */
   {6, {{0,0}, {1,0}, {2,0}, {2,1}, {2,2}, {3,0}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
- x     
-xxxx   
-  x    
+ x
+xxxx
+  x
 */
   {6, {{0,0}, {1,-1}, {1,0}, {2,0}, {2,1}, {3,0}},
    4, {0, 1, 4, 5, -1, -1, -1, -1}, 3},
 /*
-  xx   
-xxx    
-  x    
+  xx
+xxx
+  x
 */
   {6, {{0,0}, {1,0}, {2,-1}, {2,0}, {2,1}, {3,-1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
- xx    
-xxx    
-  x    
+ xx
+xxx
+  x
 */
   {6, {{0,0}, {1,-1}, {1,0}, {2,-1}, {2,0}, {2,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
-  x    
-xxx    
-x x    
+  x
+xxx
+x x
 */
   {6, {{0,0}, {0,1}, {1,0}, {2,-1}, {2,0}, {2,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 4},
 /*
-xxx    
-  xxx  
+xxx
+  xxx
 */
   {6, {{0,0}, {1,0}, {2,0}, {2,1}, {3,1}, {4,1}},
    4, {0, 1, 4, 5, -1, -1, -1, -1}, 3},
 /*
-xxx    
-  xx   
-   x   
+xxx
+  xx
+   x
 */
   {6, {{0,0}, {1,0}, {2,0}, {2,1}, {3,1}, {3,2}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
-xxx    
- xxx   
+xxx
+ xxx
 */
   {6, {{0,0}, {1,0}, {1,1}, {2,0}, {2,1}, {3,1}},
    4, {0, 1, 4, 5, -1, -1, -1, -1}, 4},
 /*
-xxx    
-  xx   
-  x    
+xxx
+  xx
+  x
 */
   {6, {{0,0}, {1,0}, {2,0}, {2,1}, {2,2}, {3,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 4},
 /*
- x     
-xxx    
-  xx   
+ x
+xxx
+  xx
 */
   {6, {{0,0}, {1,-1}, {1,0}, {2,0}, {2,1}, {3,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 4},
 /*
-xxx    
-x xx   
+xxx
+x xx
 */
   {6, {{0,0}, {0,1}, {1,0}, {2,0}, {2,1}, {3,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
-xxx    
- xx    
-  x    
+xxx
+ xx
+  x
 */
   {6, {{0,0}, {1,0}, {1,1}, {2,0}, {2,1}, {2,2}},
    4, {0, 1, 2, 3, -1, -1, -1, -1}, 4},
 /*
- x     
-xxx    
- xx    
+ x
+xxx
+ xx
 */
   {6, {{0,0}, {1,-1}, {1,0}, {1,1}, {2,0}, {2,1}},
    4, {0, 1, 2, 3, -1, -1, -1, -1}, 4},
 /*
-xxx    
-xxx    
+xxx
+xxx
 */
   {6, {{0,0}, {0,1}, {1,0}, {1,1}, {2,0}, {2,1}},
    2, {0, 1, -1, -1, -1, -1, -1, -1}, 3},
 /*
-xxx    
-  x    
-  xx   
+xxx
+  x
+  xx
 */
   {6, {{0,0}, {1,0}, {2,0}, {2,1}, {2,2}, {3,2}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
-xxx    
-  x    
- xx    
+xxx
+  x
+ xx
 */
   {6, {{0,0}, {1,0}, {1,2}, {2,0}, {2,1}, {2,2}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
- x     
-xxx    
-x x    
+ x
+xxx
+x x
 */
   {6, {{0,0}, {0,1}, {1,-1}, {1,0}, {2,0}, {2,1}},
    4, {0, 1, 2, 3, -1, -1, -1, -1}, 3},
 /*
-  xx   
-xxx    
-x      
+  xx
+xxx
+x
 */
   {6, {{0,0}, {0,1}, {1,0}, {2,-1}, {2,0}, {3,-1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
- xx    
-xxx    
-x      
+ xx
+xxx
+x
 */
   {6, {{0,0}, {0,1}, {1,-1}, {1,0}, {2,-1}, {2,0}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3},
 /*
-xx     
- xx    
-  xx   
+xx
+ xx
+  xx
 */
   {6, {{0,0}, {1,0}, {1,1}, {2,1}, {2,2}, {3,2}},
    4, {0, 1, 4, 5, -1, -1, -1, -1}, 3}};
 
 static struct pentomino_struct one_sided_pentomino[60];
 
-void make_one_sided_pentomino(void) {
+static void
+make_one_sided_pentomino(void) {
   int i,j,t,u;
 
   j=0;
-  for (i=0;i<18;i++) {
+  for (i=0;i<12;i++) {
     one_sided_pentomino[j] = pentomino[i];
     for (t=0;t<8;t++)
       if (one_sided_pentomino[j].transform_list[t]>=4) {
@@ -1486,7 +1518,8 @@ void make_one_sided_pentomino(void) {
 
 static struct hexomino_struct one_sided_hexomino[60];
 
-void make_one_sided_hexomino(void) {
+static void
+make_one_sided_hexomino(void) {
   int i,j,t,u;
 
   j=0;
@@ -1749,7 +1782,7 @@ Science News November 14, 1987 Vol 132, Pg 310
 
 */
 
-static struct {int len; point_type point[5]; 
+static struct {int len; point_type point[5];
                int transform_len, transform_list[8], max_white;} pentomino1 =
   {5, {{0,0}, {1,0}, {2,0}, {3,0}, {1,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 3};
@@ -1779,7 +1812,7 @@ int set_pentomino_puzzle1(polyominoesstruct *sp) {
 
 */
 
-static struct {int len; point_type point[6]; 
+static struct {int len; point_type point[6];
                int transform_len, transform_list[8], max_white;} hexomino1 =
   {6, {{0,0}, {1,0}, {2,0}, {3,0}, {4,0}, {1,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 4};
@@ -1811,7 +1844,7 @@ int set_hexomino_puzzle1(polyominoesstruct *sp) {
 
 */
 
-static struct {int len; point_type point[7]; 
+static struct {int len; point_type point[7];
                int transform_len, transform_list[8], max_white;} heptomino1 =
   {7, {{0,0}, {1,0}, {2,0}, {3,0}, {4,0}, {1,1}, {2,1}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 4};
@@ -1837,7 +1870,7 @@ int set_heptomino_puzzle1(polyominoesstruct *sp) {
   return 1;
 }
 
-/* The following puzzles from 
+/* The following puzzles from
 Polyominoes Puzzles, Patterns, Problems, and Packings Revised (2nd) Edition
 by Solomon W. Golomb   Princeton University Press 1994
 */
@@ -1874,9 +1907,9 @@ int set_heptomino_puzzle2(polyominoesstruct *sp) {
 
 */
 
-static struct {int len; point_type point[11]; 
+static struct {int len; point_type point[11];
                int transform_len, transform_list[8], max_white;} elevenomino1 =
-  {11, {{0,0}, {1,0}, {2,0}, 
+  {11, {{0,0}, {1,0}, {2,0},
         {0,1}, {1,1}, {2,1}, {3,1},
         {0,2}, {1,2}, {2,2}, {3,2}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 6};
@@ -1911,10 +1944,10 @@ int set_elevenomino_puzzle1(polyominoesstruct *sp) {
 
 */
 
-static struct {int len; point_type point[10]; 
+static struct {int len; point_type point[10];
                int transform_len, transform_list[8], max_white;} dekomino1 =
   {10, {       {1,-1},
-               {1,0}, 
+               {1,0},
         {0,1}, {1,1}, {2,1}, {3,1},
         {0,2}, {1,2}, {2,2}, {3,2}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 5};
@@ -1945,9 +1978,9 @@ int set_dekomino_puzzle1(polyominoesstruct *sp) {
 
 */
 
-static struct {int len; point_type point[10]; 
+static struct {int len; point_type point[10];
                int transform_len, transform_list[8], max_white;} octomino1 =
-  {8, {        {1,0}, 
+  {8, {        {1,0},
         {0,1}, {1,1}, {2,1},
         {0,2}, {1,2}, {2,2}, {3,2}},
    8, {0, 1, 2, 3, 4, 5, 6, 7}, 5};
@@ -2036,7 +2069,7 @@ init_polyominoes(ModeInfo * mi) {
 
   if (polyominoeses == NULL) {
     if ((polyominoeses
-         = (polyominoesstruct *) calloc(MI_NUM_SCREENS(mi),sizeof (polyominoesstruct))) 
+         = (polyominoesstruct *) calloc(MI_NUM_SCREENS(mi),sizeof (polyominoesstruct)))
         == NULL)
       return;
   }
@@ -2051,7 +2084,7 @@ init_polyominoes(ModeInfo * mi) {
     sp->identical = (Bool) (LRAND() & 1);
     sp->use3D = (Bool) (NRAND(4));
   } else {
-    sp->identical = identical; 
+    sp->identical = identical;
     sp->use3D = !plain;
   }
   if (sp->identical) {
@@ -2138,14 +2171,12 @@ init_polyominoes(ModeInfo * mi) {
     sp->box = box1;
   else
     sp->box = box2;
-
   if (sp->box >= 12) {
     sp->box = (sp->box/12)*12;
     create_bitmaps(mi,sp);
     if (!sp->use_bitmaps)
       free_bitmaps(sp);
-   }
-   else
+   } else
      sp->use_bitmaps = 0;
 
   if (!sp->use_bitmaps) {
@@ -2186,7 +2217,7 @@ init_polyominoes(ModeInfo * mi) {
 
   /* Clear the background. */
   MI_CLEARWINDOW(mi);
-  
+
 }
 
 void
@@ -2216,7 +2247,7 @@ draw_polyominoes(ModeInfo * mi) {
   sp->wait--;
   if (sp->wait>0) return;
 
-  memset(sp->changed_array,0,sp->width*sp->height*sizeof(int));
+  (void) memset(sp->changed_array,0,sp->width*sp->height*sizeof(int));
 
   poly_no = first_poly_no(sp);
   point_no = 0;
@@ -2225,7 +2256,7 @@ draw_polyominoes(ModeInfo * mi) {
   another_attachment_try = 1;
   find_blank(sp,&attach_point);
   if (sp->identical && sp->nr_attached < sp->nr_polyominoes)
-    memset(&REASON_TO_NOT_ATTACH(sp->nr_attached,0),0,sp->nr_polyominoes*sizeof(int));
+    (void) memset(&REASON_TO_NOT_ATTACH(sp->nr_attached,0),0,sp->nr_polyominoes*sizeof(int));
   while(!done) {
     if (sp->nr_attached < sp->nr_polyominoes) {
       while (!done && another_attachment_try) {
@@ -2282,7 +2313,7 @@ draw_polyominoes(ModeInfo * mi) {
     draw_without_bitmaps(mi,sp);
 
   if (sp->nr_attached == sp->nr_polyominoes)
-    sp->wait = 100;
+    sp->wait = 500;
   else
     sp->wait = 0;
 }
@@ -2290,11 +2321,11 @@ draw_polyominoes(ModeInfo * mi) {
 void
 release_polyominoes(ModeInfo * mi) {
   int screen;
-  
+
   if (polyominoeses != NULL) {
     for (screen=0;screen<MI_NUM_SCREENS(mi); screen++)
       free_polyominoes(&polyominoeses[screen]);
-    (void) free((void *) polyominoeses);
+    free(polyominoeses);
     polyominoeses = (polyominoesstruct *) NULL;
   }
 }
