@@ -27,7 +27,7 @@ static const char sccsid[] = "@(#)puzzle.c	5.00 2000/11/01 xlockmore";
  *              image moved off screen (-inwindow or -debug).
  * 10-May-1997: Compatible with xscreensaver
  * 15-Mar-1996: cleaned up, NUMBERED compile-time switch is now broken.
- * Feb-1996: combined with rastering.  Jouk Jansen <joukj@hrem.stm.tudelft.nl>.
+ * Feb-1996: combined with rastering.  Jouk Jansen <joukj AT hrem.nano.tudelft.nl>.
  * Feb-1995: written.  Heath Rice <hrice@netcom.com>
  */
 
@@ -40,6 +40,7 @@ static const char sccsid[] = "@(#)puzzle.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_puzzle
 #define PROGCLASS "Puzzle"
 #define HACK_INIT init_puzzle
 #define HACK_DRAW draw_puzzle
@@ -74,7 +75,7 @@ ModStruct   puzzle_description =
 #define PUZZLE_BITS    image_bits
 #include "puzzle.xbm"
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 #define PUZZLE_NAME    image_name
 #include "puzzle.xpm"
 #define DEFAULT_XPM 1
@@ -140,7 +141,7 @@ static void
 free_puzzle(Display * display, puzzlestruct * pp)
 {
 	if (pp->fixbuff != NULL) {
-		(void) free((void *) pp->fixbuff);
+		free(pp->fixbuff);
 		pp->fixbuff = (int *) NULL;
 	}
 	if (pp->bufferBox != None) {
@@ -184,7 +185,6 @@ NumberScreen(ModeInfo * mi)
 
 		pp->done = 1;
 		gcv.font = mode_font->fid;
-		XSetFont(display, MI_GC(mi), mode_font->fid);
 		gcv.graphics_exposures = False;
 		gcv.foreground = MI_WHITE_PIXEL(mi);
 		gcv.background = MI_BLACK_PIXEL(mi);
@@ -434,14 +434,14 @@ init_stuff(ModeInfo * mi)
 
 	if (pp->logo == None)
 		getImage(mi, &pp->logo, PUZZLE_WIDTH, PUZZLE_HEIGHT, PUZZLE_BITS,
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 			 DEFAULT_XPM, PUZZLE_NAME,
 #endif
 			 &pp->graphics_format, &pp->cmap, &pp->black);
-		if (pp->logo == None) {
-			free_puzzle(display, pp);
-			return False;
-		}
+	if (pp->logo == None) {
+		free_puzzle(display, pp);
+		return False;
+	}
 #ifndef STANDALONE
 	if (pp->cmap != None) {
 		setColormap(display, window, pp->cmap, MI_IS_INWINDOW(mi));
@@ -484,7 +484,7 @@ init_puzzle(ModeInfo * mi)
 	    pp->windowsize.y == MI_HEIGHT(mi))
 		return;		/* Debounce since refresh_puzzle is init_puzzle */
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 	if (pp->graphics_format >= IS_XPM) {
         	/* This is needed when another program changes the colormap. */
         	free_puzzle(display, pp);
@@ -495,7 +495,7 @@ init_puzzle(ModeInfo * mi)
 	pp->excount = MI_COUNT(mi);
 	if (pp->excount < 0) {
 		if (pp->fixbuff != NULL) {
-			(void) free((void *) pp->fixbuff);
+			free(pp->fixbuff);
 			pp->fixbuff = (int *) NULL;
 		}
 		pp->excount = NRAND(-pp->excount) + 1;
@@ -655,7 +655,7 @@ release_puzzle(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_puzzle(MI_DISPLAY(mi), &puzzles[screen]);
-		(void) free((void *) puzzles);
+		free(puzzles);
 		puzzles = (puzzlestruct *) NULL;
 	}
 #ifdef NUMBERED
