@@ -23,7 +23,7 @@ static const char sccsid[] = "@(#)pyro.c	5.00 2000/11/01 xlockmore";
  *
  * Revision History:
  * 01-Nov-2000: Allocation checks
- * 15-May-1997: jwz@jwz.org: turned into a standalone program.
+ * 15-May-1997: jwz AT jwz.org: turned into a standalone program.
  * 05-Sep-1996: Added 3d support Henrik Theiling <theiling@coli.uni-sb.de>
  * 16-Mar-1991: Written, received from David Brooks <brooks@osf.org>
  */
@@ -38,6 +38,7 @@ static const char sccsid[] = "@(#)pyro.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_pyro
 #define PROGCLASS "Pyro"
 #define HACK_INIT init_pyro
 #define HACK_DRAW draw_pyro
@@ -481,7 +482,7 @@ init_pyro(ModeInfo * mi)
 	pp->nrockets = MI_COUNT(mi);
 	if (pp->nrockets < -MINROCKETS) {
 		if (pp->rockq) {
-			(void) free((void *) pp->rockq);
+			free(pp->rockq);
 			pp->rockq = (rocket *) NULL;
 		}
 		pp->nrockets = NRAND(-pp->nrockets - MINROCKETS + 1) + MINROCKETS;
@@ -595,10 +596,10 @@ release_pyro(ModeInfo * mi)
 			pyrostruct *pp = &pyros[screen];
 
 			if (pp->rockq != NULL) {
-				(void) free((void *) pp->rockq);
+				free(pp->rockq);
 			}
 		}
-		(void) free((void *) pyros);
+		free(pyros);
 		pyros = (pyrostruct *) NULL;
 	}
 }
