@@ -60,8 +60,6 @@ static Bool invert;
 
 typedef unsigned int uint32;
 
-static XFontStruct *messagefont = None;
-
 #ifdef __VMS
 #define DEFAULT_MSG	"&0&1&2&3&4&5&6&7&8&9&a&b OpenVMS &a&b"
 #else
@@ -75,24 +73,25 @@ static XFontStruct *messagefont = None;
 
 static int pyrocnt;
 static char *modparam_msg, *modparam_fnt;
+static XFontStruct *messagefont = None;
 
 static XrmOptionDescRec opts[] = {
 	{(char *) "-invert", ".pyro2.invert", XrmoptionNoArg, (caddr_t) "on"},
 	{(char *) "+invert", ".pyro2.invert", XrmoptionNoArg, (caddr_t) "off"},
-	{(char *) "-msg", ".pyro2.msg", XrmoptionSepArg, (caddr_t) NULL	},
-	{(char *) "-fnt", ".pyro2.fnt", XrmoptionSepArg, (caddr_t) NULL	}
+	{(char *) "-msg", ".pyro2.msg", XrmoptionSepArg, (caddr_t) NULL},
+	{(char *) "-fnt", ".pyro2.fnt", XrmoptionSepArg, (caddr_t) NULL}
 };
 
 static argtype vars[] = {
 	{(void *) &invert, "invert", "Invert", DEF_INVERT, t_Bool},
-	{(void *) &modparam_msg, "msg", "Msg", "", t_String	},
-	{(void *) &modparam_fnt, "fnt", "Fnt", DEFAULT_FNT, t_String	}
+	{(void *) &modparam_msg, "msg", "Msg", "", t_String},
+	{(void *) &modparam_fnt, "fnt", "Fnt", DEFAULT_FNT, t_String}
 };
 
 static OptionStruct desc[] = {
         {"-/+invert", "turn on/off inverting of sparks for image"},
-	{"-msg str", "Pyro command (default: " DEFAULT_MSG ")"	},
-	{"-fnt font", "Font to use (default: " DEFAULT_FNT ")"	},
+	{"-msg str", "Pyro command (default: " DEFAULT_MSG ")"},
+	{"-fnt font", "Font to use (default " DEFAULT_FNT ")"}
 };
 
 ModeSpecOpt pyro2_opts =
@@ -136,6 +135,17 @@ static char * ostext = NULL;
 #include "color.h"
 #include "iostuff.h"
 
+#ifdef USE_XVMSUTILS
+#include <X11/unix_time.h>
+#endif
+#if HAVE_SYS_TIME_H
+#include <sys/time.h>
+#else
+#if HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
+#endif
+
 #if !defined(FALSE)
 #define FALSE	0
 #endif /* !defined(FALSE) */
@@ -161,6 +171,7 @@ static char * ostext = NULL;
 #define EXPLOSION	2
 #define READY		3
 
+#ifndef WIN32
 /* aliases for vars defined in the bitmap file */
 #define LOGO_WIDTH   image_width
 #define LOGO_HEIGHT    image_height
@@ -174,7 +185,6 @@ static char * ostext = NULL;
 #define DEFAULT_XPM 0
 #endif
 
-#ifndef WIN32
 #if !defined( VMS ) || ( __VMS_VER >= 70000000 )
 #include <sys/utsname.h>
 #else
@@ -203,7 +213,7 @@ typedef struct para {
 }PARA;
 
 /*
-for the moment there are 11 distinc internal explosion types.
+for the moment there are 11 distinct internal explosion types.
 Each type has its own ID. An ID is a character between
 '0' and '9' or 'a' and 'z'. 'a' corresponds to 10 and
 'z' to 35. The type ID 42 is reserved for glyph explosions.
@@ -517,18 +527,18 @@ static void explosion(ModeInfo * mi, PYRO *p)
 	++num;
 	if(e->init == FALSE) {
 		/* every spark has its own angle and initial speed (v0) */
-		e->angle = calloc(EXP_MAX_DOTS, sizeof(double));
-		e->time = calloc(EXP_MAX_DOTS, sizeof(double));
-		e->delta_t = calloc(EXP_MAX_DOTS, sizeof(double));
-		e->v0 = calloc(EXP_MAX_DOTS, sizeof(double));
-		e->ttl = calloc(EXP_MAX_DOTS, sizeof(int));
-		e->generation = calloc(EXP_MAX_DOTS, sizeof(int));
-		e->x1 = calloc(EXP_MAX_DOTS, sizeof(int));
-		e->y1 = calloc(EXP_MAX_DOTS, sizeof(int));
-		e->x2 = calloc(EXP_MAX_DOTS, sizeof(int));
-		e->y2 = calloc(EXP_MAX_DOTS, sizeof(int));
-		e->color = calloc(EXP_MAX_DOTS, sizeof(int));
-		e->m = calloc(EXP_MAX_DOTS, sizeof(double));
+		e->angle = (double *) calloc(EXP_MAX_DOTS, sizeof(double));
+		e->time = (double *) calloc(EXP_MAX_DOTS, sizeof(double));
+		e->delta_t = (double *) calloc(EXP_MAX_DOTS, sizeof(double));
+		e->v0 = (double *) calloc(EXP_MAX_DOTS, sizeof(double));
+		e->ttl = (int *) calloc(EXP_MAX_DOTS, sizeof(int));
+		e->generation = (int *) calloc(EXP_MAX_DOTS, sizeof(int));
+		e->x1 = (int *) calloc(EXP_MAX_DOTS, sizeof(int));
+		e->y1 = (int *) calloc(EXP_MAX_DOTS, sizeof(int));
+		e->x2 = (int *) calloc(EXP_MAX_DOTS, sizeof(int));
+		e->y2 = (int *) calloc(EXP_MAX_DOTS, sizeof(int));
+		e->color = (int *) calloc(EXP_MAX_DOTS, sizeof(int));
+		e->m = (double *) calloc(EXP_MAX_DOTS, sizeof(double));
 		e->current_generation = 0;
 		e->firsttime = 0;
 		e->x0 = p->para.x2;
@@ -898,8 +908,8 @@ static Bool XImage2bitmap(XImage *image, LOGO_BITMAP *bitmap, int white)
 	}
 
 	bitmap->dotcnt = dotcnt;
-	bitmap->phi = calloc(dotcnt, sizeof(float));
-	bitmap->v0 = calloc(dotcnt, sizeof(float));
+	bitmap->phi = (float *) calloc(dotcnt, sizeof(float));
+	bitmap->v0 = (float *) calloc(dotcnt, sizeof(float));
 
 	xmid = xbeg + (xend - xbeg) / 2;
 	ymid = ybeg + (yend - ybeg) / 2;
@@ -934,7 +944,7 @@ static LOGO_BITMAP *text2bitmap(ModeInfo *mi, char *text)
 {
 	LOGO_BITMAP *bitmap;
 
-	if(!(bitmap = calloc(1, sizeof(LOGO_BITMAP))))	return NULL;
+	if(!(bitmap = (LOGO_BITMAP *) calloc(1, sizeof(LOGO_BITMAP))))	return NULL;
 
 	if(!text2image(mi, text, &bitmap->image))	{
 		free(bitmap);
@@ -969,17 +979,22 @@ void init_pyro2(ModeInfo * mi)
 
 	pyrostruct *pp;
 
-	if(pyros == NULL && (pyros = calloc(MI_NUM_SCREENS(mi), sizeof (pyrostruct))) == NULL)
+	if(pyros == NULL && (pyros = (pyrostruct *) calloc(MI_NUM_SCREENS(mi), sizeof (pyrostruct))) == NULL)
 			return;
-
-        if (messagefont == None && !(messagefont = XLoadQueryFont(display, modparam_fnt)))	{
-		fprintf(stderr, "can not find font '%s'\n", modparam_fnt);
-		release_pyro2(mi);
-		return;
+        if (messagefont == None) {
+		if (modparam_fnt != NULL &&
+				!(messagefont = XLoadQueryFont(display, modparam_fnt)))	{
+			(void) fprintf(stderr, "can not find font '%s', using %s\n", modparam_fnt, DEFAULT_FNT);
+		}
+        	if (messagefont == None) {
+			if ((messagefont = getFont(display)) == None) {
+				release_pyro2(mi);
+				return;
+			}
+		}
 	}
 
 	pp = &pyros[MI_SCREEN(mi)];
-
 	if(!pp->gc) {
 		if(MI_IS_INSTALL(mi) && MI_NPIXELS(mi) > 2) {
 			XColor      color;
@@ -1015,7 +1030,7 @@ void init_pyro2(ModeInfo * mi)
 
 			pp->ncolors = PYRO_NCOLORS;
 
-			if((pp->colors = calloc(pp->ncolors,sizeof(XColor))) == NULL) {
+			if((pp->colors = (XColor *) calloc(pp->ncolors,sizeof(XColor))) == NULL) {
 				return;
 			}
 #undef PYRO_RAMP
@@ -1119,7 +1134,7 @@ void init_pyro2(ModeInfo * mi)
 #endif
 	if (!modparam_msg || !*modparam_msg) {
 		if (ostext == NULL)
-			rov = DEFAULT_MSG;
+			rov = (char *) DEFAULT_MSG;
 		else
 			rov = ostext;
 	} else
@@ -1168,10 +1183,10 @@ void init_pyro2(ModeInfo * mi)
 	}
 	if(text_in_buffer)	++pyrocnt;
 
-	pyro = calloc(pyrocnt, sizeof(PYRO));
+	pyro = (PYRO *) calloc(pyrocnt, sizeof(PYRO));
 	if (!modparam_msg || !*modparam_msg) {
 		if (ostext == NULL)
-			rov = DEFAULT_MSG;
+			rov = (char*) DEFAULT_MSG;
 		else
 			rov = ostext;
 	} else
@@ -1213,7 +1228,7 @@ void init_pyro2(ModeInfo * mi)
 				DEFAULT_XPM, LOGO_NAME,
 #endif
 				&graphics_format, &cmap, &black);
-				bitmap = calloc(1, sizeof(LOGO_BITMAP));
+				bitmap = (LOGO_BITMAP *) calloc(1, sizeof(LOGO_BITMAP));
 				XImage2bitmap(image, bitmap, 0);
 				pyro[i].bitmap = bitmap;
 				pyro[i].etype = 23;
@@ -1321,31 +1336,36 @@ void release_pyro2(ModeInfo * mi)
 			for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
 				pyrostruct *pp = &pyros[screen];
 
-				MI_WHITE_PIXEL(mi) = pp->whitepixel;
-				MI_BLACK_PIXEL(mi) = pp->blackpixel;
-#ifndef STANDALONE
-				MI_FG_PIXEL(mi) = pp->fg;
-				MI_BG_PIXEL(mi) = pp->bg;
-#endif
-
-				if(pp->colors && pp->ncolors) /* FIXME: if !no_colors */
+				if(pp->colors && pp->ncolors) { /* FIXME: if !no_colors */
 					free_colors(display, pp->cmap, pp->colors, pp->ncolors);
-
-				if(pp->colors != NULL)	free(pp->colors);
-
-				if(pp->cmap)	XFreeColormap(display, pp->cmap);
-
-				if(pp->gc != None)	XFreeGC(display, pp->gc);
+				}
+				if (pp->colors != NULL) {
+					free(pp->colors);
+				}
+				if (pp->cmap) {
+					XFreeColormap(display, pp->cmap);
+				}
+				if (pp->gc != None) {
+					MI_WHITE_PIXEL(mi) = pp->whitepixel;
+					MI_BLACK_PIXEL(mi) = pp->blackpixel;
+#ifndef STANDALONE
+					MI_FG_PIXEL(mi) = pp->fg;
+					MI_BG_PIXEL(mi) = pp->bg;
+#endif
+					XFreeGC(display, pp->gc);
+				}
 			}
 		}
-
 		free(pyros);
 		pyros = (pyrostruct *) NULL;
 	}
+	if (messagefont != None) {
+		XFreeFont(MI_DISPLAY(mi), messagefont);
+		messagefont = None;
+	}
 	if (ostext != NULL)
 		free(ostext);
 	ostext = NULL;
-
 	return;
 }
 
@@ -1357,7 +1377,6 @@ void refresh_pyro2(ModeInfo * mi)
 	else {
 		MI_CLEARWINDOW(mi);
 	}
-
 	return;
 }
 
