@@ -33,6 +33,7 @@ static const char sccsid[] = "@(#)scooter.c	5.01 2001/03/02 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_scooter
 #define PROGCLASS "Scooter"
 #define HACK_INIT init_scooter
 #define HACK_DRAW draw_scooter
@@ -211,16 +212,8 @@ static void nextdoorcolor(ModeInfo *mi, Door *door)
 	Display *display = MI_DISPLAY(mi);
 	XColor xcol;
 
-/* uncomment this to color the doors from xlock's palette (created with saturation value) */
-#if 0
-	if (MI_NPIXELS(mi) > 2) {
-		if (++colorcount >= MI_NPIXELS(mi))
-			colorcount = 0;
-		door->color = MI_PIXEL(mi,colorcount);
-	} else
-		door->color = MI_WHITE_PIXEL(mi);
-	return;
-#endif
+/* comment this to color the doors from xlock's palette (created with saturation value) */
+#if 1
 	if (door->freecolor) {
 		XFreeColors(display, MI_COLORMAP(mi), &(door->color), 1, 0);
 		door->freecolor = 0;
@@ -261,20 +254,30 @@ static void nextdoorcolor(ModeInfo *mi, Door *door)
 		door->color = xcol.pixel;
 		door->freecolor = 1;
 	}
+#else
+	if (MI_NPIXELS(mi) > 2) {
+		if (++colorcount >= MI_NPIXELS(mi))
+			colorcount = 0;
+		door->color = MI_PIXEL(mi,colorcount);
+	} else
+		door->color = MI_WHITE_PIXEL(mi);
+	return;
+#endif
 }
 
-void free_scooter(scooterstruct *sp)
+static void
+free_scooter(scooterstruct *sp)
 {
 	if (sp->doors != NULL) {
-		(void) free((void *) sp->doors);
+		free(sp->doors);
 		sp->doors = (Door *) NULL;
 	}
 	if (sp->stars != NULL) {
-		(void) free((void *) sp->stars);
+		free(sp->stars);
 		sp->stars = (Star *) NULL;
 	}
 	if (sp->zelements != NULL) {
-		(void) free((void *) sp->zelements);
+		free(sp->zelements);
 		sp->zelements = (ZElement *) NULL;
 	}
 }
@@ -286,11 +289,11 @@ void release_scooter(ModeInfo *mi)
 
                 for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
                         free_scooter(&scooters[screen]);
-                (void) free((void *) scooters);
+                free(scooters);
                 scooters = (scooterstruct *) NULL;
         }
 	if (sintable != NULL) {
-		(void) free((void *) sintable);
+		free(sintable);
 		sintable = (float *) NULL;
 	}
 }
@@ -310,7 +313,8 @@ void init_scooter(ModeInfo *mi)
 	sp->doorcount = MAX(MI_COUNT(mi),MIN_DOORS);
 	sp->speed = MI_CYCLES(mi);
 	sp->starcount = MI_SIZE(mi);
-
+	if (sp->starcount < 1)
+		sp->starcount = 1;
 	if (sp->speed < MIN_SPEED)
 		sp->speed = MIN_SPEED;
 	if (sp->speed > MAX_SPEED)
@@ -325,12 +329,13 @@ void init_scooter(ModeInfo *mi)
 	sp->halt_scooter = False;
 
 	initdoorcolors(sp);
-
 	free_scooter(sp);
-	if ((sintable = (float *) malloc(sizeof(float) *
+	if (sintable == NULL) {
+		if ((sintable = (float *) malloc(sizeof(float) *
 			SINUSTABLE_SIZE)) == NULL) {
-		release_scooter(mi);
-		return;
+			release_scooter(mi);
+			return;
+		}
 	}
 	for (i = 0; i < SINUSTABLE_SIZE; i++) {
 		sintable[i] = SINF(M_PI*2/SINUSTABLE_SIZE*i);
@@ -339,13 +344,13 @@ void init_scooter(ModeInfo *mi)
 	if ((sp->doors = (Door *) malloc(sizeof(Door) *
 			sp->doorcount)) == NULL) {
 		return;
-	} 
+	}
 
 	if ((sp->zelements = (ZElement *) malloc(sizeof(ZElement) *
 		 sp->ztotal)) == NULL) {
 		free_scooter(sp);
 		return;
-	} 
+	}
 	for (i = 0; i <  sp->doorcount; i++) {
 		sp->doors[i].zelement =
 			(sp->zelements_per_door * (i + 1)) - 1;
@@ -363,12 +368,11 @@ void init_scooter(ModeInfo *mi)
 			sp->starcount)) == NULL) {
 		free_scooter(sp);
 		return;
-	} 
+	}
 	for (i = 0; i < sp->starcount; i++) {
 		sp->stars[i].zelement = sp->ztotal * i / sp->starcount;
 		sp->stars[i].draw = 0;
 	}
-	
 
 	sp->projnorm_z = 50 * 240;
 	sp->spectator_zelement = sp->zelements_per_door;
