@@ -30,8 +30,8 @@ static const char sccsid[] = "@(#)shape.c	5.00 2000/11/01 xlockmore";
  * 27-Jun-1995: added ellipses
  * 27-Feb-1995: patch for VMS
  * 29-Sep-1994: multidisplay bug fix <epstein_caleb@jpmorgan.com>
- * 15-Jul-1994: xlock version David Bagley <bagleyd@tux.org>
- * 1992: xscreensaver version Jamie Zawinski <jwz@jwz.org>
+ * 15-Jul-1994: xlock version David Bagley <bagleyd AT verizon.net>
+ * 1992: xscreensaver version Jamie Zawinski <jwz AT jwz.org>
  */
 
 /*-
@@ -47,6 +47,7 @@ static const char sccsid[] = "@(#)shape.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_shape
 #define PROGCLASS "Shape"
 #define HACK_INIT init_shape
 #define HACK_DRAW draw_shape
@@ -114,9 +115,9 @@ static XrmOptionDescRec opts[] =
 };
 static argtype vars[] =
 {
-	{(caddr_t *) & shade, (char *) "shade", (char *) "Shade", (char *) DEF_SHADE, t_Bool},
-	{(caddr_t *) & border, (char *) "border", (char *) "Border", (char *) DEF_BORDER, t_Bool},
-	{(caddr_t *) & stipple, (char *) "stipple", (char *) "Stipple", (char *) DEF_STIPPLE, t_Bool}
+	{(void *) & shade, (char *) "shade", (char *) "Shade", (char *) DEF_SHADE, t_Bool},
+	{(void *) & border, (char *) "border", (char *) "Border", (char *) DEF_BORDER, t_Bool},
+	{(void *) & stipple, (char *) "stipple", (char *) "Stipple", (char *) DEF_STIPPLE, t_Bool}
 };
 static OptionStruct desc[] =
 {
@@ -415,7 +416,7 @@ release_shape(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_shape(MI_DISPLAY(mi), &shapes[screen]);
-		(void) free((void *) shapes);
+		free(shapes);
 		shapes = (shapestruct *) NULL;
 	}
 }
