@@ -28,12 +28,13 @@ static const char sccsid[] = "@(#)sierpinski.c	5.00 2000/11/01 xlockmore";
  * 01-Nov-2000: Allocation checks
  * 18-Sep-1997: 3D version Antti Kuntsi <kuntsi@iki.fi>.
  * 20-May-1997: Changed the name tri to sierpinski for more compatiblity
- * 10-May-1997: Jamie Zawinski <jwz@jwz.org> compatible with xscreensaver
+ * 10-May-1997: Jamie Zawinski <jwz AT jwz.org> compatible with xscreensaver
  * 05-Sep-1996: Desmond Daignault Datatimes Incorporated
  *            <tekdd@dtol.datatimes.com> .
  */
 
 #ifdef STANDALONE
+#define MODE_sierpinski
 #define PROGCLASS "Sierpinski"
 #define HACK_INIT init_sierpinski
 #define HACK_DRAW draw_sierpinski
@@ -121,7 +122,7 @@ free_sierpinski(sierpinskistruct *sp)
 
 	for (corner = 0; corner < MAXCORNERS; corner++)
 		if (sp->pointBuffer[corner] != NULL) {
-			(void) free((void *) sp->pointBuffer[corner]);
+			free(sp->pointBuffer[corner]);
 			sp->pointBuffer[corner] = (XPoint *) NULL;
 		}
 }
@@ -208,7 +209,7 @@ release_sierpinski(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_sierpinski(&tris[screen]);
-		(void) free((void *) tris);
+		free(tris);
 		tris = (sierpinskistruct *) NULL;
 	}
 }
