@@ -27,6 +27,7 @@ static const char sccsid[] = "@(#)space.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_space
 #define PROGCLASS "Space"
 #define HACK_INIT init_space
 #define HACK_DRAW draw_space
@@ -103,31 +104,31 @@ static void
 free_space(SpaceStruct *sp)
 {
 	if (sp->starX != NULL) {
-		(void) free((void *) sp->starX);
+		free(sp->starX);
 		sp->starX = (double *) NULL;
 	}
 	if (sp->starY != NULL) {
-		(void) free((void *) sp->starY);
+		free(sp->starY);
 		sp->starY = (double *) NULL;
 	}
 	if (sp->starZ != NULL) {
-		(void) free((void *) sp->starZ);
+		free(sp->starZ);
 		sp->starY = (double *) NULL;
 	}
 	if (sp->stars1a != NULL) {
-		(void) free((void *) sp->stars1a);
+		free(sp->stars1a);
 		sp->stars1a = (XPoint *) NULL;
 	}
 	if (sp->stars1b != NULL) {
-		(void) free((void *) sp->stars1b);
+		free(sp->stars1b);
 		sp->stars1b = (XPoint *) NULL;
 	}
 	if (sp->stars2a != NULL) {
-		(void) free((void *) sp->stars2a);
+		free(sp->stars2a);
 		sp->stars2a = (XPoint *) NULL;
 	}
 	if (sp->stars2b != NULL) {
-		(void) free((void *) sp->stars2b);
+		free(sp->stars2b);
 		sp->stars2b = (XPoint *) NULL;
 	}
 }
@@ -227,6 +228,8 @@ draw_space(ModeInfo * mi)
 	zoom = sp->zoom;
 	IS_SMALL = ((originX * originY) < (160 * 100));
 
+	MI_IS_DRAWN(mi) = True;
+
 	/* get cos & sin of rotations */
 	cosX = COSF(sp->ax);
 	sinX = SINF(sp->ax);
@@ -442,7 +445,7 @@ release_space(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_space(&spaces[screen]);
-		(void) free((void *) spaces);
+		free(spaces);
 		spaces = (SpaceStruct *) NULL;
 	}
 }
