@@ -23,9 +23,9 @@ static const char sccsid[] = "@(#)sphere.c	5.00 2000/11/01 xlockmore";
  *
  * Revision History:
  * 01-Nov-2000: Allocation checks
- * 30-May-1997: <jwz@jwz.org> made it go vertically as well as horizontally.
- * 27-May-1997: <jwz@jwz.org> turned into a standalone program.
- * 02-Sep-1993: xlock version David Bagley <bagleyd@tux.org>
+ * 30-May-1997: <jwz AT jwz.org> made it go vertically as well as horizontally.
+ * 27-May-1997: <jwz AT jwz.org> turned into a standalone program.
+ * 02-Sep-1993: xlock version David Bagley <bagleyd AT verizon.net>
  * 1988: Revised to use SunView canvas instead of gfxsw Sun Microsystems
  * 1982: Orignal Algorithm Tom Duff Lucasfilm Ltd.
  */
@@ -58,6 +58,7 @@ static const char sccsid[] = "@(#)sphere.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_sphere
 #define PROGCLASS "Sphere"
 #define HACK_INIT init_sphere
 #define HACK_DRAW draw_sphere
@@ -123,11 +124,11 @@ init_sphere(ModeInfo * mi)
 	sp = &spheres[MI_SCREEN(mi)];
 
 	if (sp->points != NULL) {
-		(void) free((void *) sp->points);
+		free(sp->points);
 		sp->points = (XPoint *) NULL;
 	}
-	sp->width = MI_WIDTH(mi);
-	sp->height = MI_HEIGHT(mi);
+	sp->width = MAX(MI_WIDTH(mi), 4);
+	sp->height = MAX(MI_HEIGHT(mi), 4);
 	if ((sp->points = (XPoint *) malloc(MIN(sp->width, sp->height) *
 			 sizeof (XPoint))) == NULL) {
 		return;
@@ -280,11 +281,11 @@ release_sphere(ModeInfo * mi)
 			spherestruct *sp = &spheres[screen];
 
 			if (sp->points) {
-				(void) free((void *) sp->points);
+				free(sp->points);
 				/* sp->points = NULL; */
 			}
 		}
-		(void) free((void *) spheres);
+		free(spheres);
 		spheres = (spherestruct *) NULL;
 	}
 }
