@@ -28,7 +28,7 @@ static const char sccsid[] = "@(#)spline.c	5.00 2000/11/01 xlockmore";
  * 17-Jan-1996: added compile time option, FOLLOW to erase old splines like Qix
  *              thanks to Richard Duran <rduran@cs.utep.edu>
  * 09-Mar-1995: changed how batchcount is used
- * 02-Sep-1993: xlock version: David Bagley <bagleyd@tux.org>
+ * 02-Sep-1993: xlock version: David Bagley <bagleyd AT verizon.net>
  *              reminds me of a great "Twilight Zone" episode.
  * 1992: X11 version Jef Poskanzer <jef@netcom.com>, <jef@well.sf.ca.us>
  *
@@ -49,6 +49,7 @@ static const char sccsid[] = "@(#)spline.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_spline
 #define PROGCLASS "Spline"
 #define HACK_INIT init_spline
 #define HACK_DRAW draw_spline
@@ -80,7 +81,7 @@ static XrmOptionDescRec opts[] =
 
 static argtype vars[] =
 {
-	{(caddr_t *) & erase, (char *) "erase", (char *) "Erase", (char *) DEF_ERASE, t_Bool}
+	{(void *) & erase, (char *) "erase", (char *) "Erase", (char *) DEF_ERASE, t_Bool}
 };
 
 static OptionStruct desc[] =
@@ -138,7 +139,7 @@ free_spline(splinestruct *sp)
 {
 
 	if (sp->pt != NULL) {
-		(void) free((void *) sp->pt);
+		free(sp->pt);
 		sp->pt = (splinepointstruct *) NULL;
 	}
 	if (sp->splineq != NULL) {
@@ -146,8 +147,8 @@ free_spline(splinestruct *sp)
 
 		for (spline = 0; spline < sp->nsplines; ++spline)
 			if (sp->splineq[spline] != NULL)
-				(void) free((void *) sp->splineq[spline]);
-		(void) free((void *) sp->splineq);
+				free(sp->splineq[spline]);
+		free(sp->splineq);
 		sp->splineq = (XPoint **) NULL;
 	}
 }
@@ -165,31 +166,30 @@ init_spline(ModeInfo * mi)
 	}
 	sp = &splines[MI_SCREEN(mi)];
 
-	if (MI_IS_FULLRANDOM(mi))
-		sp->erase = (Bool) (LRAND() & 1);
-	else
-		sp->erase = erase;
-
 	sp->width = MI_WIDTH(mi);
 	sp->height = MI_HEIGHT(mi);
 	/* batchcount is the upper bound on the number of points */
 	sp->points = MI_COUNT(mi);
 	if (sp->points < -MINPOINTS) {
 		if (sp->pt) {
-			(void) free((void *) sp->pt);
+			free(sp->pt);
 			sp->pt = (splinepointstruct *) NULL;
 		}
 		if (sp->erase) {
 			if (sp->splineq)
 				for (i = 0; i < sp->nsplines; ++i)
 					if (sp->splineq[i]) {
-						(void) free((void *) sp->splineq[i]);
+						free(sp->splineq[i]);
 						sp->splineq[i] = (XPoint *) NULL;
 					}
 		}
 		sp->points = NRAND(-sp->points - MINPOINTS + 1) + MINPOINTS;
 	} else if (sp->points < MINPOINTS)
 		sp->points = MINPOINTS;
+	if (MI_IS_FULLRANDOM(mi))
+		sp->erase = (Bool) (LRAND() & 1);
+	else
+		sp->erase = erase;
 	if (sp->pt == NULL)
 		if ((sp->pt = (splinepointstruct *) malloc(sp->points *
 				sizeof (splinepointstruct))) == NULL) {
@@ -433,7 +433,7 @@ release_spline(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_spline(&splines[screen]);
-		(void) free((void *) splines);
+		free(splines);
 		splines = (splinestruct *) NULL;
 	}
 }
