@@ -2,11 +2,11 @@
 /* starfish ---  */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)starfish.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)starfish.c	5.03 2003/05/01 xlockmore";
 
 #endif
 
-/* xscreensaver, Copyright (c) 1997 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1997 Jamie Zawinski <jwz AT jwz.org>
 
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -17,12 +17,14 @@ static const char sccsid[] = "@(#)starfish.c	5.00 2000/11/01 xlockmore";
  * implied warranty.
  *
  * Revision History:
+ * 01-May-2003: Changed to work in a 64x64 window.
  * 01-Nov-2000: Allocation checks
- * 24-Feb-1998: xlockmore version by Jouk Jansen <joukj@hrem.stm.tudelft.nl>
- * 1997: original xscreensaver version by Jamie Zawinski <jwz@jwz.org>
+ * 24-Feb-1998: xlockmore version by Jouk Jansen <joukj AT hrem.nano.tudelft.nl>
+ * 1997: original xscreensaver version by Jamie Zawinski <jwz AT jwz.org>
  */
 
 #ifdef STANDALONE
+#define MODE_starfish
 #define PROGCLASS "Starfish"
 #define HACK_INIT init_starfish
 #define HACK_DRAW draw_starfish
@@ -66,11 +68,11 @@ static XrmOptionDescRec opts[] =
 
 static argtype vars[] =
 {
-{(caddr_t *) & cyclespeed, (char *) "cyclespeed", (char *) "CycleSpeed", (char *) DEF_CYCLESPEED, t_Int},
-	{(caddr_t *) & rotv, (char *) "rotation", (char *) "Rotation", (char *) DEF_ROTV, t_Float},
- {(caddr_t *) & thickness, (char *) "thickness", (char *) "Thickness", (char *) DEF_THICKNESS, t_Float},
-	{(caddr_t *) & blob, (char *) "blob", (char *) "Blob", (char *) DEF_BLOB, t_Bool},
-	{(caddr_t *) & cycle_p, (char *) "cycle", (char *) "Cycle", (char *) DEF_CYCLE, t_Bool}
+	{(void *) & cyclespeed, (char *) "cyclespeed", (char *) "CycleSpeed", (char *) DEF_CYCLESPEED, t_Int},
+	{(void *) & rotv, (char *) "rotation", (char *) "Rotation", (char *) DEF_ROTV, t_Float},
+	{(void *) & thickness, (char *) "thickness", (char *) "Thickness", (char *) DEF_THICKNESS, t_Float},
+	{(void *) & blob, (char *) "blob", (char *) "Blob", (char *) DEF_BLOB, t_Bool},
+	{(void *) & cycle_p, (char *) "cycle", (char *) "Cycle", (char *) DEF_CYCLE, t_Bool}
 };
 
 static OptionStruct desc[] =
@@ -151,7 +153,7 @@ free_starfish(Display *display, starfishstruct * sp)
 		if (sp->colors != NULL) {
 			if (sp->ncolors && !sp->no_colors)
 				free_colors(display, sp->cmap, sp->colors, sp->ncolors);
-			(void) free((void *) sp->colors);
+			free(sp->colors);
 			sp->colors = (XColor *) NULL;
 		}
 		if (sp->cmap != None) {
@@ -168,11 +170,11 @@ free_starfish(Display *display, starfishstruct * sp)
 		sp->splines = (spline *) NULL;
 	}
 	if (sp->r != NULL) {
-		(void) free((void *) sp->r);
+		free(sp->r);
 		sp->r = (long *) NULL;
 	}
 	if (sp->prev != NULL) {
-		(void) free((void *) sp->prev);
+		free(sp->prev);
 		sp->prev = (XPoint *) NULL;
 	}
 }
@@ -227,10 +229,9 @@ make_starfish(ModeInfo * mi)
 	sp->size *= SCALE;
 
 	sp->max_r = sp->size;
-	sp->min_r = 0;
+	sp->min_r = SCALE;
+	/* sp->min_r = 5 * SCALE; */
 
-	if (sp->min_r < (5 * SCALE))
-		sp->min_r = (5 * SCALE);
 	/* mid = ((sp->min_r + sp->max_r) / 2); */
 
 	sp->x = sp->winwidth / 2;
@@ -255,7 +256,7 @@ make_starfish(ModeInfo * mi)
 	sp->splines = make_spline(sp->npoints);
 
 	if (sp->r != NULL)
-		(void) free((void *) sp->r);
+		free(sp->r);
 	if ((sp->r = (long *) malloc(sizeof (*sp->r) * sp->npoints)) == NULL) {
 		free_starfish(MI_DISPLAY(mi), sp);
 		return;
@@ -391,9 +392,9 @@ draw1_starfish(Display * display, Drawable drawable, GC gc, starfishstruct * sp)
 			XClearWindow(display, drawable);
 		XFillPolygon(display, drawable, gc, points, i + j,
 			     Complex, CoordModeOrigin);
-		(void) free((void *) points);
+		free(points);
 
-		(void) free((void *) sp->prev);
+		free(sp->prev);
 		sp->prev = (XPoint *) NULL;
 	}
 	if ((sp->prev = (XPoint *) malloc(sp->splines->n_points *
@@ -510,7 +511,7 @@ init_starfish(ModeInfo * mi)
 		if (sp->colors != NULL) {
 			if (sp->ncolors && !sp->no_colors)
 				free_colors(display, sp->cmap, sp->colors, sp->ncolors);
-			(void) free((void *) sp->colors);
+			free(sp->colors);
 			sp->colors = (XColor *) NULL;
 		}
 		sp->ncolors = MI_NCOLORS(mi);
@@ -572,7 +573,7 @@ init_starfish(ModeInfo * mi)
 	sp->winheight = MI_HEIGHT(mi);
 	MI_CLEARWINDOW(mi);
 	if (sp->prev != NULL) {
-		(void) free((void *) sp->prev);
+		free(sp->prev);
 		sp->prev = (XPoint *) NULL;
 	}
 	sp->stage = 0;
@@ -664,7 +665,7 @@ release_starfish(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_starfish(MI_DISPLAY(mi), &starfishes[screen]);
-		(void) free((void *) starfishes);
+		free(starfishes);
 		starfishes = (starfishstruct *) NULL;
 	}
 }
