@@ -60,7 +60,7 @@ static XrmOptionDescRec opts[] =
 
 static argtype vars[] =
 {
-        {(caddr_t *) & trackmouse, (char *) "trackmouse", (char *) "TrackMouse", (char *) DEF_TRACKMOUSE, t_Bool}
+        {(void *) & trackmouse, (char *) "trackmouse", (char *) "TrackMouse", (char *) DEF_TRACKMOUSE, t_Bool}
 };
 
 static OptionStruct desc[] =
@@ -339,8 +339,8 @@ draw_swarm(ModeInfo * mi)
 		sp->yv[NRAND(sp->beecount)] += balance_rand(3);
 	}
 	/* <=- Bees -=> */
-	sp->tick = ++(sp->tick) % (sp->taillen);
-	prev = (sp->tick) ? sp->tick - 1 : sp->taillen - 1; 
+	sp->tick = (sp->tick + 1) % (sp->taillen);
+	prev = (sp->tick) ? sp->tick - 1 : sp->taillen - 1;
 	if (sp->tick == sp->taillen - 1)
 		sp->rolloverflag = 1;
 	for (b = 0; b < sp->beecount; b++) {
@@ -365,7 +365,7 @@ draw_swarm(ModeInfo * mi)
 		/* Move */
 		X(sp->tick, b) = X(prev, b) + sp->xv[b];
 		Y(sp->tick, b) = Y(prev, b) + sp->yv[b];
- 
+
 		/* Fill the segment lists. */
 		sp->segs[b].x1 = (short) X(sp->tick, b);
 		sp->segs[b].y1 = (short) Y(sp->tick, b);
