@@ -26,7 +26,7 @@ static const char sccsid[] = "@(#)swirl.c	5.09 2003/06/30 xlockmore";
  *              xscreensaver's starfish, looks like I am not going to
  *              port the variable speed colormapping when done drawing.
  * 01-Nov-2000: Allocation checks
- * 13-May-1997: jwz@jwz.org: turned into a standalone program.
+ * 13-May-1997: jwz AT jwz.org: turned into a standalone program.
  * 21-Apr-1995: improved startup time for TrueColour displays
  *              (limited to 16bpp to save memory) S.Early <sde1000@cam.ac.uk>
  * 09-Jan-1995: fixed colour maps (more colourful) and the image now spirals
@@ -64,14 +64,14 @@ static XrmOptionDescRec opts[] =
   {(char *) "-cycle", (char *) ".swirl.cycle", XrmoptionNoArg, (caddr_t) "on"},
   {(char *) "+cycle", (char *) ".swirl.cycle", XrmoptionNoArg, (caddr_t) "off"}
 };
- 
+
 static argtype vars[] =
 {
-  {(caddr_t *) & cycle_p, (char *) "cycle", (char *) "Cycle", (char *) DEF_CYCLE, t_Bool}
+  {(void *) & cycle_p, (char *) "cycle", (char *) "Cycle", (char *) DEF_CYCLE, t_Bool}
 };
 
 static OptionStruct desc[] =
-{ 
+{
   {(char *) "-/+cycle", (char *) "turn on/off colour cycling"}
 };
 
@@ -832,7 +832,7 @@ init_swirl(ModeInfo * mi)
 		}
 	}
 	MI_CLEARWINDOW(mi);
- 
+
   /* Set up colour map */
   sp->direction = (LRAND() & 1) ? 1 : -1;
   if (MI_IS_INSTALL(mi) && MI_NPIXELS(mi) > 2) {
