@@ -31,18 +31,24 @@ static const char sccsid[] = "@(#)t3d.c	5.0 2000/11/01 xlockmore";
  *
  * Revision History:
  * 01-Nov-2000: Allocation checks
- * 26-Jan-2000: joukj@hrem.stm.tudelft.nl (Jouk Jansen) : adapted for
+ * 26-Jan-2000: joukj AT hrem.nano.tudelft.nl (Jouk Jansen) : adapted for
  *              xlockmore. Modified colour-schemes.
- * 04-Jan-1999: jwz@jwz.org -- adapted to xscreensaver framework,
+ * 04-Jan-1999: jwz AT jwz.org -- adapted to xscreensaver framework,
  *              to take advantage of the command-line options
  *              provided by screenhack.c.
  */
+#ifdef WIN32
+#include <sys/time.h>
+#else
+#include <time.h>
+#endif /* WIN32 */
 
 #define FASTDRAW
 #define FASTCOPY
 #undef USE_POLYGON
 
 #ifdef STANDALONE
+#define MODE_t3d
 #define PROGCLASS "t3d"
 #define HACK_INIT init_t3d
 #define HACK_DRAW draw_t3d
@@ -59,8 +65,8 @@ static const char sccsid[] = "@(#)t3d.c	5.0 2000/11/01 xlockmore";
 #include "color.h"
 #endif /* STANDALONE */
 
-#ifdef TIME_WITH_SYS_TIME /* for sco */
 #include <time.h>
+#ifdef TIME_WITH_SYS_TIME /* for sco */
 #include <sys/time.h>
 #endif
 
@@ -119,13 +125,13 @@ static XrmOptionDescRec opts[] =
 
 static argtype vars[] =
 {
-	{(caddr_t *) & cycle_p, (char *) "cycle", (char *) "Cycle", (char *) DEF_CYCLE, t_Bool},
-	{(caddr_t *) & move, (char *) "move", (char *) "Move", (char *) DEF_MOVE, t_Float},
-	{(caddr_t *) & wobble, (char *) "wobble", (char *) "Wobble", (char *) DEF_WOBBLE, t_Float},
-	{(caddr_t *) & mag, (char *) "mag", (char *) "Magnification", (char *) DEF_MAG, t_Float},
-	{(caddr_t *) & fastch, (char *) "fast", (char *) "Fast", (char *) DEF_FAST, t_Int},
-        {(caddr_t *) & minutes, (char *) "minutes", (char *) "Minutes", (char *) DEF_MINUTES, t_Bool},
-        {(caddr_t *) & trackmouse, (char *) "trackmouse", (char *) "TrackMouse", (char *) DEF_TRACKMOUSE, t_Bool}
+	{(void *) & cycle_p, (char *) "cycle", (char *) "Cycle", (char *) DEF_CYCLE, t_Bool},
+	{(void *) & move, (char *) "move", (char *) "Move", (char *) DEF_MOVE, t_Float},
+	{(void *) & wobble, (char *) "wobble", (char *) "Wobble", (char *) DEF_WOBBLE, t_Float},
+	{(void *) & mag, (char *) "mag", (char *) "Magnification", (char *) DEF_MAG, t_Float},
+	{(void *) & fastch, (char *) "fast", (char *) "Fast", (char *) DEF_FAST, t_Int},
+        {(void *) & minutes, (char *) "minutes", (char *) "Minutes", (char *) DEF_MINUTES, t_Bool},
+        {(void *) & trackmouse, (char *) "trackmouse", (char *) "TrackMouse", (char *) DEF_TRACKMOUSE, t_Bool}
 };
 
 static OptionStruct desc[] =
@@ -178,7 +184,7 @@ typedef struct {
    Bool        cycle_p, mono_p, no_colors;
    unsigned long blackpixel, whitepixel, fg, bg;
    int         direction;
-   struct tm *zeit;
+   struct tm  *zeit;
    kugeldat kugeln[100];
    int         color_offset;
    ModeInfo   *mi;
@@ -289,8 +295,9 @@ t3d_gettime (void)
   lt = time1.tv_sec;	/* avoid type cast lossage */
 #else
   lt = NULL;
+  time1.tv_sec = 0;
 #endif
-  zeit=localtime(&lt);
+  zeit = localtime((const time_t *) &lt);
 
   return (zeit->tm_sec+60*(zeit->tm_min+60*(zeit->tm_hour))
 	  + time1.tv_usec*1.0E-6);
@@ -355,7 +362,7 @@ t3d_turn(double feld1[], double feld2[], double winkel)
 static void
 t3d_projektion( ModeInfo* mi)
 {
-  double c1[3],c2[3],k[3],x1,y1;
+  double c1[3],c2[3],k[3],x_1,y_1;
   double cno,cnorm/*,magnit*/;
   int i;
    t3dstruct *t3dp;
@@ -387,15 +394,15 @@ t3d_projektion( ModeInfo* mi)
       t3d_vektorprodukt(c2,c1,k);
 
 
-      x1=(t3dp->startx+(t3dp->x[0]*k[0]+t3dp->x[1]*k[1]+t3dp->x[2]*k[2])*mag);
-      y1=(t3dp->starty-(t3dp->y[0]*k[0]+t3dp->y[1]*k[1]+t3dp->y[2]*k[2])*mag);
-      if(   (x1>-2000.0)
-	 && (x1<t3dp->width+2000.0)
-	 && (y1>-2000.0)
-	 && (y1<t3dp->height+2000.0))
+      x_1=(t3dp->startx+(t3dp->x[0]*k[0]+t3dp->x[1]*k[1]+t3dp->x[2]*k[2])*mag);
+      y_1=(t3dp->starty-(t3dp->y[0]*k[0]+t3dp->y[1]*k[1]+t3dp->y[2]*k[2])*mag);
+      if(   (x_1>-2000.0)
+	 && (x_1<t3dp->width+2000.0)
+	 && (y_1>-2000.0)
+	 && (y_1<t3dp->height+2000.0))
 	{
-	  t3dp->kugeln[i].x1=(int)x1;
-	  t3dp->kugeln[i].y1=(int)y1;
+	  t3dp->kugeln[i].x1=(int)x_1;
+	  t3dp->kugeln[i].y1=(int)y_1;
 	}
       else
 	{
@@ -604,7 +611,7 @@ free_t3d(Display *display, t3dstruct *t3dp)
 		if (t3dp->colors != NULL) {
 			if (t3dp->ncolors && !t3dp->no_colors)
 				free_colors(display, t3dp->cmap, t3dp->colors, t3dp->ncolors);
-			(void) free((void *) t3dp->colors);
+			free(t3dp->colors);
 			t3dp->colors = (XColor *) NULL;
 		}
 		if (t3dp->cmap != None) {
@@ -639,7 +646,7 @@ free_t3d(Display *display, t3dstruct *t3dp)
 	}
 #endif
 	if (t3dp->zeit != NULL) {
-		(void) free((void *) t3dp->zeit);
+		free(t3dp->zeit);
 		t3dp->zeit = (struct tm *) NULL;
 	}
 }
@@ -846,7 +853,7 @@ init_t3d(ModeInfo * mi)
 		if (t3dp->colors != NULL) {
 			if (t3dp->ncolors && !t3dp->no_colors)
 				free_colors(display, t3dp->cmap, t3dp->colors, t3dp->ncolors);
-			(void) free((void *) t3dp->colors);
+			free(t3dp->colors);
 			t3dp->colors = (XColor *) NULL;
 		}
 		t3dp->ncolors = MI_NCOLORS(mi);
@@ -949,7 +956,7 @@ release_t3d(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_t3d(MI_DISPLAY(mi), &t3ds[screen]);
-		(void) free((void *) t3ds);
+		free(t3ds);
 		t3ds = (t3dstruct *) NULL;
 	}
 }
