@@ -7,7 +7,7 @@ static const char sccsid[] = "@(#)thornbird.c	5.00 2000/11/01 xlockmore";
 #endif
 
 /*-
- * Copyright (c) 1996 by Tim Auckland <Tim.Auckland@Procket.com>
+ * Copyright (c) 1996 by Tim Auckland <tda10.geo AT yahoo.com>
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted,
@@ -31,6 +31,7 @@ static const char sccsid[] = "@(#)thornbird.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_thornbird
 #define PROGCLASS "Thornbird"
 #define HACK_INIT init_thornbird
 #define HACK_DRAW draw_thornbird
@@ -98,8 +99,8 @@ free_thornbird(thornbirdstruct *hp)
 
 		for (buffer = 0; buffer < hp->nbuffers; buffer++)
 			if (hp->pointBuffer[buffer] != NULL)
-				(void) free((void *) hp->pointBuffer[buffer]);
-		(void) free((void *) hp->pointBuffer);
+				free(hp->pointBuffer[buffer]);
+		free(hp->pointBuffer);
 		hp->pointBuffer = (XPoint **) NULL;
 	}
 }
@@ -182,8 +183,13 @@ draw_thornbird(ModeInfo * mi)
 	if (hp->pointBuffer == NULL)
 		return;
 
-	erase = (hp->inc + 1) % MI_CYCLES(mi);
-	current = hp->inc % MI_CYCLES(mi);
+	if (MI_CYCLES(mi) < 1) {
+		current = 0;
+		erase = 0;
+	} else {
+		erase = (hp->inc + 1) % MI_CYCLES(mi);
+		current = hp->inc % MI_CYCLES(mi);
+	}
 	k = batchcount;
 
 
@@ -191,9 +197,9 @@ draw_thornbird(ModeInfo * mi)
 
 	/* vary papameters */
 	hp->a = 1.99 + (0.4 * sin(hp->inc / hp->liss.f1) +
-					0.05 * cos(hp->inc / hp->liss.f2));
+		0.05 * cos(hp->inc / hp->liss.f2));
 	hp->c = 0.80 + (0.15 * cos(hp->inc / hp->liss.f1) +
-					0.05 * sin(hp->inc / hp->liss.f2));
+			0.05 * sin(hp->inc / hp->liss.f2));
 
 	/* vary view */
 	hp->tumble.theta += hp->tumble.dtheta;
@@ -255,7 +261,7 @@ release_thornbird(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_thornbird(&thornbirds[screen]);
-		(void) free((void *) thornbirds);
+		free(thornbirds);
 		thornbirds = (thornbirdstruct *) NULL;
 	}
 }
