@@ -8,7 +8,7 @@ static const char sccsid[] = "@(#)toneclock.c	5.00 2004/09/16 xlockmore";
 #endif
 
 /*-
- * Copyright (c) 2004 by Jouk Jansen <joukj@hrem.stm.tudelft.nl>
+ * Copyright (c) 2004 by Jouk Jansen <joukj AT hrem.nano.tudelft.nl>
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted,
@@ -468,10 +468,10 @@ init_toneclock(ModeInfo * mi)
 #ifndef NO_DBUF
 	if (tclock->dbuf != None)
 		XFreePixmap(display, tclock->dbuf);
-	        tclock->dbuf = XCreatePixmap(display, window,
-			tclock->win_width,
-			tclock->win_height,
-			MI_DEPTH(mi));
+	tclock->dbuf = XCreatePixmap(display, window,
+		tclock->win_width,
+		tclock->win_height,
+		MI_DEPTH(mi));
 	/* Allocation checked */
 	if (tclock->dbuf != None) {
 		XGCValues   gcv;
@@ -491,8 +491,8 @@ init_toneclock(ModeInfo * mi)
 		} else {
 			XFillRectangle(display, (Drawable) tclock->dbuf, tclock->dbuf_gc,
 				0, 0, tclock->win_width, tclock->win_height);
-			XSetBackground(display, MI_GC(mi), MI_BLACK_PIXEL(mi));
-			XSetFunction(display, MI_GC(mi), GXcopy);
+			/*XSetBackground(display, MI_GC(mi), MI_BLACK_PIXEL(mi));
+			XSetFunction(display, MI_GC(mi), GXcopy);*/
 		}
 	}
 #endif
