@@ -27,16 +27,16 @@ static const char sccsid[] = "@(#)triangle.c	5.00 2000/11/01 xlockmore";
  * 10-May-1997: Compatible with xscreensaver
  * 10-Mar-1996: re-arranged and re-formatted the code for appearance and
  *              to make common subroutines.  Simplified.
- *	            Ron Hitchens <ron@idiom.com>
+ *	            Ron Hitchens <ron AT idiom.com>
  * 07-Mar-1996: Removed internal delay code, set MI_PAUSE(mi) for inter-scene
  *              delays.  No other delays are needed here.
  *              Made pause time sensitive to value of cycles (in 10ths of a
  *              second).  Removed (hopefully) all references to globals.
- *	            Ron Hitchens <ron@idiom.com>
+ *	            Ron Hitchens <ron AT idiom.com>
  * 27-Feb-1996: Undid the changes listed below.  Added ModeInfo argument.
  *	            Implemented delay between scenes using the MI_PAUSE(mi)
- *              scheme.  Ron Hitchens <ron@idiom.com>
- * 27-Dec-1995: Ron Hitchens <ron@idiom.com>
+ *              scheme.  Ron Hitchens <ron AT idiom.com>
+ * 27-Dec-1995: Ron Hitchens <ron AT idiom.com>
  *              Modified logic of draw_triangle() to provide a delay
  *              (sensitive to the value of cycles) between each iteration.
  *              Because this mode is so compute intensive, when the new
@@ -49,6 +49,7 @@ static const char sccsid[] = "@(#)triangle.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_triangle
 #define PROGCLASS "Triangle"
 #define HACK_INIT init_triangle
 #define HACK_DRAW draw_triangle
@@ -85,7 +86,7 @@ ModStruct   triangle_description =
 #define MAX_LEVELS 1000
 
 #define DELTA  0.4
-#define LEFT   (-0.25)
+#define LEFT   -0.25
 #define RIGHT  1.25
 #define TOP    0.3
 #define BOTTOM 1.0
@@ -145,13 +146,15 @@ draw_atriangle(ModeInfo * mi, XPoint * p, int y_0, int y_1, int y_2, double dinv
 
 		XSetForeground(display, gc, MI_PIXEL(mi, color % MI_NPIXELS(mi)));
 		if (tp->joke) {
-			if ((Bool) (LRAND() & 1))
+			if ((Bool) (LRAND() & 1)) {
 				XDrawLines(display, window, gc, p, 4, CoordModeOrigin);
-			else
+			} else {
 				XFillPolygon(display, window, gc, p, 3, Convex, CoordModeOrigin);
+			}
 		} else if (tp->wireframe) {
 			XDrawLines(display, window, gc, p, 4, CoordModeOrigin);
 		} else {
+			/* dieing on my Sun here with gcc -g -O2, flakey */
 			XFillPolygon(display, window, gc, p, 3, Convex, CoordModeOrigin);
 		}
 	} else {
@@ -200,7 +203,7 @@ calc_points2(trianglestruct * tp, int d, int *y0_p, int *y1_p, int *y2_p, XPoint
 	p[1].y = tp->ypos[tp->j + d] - *y1_p;
 	p[2].y = tp->ypos[tp->j + d] - *y2_p;
 
-	p[3] = p[0];
+	p[3]= p[0];
 }
 
 
@@ -214,7 +217,7 @@ draw_mesh(ModeInfo * mi, trianglestruct * tp, int d, int count)
 	double      dinv = 0.2 / d;
 
 	if ((tp->j == 0) && (tp->i == 0)) {
-		MI_CLEARWINDOW(mi);
+		MI_CLEARWINDOWCOLORMAPFAST(mi, MI_GC(mi), MI_BLACK_PIXEL(mi));
 	}
 	for (; (tp->j < tp->size) && (count > 0); tp->j += ((count) ? d : 0)) {
 		for (tp->i = (first) ? tp->i : 0, first = 0;
@@ -360,7 +363,7 @@ void
 release_triangle(ModeInfo * mi)
 {
 	if (triangles != NULL) {
-		(void) free((void *) triangles);
+		free(triangles);
 		triangles = (trianglestruct *) NULL;
 	}
 }
