@@ -146,11 +146,12 @@ typedef struct {
 	int         width, height;
 	CellList   *last, *first;
 	char       *arr;
-	int         neighbors;
+	int         neighbors, polygon;
 	int         busyLoop;
 	union {
 		XPoint      hexagon[6];
 		XPoint      triangle[2][3];
+		XPoint      pentagon[4][5];
 	} shape;
 } voterstruct;
 
@@ -163,7 +164,7 @@ static voterstruct *voters = (voterstruct *) NULL;
 static int  icon_width, icon_height;
 
 static void
-drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
+drawCell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
 		Bool firstChange)
 {
 	Display    *display = MI_DISPLAY(mi);
@@ -233,7 +234,7 @@ drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
 				XFillArc(display, window, gc,
 					vp->shape.hexagon[0].x + ix,
 					vp->shape.hexagon[0].y + iy,
-					sy, sy,
+					sx, sy,
 					0, 23040);
 		}
 	} else if (vp->neighbors == 4 || vp->neighbors == 8) {
@@ -381,225 +382,366 @@ flush_list(voterstruct * vp)
 	}
 }
 
-static char
-neighbors_opinion(voterstruct * vp, int col, int row, int dir)
+
+static int
+positionOfNeighbor(voterstruct * vp, int n, int col, int row)
 {
-	if (vp->neighbors == 6) {
+	int dir = n * (360 / vp->neighbors);
+
+	if (vp->polygon == 4 || vp->polygon == 6) {
 		switch (dir) {
-			case 0:
+		case 0:
+			col = (col + 1 == vp->ncols) ? 0 : col + 1;
+			break;
+		case 45:
+			col = (col + 1 == vp->ncols) ? 0 : col + 1;
+			row = (!row) ? vp->nrows - 1 : row - 1;
+			break;
+		case 60:
+			if (!(row & 1))
 				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+			row = (!row) ? vp->nrows - 1 : row - 1;
+			break;
+		case 90:
+			row = (!row) ? vp->nrows - 1 : row - 1;
+			break;
+		case 120:
+			if (row & 1)
+				col = (!col) ? vp->ncols - 1 : col - 1;
+			row = (!row) ? vp->nrows - 1 : row - 1;
+			break;
+		case 135:
+			col = (!col) ? vp->ncols - 1 : col - 1;
+			row = (!row) ? vp->nrows - 1 : row - 1;
+			break;
+		case 180:
+			col = (!col) ? vp->ncols - 1 : col - 1;
+			break;
+		case 225:
+			col = (!col) ? vp->ncols - 1 : col - 1;
+			row = (row + 1 == vp->nrows) ? 0 : row + 1;
+			break;
+		case 240:
+			if (row & 1)
+				col = (!col) ? vp->ncols - 1 : col - 1;
+			row = (row + 1 == vp->nrows) ? 0 : row + 1;
+			break;
+		case 270:
+			row = (row + 1 == vp->nrows) ? 0 : row + 1;
+			break;
+		case 300:
+			if (!(row & 1))
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+			row = (row + 1 == vp->nrows) ? 0 : row + 1;
+			break;
+		case 315:
+			col = (col + 1 == vp->ncols) ? 0 : col + 1;
+			row = (row + 1 == vp->nrows) ? 0 : row + 1;
+			break;
+		default:
+			(void) fprintf(stderr, "wrong direction %d\n", dir);
+		}
+	} else if (vp->polygon == 3) {
+		if ((col + row) & 1) {	/* right */
+			switch (dir) {
+			case 0:
+				col = (!col) ? vp->ncols - 1 : col - 1;
+				break;
+			case 30:
+			case 40:
+				col = (!col) ? vp->ncols - 1 : col - 1;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
 				break;
 			case 60:
-				if (!(row & 1))
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-				row = (!row) ? vp->nrows - 1 : row - 1;
+				col = (!col) ? vp->ncols - 1 : col - 1;
+				if (row + 1 == vp->nrows)
+					row = 1;
+				else if (row + 2 == vp->nrows)
+					row = 0;
+				else
+					row = row + 2;
+				break;
+			case 80:
+			case 90:
+				if (row + 1 == vp->nrows)
+					row = 1;
+				else if (row + 2 == vp->nrows)
+					row = 0;
+				else
+					row = row + 2;
 				break;
 			case 120:
-				if (row & 1)
-					col = (!col) ? vp->ncols - 1 : col - 1;
-				row = (!row) ? vp->nrows - 1 : row - 1;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 150:
+			case 160:
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
 				break;
 			case 180:
-				col = (!col) ? vp->ncols - 1 : col - 1;
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+				break;
+			case 200:
+			case 210:
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+				row = (!row) ? vp->nrows - 1 : row - 1;
 				break;
 			case 240:
-				if (row & 1)
-					col = (!col) ? vp->ncols - 1 : col - 1;
-				row = (row + 1 == vp->nrows) ? 0 : row + 1;
-				break;
-			case 300:
-				if (!(row & 1))
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-				row = (row + 1 == vp->nrows) ? 0 : row + 1;
-				break;
-			default:
-				(void) fprintf(stderr, "wrong direction %d\n", dir);
-		}
-	} else if (vp->neighbors == 4 || vp->neighbors == 8) {
-		switch (dir) {
-			case 0:
-				col = (col + 1 == vp->ncols) ? 0 : col + 1;
-				break;
-			case 45:
-				col = (col + 1 == vp->ncols) ? 0 : col + 1;
 				row = (!row) ? vp->nrows - 1 : row - 1;
 				break;
-			case 90:
-				row = (!row) ? vp->nrows - 1 : row - 1;
-				break;
-			case 135:
-				col = (!col) ? vp->ncols - 1 : col - 1;
-				row = (!row) ? vp->nrows - 1 : row - 1;
-				break;
-			case 180:
-				col = (!col) ? vp->ncols - 1 : col - 1;
-				break;
-			case 225:
-				col = (!col) ? vp->ncols - 1 : col - 1;
-				row = (row + 1 == vp->nrows) ? 0 : row + 1;
-				break;
 			case 270:
-				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+			case 280:
+				if (!row)
+					row = vp->nrows - 2;
+				else if (!(row - 1))
+					row = vp->nrows - 1;
+				else
+					row = row - 2;
 				break;
-			case 315:
-				col = (col + 1 == vp->ncols) ? 0 : col + 1;
-				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+			case 300:
+				col = (!col) ? vp->ncols - 1 : col - 1;
+				if (!row)
+					row = vp->nrows - 2;
+				else if (!(row - 1))
+					row = vp->nrows - 1;
+				else
+					row = row - 2;
+				break;
+			case 320:
+			case 330:
+				col = (!col) ? vp->ncols - 1 : col - 1;
+				row = (!row) ? vp->nrows - 1 : row - 1;
 				break;
 			default:
-				(void) fprintf(stderr, "wrong direction %d\n", dir);
-		}
-	} else {		/* TRI */
-		if ((col + row) % 2) {	/* right */
-			switch (dir) {
-				case 0:
-					col = (!col) ? vp->ncols - 1 : col - 1;
-					break;
-				case 30:
-				case 40:
-					col = (!col) ? vp->ncols - 1 : col - 1;
-					row = (row + 1 == vp->nrows) ? 0 : row + 1;
-					break;
-				case 60:
-					col = (!col) ? vp->ncols - 1 : col - 1;
-					if (row + 1 == vp->nrows)
-						row = 1;
-					else if (row + 2 == vp->nrows)
-						row = 0;
-					else
-						row = row + 2;
-					break;
-				case 80:
-				case 90:
-					if (row + 1 == vp->nrows)
-						row = 1;
-					else if (row + 2 == vp->nrows)
-						row = 0;
-					else
-						row = row + 2;
-					break;
-				case 120:
-					row = (row + 1 == vp->nrows) ? 0 : row + 1;
-					break;
-				case 150:
-				case 160:
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-					row = (row + 1 == vp->nrows) ? 0 : row + 1;
-					break;
-				case 180:
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-					break;
-				case 200:
-				case 210:
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-					row = (!row) ? vp->nrows - 1 : row - 1;
-					break;
-				case 240:
-					row = (!row) ? vp->nrows - 1 : row - 1;
-					break;
-				case 270:
-				case 280:
-					if (!row)
-						row = vp->nrows - 2;
-					else if (!(row - 1))
-						row = vp->nrows - 1;
-					else
-						row = row - 2;
-
-					break;
-				case 300:
-					col = (!col) ? vp->ncols - 1 : col - 1;
-					if (!row)
-						row = vp->nrows - 2;
-					else if (!(row - 1))
-						row = vp->nrows - 1;
-					else
-						row = row - 2;
-					break;
-				case 320:
-				case 330:
-					col = (!col) ? vp->ncols - 1 : col - 1;
-					row = (!row) ? vp->nrows - 1 : row - 1;
-					break;
-				default:
-					(void) fprintf(stderr, "wrong direction %d\n", dir);
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
 			}
 		} else {	/* left */
 			switch (dir) {
-				case 0:
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-					break;
-				case 30:
-				case 40:
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-					row = (!row) ? vp->nrows - 1 : row - 1;
-					break;
-				case 60:
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-					if (!row)
-						row = vp->nrows - 2;
-					else if (row == 1)
-						row = vp->nrows - 1;
-					else
-						row = row - 2;
-					break;
-				case 80:
-				case 90:
-					if (!row)
-						row = vp->nrows - 2;
-					else if (row == 1)
-						row = vp->nrows - 1;
-					else
-						row = row - 2;
-					break;
-				case 120:
-					row = (!row) ? vp->nrows - 1 : row - 1;
-					break;
-				case 150:
-				case 160:
-					col = (!col) ? vp->ncols - 1 : col - 1;
-					row = (!row) ? vp->nrows - 1 : row - 1;
-					break;
-				case 180:
-					col = (!col) ? vp->ncols - 1 : col - 1;
-					break;
-				case 200:
-				case 210:
-					col = (!col) ? vp->ncols - 1 : col - 1;
-					row = (row + 1 == vp->nrows) ? 0 : row + 1;
-					break;
-				case 240:
-					row = (row + 1 == vp->nrows) ? 0 : row + 1;
-					break;
-				case 270:
-				case 280:
-					if (row + 1 == vp->nrows)
-						row = 1;
-					else if (row + 2 == vp->nrows)
-						row = 0;
-					else
-						row = row + 2;
-					break;
-				case 300:
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-					if (row + 1 == vp->nrows)
-						row = 1;
-					else if (row + 2 == vp->nrows)
-						row = 0;
-					else
-						row = row + 2;
-					break;
-				case 320:
-				case 330:
-					col = (col + 1 == vp->ncols) ? 0 : col + 1;
-					row = (row + 1 == vp->nrows) ? 0 : row + 1;
-					break;
-				default:
-					(void) fprintf(stderr, "wrong direction %d\n", dir);
+			case 0:
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+				break;
+			case 30:
+			case 40:
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+				row = (!row) ? vp->nrows - 1 : row - 1;
+				break;
+			case 60:
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+				if (!row)
+					row = vp->nrows - 2;
+				else if (row == 1)
+					row = vp->nrows - 1;
+				else
+					row = row - 2;
+				break;
+			case 80:
+			case 90:
+				if (!row)
+					row = vp->nrows - 2;
+				else if (row == 1)
+					row = vp->nrows - 1;
+				else
+					row = row - 2;
+				break;
+			case 120:
+				row = (!row) ? vp->nrows - 1 : row - 1;
+				break;
+			case 150:
+			case 160:
+				col = (!col) ? vp->ncols - 1 : col - 1;
+				row = (!row) ? vp->nrows - 1 : row - 1;
+				break;
+			case 180:
+				col = (!col) ? vp->ncols - 1 : col - 1;
+				break;
+			case 200:
+			case 210:
+				col = (!col) ? vp->ncols - 1 : col - 1;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 240:
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 270:
+			case 280:
+				if (row + 1 == vp->nrows)
+					row = 1;
+				else if (row + 2 == vp->nrows)
+					row = 0;
+				else
+					row = row + 2;
+				break;
+			case 300:
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+				if (row + 1 == vp->nrows)
+					row = 1;
+				else if (row + 2 == vp->nrows)
+					row = 0;
+				else
+					row = row + 2;
+				break;
+			case 320:
+			case 330:
+				col = (col + 1 == vp->ncols) ? 0 : col + 1;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
 			}
 		}
+#if 0
+	} else {
+		int orient = ((row & 1) * 2 + col) % 4;
+		switch (orient) { /* up, down, left, right */
+		case 0:
+			switch (dir) {
+			case 0:
+				col++;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col + 2 >= vp->ncols) ? 0 : col + 2;
+				break;
+			case 102: /* 7 corner */
+				col = (col + 3 >= vp->ncols) ? 1 : col + 3;
+				row = (row == 0) ? vp->nrows - 1 : row - 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col++;
+				row = (row == 0) ? vp->nrows - 1 : row - 1;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				row = (row == 0) ? vp->nrows - 1 : row - 1;
+				break;
+			case 255: /* 7 */
+				col = (col == 0) ? vp->ncols - 1 : col - 1;
+				row = (row == 0) ? vp->nrows - 1 : row - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				col = (col == 0) ? vp->ncols - 1 : col - 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 1:
+			switch (dir) {
+			case 0:
+				col--;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col == 1) ? vp->ncols - 1 : col - 2;
+				break;
+			case 102: /* 7 */
+				col = (col == 1) ? vp->ncols - 2 : col - 3;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col--;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 255: /* 7 */
+				col = (col + 1 >= vp->ncols) ? 0 : col + 1;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				col = (col + 1 >= vp->ncols) ? 0 : col + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 2:
+			switch (dir) {
+			case 0:
+				col = (col + 1 >= vp->ncols) ? 0 : col + 1;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				row = (row == 0) ? vp->nrows - 1 : row - 1;
+				col++;
+				break;
+			case 102: /* 7 */
+				col = (col == 0) ? vp->ncols - 1 : col - 1;
+				row = (row == 0) ? vp->nrows - 1 : row - 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col = (col == 0) ? vp->ncols - 2 : col - 2;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				col = (col == 0) ? vp->ncols - 1 : col - 1;
+				break;
+			case 255: /* 7 */
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				col = (col == 0) ? vp->ncols - 1 : col - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 3:
+			switch (dir) {
+			case 0:
+				col--;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col == 0) ? vp->ncols - 1 : col - 1;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 102: /* 7 */
+				col = (col + 1 >= vp->ncols) ? 0 : col + 1;
+				row = (row + 1 == vp->nrows) ? 0 : row + 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col = (col + 2 >= vp->ncols) ? 1 : col + 2;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				col = (col + 1 >= vp->ncols) ? 0 : col + 1;
+				break;
+			case 255: /* 7 */
+				col = (col + 1 >= vp->ncols) ? 0 : col + 1;
+				row = (row == 0) ? vp->nrows - 1 : row - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				row = (row == 0) ? vp->nrows - 1 : row - 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		default:
+			(void) fprintf(stderr, "wrong orient %d\n",
+				orient);
+		}
+#endif
 	}
-
-	return vp->arr[row * vp->ncols + col];
+	return (row * vp->ncols + col);
 }
 
 static void
@@ -618,7 +760,7 @@ advanceColors(ModeInfo * mi, int col, int row)
 				curr->info.age--;
 			else if (curr->info.age < 0)
 				curr->info.age++;
-			drawcell(mi, curr->info.col, curr->info.row,
+			drawCell(mi, curr->info.col, curr->info.row,
 				 (MI_NPIXELS(mi) + curr->info.age / FACTOR +
 				  (MI_NPIXELS(mi) * curr->info.kind / BITMAPS)) % MI_NPIXELS(mi),
 				 curr->info.kind, False);
@@ -650,7 +792,7 @@ refresh_voters(ModeInfo * mi)
 			for (col = 0; col < vp->ncols; col++) {
 				colrow = col + row * vp->ncols;
 				/* Draw all old, will get corrected soon if wrong... */
-				drawcell(mi, col, row,
+				drawCell(mi, col, row,
 					 (unsigned long) (MI_NPIXELS(mi) * vp->arr[colrow] / BITMAPS),
 					 vp->arr[colrow], False);
 			}
@@ -731,6 +873,7 @@ init_voters(ModeInfo * mi)
 	if (vp->neighbors == 6) {
 		int nccols, ncrows, sides;
 
+		vp->polygon = 6;
 		if (!vp->vertical) {
 			vp->height = MI_WIDTH(mi);
 			vp->width = MI_HEIGHT(mi);
@@ -775,6 +918,7 @@ init_voters(ModeInfo * mi)
 			}
 		}
 	} else if (vp->neighbors == 4 || vp->neighbors == 8) {
+		vp->polygon = 4;
 		if (vp->width < 2)
 			vp->width = 2;
 		if (vp->height < 2)
@@ -810,6 +954,7 @@ init_voters(ModeInfo * mi)
 	} else {		/* TRI */
 		int orient, sides;
 
+		vp->polygon = 3;
 		if (!vp->vertical) {
 			vp->height = MI_WIDTH(mi);
 			vp->width = MI_HEIGHT(mi);
@@ -882,7 +1027,7 @@ init_voters(ModeInfo * mi)
 			else
 				i = NRAND(vp->nparties);
 			vp->arr[colrow] = (char) i;
-			drawcell(mi, col, row, (unsigned long) (MI_NPIXELS(mi) * i / BITMAPS),
+			drawCell(mi, col, row, (unsigned long) (MI_NPIXELS(mi) * i / BITMAPS),
 				 i, False);
 			vp->number_in_party[i]++;
 		}
@@ -917,12 +1062,12 @@ draw_voters(ModeInfo * mi)
 			init_voters(mi);	/* Create a more interesting planet */
 		}
 	spineless_dude = NRAND(vp->npositions);
-	neighbor_direction = NRAND(vp->neighbors) * 360 / vp->neighbors;
+	neighbor_direction = NRAND(vp->neighbors);
 	spineless_col = spineless_dude % vp->ncols;
 	spineless_row = spineless_dude / vp->ncols;
 	old_opinion = vp->arr[spineless_dude];
-	new_opinion = neighbors_opinion(vp, spineless_col, spineless_row,
-					neighbor_direction);
+	/* neighbors opinion */
+	new_opinion = vp->arr[positionOfNeighbor(vp, neighbor_direction, spineless_col, spineless_row)];
 	if (old_opinion != new_opinion) {
 		vp->number_in_party[old_opinion]--;
 		vp->number_in_party[new_opinion]++;
@@ -943,7 +1088,7 @@ draw_voters(ModeInfo * mi)
 				return;
 			}
 		}
-		drawcell(mi, spineless_col, spineless_row,
+		drawCell(mi, spineless_col, spineless_row,
 			 (MI_NPIXELS(mi) + info.age / FACTOR +
 		  (MI_NPIXELS(mi) * new_opinion / BITMAPS)) % MI_NPIXELS(mi),
 			 new_opinion, True);
