@@ -6,7 +6,7 @@ static const char sccsid[] = "@(#)xjack.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
-/* xscreensaver, Copyright (c) 1997 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1997 Jamie Zawinski <jwz AT jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -62,7 +62,28 @@ ModStruct xjack_description =
 
 #endif
 
-#define font_height(f) ((f==None)?8:f->ascent + f->descent)
+#if USE_MB
+static int font_height(XFontStruct *f)
+{
+        XRectangle mbRect;
+
+        XmbTextExtents(fontset, "Aj", 1, NULL, &mbRect);
+        return mbRect.height;
+}
+#define font_ascent(f) (font_height(f)*4/5)
+static int char_width(XFontStruct *f)
+{
+        XRectangle mbRect;
+
+        XmbTextExtents(fontset, "A", 1, NULL, &mbRect);
+        return mbRect.width;
+}
+
+#else
+#define font_height(f) ((f==None)?15:f->ascent + f->descent)
+#define font_ascent(f) ((f==None)?12:f->ascent)
+#define char_width(f) ((mode_font->per_char?mode_font->per_char['n' - mode_font->min_char_or_byte2].rbearing:mode_font->min_bounds.rbearing))
+#endif
 
 extern XFontStruct *getFont(Display * display);
 
@@ -122,7 +143,6 @@ init_xjack(ModeInfo * mi)
 	jackstruct *jp;
 	XGCValues gcv;
 
-
 	if (jacks == NULL) {
 		if ((jacks = (jackstruct *) calloc(MI_NUM_SCREENS(mi),
 			 sizeof (jackstruct))) == NULL)
@@ -134,9 +154,8 @@ init_xjack(ModeInfo * mi)
 			release_xjack(mi);
 		}
 
-	if (jp->gc == NULL) {
+	if (jp->gc == None) {
 		gcv.font = mode_font->fid;
-		XSetFont(display, MI_GC(mi), mode_font->fid);
 		gcv.graphics_exposures = False;
 		gcv.foreground = MI_WHITE_PIXEL(mi);
 		gcv.background = MI_BLACK_PIXEL(mi);
@@ -145,7 +164,7 @@ init_xjack(ModeInfo * mi)
 				&gcv)) == None) {
 			return;
 		}
-		jp->ascent = mode_font->ascent;
+		jp->ascent = font_ascent(mode_font);
 		jp->height = font_height(mode_font);
 	}
 
@@ -153,11 +172,11 @@ init_xjack(ModeInfo * mi)
 	jp->win_height = MI_HEIGHT(mi);
 
 	jp->hspace = jp->vspace = 15;  /* pixels */
-	jp->char_width = (mode_font->per_char
-				? mode_font->per_char['n' - mode_font->min_char_or_byte2].rbearing
-				: mode_font->min_bounds.rbearing);
+	jp->char_width = char_width(mode_font);
 	if (!jp->char_width)
-		jp->char_width = 1;
+		jp->char_width = 10;
+	if (!jp->height)
+		jp->height = 15;
 	jp->line_height = jp->height + 1;
 
 	jp->columns = MAX((jp->win_width - jp->hspace - jp->hspace) / jp->char_width, 2);
@@ -317,7 +336,7 @@ draw_xjack(ModeInfo * mi)
 			/* caps typo */
 			if (c >= 'a' && c <= 'z' && (jp->caps || !NRAND(350))) {
 				c -= ('a' - 'A');
-				if (c == 'O' && LRAND() & 1)
+				if (c == 'O' && (LRAND() & 1))
 					c = '0';
 			}
 
@@ -327,7 +346,7 @@ draw_xjack(ModeInfo * mi)
 				do {
 					(void) XDrawString (display, window, jp->gc,
 							(jp->x * jp->char_width) + jp->hspace + xshift,
-							(jp->y * jp->line_height) + jp->vspace + mode_font->ascent + yshift,
+							(jp->y * jp->line_height) + jp->vspace + jp->ascent + yshift,
 							&c, 1);
 					overstrike = (xshift == 0 && yshift == 0 &&
 						(0 == (LRAND() & 3000)));
@@ -356,7 +375,7 @@ draw_xjack(ModeInfo * mi)
 		jp->s++;
 
 		if (!NRAND(200)) {
-			if (LRAND() & 1 && jp->s != source_message)
+			if ((LRAND() & 1) && jp->s != source_message)
 				jp->s--;   /* duplicate character */
 			else if (*(jp->s))
 				jp->s++;   /* skip character */
@@ -434,7 +453,7 @@ draw_xjack(ModeInfo * mi)
 			while (*n1) {
 				(void) XDrawString (display, window, jp->gc,
 									(jp->x * jp->char_width) + jp->hspace,
-									(jp->y * jp->line_height) + jp->vspace + mode_font->ascent,
+									(jp->y * jp->line_height) + jp->vspace + jp->ascent,
 									n1, 1);
 				jp->x++;
 				if (jp->x >= jp->columns)
@@ -448,7 +467,7 @@ draw_xjack(ModeInfo * mi)
 			while (*n2) {
 				(void) XDrawString (display, window, jp->gc,
 								(jp->x * jp->char_width) + jp->hspace,
-								(jp->y * jp->line_height) + jp->vspace + mode_font->ascent,
+								(jp->y * jp->line_height) + jp->vspace + jp->ascent,
 								n2, 1);
 				jp->x++;
 				if (jp->x >= jp->columns)
