@@ -14,6 +14,7 @@ static const char sccsid[] = "@(#)Xapi.c	4.07 98/04/16 xlockmore";
 
 #define NUMBER_GC		(1024)
 #define NUMBER_BITMAP	(1024)
+#define WINDOW_DEPTH	(8) /* incorrect, but we'll set it to 8 */
 
 typedef struct {
 	unsigned long	background;
@@ -62,6 +63,7 @@ typedef struct {
 typedef struct {
 	int				isActive;
 	int				needsReset;
+	int				depth;
 	TColor			color;
 	TFillRule		fillRule;
 	TFillStyle		fillStyle;
@@ -83,7 +85,7 @@ typedef struct {
 
 /* -------------------------------------------------------------------- */
 
-int GCCreate(void);
+int GCCreate(Drawable d);
 HDC GCGetDC(Drawable d, GC gc);
 int BMInfoCreate(void);
 HBITMAP DrawableGetBitmap(Drawable d);
@@ -106,12 +108,18 @@ HANDLE thread;                         /* thread handle */
 DWORD thread_id;                       /* thread id number */
 
 int      numGCInfo = 0;                /* number of GC Info */
-TGCInfo gcInfo[NUMBER_GC + 1] = { 0 }; /* information about GCs. Allow
+TGCInfo gcInfo[NUMBER_GC + 1] = { {0} }; /* information about GCs. Allow
                                           NUMBER_GC GC's, + 1 as a spill
 										  over for any modes that need
 										  more */
 int      numBMInfo = 0;                /* number of Bitmap Info */
-TBMInfo bmInfo[NUMBER_BITMAP + 1] = { 0 }; /* information about Bitmaps/Pixmaps */
+TBMInfo bmInfo[NUMBER_BITMAP + 1] = { {0} }; /* information about Bitmaps/Pixmaps */
+
+double
+pow(double x, double y)
+{
+        return exp(y * log(x));
+}
 
 /* -------------------------------------------------------------------- */
 
@@ -130,7 +138,7 @@ void SetByte(LPBYTE lpBMData, BYTE val, int loc, int maxLoc)
 void SetGraphic(LPBYTE lpBMData, XImage *image)
 {
   const int w32bb = 4 * 8;
-  const int xbmbb = 1 * 8;
+  /*const int xbmbb = 1 * 8;*/
   BYTE xbmval;
   BYTE w32val;
   int lineBitCnt = 0;
@@ -286,7 +294,7 @@ LCreateBitmap(XImage *image, HBITMAP *hBitmap, HPALETTE *hPalette)
 /* the following functions to create, destroy, modify and access the 
    GC Info structure
  */
-int GCCreate(void)
+int GCCreate(Drawable d)
 {
 	int	retVal		= 0;
 
@@ -301,6 +309,8 @@ int GCCreate(void)
 
 	gcInfo[retVal].isActive = TRUE;
 	gcInfo[retVal].needsReset = TRUE;
+	gcInfo[retVal].depth =
+		d > 0 && d < NUMBER_BITMAP ? bmInfo[d].type : WINDOW_DEPTH;
 	gcInfo[retVal].color.background = NUMCOLORS+1;
 	gcInfo[retVal].color.foreground = NUMCOLORS;
 	gcInfo[retVal].fillRule.hasFillRule = FALSE;
@@ -695,7 +705,6 @@ HDC GCGetDC(Drawable d, GC gc)
 int BMInfoCreate(void)
 {
 	int retVal	= 0;
-	HBITMAP hBitmap;
 
 	/* start at 1, not 0 as this will be None */
 	for (retVal=1; retVal<NUMBER_BITMAP; retVal++)
@@ -1099,8 +1108,8 @@ int XCopyArea(Display *display, Drawable src, Drawable dest, GC gc,
 			  int dest_x, int dest_y)
 {
 	HBITMAP	hBitmap;
-	HBITMAP	hOldBitmapS;
-	HBITMAP	hOldBitmapD;
+	HBITMAP	hOldBitmapS = NULL;
+	HBITMAP	hOldBitmapD = NULL;
 	HDC		hDestDC;
 	HDC		hSrcDC;
 
@@ -1173,13 +1182,14 @@ Colormap XCopyColormapAndFree(Display *display, Colormap colormap)
 
 /*-
  *  XCopyPlane
- *    to be implemented
+ *    copies a single plane of one drawable to another
  */
 int XCopyPlane(Display *display, Drawable src, Drawable dest, GC gc,
 			  int src_x, int src_y, unsigned width, int height, 
 			  int dest_x, int dest_y, unsigned long plane)
 {
-	return 0;
+	return XCopyArea(display, src, dest, gc,
+					 src_x, src_y, width, height, dest_x, dest_y);
 }
 
 /*-
@@ -1248,25 +1258,13 @@ GC XCreateGC(Display *display, Drawable drawable,
 	GC		newGc;
 	char	message[80];
 
-	if ((newGc = GCCreate()) == -1)
+	if ((newGc = GCCreate(drawable)) == -1)
 	{
 		sprintf(message, "Failed to create a GC");
 		xlockmore_set_debug(message);
 	}
 	else
 	{
-		/* check if drawable is a bitmap, and adjust the 1 and 0 colors
-		   (white and black) to their proper win32 values */
-		if (drawable > 0 && drawable < NUMBER_BITMAP)
-			if (bmInfo[drawable].isActive == TRUE && bmInfo[drawable].type == 1)
-			{
-				if (valuemask & GCBackground)
-					if (values->background == 0 || values->background == 1)
-						values->background = 256 + values->background;
-				if (valuemask & GCForeground)
-					if (values->foreground == 0 || values->foreground == 1)
-						values->foreground = 256 + values->foreground;
-			}
 		XChangeGC(display, newGc, valuemask, values);
 	}
 
@@ -1356,7 +1354,7 @@ Pixmap XCreatePixmapFromBitmapData(Display *display, Drawable drawable,
 	HDC		hBitmapDC;
 	HBITMAP	hOldBitmap;
 	Pixmap	pm;
-	RGBQUAD	rgbQuad[2] = { 0 };
+	RGBQUAD	rgbQuad[2] = { {0} };
 
 	/* create the bitmap */
 	if ((pm = XCreateBitmapFromData(display, drawable, data, width, height)) == None)
@@ -2178,7 +2176,7 @@ Status XGetWindowAttributes(Display *display, Window w,
 
 	window_attr_return->width = rc.right;
 	window_attr_return->height = rc.bottom;
-	window_attr_return->depth = 8; /* incorrect, but we'll set it to 8 */
+	window_attr_return->depth = WINDOW_DEPTH;
 	window_attr_return->root = (int)hwnd;
 
 	return 0;
@@ -2405,7 +2403,7 @@ int XPutPixel(XImage *ximage, int x, int y, unsigned long pixel)
 		for (i=0; i<8; i++)
 		{
 			val <<= 1;
-			val = (val & 0xFE) + ((byte_pos != i) ? oldval >> i : pixel) & 0x01;
+			val = ((val & 0xFE) + ((byte_pos != i) ? oldval >> i : pixel)) & 0x01;
 		}
 	}
 	else
@@ -2591,6 +2589,15 @@ void XSetBackground(Display *display, GC gc, unsigned long background)
 	if (gc >= 0 && gc < numGCInfo && gcInfo[gc].isActive == TRUE)
 	{
 		gcInfo[gc].needsReset = TRUE;
+
+		/* check if drawable is a bitmap, and adjust the 1 and 0 colors
+		   (white and black) to their proper win32 values */
+		if (gcInfo[gc].depth == 1)
+		{
+			if (background == 0 || background == 1)
+				background = 256 + background;
+		}
+
 		/* check that the background is in range */
 		if (background > (NUMCOLORS+1))
 			gcInfo[gc].color.background = NUMCOLORS-1;
@@ -2651,6 +2658,15 @@ void XSetForeground(Display *display, GC gc, unsigned long foreground)
 	if (gc >= 0 && gc < numGCInfo && gcInfo[gc].isActive == TRUE)
 	{
 		gcInfo[gc].needsReset = TRUE;
+
+		/* check if drawable is a bitmap, and adjust the 1 and 0 colors
+		   (white and black) to their proper win32 values */
+		if (gcInfo[gc].depth == 1)
+		{
+			if (foreground == 0 || foreground == 1)
+				foreground = 256 + foreground;
+		}
+
 		/* check that the foreground is in range */
 		if (foreground > (NUMCOLORS+1))
 			gcInfo[gc].color.foreground = NUMCOLORS-1;
@@ -2851,6 +2867,13 @@ void XUnmapWindow(Display *display, Window w)
 {
 }
 
+void glXSwapBuffers(Display *display, Window w)
+{
+	SwapBuffers(hdc);
+}
+
+
+
 #endif /* WIN32 */
 
 /* -------------------------------------------------------------------- */
