@@ -6,7 +6,9 @@
  * See xlock.c for copying information.
  *
  * Revision History:
- * Nov-99: updates by Eric Lassauge <lassauge@mail.dotcom.fr>:
+ * Nov-04: updates by Eric Lassauge <lassauge AT users.sourceforge.net>
+ * 	   - Version 3.0 : give a first try at GTK 2.x !
+ * Nov-99: updates by Eric Lassauge <lassauge AT users.sourceforge.net>
  *         - Callbacks for 'Cancel' buttons , don't SIGSEGV anymore
  *         - FontSelection dialog updates for GTK 1.2.6
  *         - Drawing of little colored rectangle simplified, now works in
@@ -14,8 +16,8 @@
  *         - Add 'About' box
  *         - Add handling of "similar to xmlock" RC file
  *         - etc...
- * Jul-98: Continual improvements by Remi Cohen-Scali <remi.cohenscali@pobox.com>
- * ?? -98: First written by Charles Vidal <vidalc@club-internet.fr>.
+ * Jul-98: Continual improvements by Remi Cohen-Scali <remi.cohenscali AT pobox.com>
+ * ?? -98: First written by Charles Vidal <cvidal AT ivsweb.com>.
  */
 
 /*-
@@ -58,6 +60,7 @@
 #define FORK_STR                        "fork"
 #endif
 
+#include <X11/Xlib.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
@@ -99,7 +102,7 @@ static pid_t *launchedprocesses =(pid_t *)NULL;
 static int num_launchedprocesses = 0;
 static int sz_launchedprocesses = 0;
 static int selected_mode = -1;
-static GtkWidget *font_sel_entry =(GtkWidget *)NULL;
+/*static GtkWidget *font_sel_entry =(GtkWidget *)NULL;*/
 static GtkTooltips *tooltips=(GtkTooltips *)NULL;
 
 /* Prototypes */
@@ -130,7 +133,7 @@ static void __PROTO( color_selection_ok_cb, STD_CB_ARGS );
 static void __PROTO( color_selection_changed_cb, STD_CB_ARGS );
 static void __PROTO( create_color_selection_dialog, STD_CB_ARGS );
 
-static void __PROTO( font_select_item_select_cb, STD_CB_ARGS );
+/*static void __PROTO( font_select_item_select_cb, STD_CB_ARGS );*/
 static void __PROTO( font_select_cancel_cb, STD_CB_ARGS );
 static void __PROTO( font_select_ok_cb, STD_CB_ARGS );
 static void __PROTO( create_font_selection_dialog, STD_CB_ARGS );
@@ -152,6 +155,12 @@ static void __PROTO( create_fntColorOptions_entries, (GtkWidget *) );
 static void __PROTO( create_genOptions_entries, (GtkWidget *) );
 static void __PROTO( create_boolOptions_buttons, (GtkWidget *) );
 
+#if GTK_CHECK_VERSION(2,0,0)
+/* GTK+ 2.x differences */
+#define    gtk_color_selection_set_opacity(colorsel,has_opacity) \
+	gtk_color_selection_set_has_opacity_control(colorsel,(gboolean)has_opacity)
+#endif
+
 /*
  * secured_malloc
  * --------------
@@ -240,11 +249,11 @@ compose_xlock_command(gchar ***cmd_argv_p,
 
 #ifdef XGLOCK_DEBUG
 # define CHECK_REALLOC(num, size, need)                                                 \
-    fprintf(stderr,                                                                     \
+    (void) fprintf(stderr,                                                              \
             "check realloc: current=%ld need=%d size=%ld\n",                            \
             num, need, size );                                                          \
     if (size <= num+need) {                                                             \
-        fprintf(stderr,                                                                 \
+        (void)  fprintf(stderr,                                                         \
                 "must realloc: size <= num+need is %s\n",                               \
                 size<=num+need?"TRUE":"FALSE");                                         \
         cmd_argv =(char **)secured_realloc((gpointer)cmd_argv,                          \
@@ -262,37 +271,39 @@ compose_xlock_command(gchar ***cmd_argv_p,
 
 #endif /* ? XGLOCK_DEBUG */
 
-    for (i = 0; i < nb_boolOpt; i++)
+    for (i = 0; i < (gint) nb_boolOpt; i++)
       if (BoolOpt[i].value) {
 
           /* Check dynamic tab size */
           CHECK_REALLOC( n, sz, 1 );
 
           /* Add the arg */
-          cmd_argv[n] = secured_malloc( strlen( BoolOpt[i].cmdarg )+2 );
+          cmd_argv[n] = (gchar *) secured_malloc( strlen( BoolOpt[i].cmdarg )+2 );
           (void) sprintf( cmd_argv[n], "-%s", BoolOpt[i].cmdarg );
           n++;
       }
 
     /* General options */
-    for (i = 0; i < nb_genOpt; i++)
+    for (i = 0; i < (gint) nb_genOpt; i++)
     {
         gchar *val;
 
         if ( !( GTK_IS_ENTRY( generalOpt[i].text_widget ) ) ) continue;
 
-        val = gtk_entry_get_text( GTK_ENTRY( generalOpt[i].text_widget ));
+        val = (gchar *)gtk_entry_get_text( GTK_ENTRY( generalOpt[i].text_widget ));
         if ( strlen( val ) > 0 )
         {
             gint j;
             gchar *quotes;
 
-            for ( j = 0; j < nb_defaultedOptions; j++ )
+            for ( j = 0; j < (gint) nb_defaultedOptions; j++ )
               if ( !strcmp( generalOpt[i].cmdarg, defaulted_options[j] ) )
                 break;
-            if ( j < nb_defaultedOptions )
+            if ( j < (gint) nb_defaultedOptions )
             {
-                gint defval = 0;
+		gint defval = 0;
+		gfloat defflt = 1.0;
+		gboolean flt = False;
 
                 switch ( j )
                 {
@@ -305,46 +316,56 @@ compose_xlock_command(gchar ***cmd_argv_p,
                   case DEF_SIZE:
                     defval = LockProcs[selected_mode].def_size; break;
                   case DEF_SATURATION:
-                    defval = LockProcs[selected_mode].def_saturation; break;
+		    flt = True;
+                    defflt = LockProcs[selected_mode].def_saturation; break;
                 }
-
-                if ( defval == atoi( val ) )
+		if (flt) {
+                  if ( defflt == atoi( val ) )
+                  {
 #ifdef XGLOCK_DEBUG
-                {
-                    fprintf(stderr,
+                    (void) fprintf(stderr,
+                            "Option %s has its default value (%g): discarded\n",
+                            generalOpt[i].cmdarg, defflt);
+#endif
+                    continue;
+                  }
+		} else {
+                  if ( defval == atoi( val ) )
+                  {
+#ifdef XGLOCK_DEBUG
+                    (void) fprintf(stderr,
                             "Option %s has its default value (%d): discarded\n",
                             generalOpt[i].cmdarg, defval);
 #endif
                     continue;
-#ifdef XGLOCK_DEBUG
+                  }
                 }
-#endif
             }
 
             /* Check dyntab size */
             CHECK_REALLOC( n, sz, 2 );
 
-            cmd_argv[n] = secured_malloc( strlen( generalOpt[i].cmdarg )+2 );
+            cmd_argv[n] = (gchar*) secured_malloc( strlen( generalOpt[i].cmdarg )+2 );
             (void) sprintf( cmd_argv[n], "-%s", generalOpt[i].cmdarg );
             n++;
 
             quotes = check_quotes( val );
 
-            cmd_argv[n] = secured_malloc( strlen( val )+3 );
+            cmd_argv[n] = (gchar *) secured_malloc( strlen( val )+3 );
             (void) sprintf( cmd_argv[n], "%s%s%s", quotes, val, quotes);
             n++;
         }
     }
 
     /* font and color options */
-    for (i = 0; i < nb_fntColorOpt; i++)
+    for (i = 0; i < (gint) nb_fntColorOpt; i++)
     {
         switch ( fntcolorOpt[i].type )
         {
             /* Handle color parameter */
           case TFNTCOL_COLOR:
           {
-              gchar *val = gtk_entry_get_text( GTK_ENTRY( fntcolorOpt[i].entry ) );
+              gchar *val = (gchar *)gtk_entry_get_text( GTK_ENTRY( fntcolorOpt[i].entry ) );
               gushort R, G, B;
               gdouble T;
               gchar   opt[50];
@@ -353,14 +374,14 @@ compose_xlock_command(gchar ***cmd_argv_p,
               {
                   CHECK_REALLOC( n, sz, 2 );
 
-                  cmd_argv[n] = secured_malloc( strlen( fntcolorOpt[i].cmdarg )+2 );
+                  cmd_argv[n] = (gchar*) secured_malloc( strlen( fntcolorOpt[i].cmdarg )+2 );
                   (void) sprintf( cmd_argv[n], "-%s", fntcolorOpt[i].cmdarg );
                   n++;
 
-                  sscanf( val, "#%4hx%4hx%4hx (%lf)", &R, &G, &B, &T );
-                  sprintf( opt, "#%04x%04x%04x", R, G, B );
+                  (void) sscanf( val, "#%4hx%4hx%4hx (%lf)", &R, &G, &B, &T );
+                  (void) sprintf( opt, "#%04x%04x%04x", R, G, B );
 
-                  cmd_argv[n] = secured_malloc( strlen( opt )+3 );
+                  cmd_argv[n] = (gchar*) secured_malloc( strlen( opt )+3 );
                   (void) sprintf( cmd_argv[n], "\"%s\"", opt );
                   n++;
               }
@@ -371,20 +392,20 @@ compose_xlock_command(gchar ***cmd_argv_p,
           case TFNTCOL_FONT:
           case TFNTCOL_FILE:
           {
-              gchar *val = gtk_entry_get_text( GTK_ENTRY( fntcolorOpt[i].entry ) );
+              gchar *val = (gchar *)gtk_entry_get_text( GTK_ENTRY( fntcolorOpt[i].entry ) );
               gchar *quotes;
 
               if (strlen( val ) > 0)
               {
                   CHECK_REALLOC( n, sz, 2 );
 
-                  cmd_argv[n] = secured_malloc( strlen( fntcolorOpt[i].cmdarg )+2 );
+                  cmd_argv[n] = (gchar *) secured_malloc( strlen( fntcolorOpt[i].cmdarg )+2 );
                   (void) sprintf( cmd_argv[n], "-%s", fntcolorOpt[i].cmdarg );
                   n++;
 
                   quotes = check_quotes( val );
 
-                  cmd_argv[n] = secured_malloc( strlen( val )+3 );
+                  cmd_argv[n] = (gchar *) secured_malloc( strlen( val )+3 );
                   (void) sprintf( cmd_argv[n], "%s%s%s", quotes, val, quotes);
                   n++;
               }
@@ -417,7 +438,7 @@ kill_xlock_cb(GtkWidget *w,
     guint i;
     gint dummy = 0;
 
-    for ( i = 0;  i < num_launchedprocesses; i++ )
+    for ( i = 0;  i < (guint) num_launchedprocesses; i++ )
       if ( launchedprocesses[i] != -1 ) {
           (void) kill( launchedprocesses[i], SIGKILL );
           (void) wait( &dummy );
@@ -445,7 +466,7 @@ launch_xlock(GtkWidget *w,
             args_cnt = 0;
     gchar **cmd_argv =(char **)NULL;
 
-    if ( selected_mode >= 0 && selected_mode < nb_mode )
+    if ( selected_mode >= 0 && selected_mode < (gint) nb_mode )
     {
         /* Alloc argv table */
         args_size = ARGS_CHUNK;
@@ -453,25 +474,25 @@ launch_xlock(GtkWidget *w,
         cmd_argv =(char **)secured_calloc( ARGS_CHUNK, sizeof(char *) );
 
         /* First arg is xlock itself */
-        cmd_argv[args_cnt] = secured_malloc( strlen( XLOCK )+1 );
+        cmd_argv[args_cnt] = (gchar *) secured_malloc( strlen( XLOCK )+1 );
         (void) strcpy(cmd_argv[args_cnt], XLOCK);
         args_cnt++;
 
         /* If launch in window/root */
         if ( launch_opt )
         {
-            cmd_argv[args_cnt] = secured_malloc( strlen( launch_opt )+1 );
+            cmd_argv[args_cnt] = (gchar *) secured_malloc( strlen( launch_opt )+1 );
             (void) strcpy(cmd_argv[args_cnt], launch_opt);
             args_cnt++;
         }
 
         /* Mode arg */
-        cmd_argv[args_cnt] = secured_malloc( strlen( MODE_STRING )+1 );
+        cmd_argv[args_cnt] = (gchar *) secured_malloc( strlen( MODE_STRING )+1 );
         (void) strcpy(cmd_argv[args_cnt], MODE_STRING);
         args_cnt++;
 
         /* Mode name */
-        cmd_argv[args_cnt] = secured_malloc( strlen( LockProcs[selected_mode].cmdline_arg )+1 );
+        cmd_argv[args_cnt] = (gchar *) secured_malloc( strlen( LockProcs[selected_mode].cmdline_arg )+1 );
         (void) strcpy(cmd_argv[args_cnt], LockProcs[selected_mode].cmdline_arg);
         args_cnt++;
 
@@ -479,20 +500,20 @@ launch_xlock(GtkWidget *w,
         compose_xlock_command( &cmd_argv, &args_size, &args_cnt );
 
 #ifdef XGLOCK_DEBUG
-        fprintf( stderr, "Running command: " );
+        (void) fprintf( stderr, "Running command: " );
         for ( i = 0; i < args_cnt; i++ )
-          fprintf( stderr, "%s ", cmd_argv[i] ? cmd_argv[i] : "\n" );
+          (void) fprintf( stderr, "%s ", cmd_argv[i] ? cmd_argv[i] : "\n" );
 #endif
         /* Allocate the launched pid_t dyntab */
         if ( !launchedprocesses )
         {
-            launchedprocesses = secured_calloc( 10, sizeof( pid_t ) );
+            launchedprocesses = (pid_t *) secured_calloc( 10, sizeof( pid_t ) );
             sz_launchedprocesses = 10;
         }
         /* Grow dyntab */
         if ( num_launchedprocesses == sz_launchedprocesses )
         {
-            launchedprocesses = secured_realloc(launchedprocesses,
+            launchedprocesses = (pid_t *) secured_realloc(launchedprocesses,
                                                 (sz_launchedprocesses+10)*sizeof( pid_t ) );
             sz_launchedprocesses+= 10;
         }
@@ -500,7 +521,7 @@ launch_xlock(GtkWidget *w,
         if ( (launchedprocesses[num_launchedprocesses] = FORK() ) == -1 )
         {
             perror( FORK_STR );
-            fprintf(stderr,
+            (void) fprintf(stderr,
                     "Cannot " FORK_STR " process for xlock !\n" );
         }
         /* Child process */
@@ -512,8 +533,8 @@ launch_xlock(GtkWidget *w,
 
     /* Free resources */
     while ( args_cnt-- )
-      (void) free( cmd_argv[args_cnt] );
-    (void) free( cmd_argv );
+      free( cmd_argv[args_cnt] );
+    free( cmd_argv );
     cmd_argv =(char **)NULL;
 }
 
@@ -532,7 +553,7 @@ exit_xglock(GtkWidget *w,
     gint dummy = 0;
 
     /* Cleanup launched processes */
-    for ( i = 0;  i < num_launchedprocesses; i++ )
+    for ( i = 0;  i < (guint) num_launchedprocesses; i++ )
       if ( launchedprocesses[i] != -1 ) {
           (void) kill( launchedprocesses[i], SIGKILL );
           (void) wait( &dummy );
@@ -572,13 +593,13 @@ static void
 bool_option_clicked_cb(GtkWidget *w,
                        gpointer udata )
 {
-    guint optnr =(guint)udata;
+    guint optnr = GPOINTER_TO_UINT(udata);
 
     if ( !( GTK_IS_CHECK_BUTTON(w) )) return;
 
     BoolOpt[optnr].value = !BoolOpt[optnr].value;
 #ifdef XGLOCK_DEBUG
-    fprintf(stderr,
+    (void) fprintf(stderr,
             "Option #%ud %s is %s\n",
             optnr, BoolOpt[optnr].cmdarg,
             BoolOpt[optnr].value ? "ACTIVE":"INACTIVE" );
@@ -601,24 +622,26 @@ static void
 mode_list_item_select_cb(GtkWidget *w,
                          gpointer mode_nr )
 {
-    gint mode =(gint)mode_nr;
+    gint mode = GPOINTER_TO_INT(mode_nr);
     gint i;
 
     if (w->state != GTK_STATE_SELECTED ) return;
 
-    if ( mode < 0 || mode >= nb_mode ) return;
+    if ( mode < 0 || mode >= (gint) nb_mode ) return;
 
     selected_mode = mode;
 
-    for ( i = 0; i < nb_defaultedOptions; i++ )
+    for ( i = 0; i < (gint) nb_defaultedOptions; i++ )
     {
         gchar temp[30];
         gint defval = 0, j;
+        gfloat defflt = 1.0;
+	gboolean flt = False;
 
-        for ( j = 0; j < nb_mode; j++ )
+        for ( j = 0; j < (gint) nb_mode; j++ )
           if ( !strcmp( generalOpt[j].cmdarg, defaulted_options[i] ) )
             break;
-        if ( j == nb_mode ) break;
+        if ( j == (gint) nb_mode ) break;
         switch ( i ) {
           case DEF_DELAY:
             defval = LockProcs[selected_mode].def_delay; break;
@@ -629,18 +652,23 @@ mode_list_item_select_cb(GtkWidget *w,
           case DEF_SIZE:
             defval = LockProcs[selected_mode].def_size; break;
           case DEF_SATURATION:
-            defval = LockProcs[selected_mode].def_saturation; break;
+	    flt = True;
+	    defflt = LockProcs[selected_mode].def_saturation; break;
+        }
+        if (flt) {
+	    (void) sprintf(temp, "%g", defflt);
+        } else {
+	    (void) sprintf(temp, "%d", defval);
         }
-        sprintf(temp, "%d", defval );
         gtk_entry_set_text(GTK_ENTRY(generalOpt[j].text_widget), temp);
     }
 
     if ( ( numberprocess = FORK() ) == -1 )
-      fprintf( stderr, "Fork error !\n" );
+      (void) fprintf( stderr, "Fork error !\n" );
     else if ( numberprocess == 0 ) {
         (void) execlp(XLOCK, XLOCK, "-parent", numberwidget,
                       MODE_STRING, LockProcs[mode].cmdline_arg, "-geometry", WINDOW_GEOMETRY, "-delay", "100000",
-                      "-nolock", "-inwindow", "+install", 0);
+                      "-nolock", "-inwindow", "+install", NULL);
     }
 }
 
@@ -701,15 +729,28 @@ color_selection_ok_cb(GtkWidget *w,
     struct_option_fntcol_callback *cbs_ptr =(struct_option_fntcol_callback *)udata;
     GtkColorSelection 		  *colorsel;
     GdkColor 	       		   gdkcol_bg;
+#if (!(GTK_CHECK_VERSION(2,0,0)))
     gdouble            		   colors[4];
+#endif
     gchar              		   colorstr[30];
 
     colorsel = GTK_COLOR_SELECTION( GTK_COLOR_SELECTION_DIALOG(cbs_ptr->fntcol_dialog)->colorsel );
 
+#if GTK_CHECK_VERSION(2,0,0)
+    gtk_color_selection_get_current_color(colorsel, &gdkcol_bg);
+    gtk_color_selection_set_current_color(colorsel, &gdkcol_bg);
+    gtk_color_selection_get_current_alpha(colorsel);
+
+    (void) sprintf(colorstr, "#%04x%04x%04x (%3.2f)",
+            gdkcol_bg.red,
+            gdkcol_bg.green,
+            gdkcol_bg.blue,
+	    (gdouble)(gtk_color_selection_get_current_alpha(colorsel)/65535.0));
+#else
     gtk_color_selection_get_color(colorsel, colors);
     gtk_color_selection_set_color(colorsel, colors);
 
-    sprintf(colorstr, "#%04x%04x%04x (%3.2f)",
+    (void) sprintf(colorstr, "#%04x%04x%04x (%3.2f)",
             (gushort)(colors[0] * 0xffff),
             (gushort)(colors[1] * 0xffff),
             (gushort)(colors[2] * 0xffff),
@@ -718,10 +759,11 @@ color_selection_ok_cb(GtkWidget *w,
     gdkcol_bg.red = (gushort)(colors[0] * 0xffff);
     gdkcol_bg.green = (gushort)(colors[1] * 0xffff);
     gdkcol_bg.blue = (gushort)(colors[2] * 0xffff);
+#endif
     gdk_color_alloc(gdkcolormap, &gdkcol_bg);
 
 #ifdef XGLOCK_DEBUG
-    fprintf(stderr,
+    (void) fprintf(stderr,
             "Parsed bg color: color=#%04x%04x%04x pixel=%lx\n",
             gdkcol_bg.red,
             gdkcol_bg.green,
@@ -749,10 +791,18 @@ color_selection_changed_cb(GtkWidget *w,
 {
     GtkWidget		 *colorsel_dialog =(GtkWidget *)udata;
     GtkColorSelection 	 *colorsel;
+#if GTK_CHECK_VERSION(2,0,0)
+    GdkColor		  gdkcol_bg;
+#else
     gdouble            	  colors[4];
+#endif
 
     colorsel = GTK_COLOR_SELECTION( GTK_COLOR_SELECTION_DIALOG(colorsel_dialog)->colorsel );
+#if GTK_CHECK_VERSION(2,0,0)
+    gtk_color_selection_get_current_color(colorsel, &gdkcol_bg);
+#else
     gtk_color_selection_get_color(colorsel, colors);
+#endif
 }
 
 /*
@@ -766,6 +816,9 @@ create_color_selection_dialog(GtkWidget *w,
                               gpointer udata)
 {
     struct_option_fntcol	  *optentry =(struct_option_fntcol *)udata;
+#if GTK_CHECK_VERSION(2,0,0)
+    GdkColor		  	   gdkcol_bg;
+#endif
     gdouble    		  	   colors[4];
     gchar      		 	  *val;
     struct_option_fntcol_callback *cbs_ptr;
@@ -780,19 +833,32 @@ create_color_selection_dialog(GtkWidget *w,
     cbs_ptr->entry = optentry->entry;
     cbs_ptr->drawing_area = optentry->drawing_area;
 
-    val = gtk_entry_get_text( GTK_ENTRY(cbs_ptr->entry) );
+    val = (gchar *)gtk_entry_get_text( GTK_ENTRY(cbs_ptr->entry) );
     if ( strlen( val ) > 0 )
     {
         gushort R, G, B;
 
-        sscanf( val, "#%04hx%04hx%04hx (%lf)", &R, &G, &B, &colors[3]);
+        (void) sscanf( val, "#%04hx%04hx%04hx (%lf)", &R, &G, &B, &colors[3]);
+        gtk_color_selection_set_opacity(GTK_COLOR_SELECTION( GTK_COLOR_SELECTION_DIALOG(cbs_ptr->fntcol_dialog)->colorsel),
+                                        colors[3] < 1.0);
+#if GTK_CHECK_VERSION(2,0,0)
+        if (colors[3] < 1.0)
+		gtk_color_selection_set_current_alpha(
+				GTK_COLOR_SELECTION( GTK_COLOR_SELECTION_DIALOG(cbs_ptr->fntcol_dialog)->colorsel),
+				(guint16)(colors[3] * 65535.0));
+	gdkcol_bg.red   = R;
+	gdkcol_bg.green = G;
+	gdkcol_bg.blue  = B;
+        gtk_color_selection_set_current_color(
+			GTK_COLOR_SELECTION( GTK_COLOR_SELECTION_DIALOG(cbs_ptr->fntcol_dialog)->colorsel),
+			&gdkcol_bg);
+#else
         colors[0] = R /(gdouble)0xffff;
         colors[1] = G /(gdouble)0xffff;
         colors[2] = B /(gdouble)0xffff;
-        gtk_color_selection_set_opacity(GTK_COLOR_SELECTION( GTK_COLOR_SELECTION_DIALOG(cbs_ptr->fntcol_dialog)->colorsel),
-                                        colors[3] < 1.0);
         gtk_color_selection_set_color(GTK_COLOR_SELECTION( GTK_COLOR_SELECTION_DIALOG(cbs_ptr->fntcol_dialog)->colorsel),
                                       colors);
+#endif
     }
 
     gtk_color_selection_set_opacity(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(cbs_ptr->fntcol_dialog)->colorsel),
@@ -883,7 +949,7 @@ create_font_selection_dialog(GtkWidget *w,
     cbs_ptr->drawing_area =(GtkWidget *)NULL;
 
     cbs_ptr->fntcol_dialog = gtk_font_selection_dialog_new("Xglock Font Selection Dialog");
-    fontname = gtk_entry_get_text(GTK_ENTRY(optentry->entry));
+    fontname = (gchar *)gtk_entry_get_text(GTK_ENTRY(optentry->entry));
     if ( fontname && strlen(fontname) )
       gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(cbs_ptr->fntcol_dialog), fontname);
     gtk_signal_connect(GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(cbs_ptr->fntcol_dialog)->ok_button), "clicked",
@@ -930,7 +996,7 @@ file_selection_ok_cb(GtkWidget *w,
     gchar       		  *filename;
 
     /* Be sure file exist. We could be more paranoia */
-    filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(cbs_ptr->fntcol_dialog));
+    filename = (gchar *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(cbs_ptr->fntcol_dialog));
     if ( stat(filename, &stbuf) != -1 )
     {
         gtk_widget_hide(cbs_ptr->fntcol_dialog);
@@ -978,21 +1044,21 @@ static void
 contextual_help_dialog(GtkWidget *w,
                        gpointer udata )
 {
-    fprintf(stderr, "contextual_help_dialog !\n");
+    (void) fprintf(stderr, "contextual_help_dialog !\n");
 }
 
 static void
 window_help_dialog(GtkWidget *w,
                    gpointer udata )
 {
-    fprintf(stderr, "window_help_dialog !\n");
+    (void) fprintf(stderr, "window_help_dialog !\n");
 }
 
 static void
 global_help_dialog(GtkWidget *w,
                    gpointer udata )
 {
-    fprintf(stderr, "global_help_dialog !\n");
+    (void) fprintf(stderr, "global_help_dialog !\n");
 }
 
 static void about_dialog_close_callback(GtkWidget *widget, gpointer   data)
@@ -1057,10 +1123,10 @@ about_dialog(GtkWidget *w,
   gtk_preview_size (GTK_PREVIEW (xpreview), logo_width, logo_height);
   temp = g_new (guchar,logo_width * 3);
   datapointer=(guchar *)header_data;
-  for (y = 0; y < logo_height; y++)
+  for (y = 0; y < (gint) logo_height; y++)
   {
     temp2=temp;
-    for (x = 0; x< logo_width; x++)
+    for (x = 0; x< (gint) logo_width; x++)
     {
       HEADER_PIXEL(datapointer,temp2);
       temp2+=3;
@@ -1077,12 +1143,15 @@ about_dialog(GtkWidget *w,
 
   xhbox = gtk_hbox_new(FALSE, 5);
   gtk_box_pack_start(GTK_BOX(xvbox), xhbox, TRUE, TRUE, 0);
-  text = "\nFirst written by Charles Vidal <vidalc@club-internet.fr>.\n"
-         "Continual improvements by Remi Cohen-Scali <remi.cohenscali@pobox.com>.\n"
-         "Updates by Eric Lassauge <lassauge@mail.dotcom.fr>.\n\n"
+  text = "\nFirst written by Charles Vidal <cvidal@ivsweb.com>.\n"
+#if GTK_CHECK_VERSION(2,0,0)
+         "Latest GTK+ 2.x by Éric Lassauge <lassauge@users.sourceforge.net>.\n\n"
+#else
+         "Updates by Eric Lassauge <lassauge@users.sourceforge.net>.\n\n"
+#endif
          "GTK+ launcher for xlockmore: the maintained version of xlock.\n"
- 	 "http://www.tux.org/~bagleyd/xlockmore.html\n"
-	 "Version 2.0\n";
+ 	 "http://www.sillycycle.com/xlockmore.html\n"
+	 "Version 3.0\n";
 
   xlabel = gtk_label_new(text);
   gtk_box_pack_start(GTK_BOX(xhbox), xlabel, TRUE, FALSE, 0);
@@ -1121,10 +1190,10 @@ create_file_menu(GtkWidget *window)
     menu_item = gtk_menu_item_new_with_label("Kill xlock");
     gtk_container_add( GTK_CONTAINER (menu), menu_item);
     gtk_widget_add_accelerator(menu_item,
-			       "activate",
-			       group,
-			       'K',
-			       0, GTK_ACCEL_VISIBLE);
+			"activate",
+			group,
+			'K',
+			(GdkModifierType) 0, GTK_ACCEL_VISIBLE);
     gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                         (GtkSignalFunc) kill_xlock_cb,
                         (gpointer) window);
@@ -1133,10 +1202,10 @@ create_file_menu(GtkWidget *window)
     menu_item = gtk_menu_item_new_with_label("Quit");
     gtk_container_add( GTK_CONTAINER (menu), menu_item);
     gtk_widget_add_accelerator(menu_item,
-			       "activate",
-			       group,
-			       'Q',
-			       0, GTK_ACCEL_VISIBLE);
+			"activate",
+			group,
+			'Q',
+			(GdkModifierType) 0, GTK_ACCEL_VISIBLE);
     gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                         (GtkSignalFunc) exit_xglock,
                         (gpointer) window);
@@ -1164,10 +1233,10 @@ create_help_menu(GtkWidget *window)
     menu_item = gtk_menu_item_new_with_label("Context help");
     gtk_container_add( GTK_CONTAINER (menu), menu_item);
     gtk_widget_add_accelerator(menu_item,
-                                   "activate",
-			       group,
-                                   'C',
-			       0, GTK_ACCEL_VISIBLE);
+			"activate",
+			group,
+			'C',
+			(GdkModifierType) 0, GTK_ACCEL_VISIBLE);
     gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                         (GtkSignalFunc) contextual_help_dialog,
                         (gpointer) window);
@@ -1176,10 +1245,10 @@ create_help_menu(GtkWidget *window)
     menu_item = gtk_menu_item_new_with_label("Window help");
     gtk_container_add( GTK_CONTAINER (menu), menu_item);
     gtk_widget_add_accelerator(menu_item,
-                                   "activate",
-			       group,
-                                   'W',
-			       0, GTK_ACCEL_VISIBLE);
+			"activate",
+			group,
+			'W',
+			(GdkModifierType) 0, GTK_ACCEL_VISIBLE);
     gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                         (GtkSignalFunc) window_help_dialog,
                         (gpointer) window);
@@ -1188,10 +1257,10 @@ create_help_menu(GtkWidget *window)
     menu_item = gtk_menu_item_new_with_label("Global help");
     gtk_container_add( GTK_CONTAINER (menu), menu_item);
     gtk_widget_add_accelerator(menu_item,
-                                   "activate",
-			       group,
-                                   'G',
-			       0, GTK_ACCEL_VISIBLE);
+			"activate",
+			group,
+			'G',
+			(GdkModifierType) 0, GTK_ACCEL_VISIBLE);
     gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                         (GtkSignalFunc) global_help_dialog,
                         (gpointer) window);
@@ -1200,10 +1269,10 @@ create_help_menu(GtkWidget *window)
     menu_item = gtk_menu_item_new_with_label("About XgLock");
     gtk_container_add( GTK_CONTAINER (menu), menu_item);
     gtk_widget_add_accelerator(menu_item,
-                                   "activate",
-			       group,
-                                   'A',
-			       0, GTK_ACCEL_VISIBLE);
+			"activate",
+			group,
+			'A',
+			(GdkModifierType) 0, GTK_ACCEL_VISIBLE);
     gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                         (GtkSignalFunc) about_dialog,
                         (gpointer) window);
@@ -1248,13 +1317,13 @@ create_fntColorOptions_entries(GtkScrolledWindow *window)
     gtk_box_pack_start(GTK_BOX(box0),table,TRUE,TRUE,XGLOCK_SPACING);
     gtk_widget_show(table);
 
-    for (i = 0; i < nb_fntColorOpt; i++) {
+    for (i = 0; i < (gint) nb_fntColorOpt; i++) {
 
         label = gtk_label_new(fntcolorOpt[i].label);
         gtk_table_attach(GTK_TABLE(table),label,
                          0,1,i,i+1,
-                         GTK_EXPAND|GTK_FILL,
-                         GTK_EXPAND|GTK_FILL,
+                         (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
+                         (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
                          XGLOCK_TABLE_SPACING,
                          XGLOCK_TABLE_SPACING);
         gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
@@ -1266,8 +1335,8 @@ create_fntColorOptions_entries(GtkScrolledWindow *window)
         gtk_tooltips_set_tip(tooltips, entry, fntcolorOpt[i].desc,(gchar *)NULL);
         gtk_table_attach(GTK_TABLE(table),entry,
                          1,5,i,i+1,
-                         GTK_EXPAND|GTK_FILL,
-                         GTK_EXPAND|GTK_FILL,
+                         (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
+                         (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
                          XGLOCK_TABLE_SPACING,
                          XGLOCK_TABLE_SPACING);
         gtk_widget_show(entry);
@@ -1279,8 +1348,8 @@ create_fntColorOptions_entries(GtkScrolledWindow *window)
             gtk_container_border_width(GTK_CONTAINER(box1),XGLOCK_TABLE_BORDER_WIDTH);
             gtk_table_attach(GTK_TABLE(table),box1,
                              5,6,i,i+1,
-                             GTK_EXPAND|GTK_FILL,
-                             GTK_EXPAND|GTK_FILL,
+                             (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
+                             (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
                              XGLOCK_TABLE_SPACING,
                              XGLOCK_TABLE_SPACING);
             gtk_widget_show(box1);
@@ -1319,8 +1388,8 @@ create_fntColorOptions_entries(GtkScrolledWindow *window)
             gtk_tooltips_set_tip(tooltips, button, fntcolorOpt[i].desc,(gchar *)NULL);
             gtk_table_attach(GTK_TABLE(table),button,
                              5,6,i,i+1,
-                             GTK_EXPAND|GTK_FILL,
-                             GTK_EXPAND|GTK_FILL,
+                             (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
+                             (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
                              XGLOCK_TABLE_SPACING,
                              XGLOCK_TABLE_SPACING);
             gtk_widget_show(button);
@@ -1335,15 +1404,15 @@ create_fntColorOptions_entries(GtkScrolledWindow *window)
             gtk_tooltips_set_tip(tooltips, button, fntcolorOpt[i].desc,(gchar *)NULL);
             gtk_table_attach(GTK_TABLE(table),button,
                              5,6,i,i+1,
-                             GTK_EXPAND|GTK_FILL,
-                             GTK_EXPAND|GTK_FILL,
+                             (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
+                             (GtkAttachOptions) (GTK_EXPAND|GTK_FILL),
                              XGLOCK_TABLE_SPACING,
                              XGLOCK_TABLE_SPACING);
             gtk_widget_show(button);
             break;
 
           default:
-            fprintf(stderr,
+            (void) fprintf(stderr,
                     "Program Inconsistency: Unknown parameter type #%d !\n",
                     fntcolorOpt[i].type );
         }
@@ -1389,13 +1458,13 @@ create_genOptions_entries(GtkScrolledWindow *window)
     gtk_container_add(GTK_CONTAINER(box0), box2);
     gtk_widget_show(box2);
 
-    for (i = 0; i < nb_genOpt; i++) {
+    for (i = 0; i < (gint) nb_genOpt; i++) {
         label = gtk_label_new(generalOpt[i].label);
         gtk_widget_set_usize(label, 60, 21);
         gtk_box_pack_start(GTK_BOX(box1), label, FALSE, TRUE, 0);
         gtk_widget_show(label);
     }
-    for (i = 0; i < nb_genOpt; i++) {
+    for (i = 0; i < (gint) nb_genOpt; i++) {
         entry = gtk_entry_new();
         gtk_widget_set_usize(entry, 350, 21);
         gtk_tooltips_set_tip(tooltips, entry, generalOpt[i].desc,(gchar *)NULL);
@@ -1420,6 +1489,7 @@ static void
 create_boolOptions_buttons(GtkScrolledWindow *parent)
 {
     gint         i;
+    guint        iptr = 0;
     GtkWidget   *box0;
     GtkWidget   *box1;
     GtkWidget   *box2;
@@ -1438,13 +1508,13 @@ create_boolOptions_buttons(GtkScrolledWindow *parent)
     gtk_container_add(GTK_CONTAINER(box0), box2);
     gtk_widget_show(box2);
 
-    for (i = 0; i < nb_boolOpt; i++) {
+    for (i = 0; i < (gint) nb_boolOpt; i++) {
         button = gtk_check_button_new_with_label(BoolOpt[i].label);
         gtk_tooltips_set_tip(tooltips, button, BoolOpt[i].desc,(gchar *)NULL);
         gtk_box_pack_start(GTK_BOX(i%2 ? box1 : box2), button, TRUE, TRUE, 0);
         gtk_signal_connect(GTK_OBJECT(button), "clicked",
                            (GtkSignalFunc) bool_option_clicked_cb,
-                           (gpointer)i);
+                           GUINT_TO_POINTER(iptr++));
         gtk_widget_show(button);
     }
 }
@@ -1455,7 +1525,7 @@ create_boolOptions_buttons(GtkScrolledWindow *parent)
 /*                                                                   */
 /* ================================================================= */
 
-gint
+int
 main(gint argc, gchar *argv[])
 {
     GtkWidget  *xglockroot;
@@ -1481,6 +1551,7 @@ main(gint argc, gchar *argv[])
     gint        i;
     gchar       temp[100];
 
+    XInitThreads();
     /* Init for some data */
     nb_mode = sizeof (LockProcs) / sizeof (LockProcs[0]);
     nb_boolOpt = sizeof (BoolOpt) / sizeof (BoolOpt[0]);
@@ -1548,8 +1619,28 @@ main(gint argc, gchar *argv[])
     gtk_box_pack_start(GTK_BOX(box0), box3, TRUE, TRUE, 0);
     gtk_widget_show(box3);
 
-    /* Preview button (button because I need here an X window) */
+    /* Preview window */
+#if GTK_CHECK_VERSION(2,0,0)
+    {
+    GdkColor color;
+    GtkTextBuffer *buffer;
+
+    /* a text view because I need here an X window (Button has no more
+     * a window in Gtk2 ! */
+    previewButton = gtk_text_view_new();
+    gtk_text_view_set_editable(GTK_TEXT_VIEW(previewButton), FALSE);
+    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(previewButton), FALSE);
+    gtk_text_view_set_justification(GTK_TEXT_VIEW(previewButton), GTK_JUSTIFY_CENTER);
+    gtk_text_view_set_pixels_above_lines(GTK_TEXT_VIEW(previewButton),30);
+    buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (previewButton));
+    gtk_text_buffer_set_text (buffer, "Preview window", -1);
+    gdk_color_parse ("red", &color);
+    gtk_widget_modify_text( previewButton, GTK_STATE_NORMAL, &color);
+    }
+#else
+    /* button because I need here an X window */
     previewButton = gtk_button_new_with_label( "Preview window" );
+#endif
     gtk_widget_set_name(previewButton,"preview_button");
     gtk_widget_set_usize(previewButton, WINDOW_WIDTH, WINDOW_HEIGHT );
     gtk_box_pack_start(GTK_BOX(box3), previewButton, TRUE, FALSE, 10);
@@ -1588,7 +1679,7 @@ main(gint argc, gchar *argv[])
     GTK_WIDGET_UNSET_FLAGS (GTK_SCROLLED_WINDOW (scrolled_win)->vscrollbar, GTK_CAN_FOCUS);
     gtk_tooltips_set_tip(tooltips, list, "Select the mode you want to try",(gchar *)NULL);
 
-    for (i = 0; i < nb_mode; i++) {
+    for (i = 0; i < (gint) nb_mode; i++) {
         sprintf(temp, "%s :%s", LockProcs[i].cmdline_arg, LockProcs[i].desc);
         list_item = gtk_list_item_new_with_label(temp);
         gtk_container_add(GTK_CONTAINER(list), list_item);
@@ -1679,7 +1770,7 @@ main(gint argc, gchar *argv[])
 
     /* launch in root button */
     button = gtk_button_new_with_label("Launch in root");
-    gtk_widget_set_usize(button, 95, 30);
+    gtk_widget_set_usize(button, 105, 30);
     GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
     gtk_tooltips_set_tip(tooltips, button, "Launch xlock in root window",(gchar *)NULL);
     gtk_signal_connect(GTK_OBJECT(button), "clicked",
@@ -1690,7 +1781,7 @@ main(gint argc, gchar *argv[])
 
     /* launch -- in window -- button */
     button = gtk_button_new_with_label("Launch in window");
-    gtk_widget_set_usize(button, 115, 30);
+    gtk_widget_set_usize(button, 125, 30);
     GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
     gtk_tooltips_set_tip(tooltips, button, "Launch xlock in a window",(gchar *)NULL);
     gtk_signal_connect(GTK_OBJECT(button), "clicked",
