@@ -10,7 +10,7 @@ static const char sccsid[] = "@(#)automata.c	4.10 98/04/28 xlockmore";
  *
  * Revision History:
  *
- * Changes maintained by David Bagley <bagleyd@tux.org>
+ * Changes maintained by David Bagley <bagleyd AT verizon.net>
  * 20-Apr-98: Separated out of util.c
  *
  */
@@ -19,7 +19,6 @@ static const char sccsid[] = "@(#)automata.c	4.10 98/04/28 xlockmore";
 #include "utils.h"
 #else
 #include "xlock.h"
-int         neighbors;
 
 #endif
 #include "automata.h"
@@ -48,6 +47,38 @@ XPoint      triangleUnit[2][3] =
 	}
 };
 
+XPoint      pentagonUnit[4][5] =
+{
+	{
+		{0, 0},
+		{-2, -1},
+		{1, -2},
+		{2, 0},
+		{1, 2}
+	},
+	{ /* left */
+		{0, 0},
+		{1, -2},
+		{2, 1},
+		{0, 2},
+		{-2, 1}
+	},
+	{ /* up */
+		{0, 0},
+		{2, 1},
+		{-1, 2},
+		{-2, 0},
+		{-1, -2}
+	},
+	{ /* right */
+		{0, 0},
+		{-1, 2},
+		{-2, -1},
+		{0, -2},
+		{2, -1}
+	}
+};
+
 unsigned char stipples[NUMSTIPPLES][STIPPLESIZE] =
 {
 	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},	/* white */
