@@ -21,7 +21,7 @@ static const char sccsid[] = "@(#)color.c	4.00 97/01/01 xlockmore";
  * Create an HSB ramp.
  *
  * Revision History:
- * Changes maintained by David Bagley <bagleyd@tux.org>
+ * Changes maintained by David Bagley <bagleyd AT verizon.net>
  * 22-Jun-94: Modified for VMS
  *            <Anthony.D.Clarke@Support.Hatfield.Raytheon.bae.eurokom.ie>
  * Changes of Patrick J. Naughton
@@ -233,10 +233,8 @@ fixColormap(ModeInfo * mi, int ncolors, float saturation,
 	MI_BLACK_PIXEL(mi) = BlackPixelOfScreen(scr);
 	MI_WHITE_PIXEL(mi) = WhitePixelOfScreen(scr);
 #else
-	MI_BLACK_PIXEL(mi) = allocPixel(display, cmap, (char *) "Black", (char
-*) "Black");
-	MI_WHITE_PIXEL(mi) = allocPixel(display, cmap, (char *) "White", (char
-*) "White");
+	MI_BLACK_PIXEL(mi) = allocPixel(display, cmap, (char *) "Black", (char *) "Black");
+	MI_WHITE_PIXEL(mi) = allocPixel(display, cmap, (char *) "White", (char *) "White");
 #endif
 	MI_BG_PIXEL(mi) = allocPixel(display, cmap, background, (char *) "White");
 	MI_FG_PIXEL(mi) = allocPixel(display, cmap, foreground, (char *) "Black");
@@ -402,7 +400,7 @@ reserveColors(ModeInfo * mi, Colormap cmap, unsigned long *black)
 /* the remaining of this file was hacked from colors.c and hsv.c from
  * xscreensaver
  *
- * xscreensaver, Copyright (c) 1997 Jamie Zawinski <jwz@jwz.org>
+ * xscreensaver, Copyright (c) 1997 Jamie Zawinski <jwz AT jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -412,7 +410,7 @@ reserveColors(ModeInfo * mi, Colormap cmap, unsigned long *black)
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *
- * Modified for the use with xlockmore by Jouk Jansen <joukj@hrem.stm.tudelft.nl>
+ * Modified for the use with xlockmore by Jouk Jansen <joukj AT hrem.nano.tudelft.nl>
  * 12 June 1998
  */
 
@@ -1069,7 +1067,7 @@ rotate_colors(Display * dpy, Colormap cmap,
 	free(colors2);
 }
 
-/* xscreensaver, Copyright (c) 1992, 1997 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1992, 1997 Jamie Zawinski <jwz AT jwz.org>
 
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -1328,6 +1326,56 @@ visual_depth (Screen *screen, Visual *visual)
 }
 
 
+/* You very probably don't want to be using this.
+   Pixmap depth doesn't refer to the depths of pixmaps, but rather, to
+   the depth of protocol-level on-the-wire pixmap data, that is, XImages.
+   To get this info, you should be looking at XImage->bits_per_pixel
+   instead.  (And allocating the data for your XImage structures by
+   multiplying ximage->bytes_per_line by ximage->height.)
+
+   Still, it can be useful to know bits_per_pixel before the XImage exists.
+
+   XCreateImage calls _XGetBitsPerPixel to figure this out, but that function
+   is private to Xlib.
+
+   For some reason, _XGetBitsPerPixel tries a hard-coded list of depths if
+   it doesn't find a matching pixmap format, but I (Dave Odell) couldn't
+   find any justification for this in the X11 spec. And the XFree86 CVS
+   repository doesn't quite go back far enough to shed any light on what
+   the deal is with that.
+   http://cvsweb.xfree86.org/cvsweb/xc/lib/X11/ImUtil.c
+
+   The hard-coded list apparently was added between X11R5 and X11R6.
+   See <ftp://ftp.x.org/pub/>.
+ */
+int
+visual_pixmap_depth (Screen *screen, Visual *visual)
+{
+#ifdef WIN32
+  return 1; /* XApi.c XPutImage only does bitmaps. */
+#else
+  Display *dpy = DisplayOfScreen (screen);
+  int vdepth = visual_depth (screen, visual);
+  int pdepth = vdepth;
+  int i, pfvc = 0;
+  XPixmapFormatValues *pfv = XListPixmapFormats (dpy, &pfvc);
+
+  /* Return the first matching depth in the pixmap formats.  If there are no
+     matching pixmap formats (which shouldn't be able to happen at all) then
+     return the visual depth instead. */
+  for (i = 0; i < pfvc; i++)
+    if (pfv[i].depth == vdepth)
+      {
+	pdepth = pfv[i].bits_per_pixel;
+	break;
+      }
+  if (pfv)
+    XFree (pfv);
+  return pdepth;
+#endif
+}
+
+
 Bool
 allocate_alpha_colors (Screen *screen, Visual *visual, Colormap cmap,
 		       int *nplanesP, Bool additive_p,
