@@ -1,5 +1,5 @@
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)iostuff.c	4.10 98/10/23 xlockmore";
+static const char sccsid[] = "@(#)iostuff.c	5.03 2001/11/16 xlockmore";
 
 #endif
 
@@ -10,8 +10,12 @@ static const char sccsid[] = "@(#)iostuff.c	4.10 98/10/23 xlockmore";
  *
  * Revision History:
  *
- * Changes maintained by David Bagley <bagleyd@tux.org>
- * 23-Apr-98: Separated out of util.c
+ * Changes maintained by David Bagley <bagleyd AT verizon.net>
+ * 16-Nov-2001: Keep same sel_image wildcard behavior as xlockmore 5.02 except
+ *              on VMS so it works for directories. <tschmidt AT micron.com>
+ * 29-Oct-2001: Bug fix for sel_image by Jouk Jansen <joukj AT hrem.nano.tudelft.nl>
+ *              It treated wildcards like wild*.* as being *wild*.*.
+ * 23-Apr-1998: Separated out of util.c
  *
  */
 #ifdef STANDALONE
@@ -35,8 +39,11 @@ char       *messagefontname;
 #include "magick.h"
 #endif
 
-
+#ifndef WIN32
 #include <X11/Xutil.h>
+#else
+#include "Xapi.h"
+#endif
 #include <sys/stat.h>
 #include <stdio.h>
 
@@ -69,6 +76,25 @@ my_fopen(char *filename, const char *type)
 	return fp;
 }
 
+FILE       *
+my_fopenSize(char *filename, const char *type, int *size)
+{
+	FILE       *fp = (FILE *) NULL;
+	int         s;
+	struct stat fileStat;
+
+	s = stat(filename, &fileStat);
+	*size = fileStat.st_size;
+	if ((s >= 0 && S_ISREG(fileStat.st_mode)) || (s < 0 && type[0] == 'w')) {
+		if ((fp = fopen(filename, type)) == NULL)
+			return (FILE *) NULL;
+	} else {
+		return (FILE *) NULL;
+	}
+
+	return fp;
+}
+
 #if HAVE_DIRENT_H
 static void
 randomFileFromList(char *directory,
@@ -198,6 +224,16 @@ sel_image(struct dirent *name)
 	int         numfrag = -1, ip, i = -1;
 #define MAX_FRAGS 64
 	char *frags[MAX_FRAGS];
+#ifdef VMS
+        Bool not_first_star = True;
+
+	upcase( name_tmp ); /* VMS-file system is not case selective */
+        if ( *filename_tmp == '*' )
+	  {
+	    not_first_star = False;
+	    filename_tmp++;
+	  }
+#endif
 
 	if (numfrag == -1) {
 		++numfrag;
@@ -217,7 +253,7 @@ sel_image(struct dirent *name)
 		}
 		if ( strlen( filename_tmp ) != 0 ) {
 			if (numfrag > 0 && frags[numfrag] != NULL)
-				(void) free((void *) frags[numfrag]);
+				free(frags[numfrag]);
 			if ((frags[numfrag] = (char *) malloc(strlen(filename_tmp) +
 					1)) == NULL) {
 				numfrag--;
@@ -227,14 +263,33 @@ sel_image(struct dirent *name)
 		} else
 			numfrag--;
 	}
-	for (i = 0; i <= numfrag; ++i) {
-		ip = index_dir(name_tmp, frags[i]);
-		(void) free((void *) frags[i]);
-		if (ip == 0) {
+
+#ifdef VMS
+	i = 0;
+	ip = index_dir(name_tmp, frags[i]);
+	free(frags[i]);
+	if ( not_first_star ) {
+		if ( ip != 1 )
+			goto FAIL;
+	} else {
+		if ( ip == 0 )
 			goto FAIL;
-		}
-		name_tmp = name_tmp + ip;
 	}
+	name_tmp = name_tmp + ip;
+
+	if ( numfrag > 0 )
+		for (i = 1; i <= numfrag; ++i)
+#else
+		for (i = 0; i <= numfrag; ++i)
+#endif
+		{
+			ip = index_dir(name_tmp, frags[i]);
+			free(frags[i]);
+			if (ip == 0) {
+				goto FAIL;
+			}
+			name_tmp = name_tmp + ip;
+		}
 
 	return (1);
 FAIL:
@@ -242,7 +297,7 @@ FAIL:
 		int         j;
 
 		for (j = i + 1; j <= numfrag; ++j)
-			(void) free((void *) frags[j]);
+			free(frags[j]);
 		return (0);
 	}
 }
@@ -256,7 +311,7 @@ scan_dir(const char *directoryname, struct dirent ***namelist,
 	 int         (*compare) (const void *, const void *))
 {
 	DIR        *dirp;
-	struct dirent *new_entry, **namelist_tmp;
+	struct dirent *new_entry, **namelist_tmp, **namelist_buf;
 	int         size_tmp, num_list_tmp;
 
 	if (debug)
@@ -275,6 +330,7 @@ scan_dir(const char *directoryname, struct dirent ***namelist,
 			sizeof (struct dirent *))) == NULL) {
 		if (debug)
 			(void) printf("scan_dir no memory\n");
+		(void) closedir(dirp);
 		return (-1);
 	}
 	num_list_tmp = 0;
@@ -286,22 +342,44 @@ scan_dir(const char *directoryname, struct dirent ***namelist,
 		if (specify != NULL && !(*specify) (new_entry))
 			continue;
 		if (++num_list_tmp >= size_tmp) {
+			int oldsize = size_tmp;
+
 			size_tmp = size_tmp + _MEMBL_;
-			if ((namelist_tmp = (struct dirent **) realloc(
+			if ((namelist_buf = (struct dirent **) realloc(
 					(void *) namelist_tmp, size_tmp *
 					sizeof (struct dirent *))) == NULL) {
+				int i;
 				if (debug)
 					(void) printf("scan_dir no memory\n");
+				(void) closedir(dirp);
+				for (i = 0; i < oldsize; i++) {
+					if (namelist_tmp[i] != NULL) {
+						free(namelist_tmp[i]);
+					}
+				}
+				free(namelist_tmp);
 				return (-1);
 			}
+			/* wait to see if error else need to free */
+			namelist_tmp = namelist_buf;
 		}
 		if ((namelist_tmp[num_list_tmp - 1] =
 			(struct dirent *) malloc(sizeof (struct dirent)
 #ifdef SVR4
 			 + strlen(new_entry->d_name)
 #endif
-		)) == NULL)
+		)) == NULL) {
+				int i;
+
+				(void) closedir(dirp);
+				for (i = 0; i < num_list_tmp - 1; i++) {
+					if (namelist_tmp[i] != NULL) {
+						free(namelist_tmp[i]);
+					}
+				}
+				free(namelist_tmp);
 				return (-1);
+		}
 
 		(void) strcpy(namelist_tmp[num_list_tmp - 1]->d_name, new_entry->d_name);
 
@@ -336,15 +414,15 @@ getRandomFile(char *randomfile, char *randomfile_local)
 
 		for (i = 0; i < num_list; i++) {
 			if (image_list[i] != NULL)
-				(void) free((void *) image_list[i]);
+				free(image_list[i]);
 		}
-		(void) free((void *) image_list);
+		free(image_list);
 		image_list = (struct dirent **) NULL;
 	}
 	if ((images_list = (struct dirent ***) malloc(sizeof (struct dirent **))) != NULL) {
 		num_list = scan_dir(directory_r, images_list, sel_image, NULL);
 		image_list = *images_list;
-		(void) free((void *) images_list);
+		free(images_list);
 		if (num_list > 0) {
 			randomFileFromList(directory_r, image_list, num_list,
 				randomfile_local);
@@ -359,14 +437,10 @@ getRandomFile(char *randomfile, char *randomfile_local)
 #endif
 
 extern int  XbmReadFileToImage(char *filename,
-			       int *width, int *height, unsigned char **bits);
+			       int *width, int *height, char **bits);
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
-#if USE_XPMINC
-#include <xpm.h>
-#else
-#include <X11/xpm.h>		/* Normal spot */
-#endif
+#ifdef HAVE_XPM
+#include <X11/xpm.h>
 #endif
 #ifndef USE_MAGICK
 # include "ras.h"
@@ -383,7 +457,7 @@ static XImage blogo =
 void
 getImage(ModeInfo * mi, XImage ** logo,
 	 int default_width, int default_height, unsigned char *default_bits,
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 	 int default_xpm, char **name,
 #endif
 	 int *graphics_format, Colormap * ncm,
@@ -393,7 +467,7 @@ getImage(ModeInfo * mi, XImage ** logo,
 	static char *bitmap_local = (char *) NULL;
 
 #ifndef STANDALONE
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 	XpmAttributes attrib;
 
 #endif
@@ -407,7 +481,7 @@ getImage(ModeInfo * mi, XImage ** logo,
 	else
 		*ncm = None;
 #endif
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 	attrib.visual = MI_VISUAL(mi);
 	if (*ncm == None) {
 		attrib.colormap = MI_COLORMAP(mi);
@@ -421,7 +495,7 @@ getImage(ModeInfo * mi, XImage ** logo,
 	*graphics_format = 0;
 
 	if (bitmap_local != NULL) {
-		(void) free((void *) bitmap_local);
+		free(bitmap_local);
 		bitmap_local = (char *) NULL;
 	}
 	if (MI_BITMAP(mi) && strlen(MI_BITMAP(mi))) {
@@ -445,13 +519,19 @@ getImage(ModeInfo * mi, XImage ** logo,
 	     {
 		if ( MI_NPIXELS( mi ) > 2 )
 		  {
+		     Colormap magick_colormap;
+		     if (*ncm == None) {
+			magick_colormap = MI_COLORMAP(mi);
+		     } else {
+			magick_colormap = *ncm;
+		     }
 		     if ( MagickSuccess == MagickFileToImage( mi ,
 							     bitmap_local ,
-							     logo) )
+							     logo ,
+							      magick_colormap 
+							      ) )
 		       {
 			  *graphics_format = IS_MAGICKFILE;
-			  if ( !fixedColors( mi ) )
-			    SetImageColors( display , *ncm );
 			  *black = GetColor(mi, MI_BLACK_PIXEL(mi));
 			  (void) GetColor(mi, MI_WHITE_PIXEL(mi));
 			  (void) GetColor(mi, MI_BG_PIXEL(mi));
@@ -468,10 +548,16 @@ getImage(ModeInfo * mi, XImage ** logo,
 # ifndef STANDALONE
 		if (readable(bitmap_local)) {
 			if (MI_NPIXELS(mi) > 2) {
-				if (RasterSuccess == RasterFileToImage(mi, bitmap_local, logo)) {
+			   Colormap ras_colormap;
+			   if (*ncm == None) {
+			      ras_colormap = MI_COLORMAP(mi);
+			   } else {
+			      ras_colormap = *ncm;
+			   }
+				if (RasterSuccess == RasterFileToImage(mi,
+					       bitmap_local, logo ,
+					       ras_colormap )) {
 					*graphics_format = IS_RASTERFILE;
-					if (!fixedColors(mi))
-						SetImageColors(display, *ncm);
 					*black = GetColor(mi, MI_BLACK_PIXEL(mi));
 					(void) GetColor(mi, MI_WHITE_PIXEL(mi));
 					(void) GetColor(mi, MI_BG_PIXEL(mi));
@@ -482,7 +568,7 @@ getImage(ModeInfo * mi, XImage ** logo,
 			(void) fprintf(stderr,
 			       "could not read file \"%s\"\n", bitmap_local);
 		}
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 #ifndef USE_MONOXPM
 		if (MI_NPIXELS(mi) > 2)
 #endif
@@ -501,8 +587,8 @@ getImage(ModeInfo * mi, XImage ** logo,
 		if (*graphics_format <= 0) {
 			if (!blogo.data) {
 				if (BitmapSuccess == XbmReadFileToImage(bitmap_local,
-						 &blogo.width, &blogo.height,
-					   (unsigned char **) &blogo.data)) {
+						&blogo.width, &blogo.height,
+						&blogo.data)) {
 					blogo.bytes_per_line = (blogo.width + 7) / 8;
 					*graphics_format = IS_XBMFILE;
 					*logo = &blogo;
@@ -519,7 +605,7 @@ getImage(ModeInfo * mi, XImage ** logo,
 				       bitmap_local);
 	}
 #ifndef STANDALONE
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 	if (*graphics_format <= 0 &&
 	    ((MI_IS_FULLRANDOM(mi)) ? LRAND() & 1: default_xpm))
 #ifndef USE_MONOXPM
@@ -560,7 +646,7 @@ destroyImage(XImage ** logo, int *graphics_format)
 			break;
 		case IS_XBMFILE:
 			if (blogo.data) {
-				(void) free((void *) blogo.data);
+				free(blogo.data);
 				blogo.data = (char *) NULL;
 			}
 			break;
@@ -618,7 +704,7 @@ getPixmap(ModeInfo * mi, Drawable drawable,
 	static char *bitmap_local = (char *) NULL;
 
 	if (bitmap_local) {
-		(void) free((void *) bitmap_local);
+		free(bitmap_local);
 		bitmap_local = (char *) NULL;
 	}
 	if (MI_BITMAP(mi) && strlen(MI_BITMAP(mi))) {
@@ -647,7 +733,7 @@ getModeFont(char *infont)
 	static char *localfont = (char *) NULL;
 
 	if (localfont != NULL) {
-		(void) free((void *) localfont);
+		free(localfont);
 		localfont = (char *) NULL;
 	}
 	if (infont && strlen(infont)) {
@@ -669,17 +755,17 @@ getModeFont(char *infont)
 		(void) fprintf(stderr,
 		       "could not read file \"%s\"\n", localfont);
 		if (localfont) {
-			(void) free((void *) localfont);
+			free(localfont);
 			localfont = (char *) NULL;
 		}
 	}
 	return localfont;
 }
 
-#define FROM_PROGRAM 1
-#define FROM_FORMATTEDFILE    2
-#define FROM_FILE    3
-#define FROM_RESRC   4
+#define FROM_RESRC   1
+#define FROM_FILE    2
+#define FROM_FORMATTEDFILE    3
+#define FROM_PROGRAM 4
 
 static char *def_words = (char *) "I'm out running around.";
 static int  getwordsfrom;
@@ -700,7 +786,7 @@ isRibbon(void)
 	return (getwordsfrom == FROM_RESRC);
 }
 
-#if !defined(__cplusplus) && !defined(c_plusplus)
+#if !defined(WIN32) && !defined(__cplusplus) && !defined(c_plusplus)
 extern int  pclose(FILE *);
 #endif
 
@@ -734,6 +820,80 @@ getWords(int screen, int screens)
 	}
 
 	switch (getwordsfrom) {
+		case FROM_RESRC:
+			p = message;
+			break;
+		case FROM_FILE:
+			if ((pp = my_fopen(messagefile, "r")) != NULL) {
+				while (fgets(p, BUFSIZ - strlen(&buf[screen * BUFSIZ]), pp)) {
+					if (strlen(&buf[screen * BUFSIZ]) + 1 < BUFSIZ)
+						p = &buf[screen * BUFSIZ] + strlen(&buf[screen * BUFSIZ]);
+					else
+						break;
+				}
+				(void) fclose(pp);
+				if (!buf[screen * BUFSIZ])
+					(void) sprintf(&buf[screen * BUFSIZ],
+					"file \"%s\" is empty!", messagefile);
+				p = &buf[screen * BUFSIZ];
+			} else {
+				(void) sprintf(&buf[screen * BUFSIZ],
+				 "could not read file \"%s\"!", messagefile);
+				p = &buf[screen * BUFSIZ];
+			}
+			break;
+		case FROM_FORMATTEDFILE:
+			if ((pp = my_fopen(messagesfile, "r")) != NULL) {
+				int         len_mess_file;
+
+				if (fscanf(pp, "%d", &len_mess_file)) {
+					int         no_quote;
+
+					if (len_mess_file <= 0) {
+						buf[screen * BUFSIZ] = '\0';
+					} else {
+						(void) fgets(p, BUFSIZ - strlen(&buf[screen * BUFSIZ]), pp);
+						/* get first '%%' (the one after the number of quotes) */
+						(void) fgets(p, BUFSIZ - strlen(&buf[screen * BUFSIZ]), pp);
+						no_quote = NRAND(len_mess_file);
+						for (i = 0; i <= no_quote; ++i) {
+							unsigned int len_cur = 0;
+
+							buf[screen * BUFSIZ] = '\0';
+							p = &buf[screen * BUFSIZ] + strlen(&buf[screen * BUFSIZ]);
+							while (fgets(p, BUFSIZ - strlen(&buf[screen * BUFSIZ]), pp)) {
+								if (strlen(&buf[screen * BUFSIZ]) + 1 < BUFSIZ) {
+									/* a line with '%%' contains 3 characters */
+									if ((strlen(&buf[screen * BUFSIZ]) == len_cur + 3) &&
+									    (p[0] == '%') && (p[1] == '%')) {
+										p[0] = '\0';	/* get rid of "%%" in &buf[screen * BUFSIZ] */
+										break;
+									} else {
+										p = &buf[screen * BUFSIZ] + strlen(&buf[screen * BUFSIZ]);
+										len_cur = strlen(&buf[screen * BUFSIZ]);
+									}
+								} else {
+									break;
+								}
+							}
+						}
+					}
+					if (!buf[screen * BUFSIZ])
+						(void) sprintf(&buf[screen * BUFSIZ],
+							       "file \"%s\" is empty!", messagesfile);
+					p = &buf[screen * BUFSIZ];
+				} else {
+					(void) sprintf(&buf[screen * BUFSIZ],
+						       "file \"%s\" not in correct format!", messagesfile);
+					p = &buf[screen * BUFSIZ];
+				}
+				(void) fclose(pp);
+			} else {
+				(void) sprintf(&buf[screen * BUFSIZ],
+				"could not read file \"%s\"!", messagesfile);
+				p = &buf[screen * BUFSIZ];
+			}
+			break;
 #ifndef VMS
 		case FROM_PROGRAM:
 /*-
@@ -758,46 +918,54 @@ getWords(int screen, int screens)
 						      (char *) DEF_PROGRAM : program);
 					(void) strcat(progerr, ": not found\n");
 					if (!strcmp(&buf[screen * BUFSIZ], progerr))
-						switch (NRAND(12)) {
+						switch (NRAND(14)) {
 							case 0:
 								(void) strcat(&buf[screen * BUFSIZ],
-									      "( Get with the program, bub. )\n");
+									      "(Get with the program, bub.)\n");
 								break;
 							case 1:
 								(void) strcat(&buf[screen * BUFSIZ],
-									      "( I blow my nose at you, you silly person! )\n");
+									      "(I blow my nose at you, you silly person!)");
 								break;
 							case 2:
 								(void) strcat(&buf[screen * BUFSIZ],
-									      "\nThe resource you want to\nset is `program'.\n");
+									      "(The resource you want to\nset is `program'.)");
 								break;
 							case 3:
 								(void) strcat(&buf[screen * BUFSIZ],
-									      "\nHelp!!  Help!!\nAAAAAAGGGGHHH!!  \n\n");
+									      "(Help!!  Help!!\nAAAAAAGGGGHHH!!)");
 								break;
 							case 4:
 								(void) strcat(&buf[screen * BUFSIZ],
-									      "( Hey, who called me `Big Nose'? )\n");
+									      "(Hey, who called me `Big Nose'?)");
 								break;
 							case 5:
 								(void) strcat(&buf[screen * BUFSIZ],
-									      "( Hello?  Are you paying attention? )\n");
+									      "(Hello?  Are you paying attention?)");
 								break;
 							case 6:
 								(void) strcat(&buf[screen * BUFSIZ],
-									      "sh: what kind of fool do you take me for? \n");
+									      "(sh: what kind of fool do you take me for?)");
 								break;
 							case 7:
 								(void) strcat(&buf[screen * BUFSIZ],
-									      "\nRun me with -program \"fortune -o\".\n");
+									      "(Run me with -program \"fortune -o\".)");
 								break;
 							case 8:
 								(void) strcat(&buf[screen * BUFSIZ],
-									      "( Where is your fortune? )\n");
+									      "(Where is your fortune?  This is not good.)");
 								break;
 							case 9:
 								(void) strcat(&buf[screen * BUFSIZ],
-									      "( Your fortune has not been written yet!! )");
+									      "(Your fortune has not been written yet!!)");
+								break;
+							case 10:
+								(void) strcat(&buf[screen * BUFSIZ],
+									      "(Hey, dude when are you going to fix this?)");
+								break;
+							case 11:
+								(void) strcat(&buf[screen * BUFSIZ],
+									      "(Oh, what sad times are these when\none can not get a harmless fortune.)");
 								break;
 						}
 				}
@@ -808,79 +976,6 @@ getWords(int screen, int screens)
 			}
 			break;
 #endif
-		case FROM_FORMATTEDFILE:
-			if ((pp = my_fopen(messagesfile, "r")) != NULL) {
-				int         len_mess_file;
-
-				if (fscanf(pp, "%d", &len_mess_file)) {
-					int         no_quote;
-
-					if (len_mess_file <= 0)
-						buf[screen * BUFSIZ] = '\0';
-					else {
-						(void) fgets(p, BUFSIZ - strlen(&buf[screen * BUFSIZ]), pp);
-						/* get first '%%' (the one after the number of quotes) */
-						(void) fgets(p, BUFSIZ - strlen(&buf[screen * BUFSIZ]), pp);
-						no_quote = NRAND(len_mess_file);
-						for (i = 0; i <= no_quote; ++i) {
-							unsigned int len_cur = 0;
-
-							buf[screen * BUFSIZ] = '\0';
-							p = &buf[screen * BUFSIZ] + strlen(&buf[screen * BUFSIZ]);
-							while (fgets(p, BUFSIZ - strlen(&buf[screen * BUFSIZ]), pp)) {
-								if (strlen(&buf[screen * BUFSIZ]) + 1 < BUFSIZ) {
-									/* a line with '%%' contains 3 characters */
-									if ((strlen(&buf[screen * BUFSIZ]) == len_cur + 3) &&
-									    (p[0] == '%') && (p[1] == '%')) {
-										p[0] = '\0';	/* get rid of "%%" in &buf[screen * BUFSIZ] */
-										break;
-									} else {
-										p = &buf[screen * BUFSIZ] + strlen(&buf[screen * BUFSIZ]);
-										len_cur = strlen(&buf[screen * BUFSIZ]);
-									}
-								} else
-									break;
-							}
-						}
-					}
-					(void) fclose(pp);
-					if (!buf[screen * BUFSIZ])
-						(void) sprintf(&buf[screen * BUFSIZ],
-							       "file \"%s\" is empty!", messagesfile);
-					p = &buf[screen * BUFSIZ];
-				} else {
-					(void) sprintf(&buf[screen * BUFSIZ],
-						       "file \"%s\" not in correct format!", messagesfile);
-					p = &buf[screen * BUFSIZ];
-				}
-			} else {
-				(void) sprintf(&buf[screen * BUFSIZ],
-				"could not read file \"%s\"!", messagesfile);
-				p = &buf[screen * BUFSIZ];
-			}
-			break;
-		case FROM_FILE:
-			if ((pp = my_fopen(messagefile, "r")) != NULL) {
-				while (fgets(p, BUFSIZ - strlen(&buf[screen * BUFSIZ]), pp)) {
-					if (strlen(&buf[screen * BUFSIZ]) + 1 < BUFSIZ)
-						p = &buf[screen * BUFSIZ] + strlen(&buf[screen * BUFSIZ]);
-					else
-						break;
-				}
-				(void) fclose(pp);
-				if (!buf[screen * BUFSIZ])
-					(void) sprintf(&buf[screen * BUFSIZ],
-					"file \"%s\" is empty!", messagefile);
-				p = &buf[screen * BUFSIZ];
-			} else {
-				(void) sprintf(&buf[screen * BUFSIZ],
-				 "could not read file \"%s\"!", messagefile);
-				p = &buf[screen * BUFSIZ];
-			}
-			break;
-		case FROM_RESRC:
-			p = message;
-			break;
 		default:
 			p = def_words;
 			break;
@@ -924,9 +1019,17 @@ getFontSet(Display * display)
 	int n_miss;
 	XFontSet fs;
 
-	fs = XCreateFontSet(display,
-			    DEF_MESSAGEFONTSET,
-			    &miss, &n_miss, &def);
+	if ((fs = XCreateFontSet(display,
+			(messagefontname) ? messagefontname : DEF_MESSAGEFONT,
+			&miss, &n_miss, &def)) == NULL) {
+		if (messagefontname) {
+			(void) fprintf(stderr, "can not find font: %s, using %s...\n",
+				messagefontname, DEF_MESSAGEFONT);
+			fs = XCreateFontSet(display,
+				DEF_MESSAGEFONTSET,
+				&miss, &n_miss, &def);
+		}
+	}
 	return fs;
 }
 #endif
