@@ -26,7 +26,7 @@ static const char sccsid[] = "@(#)logout.c	4.02 97/04/01 xlockmore";
  *            Anthony Thyssen <anthony@cit.gu.edu.au>.
  * 24-Feb-95: fullLock rewritten to handle non-default group names from
  *            Dale A. Harris <rodmur@ecst.csuchico.edu>
- * 13-Feb-95: Heath A. Kehoe <hakehoe@icaen.uiowa.edu>.
+ * 13-Feb-95: Heath A. Kehoe <hakehoe AT icaen.uiowa.edu>.
  *            Mostly taken from bomb.c
  * 1994:      bomb.c written.  Copyright (c) 1994 Dave Shield
  *            Liverpool Computer Science
@@ -40,6 +40,9 @@ static const char sccsid[] = "@(#)logout.c	4.02 97/04/01 xlockmore";
 #if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
 #include <syslog.h>
 #endif
+#if defined(__CYGWIN__) || defined(SOLARIS2) || defined(__FreeBSD__)
+#include <signal.h>
+#endif
 #include <sys/signal.h>
 
 extern Bool inroot, inwindow, nolock, debug;
@@ -306,6 +309,17 @@ closedownLogout(Display * display, int screens)
 
 #endif /* CLOSEDOWN_LOGOUT */
 
+#ifdef SunCplusplus
+/* #include <signal.h> */
+extern void (*signal(int, void (*)(int))) (int);
+extern int kill(pid_t, int);
+#else
+#if 0
+extern int signal(int, void *);
+extern int kill(int, int);
+#endif
+#endif
+
 #ifdef SESSION_LOGOUT
 static void
 sessionLogout(void)
@@ -314,24 +328,13 @@ sessionLogout(void)
 
 	pidstr = getenv("XSESSION");
 	if (pidstr) {
-		kill(atoi(pidstr), SIGTERM);
+		(void) kill(atoi(pidstr), SIGTERM);
 		(void) sleep(NAP_TIME);
 	}
 }
 
 #endif /* SESSION_LOGOUT */
 
-#ifdef SunCplusplus
-/* #include <signal.h> */
-extern void (*signal(int, void (*)(int))) (int);
-extern int  kill(pid_t, int);
-#else
-#if 0
-extern int  signal(int, void *);
-extern int  kill(int, int);
-#endif
-#endif
-
 static void
 uglyLogout(void)
 {
@@ -365,6 +368,10 @@ uglyLogout(void)
 	exit(-1);
 }
 
+#if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
+extern void syslogStop(char *);
+#endif
+
 void
 logoutUser(Display * display
 #ifdef CLOSEDOWN_LOGOUT
@@ -374,8 +381,6 @@ logoutUser(Display * display
 {
 
 #if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
-	extern      syslogStop(char *);
-
 	syslog(SYSLOG_INFO, "%s: expired. closing down (uid %d) on %s\n",
 	       ProgramName, getuid(), getenv("DISPLAY"));
 	syslogStop(XDisplayString(display));
@@ -458,6 +463,10 @@ logoutUser(Display * display
 #endif
 #endif
 
+#if defined( USE_BUTTON_LOGOUT ) && !defined( NO_NEGATIVE_LOGOUT )
+extern int  logoutButton;
+#endif
+
 int
 fullLock(void)
 {
@@ -476,9 +485,12 @@ fullLock(void)
 	struct group *gp;
 	FILE       *fp;
 	char        buf[BUFSIZ];
-	extern int  logoutButton;
-
-	if (inwindow || inroot || nolock || logoutButton < 0 /*|| debug */ )
+#if defined( USE_BUTTON_LOGOUT ) && !defined( NO_NEGATIVE_LOGOUT )
+	if (logoutButton < 0)
+		return (FULL_LOCK);
+#endif
+	/* The debug portion may depend on what you are debugging.  :) */
+	if (inwindow || inroot || nolock /*|| debug */ )
 		return (FULL_LOCK);	/* (mostly) harmless user */
 	uid = getuid();
 	/* Do not try to logout root! */
