@@ -6,7 +6,7 @@ static const char sccsid[] = "@(#)magick.c     5.00 00/10/26 xlockmore";
 /*-
  * Utilities for Reading images using ImageMagick
  *
- * Copyright (c) 2000 by J.Jansen (joukj@hrem.nano.tudelft.nl)
+ * Copyright (c) 2000-2011 by J.Jansen (joukj AT hrem.nano.tudelft.nl)
  *
  * See xlock.c for copying information.
  *
@@ -14,6 +14,7 @@ static const char sccsid[] = "@(#)magick.c     5.00 00/10/26 xlockmore";
  * 26-Oct-00: Created
  * 15-Mar-06: Corrected for other QuantumDepth than 8
  * 04-Apr-06: Corrected for TrueColor visuals
+ * 18-Oct-11: Upgraded for current version of Imagemagick
  */
 
 #include "xlock.h"
@@ -30,9 +31,15 @@ MagickFileToImage(ModeInfo * mi, char *filename, XImage ** image ,
 {
    Image* imageData = (Image*) NULL;
    ImageInfo imageInfo;
+#if MagickLibVersion < 0x700
    ExceptionInfo exception;
    PixelPacket* pixdata;
-   IndexPacket* tmpdata0;
+#else
+   ExceptionInfo* exception;
+   const Quantum* pixdata;
+#endif
+   PixelPacket *pixtmp2, *pixtmp2_save;
+   int* tmpdata0;
    int i, fuzz, depth, format;
    unsigned long black, white, fgpix, bgpix;
    XColor      fgcol, bgcol;
@@ -48,8 +55,13 @@ MagickFileToImage(ModeInfo * mi, char *filename, XImage ** image ,
    GetImageInfo( &imageInfo );
    imageInfo.dither = 0;
    (void) strcpy( imageInfo.filename , filename );
+#if MagickLibVersion < 0x700
    GetExceptionInfo( &exception );
    imageData = ReadImage ( &imageInfo, &exception );
+#else
+   exception = AcquireExceptionInfo();
+   imageData = ReadImage ( &imageInfo, exception );
+#endif
    if ( imageData == (Image*) NULL )
      {
 	(void) fprintf ( stderr , "Error reading image %s\n", imageInfo.filename );
@@ -98,85 +110,103 @@ MagickFileToImage(ModeInfo * mi, char *filename, XImage ** image ,
 	return MagickColorError;
      }
 
-   if ( imageData->colors == 0 )
      {
+#if MagickLibVersion < 0x700
 	PixelPacket* pixtmp;
 
+#if 1
+	/* Deprecated but seems to work */
 /* PURIFY reports a memory leak on the next line */
 	AllocateImageColormap ( imageData , COLORMAP_SIZE );
 	pixdata = GetImagePixels( imageData , 0 , 0 , imageData->columns ,
 					     imageData->rows );
-	imageData->colors = imageData->columns*imageData->rows;
-	tmpdata0 = (IndexPacket *) malloc((int) ( xlockimage.width *
-						  xlockimage.height *
-						  sizeof(IndexPacket)));
+#else
+	/* Get undefined reference */
+	AcquireImageColormap ( imageData , COLORMAP_SIZE );
+	pixdata = GetAuthenticPixels( imageData , 0 , 0 , imageData->columns ,
+					     imageData->rows , &exception );
+#endif
+#else
+	const Quantum* pixtmp;
 
-#if (QuantumDepth == 8)
+	pixdata = GetVirtualPixels( imageData , 0 , 0 ,
+						     imageData->columns ,
+						     imageData->rows ,
+						     exception );
+#endif
+	
+	imageData->colors = COLORMAP_SIZE + 1;
+	tmpdata0 = (int *) malloc((int) ( xlockimage.width *
+						  xlockimage.height *
+						  sizeof(int)));
+
+	pixtmp2 = (PixelPacket*) malloc( imageData->columns * imageData->rows *
+					 sizeof( PixelPacket ) );
+#if (QuantumDepth == 8) || (MAGICKCORE_QUANTUM_DEPTH == 8 )
 	for ( fuzz=0; imageData->colors > COLORMAP_SIZE; fuzz++ )
-#elif (QuantumDepth == 16)
+#elif (QuantumDepth == 16) || (MAGICKCORE_QUANTUM_DEPTH == 16 ) 
 	for ( fuzz=0; imageData->colors > COLORMAP_SIZE; fuzz+=256 )
-#elif (QuantumDepth == 32)
+#elif (QuantumDepth == 32) || (MAGICKCORE_QUANTUM_DEPTH == 32 )
 	for ( fuzz=0; imageData->colors > COLORMAP_SIZE; fuzz+=65536 )
-#elif (QuantumDepth == 64)
+#elif (QuantumDepth == 64) || (MAGICKCORE_QUANTUM_DEPTH == 64 )
 	for ( fuzz=0; imageData->colors > COLORMAP_SIZE; fuzz+=4294967296 )
 #else
 #error "ImageMagick configuration error"
 #endif
 	  {
 	     int num_col , j , x , y;
+	     Quantum red , green , blue;
 
 	     pixtmp = pixdata;
 	     num_col = 0;
+	     pixtmp2_save = pixtmp2;
 	     for ( i=0; i<imageData->columns*imageData->rows; i++)
 	       {
-		  PixelPacket* pixtmp2;
-
-		  pixtmp2 = imageData->colormap;
+#if MagickLibVersion < 0x700
+		  red = (int) pixtmp->red;
+		  green = (int) pixtmp->green;
+		  blue = (int) pixtmp->blue;
+#else
+		  red = GetPixelRed(imageData,pixtmp);
+		  green = GetPixelGreen(imageData,pixtmp);
+		  blue = GetPixelBlue(imageData,pixtmp);
+#endif
 		  for ( j=0; j<num_col; j++ )
 		    {
-		       if ( ABS( pixtmp->red - pixtmp2->red ) +
-			 ABS( pixtmp->green - pixtmp2->green ) +
-			 ABS( pixtmp->blue - pixtmp2->blue ) < fuzz )
-			 break;
+		       if ( ABS( red - pixtmp2->red ) +
+			    ABS( green - pixtmp2->green ) +
+			    ABS( blue - pixtmp2->blue ) < fuzz )
+			 {
+			    pixtmp2 = pixtmp2_save;
+			    break;
+			 }
 		       pixtmp2++;
 		    }
 		  if ( j == num_col )
 		    {
-		       pixtmp2->red = pixtmp->red;
-		       pixtmp2->green = pixtmp->green;
-		       pixtmp2->blue = pixtmp->blue;
+		       pixtmp2->red = red;
+		       pixtmp2->green = green;
+		       pixtmp2->blue = blue;
 		       num_col++;
 		       if ( num_col == COLORMAP_SIZE )
 			 {
 			    num_col++;
+			    pixtmp2 = pixtmp2_save;
 			    break;
 			 }
 		    }
 		  tmpdata0[ i ] = j;
+#if MagickLibVersion < 0x700
 		  pixtmp++;
+#else
+		  pixtmp += GetPixelChannels(imageData);
+#endif
+		  pixtmp2 = pixtmp2_save;
 	       }
 	     imageData->colors = num_col;
+	     pixtmp2 = pixtmp2_save;
 	  }
      }
-   else
-     {
-	if ( imageData->colors > COLORMAP_SIZE )
-	  {
-	     imageData->colors = COLORMAP_SIZE;
-	     (void) fprintf( stderr ,
-			    "wrong colour reducing algorithm used\n" );
-	     (void) fprintf( stderr ,
-	     "Please notify the maintainer that this statement is reached\n" );
-	  }
-   /* get pixel information */
-   if ( !( pixdata = GetImagePixels( imageData , 0 , 0 , imageData->columns ,
-				    imageData->rows ) ) )
-     {
-	(void) fprintf( stderr , "Error getting pixels\n" );
-	return MagickFileInvalid;
-     }
-	tmpdata0 = GetIndexes ( imageData );
-     }
 
    /*set up colourmap */
 	black = MI_BLACK_PIXEL(mi);
@@ -212,26 +242,27 @@ MagickFileToImage(ModeInfo * mi, char *filename, XImage ** image ,
    /* supply data and colours*/
 	xlockimage.colors = imageData->colors;
 	pixel_num = malloc( imageData->colors * sizeof( unsigned long ) );
+	pixtmp2_save = pixtmp2;
 	for ( i=0 ; i<xlockimage.colors ; i++ )
 	  {
 	     XColor Xcol;
 	     
-#if (QuantumDepth == 8)
-	     xlockimage.red[ i ] =  imageData->colormap->red * 257;
-	     xlockimage.green[ i ] = imageData->colormap->green *257;
-	     xlockimage.blue[ i ] = imageData->colormap->blue * 257;
-#elif (QuantumDepth == 16)
-	     xlockimage.red[ i ] =  imageData->colormap->red;
-	     xlockimage.green[ i ] = imageData->colormap->green;
-	     xlockimage.blue[ i ] = imageData->colormap->blue;
-#elif (QuantumDepth == 32)
-	     xlockimage.red[ i ] =  imageData->colormap->red / 256;
-	     xlockimage.green[ i ] = imageData->colormap->green / 256;
-	     xlockimage.blue[ i ] = imageData->colormap->blue/ 256;
-#elif (QuantumDepth == 64)
-	     xlockimage.red[ i ] =  imageData->colormap->red / 65537;
-	     xlockimage.green[ i ] = imageData->colormap->green / 65537;
-	     xlockimage.blue[ i ] = imageData->colormap->blue/ 65537;
+#if (QuantumDepth == 8) || (MAGICKCORE_QUANTUM_DEPTH == 8 )
+	     xlockimage.red[ i ] =  pixtmp2->red * 257;
+	     xlockimage.green[ i ] = pixtmp2->green *257;
+	     xlockimage.blue[ i ] = pixtmp2->blue * 257;
+#elif (QuantumDepth == 16) || (MAGICKCORE_QUANTUM_DEPTH == 16 )
+	     xlockimage.red[ i ] =  pixtmp2->red;
+	     xlockimage.green[ i ] = pixtmp2->green;
+	     xlockimage.blue[ i ] = pixtmp2->blue;
+#elif (QuantumDepth == 32) || (MAGICKCORE_QUANTUM_DEPTH == 32 )
+	     xlockimage.red[ i ] =  pixtmp2->red / 256;
+	     xlockimage.green[ i ] = pixtmp2->green / 256;
+	     xlockimage.blue[ i ] = pixtmp2->blue/ 256;
+#elif (QuantumDepth == 64) || (MAGICKCORE_QUANTUM_DEPTH == 64 )
+	     xlockimage.red[ i ] =  pixtmp2->red / 65537;
+	     xlockimage.green[ i ] = pixtmp2->green / 65537;
+	     xlockimage.blue[ i ] = pixtmp2->blue/ 65537;
 #else
 #error "ImageMagick configuration error"
 #endif
@@ -240,9 +271,9 @@ MagickFileToImage(ModeInfo * mi, char *filename, XImage ** image ,
 	     Xcol.blue = xlockimage.blue[ i ];
 	     XAllocColor( MI_DISPLAY(mi) , m_cmap , &Xcol);
 	     pixel_num[i] = Xcol.pixel;
-	     imageData->colormap++;
+	     pixtmp2++;
 	  }
-
+   free( pixtmp2_save );
    if ( MI_IS_VERBOSE( mi ) )
      {
 	(void) printf ( "xlockimage.colors %d\n", xlockimage.colors );
@@ -252,7 +283,7 @@ MagickFileToImage(ModeInfo * mi, char *filename, XImage ** image ,
      xlockimage.red[0xff] = xlockimage.green[0xff] = xlockimage.blue[0xff] = 0;
 
    {
-	     IndexPacket* tmpdata;
+	     int* tmpdata;
 	     int x , y;
 
 	     tmpdata = tmpdata0;
