@@ -5,7 +5,7 @@
 
 /*-
  * Utilities for Reading images using ImageMagickSun rasterfile processing
- * 
+ *
  * Copyright (c) 2000 by J.Jansen
  *
  * See xlock.c for copying information.
@@ -17,7 +17,8 @@
 #ifdef USE_MAGICK
 #include <time.h>
 #include <sys/types.h>
-#include <magick/api.h>
+#include <stdarg.h>
+#include <magick/MagickCore.h>
 
 #include "xlockimage.h"
 
@@ -26,6 +27,7 @@
 #define MagickFileInvalid -2
 #define MagickNoMemory    -3
 
-extern int  MagickFileToImage(ModeInfo * mi, char *filename, XImage ** image);
+extern int  MagickFileToImage(ModeInfo * mi, char *filename, XImage ** image,
+			      Colormap m_cmap );
 
 #endif /* USE_MAGICK */
