@@ -10,13 +10,16 @@ static const char sccsid[] = "@(#)mode.c	5.12 04/03/09 xlockmore";
  * Revision History:
  * 01-Apr-97: Fixed memory leak in XStringListToTextProperty.
  * 23-Feb-96: Extensive revision to implement new mode hooks and stuff
- *		Ron Hitchens <ron@idiom.com>
+ *		Ron Hitchens <ron AT idiom.com>
  * 04-Sep-95: Moved over from mode.h (previously resource.h) with new
- *            "&*_opts" by Heath A. Kehoe <hakehoe@icaen.uiowa.edu>.
+ *            "&*_opts" by Heath A. Kehoe <hakehoe AT icaen.uiowa.edu>.
  *
  */
 
 #include "xlock.h"
+#ifdef USE_GL
+# include "visgl.h"
+#endif
 
 
 /* -------------------------------------------------------------------- */
@@ -40,15 +43,15 @@ LockStruct  LockProcs[] =
 	 (char *) "Shows wiggling tentacles", 0, NULL},
 #endif
 #ifdef MODE_ant
-	{(char *) "ant", init_ant, draw_ant, release_ant,
-	 refresh_ant, init_ant, (ModeHook *) NULL, &ant_opts,
+	{(char *) "ant", init_ant, draw_ant, (ModeHook *) NULL,
+	 refresh_ant, init_ant, free_ant, &ant_opts,
 	 1000, -3, 40000, -12, 64, 1.0, (char *) "",
 	 (char *) "Shows Langton's and Turk's generalized ants", 0, NULL},
 #endif
 #ifdef MODE_ant3d
 	{(char *) "ant3d", init_ant3d, draw_ant3d, release_ant3d,
 	 refresh_ant3d, init_ant3d, (ModeHook *) NULL, &ant3d_opts,
-	 5000, -3, 100000, 1, 64, 1.0, (char *) "",
+	 5000, -3, 10000, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows 3D ants", 0, NULL},
 #endif
 #ifdef MODE_apollonian
@@ -87,12 +90,6 @@ LockStruct  LockProcs[] =
 	 200000, 6, 30, 1, 64, 0.3, (char *) "",
 	 (char *) "Shows billiards", 0, NULL},
 #endif
-#ifdef MODE_biof
-	{(char *) "biof", init_biof, draw_biof, release_biof,
-	 draw_biof, init_biof, (ModeHook *) NULL, &biof_opts,
-	 10000, 800, 1, 0, 64, 1.0, (char *) "",
-	 (char *) "Shows 3D bioform", 0, NULL},
-#endif
 #ifdef MODE_blot
 	{(char *) "blot", init_blot, draw_blot, release_blot,
 	 refresh_blot, init_blot, (ModeHook *) NULL, &blot_opts,
@@ -414,7 +411,7 @@ LockStruct  LockProcs[] =
 #ifdef MODE_life1d
 	{(char *) "life1d", init_life1d, draw_life1d, release_life1d,
 	 refresh_life1d, init_life1d, (ModeHook *) NULL, &life1d_opts,
-	 10000, 1, 10, 0, 64, 1.0, (char *) "",
+	 100000, -5, 1200, -15, 64, 1.0, (char *) "",
 	 (char *) "Shows Wolfram's game of 1D Life", 0, NULL},
 #endif
 #ifdef MODE_life3d
@@ -707,7 +704,7 @@ LockStruct  LockProcs[] =
 #ifdef MODE_strange
 	{(char *) "strange", init_strange, draw_strange, release_strange,
 	 init_strange, init_strange, (ModeHook *) NULL, &strange_opts,
-	 1000, 1, 1, 1, 64, 1.0, (char *) "",
+	 10000, 1, 1, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows strange attractors", 0, NULL},
 #endif
 #ifdef MODE_superquadrics
@@ -855,16 +852,16 @@ LockStruct  LockProcs[] =
 	 refresh_bomb, change_bomb, (ModeHook *) NULL, &bomb_opts,
 	 100000, 10, 20, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows a bomb and will autologout after a time", 0, NULL},
-	{(char *) "random", init_random, draw_random, release_random,
-	 refresh_random, change_random, (ModeHook *) NULL, &random_opts,
-	 1, 1, 1, 1, 64, 1.0, (char *) "",
-	 (char *) "Shows a random mode (except blank and bomb)", 0, NULL},
-#else
-	{(char *) "random", init_random, draw_random, release_random,
-	 refresh_random, change_random, (ModeHook *) NULL, &random_opts,
-	 1, 1, 1, 1, 64, 1.0, (char *) "",
-	 (char *) "Shows a random mode (except blank)", 0, NULL},
 #endif
+	{(char *) "random", init_random, draw_random, release_random,
+	 refresh_random, change_random, (ModeHook *) NULL, &random_opts,
+	 1, 1, 1, 1, 64, 1.0, (char *) "",
+#ifdef MODE_bomb
+	 (char *) "Shows a random mode (except blank and bomb)",
+#else
+	 (char *) "Shows a random mode (except blank)",
+#endif
+	0, NULL},
 };
 
 int         numprocs = sizeof (LockProcs) / sizeof (LockProcs[0]);
@@ -1242,9 +1239,28 @@ call_release_hook(LockStruct * ls, ModeInfo * mi)
 		return;
 	}
 	MI_LOCKSTRUCT(mi) = ls;
+
+	if (ls->free_hook != NULL) {
+		int old_screen = MI_SCREEN(mi);
+		for (MI_SCREEN(mi) = 0; MI_SCREEN(mi) != MI_NUM_SCREENS(mi); ++MI_SCREEN(mi))
+			ls->free_hook (mi);
+		MI_SCREEN(mi) = old_screen;
+	}
+
 	if (ls->release_hook != NULL) {
 		ls->release_hook(mi);
 	}
+
+	if (ls->state_array) { /* MI_INIT modes only. */
+		free(*ls->state_array);
+		*ls->state_array = NULL;
+		ls->state_array = NULL;
+
+#ifdef USE_GL
+		FreeAllGL(mi); /* Should be a safe no-op for Xlib modes. */
+#endif
+	}
+
 	ls->flags &= ~(LS_FLAG_INITED);
 
 	last_initted_mode = (LockStruct *) NULL;
@@ -1261,6 +1277,25 @@ release_last_mode(ModeInfo * mi)
 	last_initted_mode = (LockStruct *) NULL;
 }
 
+void
+mi_init(ModeInfo *mi, size_t state_size, void **state_array)
+{
+	LockStruct *ls = MI_LOCKSTRUCT(mi);
+	ls->state_array = state_array;
+
+	if (!*state_array) {
+		*state_array = calloc (MI_NUM_SCREENS(mi), state_size);
+
+		if (!*state_array) {
+			return;
+		}
+	}
+
+	if (ls->free_hook)
+		ls->free_hook (mi);
+	memset ((char *)(*ls->state_array) + MI_SCREEN(mi) * state_size, 0, state_size);
+}
+
 /* -------------------------------------------------------------------- */
 
 /*-
