@@ -10,7 +10,7 @@
  * mode.h - mode management for xlock, the X Window System lockscreen.
  *
  * Copyright (c) 1991 by Patrick J. Naughton.
- * xscreensaver code, Copyright (c) 1997 Jamie Zawinski <jwz@jwz.org>
+ * xscreensaver code, Copyright (c) 1997 Jamie Zawinski <jwz AT jwz.org>
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted,
@@ -26,8 +26,8 @@
  *
  * Revision History:
  *
- * Changes maintained by David Bagley <bagleyd@tux.org>
- * 18-Mar-96: Ron Hitchens <ron@idiom.com>
+ * Changes maintained by David Bagley <bagleyd AT verizon.net>
+ * 18-Mar-96: Ron Hitchens <ron AT idiom.com>
  *		Extensive revision to define new data types for
  *		the new mode calling scheme.
  * 02-Jun-95: Extracted out of resource.c.
@@ -45,7 +45,7 @@
 /* xscreensaver compatibility layer for xlockmore modules. */
 
 /*-
- * xscreensaver, Copyright (c) 1997, 1998 Jamie Zawinski <jwz@jwz.org>
+ * xscreensaver, Copyright (c) 1997, 1998 Jamie Zawinski <jwz AT jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -57,7 +57,7 @@
  *
  * The definitions in this file make it possible to compile an xlockmore
  * module into a standalone program, and thus use it with xscreensaver.
- * By Jamie Zawinski <jwz@jwz.org> on 10-May-97; based on the ideas
+ * By Jamie Zawinski <jwz AT jwz.org> on 10-May-97; based on the ideas
  * in the older xlock.h by Charles Hannum <mycroft@ai.mit.edu>.  (I had
  * to redo it, since xlockmore has diverged so far from xlock...)
  */
@@ -214,7 +214,7 @@ typedef struct LockStruct_s {
 	ModeHook   *release_hook;	/* func to shutdown a mode */
 	ModeHook   *refresh_hook;	/* tells mode to repaint */
 	ModeHook   *change_hook;	/* user wants mode to change */
-	ModeHook   *unused_hook;	/* for future expansion */
+	ModeHook   *free_hook;	/* free a single screen */
 	ModeSpecOpt *msopt;	/* this mode's def resources */
 	int         def_delay;	/* default delay for mode */
 	int         def_count;
@@ -226,6 +226,7 @@ typedef struct LockStruct_s {
 	char       *desc;	/* text description of mode */
 	unsigned int flags;	/* state flags for this mode */
 	void       *userdata;	/* for use by the mode */
+	void       **state_array;	/* array of per-screen state, set from MI_INIT */
 } LockStruct;
 
 #define LS_FLAG_INITED		1
@@ -492,7 +493,6 @@ extern void release_last_mode(ModeInfo *);
 #ifndef NICE_ONLY
 /* comment out following defines to remove modes */
 #ifdef USE_GL
-#define MODE_biof
 #define MODE_cage
 #define MODE_fire
 #define MODE_gears
@@ -543,9 +543,7 @@ extern void release_last_mode(ModeInfo *);
 #ifdef USE_UNSTABLE
 #define MODE_run /* VMS problems as well as -debug problems */
 #endif	/* USE_UNSTABLE */
-#ifndef WIN32
 #define MODE_anemone
-#endif
 #define MODE_ant
 #define MODE_ant3d
 #define MODE_apollonian
@@ -555,15 +553,12 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_bouboule
 #define MODE_bounce
 #define MODE_braid
-#ifndef WIN32
 #define MODE_bubble
-#endif /* !WIN32 */
 #define MODE_bug
 #define MODE_clock
 #define MODE_coral
 #define MODE_crystal
 #define MODE_daisy
-#ifndef WIN32
 #define MODE_dclock
 /*-
  * Comment out dclock sub-modes to disable them from random selection.
@@ -572,12 +567,11 @@ extern void release_last_mode(ModeInfo *);
  */
 /* #define MODE_dclock_y2k */
 /* #define MODE_dclock_millennium */
+#ifndef WIN32
 #define MODE_decay
 #endif /* !WIN32 */
 #define MODE_deco
-#ifndef WIN32
 #define MODE_deluxe
-#endif /* !WIN32 */
 #define MODE_demon
 #define MODE_dilemma
 #define MODE_discrete
@@ -605,9 +599,7 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_hop
 #define MODE_hyper
 #define MODE_ico
-#ifndef WIN32
 #define MODE_ifs
-#endif /* !WIN32 */
 #define MODE_image
 #define MODE_juggle
 #define MODE_julia
@@ -655,12 +647,12 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_spline
 #define MODE_star
 #define MODE_starfish
-#ifndef WIN32
 #define MODE_strange
-#endif /* !WIN32 */
 #define MODE_swarm
 #ifndef WIN32
 #define MODE_swirl
+#endif /* !WIN32 */
+#ifndef WIN32
 #define MODE_t3d
 #endif /* !WIN32 */
 #define MODE_tetris
@@ -768,8 +760,8 @@ extern ModeSpecOpt anemone_opts;
 #ifdef MODE_ant
 extern ModeHook init_ant;
 extern ModeHook draw_ant;
-extern ModeHook release_ant;
 extern ModeHook refresh_ant;
+extern ModeHook free_ant;
 extern ModeSpecOpt ant_opts;
 #endif
 
@@ -823,15 +815,6 @@ extern ModeHook refresh_bat;
 extern ModeSpecOpt bat_opts;
 #endif
 
-#ifdef MODE_biof
-extern ModeHook init_biof;
-extern ModeHook draw_biof;
-extern ModeHook release_biof;
-extern ModeHook refresh_biof;
-extern ModeHook change_fire;
-extern ModeSpecOpt biof_opts;
-#endif
-
 #ifdef MODE_billiards
 extern ModeHook init_billiards;
 extern ModeHook draw_billiards;
@@ -1885,6 +1868,16 @@ extern int  numprocs;
   }
 #endif
 
+#define MI_INIT(mi, state_array) \
+  { \
+    mi_init ((mi), sizeof(*(state_array)), (void **)&(state_array)); \
+    if (!state_array) \
+      return; \
+  }
+extern void mi_init (ModeInfo *, size_t, void **);
+
+#define XSCREENSAVER_MODULE(CLASS,PREFIX) 
+
 #endif /* STANDALONE */
 /* -------------------------------------------------------------------- */
 #endif /* __XLOCK_MODE_H__ */
