@@ -10,7 +10,7 @@ static const char sccsid[] = "@(#)passwd.c	5.00 2000/11/01 xlockmore";
  *
  * Revision History:
  *
- * Changes maintained by David Bagley <bagleyd@tux.org>
+ * Changes maintained by David Bagley <bagleyd AT verizon.net>
  * 23-Feb-2003: Timothy Reed <treed1@twcny.rr.com>
  *              Updated PAM_conv to call over to xlock to display messages
  *              and return responses to the calling PAM
@@ -28,11 +28,11 @@ static const char sccsid[] = "@(#)passwd.c	5.00 2000/11/01 xlockmore";
  *              Added in ability for any of the root accounts to unlock
  *              screen.  Message now gets sent to syslog if root does the
  *              unlocking.
- * 23-Dec-1995: Ron Hitchens <ron@idiom.com> reorganized.
+ * 23-Dec-1995: Ron Hitchens <ron AT idiom.com> reorganized.
  * 10-Dec-1995: More context handling stuff for DCE thanks to
  *              Terje Marthinussen <terjem@cc.uit.no>.
  * 01-Sep-1995: DCE code added thanks to Heath A. Kehoe
- *              <hakehoe@icaen.uiowa.edu>.
+ *              <hakehoe AT icaen.uiowa.edu>.
  * 24-Jun-1995: Extracted from xlock.c, encrypted passwords are now fetched
  *              on start-up to ensure correct operation (except Ultrix).
  */
@@ -56,11 +56,11 @@ extern Bool grabmouse;
 extern Bool nolock;
 extern char *cpasswd;
 
-#ifdef PAM
+#ifdef USE_PAM
 #include <X11/Xlib.h>
 #endif
 
-#ifdef __CYGWIN__
+#if defined( CYGWIN ) || defined( SOLARIS2 )
 #include <crypt.h>
 #endif
 
@@ -88,7 +88,11 @@ static login_cap_t *rlc = NULL;
 #endif
 
 #include <errno.h>
-extern int  errno;
+#ifndef errno
+extern int errno;
+#endif
+
+extern char error_buf[ERROR_BUF];
 
 #if ( HAVE_FCNTL_H && (defined( USE_MULTIPLE_ROOT ) || defined( USE_MULTIPLE_USER )))
 #include <fcntl.h>
@@ -98,7 +102,6 @@ void        set_multiple(int uid);
 
 #define BUFMAX 1024		/* Maximum size of pipe buffer */
 
-
 /* Linked list to keep track of everyone that's authorized * to unlock the
    screen. */
 struct pwln {
@@ -273,7 +276,7 @@ char        user[PASSLENGTH];
 char        global_user[PASSLENGTH];
 #endif
 
-#ifdef PAM
+#ifdef USE_PAM
 
 /* used to pass the password to the conversation function */
 static char *PAM_password;
@@ -285,10 +288,13 @@ static char *PAM_password;
  */
 
 int
-PAM_conv(int num_msg, 
-	 const struct pam_message **msg,
-	 struct pam_response **resp,
-	 void *appdata_ptr)
+PAM_conv(int num_msg,
+#ifndef SOLARIS2
+		const
+#endif
+		struct pam_message **msg,
+		struct pam_response **resp,
+		void *appdata_ptr)
 {
 	int         replies = 0;
 	struct pam_response *reply = NULL;
@@ -299,75 +305,82 @@ PAM_conv(int num_msg,
 	reply = (struct pam_response *) malloc(sizeof (struct pam_response) *
 					       num_msg);
 
+/*
+ * reply[] members is not initialized!
+ * As a result - abort trap when PAM tries to free reply structure
+ * after PAM_ERROR_MSG processing.
+ */
 	if (!reply)
-		return PAM_CONV_ERR;
+		return PAM_BUF_ERR;
+
+/*
+ * So I just initialize reply here with default values and drop
+ * initialization from code below (if code matches).
+ */
+	reply[replies].resp_retcode = PAM_SUCCESS; /* be optimistic */
+	reply[replies].resp = NULL;
 
 	for (replies = 0; replies < num_msg; replies++) {
+#ifdef DEBUG
+		(void) printf( "PAM_conv: message of style (%d) received\n", msg[replies]->msg_style );
+		(void) printf( " + Message is: (%s)\n", msg[replies]->msg );
+#endif
 		switch (msg[replies]->msg_style) {
 			case PAM_PROMPT_ECHO_ON:
-  			        PAM_putText( msg[replies], &reply[replies], True );
 #ifdef DEBUG
-			        (void) printf( "Back From PAM_putText: PAM_PROMPT_ECHO_ON\n" );
-			        (void) printf( "Response is: (%s)\n, Return Code is: (%d)\n", 
-					 reply[replies].resp, reply[replies].resp_retcode );
+				(void) printf( " + Message style: PAM_PROMPT_ECHO_ON\n" );
 #endif
+				PAM_putText( msg[replies], &reply[replies], True );
 				/* PAM frees resp */
 				break;
 			case PAM_PROMPT_ECHO_OFF:
-			  if( strstr( msg[replies]->msg, "Password" ) == NULL )
-			  {
-			    PAM_putText( msg[replies], &reply[replies], False );
-			  }
-			  else
-			  {
-			    reply[replies].resp_retcode = PAM_SUCCESS;
-			    reply[replies].resp = COPY_STRING(PAM_password);
-			  }
 #ifdef DEBUG
-			        (void) printf( "Back From PAM_putText: PAM_PROMPT_ECHO_OFF\n" );
-			        (void) printf( "Response is: (%s)\n, Return Code is: (%d)\n", 
-					 reply[replies].resp, reply[replies].resp_retcode );
+				(void) printf( " + Message style: PAM_PROMPT_ECHO_OFF\n" );
 #endif
+/* Hard coding "Password"  is an issue for  non-English locales. */
+/* A list of prompts should be obtained at initial phase of authentication. */
+#ifdef PAM_PASSWORD_PROMPT
+				if( strstr( msg[replies]->msg, "Password" ) == NULL ) {
+					PAM_putText( msg[replies], &reply[replies], False );
+				} else
+#endif
+					reply[replies].resp = COPY_STRING(PAM_password);
 				/* PAM frees resp */
 				break;
 			case PAM_TEXT_INFO:
-			  if( strstr( msg[replies]->msg, "Password" ) == NULL )
-			  {
-			    PAM_putText( msg[replies], &reply[replies], False );
-			  }
-			  else
-			  {
-			    reply[replies].resp_retcode = PAM_SUCCESS;
-			    reply[replies].resp = NULL;
-			  }
 #ifdef DEBUG
-			        (void) printf( "Back From PAM_putText: PAM_PROMPT_ECHO_ON\n" );
-			        (void) printf( "Response is: (%s)\n, Return Code is: (%d)\n", 
-					 reply[replies].resp, reply[replies].resp_retcode );
+				(void) printf( " + Message style: PAM_TEXT_INFO\n" );
+#endif
+#ifdef PAM_PASSWORD_PROMPT
+				if( strstr( msg[replies]->msg, "Password" ) == NULL ) {
+					PAM_putText( msg[replies], &reply[replies], False );
+				}
 #endif
 				/* PAM frees resp */
 				break;
 			case PAM_ERROR_MSG:
-			  if( strstr( msg[replies]->msg, "Password" ) == NULL )
-			  {
-			    PAM_putText( msg[replies], &reply[replies], False );
-			  }
-			  else
-			  {
-			    reply[replies].resp_retcode = PAM_SUCCESS;
-			    reply[replies].resp = NULL;
-			  }
 #ifdef DEBUG
-			        (void) printf( "Back From PAM_putText: PAM_PROMPT_ECHO_ON\n" );
-			        (void) printf( "Response is: (%s)\n, Return Code is: (%d)\n", 
-					 reply[replies].resp, reply[replies].resp_retcode );
+				(void) printf( " + Message style: PAM_ERROR_MSG\n" );
 #endif
+#ifdef PAM_PASSWORD_PROMPT
+				if( strstr( msg[replies]->msg, "Password" ) == NULL ) {
+					PAM_putText( msg[replies], &reply[replies], False );
+				}
+#endif
+				/* PAM frees resp */
 				break;
 			default:
 				/* Must be an error of some sort... */
+#ifdef DEBUG
+				(void) printf( " + Message style: unknown\n" );
+#endif
 				free(reply);
 				return PAM_CONV_ERR;
 		}
+#ifdef DEBUG
+		(void) printf( " + Response is: (%s). Return Code is: (%d)\n",
+			reply[replies].resp, reply[replies].resp_retcode );
+#endif
 	}
 	*resp = reply;
 	return PAM_SUCCESS;
@@ -379,11 +392,12 @@ static const struct pam_conv PAM_conversation =
 	NULL
 };
 
-#endif /* PAM */
+#endif /* USE_PAM */
 
-#if !defined( ultrix ) && !defined( PAM )
+#if !defined( ultrix ) && !defined( USE_PAM )
 static char userpass[PASSLENGTH];
 static char rootpass[PASSLENGTH];
+static char *crptpass = (char *) NULL;
 #ifdef USE_XLOCK_GROUP
 #include <grp.h>
 static char *grouppass = (char *) NULL;
@@ -395,7 +409,7 @@ static int checkGroupXlockPasswds(char *buffer);
 static char root[] = ROOT;
 
 #endif
-#endif /* !ultrix && !PAM */
+#endif /* !ultrix && !USE_PAM */
 
 #ifdef DCE_PASSWD
 static int  usernet, rootnet;
@@ -443,7 +457,7 @@ static int	run_helper(const char *user, const char *pass);
 static struct passwd *
 my_passwd_entry(void)
 {
-	int         uid;
+	int uid;
 	struct passwd *pw;
 
 #ifdef HAVE_SHADOW
@@ -462,7 +476,7 @@ my_passwd_entry(void)
 			logname = getenv("USER");
 		if (logname != NULL) {
 			pw = getpwnam(logname);
-			if (pw && (pw->pw_uid != uid))
+			if (pw && ((int) pw->pw_uid != uid))
 				pw = 0;
 		}
 	}
@@ -472,7 +486,7 @@ my_passwd_entry(void)
 	if (!pw)
 		return (pw);
 #ifdef HAVE_SHADOW
-/*- 
+/*-
  * PURIFY on Solaris 2 reports a 1024 byte memory leak on the following line
  * if using NIS compatibility mode, and is internal to libc.so */
 	if ((spw = getspnam(pw->pw_name)) != NULL) {
@@ -493,7 +507,6 @@ int
 checkDynamic()
 {
 	struct passwd *pw;
-	char       *buf;
 
 #ifndef SAFELOG_SWCHECK
 #define SAFELOG_SWCHECK "/progs/safelog/swcheck"
@@ -505,11 +518,10 @@ checkDynamic()
 
 	if (!(pw = my_passwd_entry())) {
 		/*if ((pw = (struct passwd *) getpwuid(getuid())) == NULL) */
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-			     "%s: could not get user shell.\n", ProgramName);
-		error(buf);
-		free(buf);	/* Should never get here */
+		(void) sprintf(error_buf, "%s: could not get user shell.\n",
+			(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+			ProgramName : DEFAULT_NAME);
+		error(error_buf);
 	}
 	return (!(strcmp(pw->pw_shell, SAFELOG_SWCHECK) &&
 		  strcmp(pw->pw_shell, SAFELOG_IDENT)));
@@ -538,7 +550,6 @@ getUserName(void)
 	struct passwd *pw;
 	int         pipefd[2], n, total = 0, stat_loc;
 	pid_t       pid;
-	char       *buf;
 
 	pipe(pipefd);
 
@@ -550,11 +561,11 @@ getUserName(void)
 		_exit(0);
 	}
 	if (pid < 0) {
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-			       "%s: could not get user password (fork failed)\n", ProgramName);
-		error(buf);
-		free(buf);	/* Should never get here */
+		(void) sprintf(error_buf,
+			"%s: could not get user password (fork failed)\n",
+			(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+			ProgramName : DEFAULT_NAME);
+		error(error_buf);
 	}
 	(void) close(pipefd[1]);
 
@@ -564,39 +575,36 @@ getUserName(void)
 	wait(&stat_loc);
 
 	if (n < 0) {
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-		 "%s: could not get user name (read failed)\n", ProgramName);
-		error(buf);
-		free(buf);	/* Should never get here */
+		(void) sprintf(error_buf,
+			"%s: could not get user name (read failed)\n",
+			(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+			ProgramName : DEFAULT_NAME);
+		error(error_buf);
 	}
 	user[total] = 0;
 
 	if (total < 1) {
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-			       "%s: could not get user name (lookups failed)\n", ProgramName);
-		error(buf);
-		free(buf);	/* Should never get here */
+		(void) sprintf(error_buf,
+			"%s: could not get user name (lookups failed)\n",
+			(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+			ProgramName : DEFAULT_NAME);
+		error(error_buf);
 	}
 #else /* !HP_PASSWDETC */
 #ifdef OSF1_ENH_SEC
 	struct pr_passwd *pw;
-	char       *buf;
 
 	/*if ((pw = getprpwuid(getuid())) == NULL) */
 	if ((pw = getprpwuid(starting_ruid())) == NULL) {
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-			       "%s: could not get user name.\n", ProgramName);
-		error(buf);
-		free(buf);	/* Should never get here */
+		(void) sprintf(error_buf, "%s: could not get user name.\n",
+			(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+			ProgramName : DEFAULT_NAME);
+		error(error_buf);
 	}
 	(void) strcpy(user, pw->ufld.fd_name);
 #else /* !OSF1_ENH_SEC */
 
 	struct passwd *pw;
-	char       *buf;
 
 #ifdef HAVE_KRB5
 	krb5_context context;
@@ -645,11 +653,10 @@ getUserName(void)
 #endif /* HAVE_KRB5 */
 	if (!(pw = my_passwd_entry())) {
 		/*if ((pw = (struct passwd *) getpwuid(getuid())) == NULL) */
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-			       "%s: could not get user name.\n", ProgramName);
-		error(buf);
-		free(buf);	/* Should never get here */
+		(void) sprintf(error_buf, "%s: could not get user name.\n",
+			(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+			ProgramName : DEFAULT_NAME);
+		error(error_buf);
 	}
 	(void) strcpy(user, pw->pw_name);
 
@@ -670,34 +677,37 @@ getUserName(void)
 
 #define CPASSLENGTH 14
 #define CPASSCHARS 64
-static void
+static Bool
 gpasskey(char *pass)
 {
 	static char saltchars[CPASSCHARS + 1] =
 	"abcdefghijklmnopwrstuvwxyzABCDEFGHIJKLMNOPWRSTUVWXYZ1234567890./";
-	char        pw[9], salt[3], *pw2;
+	char        *pw, salt[3], key[9];
 #if 0
 	char       *getpass(const char *);
 #endif
-
-	(void) strncpy(pw, (char *) getpass("Key: "), sizeof (pw));
-	pw2 = (char *) getpass("Again:");
-	if (strcmp(pw, pw2) != 0) {
+	pw = (char *) getpass("  Key: ");
+	(void) strncpy(key, pw, 8);
+	key[8] = '\0';
+	pw = (char *) getpass("Again: ");
+	if (strncmp(pw, key, 8) != 0) {
 		(void) fprintf(stderr, "%s: Mismatch, try again\n", ProgramName);
-		exit(1);
+		return False;
 	}
+	(void) strncpy(key, "********", 8);
 	/* grab a random printable character that is not a colon */
 	salt[0] = saltchars[LRAND() & (CPASSCHARS - 1)];
 	salt[1] = saltchars[LRAND() & (CPASSCHARS - 1)];
 	salt[2] = '\0';
 	(void) strncpy(pass, (char *) crypt(pw, salt), CPASSLENGTH);
+	return True;
 }
 
 static void
 gpass(void)
 {
 	FILE       *fp;
-	char        xlockrc[MAXPATHLEN], *home;
+	char        xlockrc[MAXPATHLEN+1], *home;
 
 	if (!cpasswd || !*cpasswd) {
 		/*
@@ -709,16 +719,17 @@ gpass(void)
 			struct passwd *p = my_passwd_entry();
 
 			if (p == 0) {
-				char       *buf = (char *) malloc(strlen(ProgramName) + 80);
-
-				(void) sprintf(buf,
-					  "%s: Who are you?\n", ProgramName);
-				error(buf);
-				free(buf);	/* Should never get here */
+				(void) sprintf(error_buf,
+					"%s: Who are you?\n",
+					(strlen(ProgramName) >
+					ERROR_BUF - ERROR_LINE) ?
+					ProgramName : DEFAULT_NAME);
+				error(error_buf);
 			}
 			home = p->pw_dir;
 		}
 		(void) strncpy(xlockrc, home, MAXPATHLEN);
+		xlockrc[MAXPATHLEN] = '\0';
 		(void) strncat(xlockrc, "/.xlockrc", MAXPATHLEN);
 		if ((fp = my_fopen(xlockrc, "r")) == NULL) {
 			if ((fp = my_fopen(xlockrc, "w")) != NULL)
@@ -726,7 +737,10 @@ gpass(void)
 #if defined(HAVE_KRB4) || defined(HAVE_KRB5)
 			(void) strcpy(userpass, "*");
 #else
-			gpasskey(userpass);
+			if (!gpasskey(userpass)) {
+				remove(xlockrc); /* else creates annoying null file */
+				exit(1);
+			}
 #endif /* KRB4 || KRB5 */
 			if (fp)
 				(void) fprintf(fp, "%s\n", userpass);
@@ -745,7 +759,10 @@ gpass(void)
 				(void) fprintf(stderr, "%s: %s crypted password %s\n", xlockrc,
 				       buf[0] == '\0' ? "null" : "bad", buf);
 #if defined(HAVE_KRB4) || defined(HAVE_KRB5)
-				gpasskey(buf);
+				if (!gpasskey(buf)) {
+					remove(xlockrc); /* else creates annoying null file */
+					exit(1);
+				}
 #else
 				exit(1);
 #endif
@@ -768,7 +785,7 @@ gpass(void)
 
 #endif /* USE_XLOCKRC || FALLBACK_XLOCKRC */
 
-#if !defined( ultrix ) && !defined( DCE_PASSWD ) && !defined( BSD_AUTH )  && !defined ( PAM )
+#if !defined( ultrix ) && !defined( DCE_PASSWD ) && !defined( BSD_AUTH )  && !defined ( USE_PAM )
 #ifndef USE_XLOCKRC
 
 #if defined( HAVE_SHADOW ) && !defined( AFS ) && !defined( SIA )
@@ -825,7 +842,6 @@ getCryptedUserPasswd(void)
 	struct passwd *pw;
 	int         pipefd[2], n, total = 0, stat_loc;
 	pid_t       pid;
-	char       *buf;
 
 	pipe(pipefd);
 
@@ -837,11 +853,11 @@ getCryptedUserPasswd(void)
 		_exit(0);
 	}
 	if (pid < 0) {
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-			       "%s: could not get user password (fork failed)\n", ProgramName);
-		error(buf);
-		free(buf);	/* Should never get here */
+		(void) sprintf(error_buf,
+			"%s: could not get user password (fork failed)\n",
+			(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+			ProgramName : DEFAULT_NAME);
+		error(error_buf);
 	}
 	(void) close(pipefd[1]);
 
@@ -851,33 +867,32 @@ getCryptedUserPasswd(void)
 	wait(&stat_loc);
 
 	if (n < 0) {
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-			       "%s: could not get user password (read failed)\n", ProgramName);
-		error(buf);
-		free(buf);	/* Should never get here */
+		(void) sprintf(error_buf,
+			"%s: could not get user password (read failed)\n",
+			(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+			ProgramName : DEFAULT_NAME);
+		error(error_buf);
 	}
 	user[total] = 0;
 
 	if (total < 1) {
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-			       "%s: could not get user password (lookups failed)\n", ProgramName);
-		error(buf);
-		free(buf);	/* Should never get here */
+		(void) sprintf(error_buf,
+			"%s: could not get user password (lookups failed)\n",
+			(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+			ProgramName : DEFAULT_NAME);
+		error(error_buf);
 	}
 #else /* !HP_PASSWDETC */
 #ifdef OSF1_ENH_SEC
 	struct pr_passwd *pw;
-	char       *buf;
 
 	/*if ((pw = getprpwuid(getuid())) == NULL) */
 	if ((pw = getprpwuid(starting_ruid())) == NULL) {
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-		"%s: could not get encrypted user password.\n", ProgramName);
-		error(buf);
-		free(buf);	/* Should never get here */
+		(void) sprintf(error_buf,
+			"%s: could not get encrypted user password.\n",
+			(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+			ProgramName : DEFAULT_NAME);
+		error(error_buf);
 	}
 	(void) strcpy(userpass, pw->ufld.fd_encrypt);
 	/* also save encryption algorithm associated with encrypted password */
@@ -885,14 +900,13 @@ getCryptedUserPasswd(void)
 #else /* !OSF1_ENH_SEC */
 
 	struct passwd *pw;
-	char       *buf;
 
 	if (!(pw = my_passwd_entry())) {
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-		"%s: could not get encrypted user password.\n", ProgramName);
-		error(buf);
-		free(buf);	/* Should never get here */
+		(void) sprintf(error_buf,
+			"%s: could not get encrypted user password.\n",
+			(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+			ProgramName : DEFAULT_NAME);
+		error(error_buf);
 	}
 	/*if ((pw = (struct passwd *) getpwuid(getuid())) == NULL) */
 	/* Check if there is any chance of unlocking the display later...  */
@@ -901,11 +915,10 @@ getCryptedUserPasswd(void)
 #ifdef HAVE_SHADOW
 #if !defined ( AFS ) && !defined ( SIA )
 	if (passwd_invalid(pw->pw_passwd)) {
-		buf = (char *) malloc(strlen(ProgramName) + 80);
 		if (verbose || debug) {
 			(void) fprintf(stderr,
-			  "%s: it looks like you have shadow passwording.\n",
-				       ProgramName);
+				"%s: it looks like you have shadow passwording.\n",
+				ProgramName);
 			(void) fprintf(stderr, "Contact your administrator to setgid or\n");
 			(void) fprintf(stderr, "setuid %s for /etc/shadow read access.\n",
 				       ProgramName);
@@ -968,7 +981,6 @@ getCryptedRootPasswd(void)
 	struct passwd *pw;
 	int         pipefd[2], n, total = 0, stat_loc;
 	pid_t       pid;
-	char       *buf;
 
 	pipe(pipefd);
 
@@ -980,11 +992,11 @@ getCryptedRootPasswd(void)
 		_exit(0);
 	}
 	if (pid < 0) {
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-			       "%s: could not get root password (fork failed)\n", ProgramName);
-		error(buf);
-		free(buf);	/* Should never get here */
+		(void) sprintf(error_buf,
+			"%s: could not get root password (fork failed)\n",
+			(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+			ProgramName : DEFAULT_NAME);
+		error(error_buf);
 	}
 	(void) close(pipefd[1]);
 
@@ -994,39 +1006,37 @@ getCryptedRootPasswd(void)
 	wait(&stat_loc);
 
 	if (n < 0) {
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-			       "%s: could not get root password (read failed)\n", ProgramName);
-		error(buf);
-		free(buf);	/* Should never get here */
+		(void) sprintf(error_buf,
+			"%s: could not get root password (read failed)\n",
+			(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+			ProgramName : DEFAULT_NAME);
+		error(error_buf);
 	}
 	rootpass[total] = 0;
 
 	if (total < 1) {
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-			       "%s: could not get root password (lookups failed)\n", ProgramName);
-		error(buf);
-		free(buf);	/* Should never get here */
+		(void) sprintf(error_buf,
+			"%s: could not get root password (lookups failed)\n",
+			(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+			ProgramName : DEFAULT_NAME);
+		error(error_buf);
 	}
 #else /* !HP_PASSWDETC */
 #ifdef OSF1_ENH_SEC
 	struct pr_passwd *pw;
-	char       *buf;
 
 	if ((pw = getprpwnam(ROOT)) == NULL) {
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-		"%s: could not get encrypted root password.\n", ProgramName);
-		error(buf);
-		free(buf);	/* Should never get here */
+		(void) sprintf(error_buf,
+			"%s: could not get encrypted root password.\n",
+			(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+			ProgramName : DEFAULT_NAME);
+		error(error_buf);
 	}
 	(void) strcpy(rootpass, pw->ufld.fd_encrypt);
 	/* also save encryption algorithm associated with encrypted password */
 	root_oldcrypt = pw->ufld.fd_oldcrypt;
 #else /* !OSF1_ENH_SEC */
 	struct passwd *pw;
-	char       *buf;
 
 #ifdef HAVE_SHADOW
 	struct spwd *spw;
@@ -1036,11 +1046,11 @@ getCryptedRootPasswd(void)
 	if (!(pw = getpwnam(ROOT)))
 		if (!(pw = getpwuid(0))) {
 			/*if ((pw = (struct passwd *) getpwuid(0)) == NULL) */
-			buf = (char *) malloc(strlen(ProgramName) + 80);
-			(void) sprintf(buf,
-				       "%s: could not get encrypted root password.\n", ProgramName);
-			error(buf);
-			free(buf);	/* Should never get here */
+			(void) sprintf(error_buf,
+				"%s: could not get encrypted root password.\n",
+				(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+				ProgramName : DEFAULT_NAME);
+			error(error_buf);
 		}
 #ifdef HAVE_SHADOW
 /*-
@@ -1066,7 +1076,7 @@ getCryptedRootPasswd(void)
 #endif /* !VMS */
 }
 
-#endif /* !ultrix && !DCE_PASSWD && !BSD_AUTH && !PAM */
+#endif /* !ultrix && !DCE_PASSWD && !BSD_AUTH && !USE_PAM */
 
 
 #ifdef GLOBAL_UNLOCK
@@ -1088,7 +1098,7 @@ checkUser(char *buffer)
 		(void) strcpy(userpass, "*");
 	else {
 #ifdef HAVE_SHADOW
-/*- 
+/*-
  * PURIFY on Solaris 2 reports a 1024 byte memory leak on the following line
  * if using NIS compatibility mode, and is internal to libc.so */
 		if ((spw = getspnam(pw->pw_name)) != NULL) {
@@ -1148,7 +1158,8 @@ checkPasswd(char *buffer)
 	if (usernet)
 		done = check_dce_net_passwd(user, buffer);
 	else
-		done = !strcmp(userpass, crypt(buffer, userpass));
+		done = ((crptpass = crypt(buffer, userpass)) &&
+			!strcmp(userpass, crptpass));
 
 	if (done)
 		return True;
@@ -1158,10 +1169,11 @@ checkPasswd(char *buffer)
 	if (rootnet)
 		done = check_dce_net_passwd(ROOT, buffer);
 	else
-		done = !strcmp(rootpass, crypt(buffer, rootpass));
+		done = ((crptpass = crypt(buffer, rootpass)) &&
+			!strcmp(rootpass, crptpass));
 #else /* !DCE_PASSWD */
 
-#ifdef PAM
+#ifdef USE_PAM
 /*-
  * Use PAM to do authentication.  No session logging, only authentication.
  * Bail out if there are any errors.
@@ -1171,29 +1183,42 @@ checkPasswd(char *buffer)
  * Query: should we be using PAM_SILENT to shut PAM up?
  */
 	pam_handle_t *pamh;
-	int         pam_error;
+	int pam_error;
+	char *display;
 #ifdef BAD_PAM
-	uid_t       ruid;
+	uid_t ruid;
 #define BAD_PAM_SETUID	seteuid(ruid);
 #else
 #define BAD_PAM_SETUID
 #endif
 
 #define PAM_BAIL if (pam_error != PAM_SUCCESS) { \
-    pam_end(pamh, 0); BAD_PAM_SETUID return False; \
+    pam_end(pamh, pam_error); BAD_PAM_SETUID return False; \
 }
 #ifdef BAD_PAM
 	ruid = getuid();  /* the real user we are running as */
 	(void) seteuid(0); /* temporarily go to root so that pam can get shadow password */
+#endif
+#ifdef DEBUG
+	(void) printf("PAM: Before: UID (%d), EUID (%d)\n", getuid(), geteuid());
 #endif
 	PAM_password = buffer;
-	pam_error = pam_start("xlock", user, &PAM_conversation, &pamh);
+	pam_error = pam_start(DEFAULT_NAME, user, &PAM_conversation, &pamh);
 	PAM_BAIL;
+
+	display = getenv("DISPLAY");
+	if (display != NULL) {
+		pam_error = pam_set_item(pamh, PAM_TTY, display);
+		PAM_BAIL;
+	}
+
 	pam_error = pam_authenticate(pamh, 0);
+#ifdef DEBUG
+	(void) printf("PAM: pam_authenticate returns code (%d)\n", pam_error);
+#endif
 	if (pam_error != PAM_SUCCESS) {
                 if (!allowroot) {
-                        pam_end(pamh, 0);
-                        return False;
+					PAM_BAIL;
                 }
 
 		/* Try as root; bail if no success there either */
@@ -1205,15 +1230,27 @@ checkPasswd(char *buffer)
 #ifdef BAD_PAM
 	(void) seteuid(ruid); /* back to user's privileges */
 #endif
-	/* Do not do account management or credentials; credentials
-	 * are not needed and account management would just lock up
-	 * a computer and require root to come and unlock it.  Blech.
-	 */
+#ifdef DEBUG
+	(void) printf("PAM: After: UID (%d), EUID (%d)\n", getuid(), geteuid());
+#endif
+
+/* Seems to cause problems on several systems including GenToo if defined */
+/* https://bugs.gentoo.org/show_bug.cgi?id=96212 */
+#ifdef GOOD_PAM
+	pam_error = pam_acct_mgmt(pamh, 0);
+#endif
+	if (pam_error == PAM_NEW_AUTHTOK_REQD)
+		pam_error = pam_chauthtok(pamh, PAM_CHANGE_EXPIRED_AUTHTOK);
+	PAM_BAIL;
+
+	pam_error = pam_setcred(pamh, PAM_REINITIALIZE_CRED);
+	PAM_BAIL;
+
 	pam_end(pamh, PAM_SUCCESS);
 	/* If this point is reached, the user has been authenticated. */
 	done = True;
 
-#else /* !PAM */
+#else /* !USE_PAM */
 
 #ifdef ultrix
 
@@ -1224,11 +1261,11 @@ checkPasswd(char *buffer)
 	(void) setregid(rgid, egid);
 #endif
 #endif
-	done = ((authenticate_user((struct passwd *) getpwnam(user),
-				   buffer, NULL) >= 0);
+	done = (authenticate_user((struct passwd *) getpwnam(user),
+		buffer, NULL) >= 0);
 	if (!done && allowroot &&
-		 (authenticate_user((struct passwd *) getpwnam(ROOT),
-					    buffer, NULL) >= 0))) {
+			(authenticate_user((struct passwd *) getpwnam(ROOT),
+			buffer, NULL) >= 0)) {
 		done = True;
 		if (!*buffer)
 			/*
@@ -1321,7 +1358,7 @@ checkPasswd(char *buffer)
 #endif /* AFS */
 #ifdef SIA
 	{
-		if ((sia_validate_user(NULL, "xlock", 1, NULL, user, NULL,
+		if ((sia_validate_user(NULL, DEFAULT_NAME, 1, NULL, user, NULL,
 				0, NULL, buffer)) == SIASUCCESS)
 			return True;
 	}
@@ -1354,7 +1391,8 @@ checkPasswd(char *buffer)
 		 * to make this more secure.
 		 */
 		for (pwll = pwllh; pwll->next; pwll = pwll->next)
-			if (!strcmp((char *) crypt(buffer, pwll->pw_passwd), pwll->pw_passwd)) {
+			if ((crptpass = crypt(buffer, pwll->pw_passwd)) &&
+					!strcmp((char *) crptpass, pwll->pw_passwd)) {
 #if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
 				if (strcmp(user, pwll->pw_name) != 0)
 					syslog(SYSLOG_NOTICE, "%s: %s unlocked screen",
@@ -1365,11 +1403,13 @@ checkPasswd(char *buffer)
 	}
 #endif
 	if (!done) {
+		done = (
 #ifdef OSF1_ENH_SEC
-		done = (!strcmp((char *) dispcrypt(buffer, userpass, user_oldcrypt), userpass));
+			(crptpass = dispcrypt(buffer, userpass, user_oldcrypt)) &&
 #else
-		done = (!strcmp((char *) crypt(buffer, userpass), userpass));
+			(crptpass = crypt(buffer, userpass)) &&
 #endif
+			!strcmp((char *) crptpass, userpass));
 		/* userpass is used */
 		if (!*userpass && *buffer)
 			/*
@@ -1385,17 +1425,16 @@ checkPasswd(char *buffer)
 #endif
 	if (!done && allowroot &&
 #ifdef OSF1_ENH_SEC
-		!strcmp((char *) dispcrypt(buffer, rootpass, root_oldcrypt), rootpass)
+			(crptpass = dispcrypt(buffer, rootpass, root_oldcrypt)) &&
 #else
-		!strcmp((char *) crypt(buffer, rootpass), rootpass)
+			(crptpass = crypt(buffer, rootpass)) &&
 #endif
-	) {
-		done = True;	
-		if (!(rootpass && *buffer))
+			!strcmp((char *) crptpass, rootpass)) {
+		done = True;
+		if (!*rootpass && *buffer)
 			/*
 			 * root has no password, don't let him in...
 			 */
-
 			done = False;
 #if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
 		else
@@ -1408,7 +1447,7 @@ checkPasswd(char *buffer)
 #endif
 #endif /* !BSD_AUTH */
 #endif /* !ultrix */
-#endif /* !PAM */
+#endif /* !USE_PAM */
 #endif /* !DCE_PASSWD */
 #endif /* !VMS */
 
@@ -1425,9 +1464,9 @@ checkPasswd(char *buffer)
  *
  * Revision History:
  * 21-Aug-95: Added fallback to static password file [HAK]
- * 06-Jul-95: Mods by Heath A. Kehoe <hakehoe@icaen.uiowa.edu> for
+ * 06-Jul-95: Mods by Heath A. Kehoe <hakehoe AT icaen.uiowa.edu> for
  *            inclusion into xlockmore
- * May-95: Created by Phil O'Connell <philo@icaen.uiowa.edu>
+ * May-95: Created by Phil O'Connell <philo AT icaen.uiowa.edu>
  */
 
 #ifdef DCE_PASSWD
@@ -1449,8 +1488,6 @@ initDCE(void)
 	struct userpw *userpwd;
 
 #endif
-	char       *buf;
-
 	/* test to see if this user exists on the network registry */
 	valid = sec_login_setup_identity((unsigned_char_p_t) user,
 			  sec_login_no_flags, &login_context, &error_status);
@@ -1463,11 +1500,12 @@ initDCE(void)
 					       ProgramName);
 				break;
 			case sec_login_s_no_memory:
-				buf = (char *) malloc(strlen(ProgramName) + 80);
-				(void) sprintf(buf,
-					 "%s: out of memory\n", ProgramName);
-				error(buf);
-				free(buf);	/* Should never get here */
+				(void) sprintf(error_buf,
+					"%s: out of memory\n",
+					(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+					ProgramName : DEFAULT_NAME);
+				error(error_buf);
+				/* NOTREACHED */
 			default:
 				(void) fprintf(stderr,
 					       "%s: sec_login_setup_identity() returned status %d\n",
@@ -1477,11 +1515,11 @@ initDCE(void)
 
 		pwd = getpwnam(user);
 		if (!pwd || strlen(pwd->pw_passwd) < 10) {
-			buf = (char *) malloc(strlen(ProgramName) + 80);
-			(void) sprintf(buf,
-			   "%s: could not get user password\n", ProgramName);
-			error(buf);
-			free(buf);	/* Should never get here */
+			(void) sprintf(error_buf,
+				"%s: could not get user password\n",
+				(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+				ProgramName : DEFAULT_NAME);
+			error(error_buf);
 		}
 		usernet = 0;
 		(void) strcpy(userpass, pwd->pw_passwd);
@@ -1510,22 +1548,24 @@ initDCE(void)
 						       ProgramName);
 					break;
 				case sec_login_s_no_memory:
-					buf = (char *) malloc(strlen(ProgramName) + 80);
-					(void) sprintf(buf,
-					 "%s: out of memory\n", ProgramName);
-					error(buf);
-					free(buf);	/* Should never get here */
+					(void) sprintf(error_buf,
+						"%s: out of memory\n",
+						(strlen(ProgramName) < ERROR_BUF - ERROR_LINE) ?
+						ProgramName : DEFAULT_NAME);
+					error(error_buf);
+					/* NOTREACHED */
 				default:
 					(void) fprintf(stderr,
-						       "%s: sec_login_setup_identity() returned status %d\n",
-					    ProgramName, (int) error_status);
+					       "%s: sec_login_setup_identity() returned status %d\n",
+						ProgramName, (int) error_status);
 					break;
 			}
 
 			pwd = getpwuid(0);
 			if (!pwd || strlen(pwd->pw_passwd) < 10) {
 				(void) fprintf(stderr,
-					       "%s: could not get root password\n", ProgramName);
+					"%s: could not get root password\n",
+					ProgramName);
 				allowroot = 0;
 			}
 			rootnet = 0;
@@ -1585,7 +1625,7 @@ error_string(error_status_t error_status)
  *       Server used to set up and validate a login context is legitimate.
  *
  *       Created by Phil O'Connell
- *     philo@icaen.uiowa.edu
+ *     philo AT icaen.uiowa.edu
  *     Student Programmer
  *
  *-----------------------------------------------------------------------
@@ -1681,9 +1721,9 @@ check_dce_net_passwd(char *usr, char *pass)
 				(void) fprintf(stderr, "sec_login_validate_identity failed: %s\n",
 					       error_string(error_status));
 			}
+			/* done with the copy of the password */
+			free(passcpy);
 		}
-		/* done with the copy of the password */
-		free(passcpy);
 	}
 	/* make sure that the authentication service is not an imposter */
 	if (valid) {
@@ -1707,13 +1747,14 @@ krb_check_password(struct passwd *pwd, char *pass)
 	char        tkfile[MAXPATHLEN];
 
 	/* find local realm */
-	if (krb_get_lrealm(realm, 1) != KSUCCESS)
+	if (krb_get_lrealm(realm, 1) != KSUCCESS) {
 #ifdef KRB_REALM
 		/* krb_get_default_realm() may not work well on Solaris */
 		(void) strncpy(realm, KRB_REALM, sizeof (realm));
 #else
 		(void) strncpy(realm, krb_get_default_realm(), sizeof (realm));
 #endif
+	}
 	/* Construct a ticket file */
 	(void) sprintf(tkfile, "/tmp/tkt_%d", pwd->pw_uid);
 
@@ -1744,7 +1785,7 @@ krb_check_password(struct passwd *pwd, char *pass)
 
 extern krb5_error_code xlock_prompter(krb5_context, void *, const char *,
 				      const char *, int, krb5_prompt []);
-	
+
 static int
 krb_check_password(char *user, char *pass)
 {
@@ -1769,7 +1810,7 @@ krb_check_password(char *user, char *pass)
 
 	if (krb5_valid)
 		code = krb5_parse_name(kcontext, user, &me);
-	
+
 	if (! krb5_valid || (krb5_valid && code)) {
 		if ((code = krb5_cc_default(kcontext, &ccache))) {
 			com_err(ProgramName, code, "while getting default "
@@ -1792,16 +1833,16 @@ krb_check_password(char *user, char *pass)
 	/*
 	 * Set all of our various init_creds options
 	 */
-	
+
 	option = 0;
-	krb5_appdefault_boolean(kcontext, "xlock",
+	krb5_appdefault_boolean(kcontext, DEFAULT_NAME,
 				krb5_princ_realm(context, me), "forwardable",
 				option, &option);
 	krb5_get_init_creds_opt_set_forwardable(&options, option);
 
-	krb5_appdefault_string(kcontext, "xlock", krb5_princ_realm(context, me),
+	krb5_appdefault_string(kcontext, DEFAULT_NAME, krb5_princ_realm(context, me),
 			       "default_lifetime", "", &lifetimestr);
-	
+
 	if (lifetimestr && *lifetimestr) {
 		code = krb5_string_to_deltat(lifetimestr, &lifetime);
 		if (code == 0)
@@ -1840,7 +1881,7 @@ krb_check_password(char *user, char *pass)
 	 * Since the verification has succeeded, a failure at this point
 	 * will still result in returning a True.
 	 */
-	
+
 	if (!ccache && (code = krb5_cc_default(kcontext, &ccache)))
 		com_err(ProgramName, code, "while retrieving default "
 			"credential cache");
@@ -1848,7 +1889,7 @@ krb_check_password(char *user, char *pass)
 	if (!code && (code = krb5_cc_initialize(kcontext, ccache, me)))
 		com_err(ProgramName, code, "while initializing credential "
 			"cache");
-	
+
 	if (!code && (code = krb5_cc_store_cred(kcontext, ccache, &my_creds)))
 		com_err(ProgramName, code, "while storing credentials");
 
@@ -1940,7 +1981,7 @@ set_multiple(int uid)
 		}
 	}
 
-	if (pwll->next = new_pwlnode())
+	if ((pwll->next = new_pwlnode()) != NULL)
 		pwll = pwll->next;
 #else /* !BSD_AUTH */
 	/* If you thought the above was sick, then you will think this is
@@ -1985,7 +2026,7 @@ set_multiple(int uid)
 			if (pw->pw_uid != uid)
 				continue;
 #ifdef HAVE_SHADOW
-/*- 
+/*-
  * PURIFY on Solaris 2 reports a 1024 byte memory leak on the following line
  * if using NIS compatibility mode, and is internal to libc.so */
 			if ((spw = getspnam(pw->pw_name)) != NULL) {
@@ -2126,7 +2167,7 @@ void
 initPasswd(void)
 {
 	getUserName();
-#if !defined( ultrix ) && !defined( DCE_PASSWD ) && !defined( PAM )
+#if !defined( ultrix ) && !defined( DCE_PASSWD ) && !defined( USE_PAM )
 	if (!nolock && !inroot && !inwindow && grabmouse) {
 #ifdef BSD_AUTH
 		struct passwd *pwd = getpwnam(user);
@@ -2154,7 +2195,7 @@ initPasswd(void)
 #endif
 #endif /* !BSD_AUTH */
 	}
-#endif /* !ultrix && !DCE_PASSWD && !PAM */
+#endif /* !ultrix && !DCE_PASSWD && !USE_PAM */
 #ifdef DCE_PASSWD
 	initDCE();
 #endif
@@ -2171,7 +2212,7 @@ initPasswd(void)
 */
 
 #ifndef GROUP
-#define GROUP "xlock"
+#define GROUP DEFAULT_NAME
 #endif
 
 #define PASSLENGTH1 (PASSLENGTH+1)
@@ -2205,7 +2246,7 @@ getCryptedXlockGroupPasswds()
 		else {
 
 #ifdef HAVE_SHADOW
-/*- 
+/*-
  * PURIFY on Solaris 2 reports a 1024 byte memory leak on the following line
  * if using NIS compatibility mode, and is internal to libc.so */
 			if ((spw = getspnam(pw->pw_name)) != NULL) {
@@ -2240,11 +2281,13 @@ int checkGroupXlockPasswds(char *buffer)
 		return False;
 	}
 	while (grp->gr_mem[i]) {
+		unlock = (
 #ifdef OSF1_ENH_SEC
-		unlock = (!strcmp((char *) dispcrypt(buffer, &grouppass[i * PASSLENGTH1], user_oldcrypt), &grouppass[i * PASSLENGTH1]));
+			(crptpass = dispcrypt(buffer, &grouppass[i * PASSLENGTH1], user_oldcrypt)) &&
 #else
-		unlock = (!strcmp((char *) crypt(buffer, &grouppass[i * PASSLENGTH1]), &grouppass[i * PASSLENGTH1]));
+			(crptpass = crypt(buffer, &grouppass[i * PASSLENGTH1])) &&
 #endif
+			!strcmp((char *) crptpass, &grouppass[i * PASSLENGTH1]));
 #if 0
 		/* For debugging only! */
 		(void) printf("gbuffer=%s, encrypt=%s, grouppass=%s,\n", buffer,
@@ -2293,7 +2336,7 @@ run_helper(const char *user, const char *pass)
       dup2(pfd[0], 0);
 
     /* Helper is invoked as helper service-name [user] */
-    execlp(PASSWD_HELPER_PROGRAM, PASSWD_HELPER_PROGRAM,"xlock", user, NULL);
+    execlp(PASSWD_HELPER_PROGRAM, PASSWD_HELPER_PROGRAM, DEFAULT_NAME, user, NULL);
     error("Failed to execute " PASSWD_HELPER_PROGRAM);
     exit(1);
   }
