@@ -9,12 +9,36 @@ static const char sccsid[] = "@(#)sound.c	4.00 97/01/01 xlockmore";
  * See xlock.c for copying information.
  *
  * Revision History:
- * 26-Oct-01: Allow wild cards on sound-files. (joukj@hrem.stm.tudelft.nl)
+ * 26-Oct-01: Allow wild cards on sound-files. (joukj AT hrem.nano.tudelft.nl)
  */
 
 #include "xlock.h"
 #include "iostuff.h"
 
+#ifndef SOUNDPATH
+#if 0
+#define SOUNDPATH "/usr/share/xlock/sounds"
+#else
+#define SOUNDPATH "/usr/local/share/xlock/sounds"
+#endif
+#endif
+
+#ifdef DEF_PLAY
+static void
+play_sound(char *filename, Bool verbose)
+{
+	char        *progrun = NULL;
+
+	if ((progrun = (char *) malloc(strlen(DEF_PLAY) + strlen(filename) + 11)) != NULL) {
+		(void) sprintf(progrun, "( %s %s ) 2>&1", DEF_PLAY, filename);
+		/*(void) printf("%s\n", progrun);*/
+		(void) system(progrun);
+		free(progrun);
+	}
+}
+
+#endif
+
 #ifdef USE_RPLAY
 #include <rplay.h>
 
@@ -75,7 +99,7 @@ extern void PLAY_SOUND_MMOV(char* filename, Bool verbose);
 
 #ifdef USE_VMSPLAY
 /*-
- * Jouk Jansen <joukj@hrem.stm.tudelft.nl> contributed this
+ * Jouk Jansen <joukj AT hrem.nano.tudelft.nl> contributed this
  * which he found at http://axp616.gsi.de:8080/www/vms/mzsw.html
  *
  * quick hack for sounds in xlockmore on VMS
@@ -139,32 +163,8 @@ play_sound(char* filename, Bool verbose)
 }
 #endif
 
-#ifdef DEF_PLAY
-static void
-play_sound(char *filename, Bool verbose)
-{
-	char        *progrun = NULL;
-
-	if ((progrun = (char *) malloc(strlen(DEF_PLAY) + strlen(filename) + 11)) != NULL) {
-		(void) sprintf(progrun, "( %s %s ) 2>&1", DEF_PLAY, filename);
-		/*(void) printf("%s\n", progrun); */
-		(void) system(progrun);
-		free(progrun);
-	}
-}
-
-#endif
-
 #ifdef USE_ESOUND
 
-#ifndef DEFAULT_SOUND_DIR
-#if 1
-#define DEFAULT_SOUND_DIR "/usr/lib/X11/xlock/sounds/"
-#else
-#define DEFAULT_SOUND_DIR "@PREFIX@/lib/X11/xlock/sounds/"
-#endif
-#endif
-
 #ifdef HAVE_LIBESD
 
 #include <sys/stat.h>
@@ -225,7 +225,7 @@ void shutdown_sound(void)
 }
 
 /* Found this snippet from modes/glx/xpm-image.c by way of
-   Jamie Zawinski <jwz@jwz.org> Copyright (c) 1998 */
+   Jamie Zawinski <jwz AT jwz.org> Copyright (c) 1998 */
 static Bool
 bigendian (void)
 {
@@ -252,11 +252,10 @@ sound_esd_load_sample(char *file)
    char 	      *origfile;
    char 	       fullfile[MAXPATHLEN];
 
+   s = EsdSamplesList;
 #ifdef DEBUG
    (void) fprintf(stderr, "sound_esd_load_sample: %s\n", s->file);
 #endif
-
-   s = EsdSamplesList;
    while ( s && strcmp( file, s->file ) )
      s = s->next;
    if ( s && !strcmp( file, s->file ) )
@@ -264,7 +263,7 @@ sound_esd_load_sample(char *file)
 
    origfile = strdup( file ? file : "" );
 #ifdef DEBUG
-   (void) fprintf(stderr, "sound_esd_load_sample: sample not loaded: loading ...\n", origfile);
+   (void) fprintf(stderr, "sound_esd_load_sample: sample not loaded: loading %s\n", origfile);
 #endif
 
    if (file[0] != '/')
@@ -287,7 +286,7 @@ sound_esd_load_sample(char *file)
    s = EsdSamplesList;
    if ( s )
    {
-       while ( s && s->next )
+       while ( s->next )
          s = s->next;
        s->next = (EsdSample_t *) malloc(sizeof(EsdSample_t));
        if ( !s->next )
