@@ -9,7 +9,7 @@ static const char sccsid[] = "@(#)vis.c	4.07 97/01/30 xlockmore";
  * Copyright (c) 1991 by Patrick J. Naughton.
  *
  * Revision History:
- * 19-Feb-01: Separated out pure GL stuff <lassauge@mail.dotcom.fr>
+ * 19-Feb-01: Separated out pure GL stuff <lassauge AT users.sourceforge.net>
  * 23-Apr-97: Name conflict with xscreensaver, name changed.
  * 15-Oct-97: Separated out and modified.
  *
@@ -17,6 +17,7 @@ static const char sccsid[] = "@(#)vis.c	4.07 97/01/30 xlockmore";
 
 #include "xlock.h"
 #include "vis.h"
+#include "visgl.h"
 
 extern Bool verbose;
 extern Bool debug;
@@ -130,6 +131,7 @@ nameOfVisualClass(int visualclass)
 	return ("[Unknown Visual Class]");
 }
 
+#if 0
 /*-
  * setupColormap
  *
@@ -157,7 +159,7 @@ unsigned long *red_mask, unsigned long *blue_mask, unsigned long *green_mask)
 
 	return !fixedColors(mi);
 }
-
+#endif
 
 extern Bool mono;
 
@@ -183,14 +185,17 @@ extern int  VisualClassWanted;
 void
 defaultVisualInfo(Display * display, int screen)
 {
-	XVisualInfo *info_list = (XVisualInfo *) NULL,
-		*info = (XVisualInfo *) NULL, vTemplate;
-	int         n = 0;
+	XVisualInfo *info = (XVisualInfo *) NULL;
+
+#ifndef WIN32
+	XVisualInfo *info_list = (XVisualInfo *) NULL, vTemplate;
 
 	vTemplate.screen = screen;
 	vTemplate.depth = DisplayPlanes(display, screen);
 	{
-#if defined( USE_GL )		/* go with what init_GL found */
+		int         n = 0;
+#if defined( USE_GL )	/* go with what init_GL found */
+
 		XVisualInfo *wantVis = (XVisualInfo *) NULL;
 
 		if (!glOK) {
@@ -286,24 +291,29 @@ defaultVisualInfo(Display * display, int screen)
 		}
 #endif
 	}
+#endif
 	if (verbose) {
 		showVisualInfo(info);
 	}
-	Scr[screen].depth = info->depth;
-	Scr[screen].visual = info->visual;
-	Scr[screen].visualclass = info->CLASS;
-	Scr[screen].colormap_size = info->colormap_size;
-	Scr[screen].red_mask = info->red_mask;
-	Scr[screen].green_mask = info->green_mask;
-	Scr[screen].blue_mask = info->blue_mask;
+	if (info != NULL) {
+		Scr[screen].depth = info->depth;
+		Scr[screen].visual = info->visual;
+		Scr[screen].visualclass = info->CLASS;
+		Scr[screen].colormap_size = info->colormap_size;
+		Scr[screen].red_mask = info->red_mask;
+		Scr[screen].green_mask = info->green_mask;
+		Scr[screen].blue_mask = info->blue_mask;
+	}
+#ifndef WIN32
 	XFree((char *) info_list);	/* NOT info ... its modified */
+#endif
 }
 
 /* has_writeable was hacked out of xscreensaver and modified for xlockmore
- * by Jouk Jansen <joukj@hrem.stm.tudelft.nl>
+ * by Jouk Jansen <joukj AT hrem.nano.tudelft.nl>
  *
  * xscreensaver, Copyright (c) 1993, 1994, 1995, 1996, 1997, 1998
- *  by Jamie Zawinski <jwz@jwz.org>
+ *  by Jamie Zawinski <jwz AT jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -319,13 +329,12 @@ has_writable_cells(ModeInfo * mi)
 	switch (MI_VISUALCLASS(mi)) {
 		case GrayScale:	/* Mappable grays. */
 		case PseudoColor:	/* Mappable colors. */
+		case DirectColor:	/* DirectColor visuals are like TrueColor, but have
+					   three colormaps - one for each component of RGB. */
 			return True;
 		case StaticGray:	/* Fixed grays. */
 		case TrueColor:	/* Fixed colors. */
 		case StaticColor:	/* (What's the difference again?) */
-		case DirectColor:	/* DirectColor visuals are like TrueColor, but have
-					   three colormaps - one for each component of RGB.
-					   Screw it. */
 			return False;
 		default:
 			abort();
