@@ -13,12 +13,11 @@ static const char sccsid[] = "@(#)xlockimage.c	4.19 00/10/26 xlockmore";
  * Revision History:
  *  3-Mar-96: Added random image selection.
  * 12-Dec-95: Modified to be a used in more than one mode
- *            <joukj@hrem.stm.tudelft.nl>
+ *            <joukj AT hrem.nano.tudelft.nl>
  * 22-May-95: Written.
  */
 
 #include "xlock.h"
-#include "iostuff.h"
 #include "xlockimage.h"
 #include <time.h>
 
